<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Log, NodeGW, DB;

class AuctionBidButler extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'auction:bidButler';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Auction Bid Butler';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $itemsQuery = \App\Models\AuctionItems::whereRaw('status = 0 AND (UNIX_TIMESTAMP(ends_at) - UNIX_TIMESTAMP(NOW())) >= 1 AND (UNIX_TIMESTAMP(ends_at) - UNIX_TIMESTAMP(NOW())) <= 3');
        $items = $itemsQuery->get();
        $itemIds = $items->pluck('id');

        if( $items->count() > 0 ) {
            \App\Models\AuctionItems::whereIn('id', $itemIds->all())->update([
                'status' => 1
            ]);

            foreach( $items as $item ) {
                $bidButlers = $item->bidButlers;
                $bidButlersCount = $bidButlers->count();
                $newBids = [];

                if( $bidButlersCount > 0 ) {

                    if( $bidButlersCount == 1 ) {
                        if( $item->lastBidder()->user_id == $bidButlers->first()->user_id ) {
                            break;
                        }
                    }

                    $randomBids = $bidButlers->shuffle()->all();

                    foreach( $randomBids as $rb ) {

                        if( $rb->user->hasBids() > 0 ) {
                            $deduces = $rb->user->deduceBid($item->per_bid_price);
                            
                            $newBid = new \App\Models\AuctionBids;
                            $newBid->item_id = $item->id;
                            $newBid->user_id = $rb->user_id;
                            $newBid->type = 2;
                            $newBid->price = $item->per_bid_price;
                            $newBid->package_bid_price = $deduces[ 0 ]['price'];
                            $newBid->save();

                            $newBid->created_at_format = $newBid->created_at->format('d.m.y H:i:s');
                            $newBid->username = $newBid->user->username;

                            $newBids[] = $newBid->toArray();

                            if( $rb->counter == 1 ) 
                                $rb->delete();
                            else {
                                $rb->counter = $rb->counter - 1;
                                $rb->save();
                            }
                        }
                    }

                    $item->current_price = $item->current_price + ( $item->per_bid_price * $bidButlersCount );
                    $item->ends_at = $item->ends_at->addSeconds(30);


                    $item->last_bid_id = $newBid->id;

                    $item->save();

                    $item->ends_at_ts = $item->ends_at->timestamp;

                    NodeGW::send('api/socketio/auction/newBid', [
                        'item' => $item->toArray(),
                        'newBids' => $newBids
                    ]);
                }
            }

            \App\Models\AuctionItems::whereIn('id', $itemIds->all())->update([
                'status' => 0
            ]);
        }
    }
}