<?php

namespace App\Console\Commands;

use App\Facades\NodeGWManager;
use App\Models\AuctionItems;
use App\Models\User;
use Illuminate\Console\Command;
use Illuminate\Support\Arr;
use Log, NodeGW, Carbon, Config, DB, File,  MailManager;

class AuctionValidation extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'auction:validation';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Auction validation';

    protected $users = [];

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->users = User::getBotIds();

        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {

        //$this->writeLogLine(true, 'Auction Validation Started');
        //$this->info(Carbon::now() . ' - Start of command: auction:validation');
        $items = \App\Models\AuctionItems::
            where('ends_at', '<=', Carbon\Carbon::now()->subSecond()->format('Y-m-d H:i:s'))
            ->where('status', 0)
            ->get();

        if( $items && $items->count() > 0 ) {
            foreach( $items as $item ) {
                $last_bid = \App\Models\AuctionBids::where('item_id', $item->id)->orderby('id', 'desc')->first();

                if(!$last_bid){
                    $this->renew($item);
                    continue;
                }

                $last_bidder_user = \App\Models\User::where('id', $last_bid->user_id)->first();

                if(!$last_bidder_user){
                    $this->renew($item);
                    continue;
                }

                if($item->bot_need_win == 1){
                    $bot = \App\Models\User::where('id', $this->users[$this->__generateRandom(0, (int) count($this->users) - 1)])->first();
                    //$this->info(Carbon::now() . ' - Selected user: ' . $bot->id);

                    $is_bot_last_bidder = false;
                    for($i = 0; $i < count($this->users); $i++)
                    {
                        if($last_bidder_user->id == $this->users[$i]){
                            $is_bot_last_bidder = true;
                            break;
                        }
                    }

                    //$this->info(Carbon::now() . ' - Bot bidder? ' . $is_bot_last_bidder);

                    ///////////////////////////////
                    //if($is_bot_last_bidder == false){
                    //    $this->autoBid_ManualBid($item, $bot, $last_bid);
                    //    $last_bidder_user = $bot;
                    //}
                    ///////////////////////////////

                    //$this->info(Carbon::now() . ' - Last bidder: ' . $last_bidder_user->id);
                }

                $item->status = 2;
                $item->save();

                dispatch(new \App\Jobs\ArchiveData('auction_bids', 20, 'item_id', $item->id));

                if(isset($last_bidder_user)){
                    $this->sendMail($item, $last_bidder_user);
                }

                if($item->auto_renew){
                    $this->renew($item);
                }
               

                \App\Facades\NotificationManager::create([
                    'user_id' => $last_bidder_user->id,
                    'type' => 'auction-wonitem',
                    //'content_id' => $last_bid_id
                    'content_id' => $item->id
                ], $last_bidder_user);
                NodeGWManager::send('api/socketio/auction/ended',
                [
                    'item' => $item->toArray()
                ]);
                /*$last_bid = \App\Models\AuctionBids::where('item_id', $item->id)->orderby('id', 'desc')->first();
                $last_bidder_user = \App\Models\User::where('id', $last_bid->user_id)->first();

                NodeGWManager::send('api/socketio/user/wonItemNotification',
                [
                    'item' => $item->toArray(),
                    'user' =>  $last_bidder_user->id
                ]);*/
            }
        }

        //$this->info(Carbon::now() . ' - End of command: auction:validation');
    }
    function renew($item){	
        if($item->auto_renew) {


            $new = new AuctionItems();
            $new->type_id = $item->type_id;
            $new->category_id = $item->category_id;
            $new->per_bid_price = $item->per_bid_price;
            $new->start_price = $item->start_price;

            $new->buy_price = $item->buy_price;
            $new->bot_bids_start_from = $item->bot_bids_start_from;
            $new->bot_bids_target_price = $item->bot_bids_target_price;

            ///////////////////////////
            $new->parent_item_id = $item->id;
            $new->user_bidders_price = $item->user_bidders_price;
            $new->inheritance_cycle = $item->inheritance_cycle + 1;

            $new->user_win_price_limit = $item->user_win_price_limit;

            ///////////////////////////

            if(!is_null($item->user_win_price_limit) && $item->user_win_price_limit != 0) {
                //If the income limit is set
                if ($item->user_win_price_limit <= $item->user_bidders_price) {
                    //If we reached the income limit
                    if(!in_array($item->lastBidder()->first()->user_id, User::getBotIds())){
                        //So we are here, when the last cycle ended with user win
                        //We have to restart the item cycle with initialized parameters
                        $new->user_bidders_price = 0;
                        $new->bot_need_win = 1;
                    }
                    else {
                        //When it reached, but bot won, in the next cycle, user can win it.
                        $new->bot_need_win = 0;
                    }

                }
                else {
                    //We not reached the income limit
                    $new->bot_need_win = 1;
                }
            }
            else {
                //If the income limit is not defined
                $new->bot_need_win = $item->bot_need_win;
            }
            //$new->bot_need_win = $item->bot_need_win;
            //$new->auto_renew = $item->auto_renew;

            ///////////////////////////

            //Infinite loop of item generation
            $new->auto_renew = $item->auto_renew;

            $new->ends_at = \Illuminate\Support\Carbon::now()->addDay(env('RENEW_TIME', 30));
            $new->isbusy = 0;
            //$new->current_price = 0;
            $new->status = 0;



            $new->current_price = $item->start_price;


            foreach (\Illuminate\Support\Facades\Config::get('app.languages') as $key => $lang) {
                $new->translateOrNew($key)->title = $item->translate($key)->title;
                $new->translateOrNew($key)->description = $item->translate($key)->description;
            }

            if ($new->save()) {
                foreach ($item->images as $image) {
                    $ext = pathinfo($image->image, PATHINFO_EXTENSION);
                    if ((!file_exists(storage_path() . '/app/public/auction/items/' . $image->image)) || (!file_exists(storage_path() . '/app/public/auction/items/thumbnails/' . $image->image))) {
                        continue;
                    }
                    do {
                        $new_file = str_random(40) . '.' . $ext;
                    } while (file_exists(storage_path() . '/app/public/auction/items/' . $new_file));

                    \Illuminate\Support\Facades\File::copy(storage_path() . '/app/public/auction/items/' . $image->image, storage_path() . '/app/public/auction/items/' . $new_file);
                    \Illuminate\Support\Facades\File::copy(storage_path() . '/app/public/auction/items/thumbnails/' . $image->image, storage_path() . '/app/public/auction/items/thumbnails/' . $new_file);

                    $new_image = new \App\Models\AuctionImages;
                    $new_image->item_id = $new->id;
                    $new_image->image = $new_file;
                    $new_image->order = $image->order;
                    $new_image->save();
                }

                $item->auto_renew = 0;
                $item->save();
            }
            //return false;
        }
    }
    public function sendMail($item,$winner){
        //print ("sending mail regarding the following auction:".$item->id);
        \App\Facades\MailManager::send('auctions.won',  trans('mail.auction.youWon',[],$winner->language) , ['id' => $item->id, 'title'=>$item->title ], $winner);

    }
    public function __generateRandom( $min, $max ) {
        return (int) random_int( (int) $min, (int) $max );
    }

    public function autoBid_ManualBid($item, $bot, $last_bid)
    {
        //$this->info(Carbon::now() . ' - Bidding: ' . $bot->id);
        if (!isset($bot))
            return false;
        if (!$bot->hasBids()) {
            $new_package = new \App\Models\AuctionUsersPackages;
            $new_package->user_id = $bot->id;
            $new_package->package_id = 2;
            $new_package->bids = 1000000;
            $new_package->save();
        }

        if(Carbon::parse($item->ends_at)->subSeconds(1) >  $last_bid->created_at){
            $new_bid_time = Carbon::parse($item->ends_at)->subSeconds(1);
        }
        else{
            $new_bid_time = $item->ends_at;
        }

        $newBid = new \App\Models\AuctionBids;
        $newBid->item_id = $item->id;
        $newBid->user_id = $bot->id;
        $newBid->type = 1;
        $newBid->price = $item->per_bid_price;
        $newBid->created_at = $new_bid_time;
        $newBid->updated_at = $new_bid_time;


        $deduces = $bot->deduceBid($item->per_bid_price);
        $newBid->package_bid_price = $deduces[0]['price'];

        if ($newBid->save()) {

            $item->current_price = $item->current_price + $newBid->price;

            $item->last_bid_id = $newBid->id;

            $item->save();
        }

    }

    public function writeLogLine($with_ts, $output){
        if($with_ts) {
            \Illuminate\Support\Facades\Log::info(\Illuminate\Support\Carbon::now(). ' - ' . $output);
        }
        else{
            \Illuminate\Support\Facades\Log::info($output);
        }
    }
}