<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Log, Carbon, NodeGW;

class AutoAuctionSimulation extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'auction:simulation';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Auction simulation';

    protected $itemsNumber = 6;
    protected $autoItems = [];
    protected $users = [];

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();

        $items = [
            [
                'type_id' => 1,
                'category_id' => 2,
                'title' => 'Apple iPhone 7',
                'description' => 'A brand new Apple iPhone 7',
                'buy_price' => 850.00,
                'ends_at' => 25,
                'images' => [9999]
            ],
            [
                'type_id' => 1,
                'category_id' => 2,
                'title' => 'Apple iPhone 7 Plus',
                'description' => 'A brand new Apple iPhone 7 plus',
                'buy_price' => 1050.00,
                'ends_at' => 60,
                'images' => [9999]
            ],
            [
                'type_id' => 1,
                'category_id' => 2,
                'title' => 'Apple TV',
                'description' => 'A brand new Apple TV',
                'buy_price' => 400.00,
                'ends_at' => 30,
                'images' => [9997]
            ],
            [
                'type_id' => 1,
                'category_id' => 2,
                'title' => 'Apple iPhone X',
                'description' => 'A brand new Apple iPhone X',
                'buy_price' => 1500.00,
                'ends_at' => 40,
                'images' => [9998]
            ],
            [
                'type_id' => 1,
                'category_id' => 3,
                'title' => 'Samsung UHD Tv 65"',
                'description' => 'A brand new Samsung UHD Tv 65"',
                'buy_price' => 3100.00,
                'ends_at' => 15,
                'images' => [9995]
            ],
            [
                'type_id' => 1,
                'category_id' => 3,
                'title' => 'LG OLed Tv 50"',
                'description' => 'A brand new LG OLed Tv 50"',
                'buy_price' => 2200.00,
                'ends_at' => 10,
                'images' => [9996]
            ],
        ];

        foreach( $items as $item ) {
            $this->autoItems[] = (object) $item;
        }

        $this->users = [
            900001,
            900002,
            900003,
            900004,
            900005,
            900006,
            /*900007,
            900008,
            900009,
            900010*/
        ];
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        //if( !($this->__generateRandom(1, 10) % 2) ) return;

        $items = \App\Models\AuctionItems::where('status', 0)->get();

        /* Create elements if some ended */
        if( $items->count() < $this->itemsNumber ) {
            $this->__createItems( $items->count() );
            $items = \App\Models\AuctionItems::where('status', 0)->get()->random( $this->__generateRandom(1, count($this->itemsNumber)) )->all();
        } else {
            $items = $items->random( $this->__generateRandom(1, count($this->itemsNumber)) )->all();
        }

        foreach( $items as $i ) {
            $users = \App\Models\User::whereIn('id', $this->users)->get()->random( $this->__generateRandom(1, count($this->users)) )->all();

            foreach( $users as $u ) {
                if( !$u->hasBids() ) {
                    $new_package = new \App\Models\AuctionUsersPackages;
                    $new_package->user_id = $u->id;
                    $new_package->package_id = 2;
                    $new_package->bids = 1000000;
                    $new_package->save();
                }

                $whatToDo = $this->__generateRandom( 1, 10 );

                switch ( $whatToDo ) {
                    /* Place manual bid */
                    case 1:

                        if( 
                            //is_null( $bidButler = \App\Models\AuctionBidButlers::where('item_id', $i->id)->where('user_id', $u->id)->first() ) 
                            //&&
                            $i->ends_at->timestamp < Carbon::now()->addHours(3)->timestamp
                            &&
                            $i->ends_at->timestamp > Carbon::now()->addMinutes(10)->timestamp
                        ) {
                            $newBid = new \App\Models\AuctionBids;
                            $newBid->item_id = $i->id;
                            $newBid->user_id = $u->id;
                            $newBid->type = 1;
                            $newBid->price = $i->per_bid_price;

                            $deduces = $u->deduceBid($i->per_bid_price);
                            $newBid->package_bid_price = $deduces[0]['price'];

                            if( $newBid->save() ) {

                                $i->current_price = $i->current_price + $newBid->price;
                                $i->ends_at = $i->ends_at->addSeconds(10);

                                $i->last_bid_id = $newBid->id;

                                if( $i->save() ) {

                                    $newBid->username = $u->username;
                                    $newBid->created_at_format = $newBid->created_at->format('d.m.y H:i:s');
                                    $i->ends_at_ts = $i->ends_at->timestamp;

                                    NodeGW::send('api/socketio/auction/newBid', [
                                        'item' => $i->toArray(),
                                        'newBids' => [ $newBid->toArray() ]
                                    ]);
                                }
                            }
                        }
                        
                    break;
                    /* Place bid butler */
                    case 8:
                        if( 
                            $i->ends_at->timestamp > Carbon::now()->addSeconds(30)->timestamp
                            //&&
                            //\App\Models\AuctionBids::where('item_id', $i->id)->where('user_id', $u->id)->count() == 0
                            &&
                            \App\Models\AuctionBidButlers::where('item_id', $i->id)->count() <= 3
                        ) {
                            $bidButler = \App\Models\AuctionBidButlers::where('item_id', $i->id)->where('user_id', $u->id)->first();
                            $counter = $this->__generateRandom( 10, 20 );

                            if( 
                                !is_null( $bidButler ) 
                            ) {
                                $bidButler->counter = $bidButler->counter + $counter;
                                $bidButler->save();
                            } else {
                                $bidButler = new \App\Models\AuctionBidButlers;
                                $bidButler->item_id = $i->id;
                                $bidButler->user_id = $u->id;
                                $bidButler->counter = $counter;
                                $bidButler->save();
                            }
                        }
                    break;

                    default:
                        /* Do nothing */
                    break;
                }
            }
        }
    }

    public function __generateRandom( $min, $max ) {
        return (int) random_int( (int) $min, (int) $max );
    }

    public function __createItems( $count ) {
        $num = $this->itemsNumber - $count;
        $autoItems = collect( $this->autoItems )->random( $num );

        foreach( $autoItems->all() as $item ) {
            $newItem = new \App\Models\AuctionItems;
            $newItem->type_id = $item->type_id;
            $newItem->category_id = $item->category_id;
            $newItem->title = $item->title;
            $newItem->description = $item->description;
            $newItem->per_bid_price = 0.01;
            $newItem->start_price = 0.00;
            $newItem->current_price = 0.00;
            $newItem->buy_price = $item->buy_price;
            $newItem->status = 0;
            $newItem->ends_at = Carbon::now()->addMinutes( $item->ends_at );
            if( $newItem->save() ) {
                $images = \App\Models\AuctionImages::whereIn('id', $item->images)->get();

                foreach( $images as $image ) {
                    $newImage = new \App\Models\AuctionImages;
                    $newImage->item_id = $newItem->id;
                    $newImage->image = $image->image;
                    $newImage->order = $image->order;
                    $newImage->save();
                }
            }
        }
    }
}