<?php

namespace App\Console\Commands;

use App\Events\ManualBidEvent;
use App\Facades\NodeGWManager;
use Illuminate\Console\Command;
use Log, Carbon, NodeGW;

class AutoBid extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'auction:autobid';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Auction simulation';

    protected $itemsNumber = 6;
    protected $autoItems = [];
    protected $users = [];

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {

        $this->users = [
            100000001,
            100000002,
            100000003,
            100000004,
            100000005,
            100000006,
            100000007,
            100000008,
            100000009,
            1000000010,
            1000000011,
            1000000012,
            1000000013,
            1000000014,
            1000000015,
            1000000016,
            1000000017,
            1000000018,
            1000000019,
            1000000020,
            1000000021,/*
            1000000022,*/
        ];

        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {

        //$this->writeLogLine(true, 'Auction AutoBid Started');
        //if( !($this->__generateRandom(1, 10) % 2) ) return;

        //$this->info(Carbon::now() . ' - Start of command: auction:autobid');

        $items = \App\Models\AuctionItems::
            where('status', 0)
            ->whereNotNull('bot_bids_start_from')
            ->whereNotNull('bot_bids_target_price')
            ->where('bot_bids_start_from', '<', Carbon::now())
            ->where('bot_bids_target_price', '>', 'current_price')
            ->inRandomOrder()
            ->get();
        foreach( $items as $item ) {

            //$this->info(Carbon::now() . ' - Item found: ' . $item->id);

            $users = \App\Models\User::whereIn('id', $this->users)->get()->random( $this->__generateRandom(1, (int) count($this->users) / 2) )->all();

            foreach( $users as $u ) {

                //$this->info(Carbon::now() . ' - Selected user: ' . $u->id);

                $item_old = $item;
                $sleep = random_int(1, 5);
                sleep($sleep);

                //To be sure the ends_at timestamp always up to date
                //$item = $item->where('id', $item->id)->first();

                $item = \App\Models\AuctionItems::
                    where('id', $item_old->id)
                    ->where('status', 0)
                    ->whereNotNull('bot_bids_start_from')
                    ->whereNotNull('bot_bids_target_price')
                    ->where('bot_bids_start_from', '<', Carbon::now())
                    ->where('bot_bids_target_price', '>', 'current_price')->first();

                if(!is_null($item)) {
                    if ($item->status != 0) {
                        continue;
                    }
                }
                else{
                    continue;
                }

                if( !$u->hasBids() ) {
                    $new_package = new \App\Models\AuctionUsersPackages;
                    $new_package->user_id = $u->id;
                    $new_package->package_id = 2;
                    $new_package->bids = 1000000;
                    $new_package->save();
                }

                $whatToDo = $this->__generateRandom( 1, 20 );


                switch ( $whatToDo ) {
                    case 11:
                        // Place bid butler
                        $this->autoBid_BidButler($item, $u);
                        break;
                    case 19:
                        // Place manual bid
                        $this->autoBid_ManualBid($item, $u);
                        break;
                    default:
                        // Do nothing
                    break;
                }
            }
        }

        //$this->info(Carbon::now() . ' - End of command: auction:autobid');
    }

    public function __generateRandom( $min, $max ) {
        return (int) random_int( (int) $min, (int) $max );
    }

    public function autoBid_ManualBid($item, $bot){
        //if(($item->ends_at->timestamp < Carbon::now()->addDays(30)->timestamp) && ($item->ends_at->timestamp > Carbon::now()->addMinutes(10)->timestamp)) {
            $last_bid = \App\Models\AuctionBids::where('item_id', $item->id)->orderby('id', 'desc')->first();
            $last_bidder_user = null;

            if($last_bid){
                $last_bidder_user = \App\Models\User::where('id', $last_bid->user_id)->first();
            }

            if($last_bidder_user && $last_bidder_user->id == $bot->id){
                return false;
            }

            if($item->isbusy == 0) {

                $newBid = new \App\Models\AuctionBids;
                $newBid->item_id = $item->id;
                $newBid->user_id = $bot->id;
                $newBid->type = 1;
                $newBid->price = $item->per_bid_price;


                $deduces = $bot->deduceBid($item->per_bid_price);
                $newBid->package_bid_price = $deduces[0]['price'];

                if ($newBid->save()) {

                    $item->current_price = $item->current_price + $newBid->price;
                    $item->ends_at = $item->ends_at->addSeconds(10);

                    $item->last_bid_id = $newBid->id;

                    if ($item->save()) {

                        $newBid->username = $bot->username;
                        $newBid->created_at_format = $newBid->created_at->format('Y.m.d H:i:s');

                        $data = $item->toArray();
                        $data["ends_at_ts"] = $item->ends_at->timestamp;

                        if($last_bidder_user && $last_bidder_user->id != $bot->id) {
                            \App\Facades\NotificationManager::create([
                                'user_id' => $last_bidder_user->id,
                                'type' => 'auction-overbid',
                                'content_id' => $item->id
                            ], $last_bidder_user);
                        }

                        NodeGWManager::send('api/socketio/auction/newBid', [
                            'item' => $data,
                            'newBids' => [$newBid->toArray()]
                        ]);

                        event(new ManualBidEvent($bot->id, $item->id));
                    }
                }
            }
            else
            {
                $waiting_stack = new \App\Models\WaitingUserBids;
                $waiting_stack->item_id = $item->id;
                $waiting_stack->user_id = $bot->id;
                $waiting_stack->type = 1;
                $item->current_price = $item->current_price + $item->per_bid_price;
                $waiting_stack->save();
            }
        //}
    }

    public function autoBid_BidButler($item, $bot){
        //if(($item->ends_at->timestamp > Carbon::now()->addSeconds(30)->timestamp) && (\App\Models\AuctionBidButlers::where('item_id', $item->id)->count() <= 3)) {
            $bidButler = \App\Models\AuctionBidButlers::where('item_id', $item->id)->where('user_id', $bot->id)->first();
            $counter = $this->__generateRandom( 3, 5 );

            if(
            !is_null( $bidButler )
            ) {
                $bidButler->counter = $bidButler->counter + $counter;
                $bidButler->save();

            } else {
                $bidButler = new \App\Models\AuctionBidButlers;
                $bidButler->item_id = $item->id;
                $bidButler->user_id = $bot->id;
                $bidButler->counter = $counter;
                $bidButler->save();
            }

            event(new ManualBidEvent($bot->id, $item->id));
        //}
    }

    public function writeLogLine($with_ts, $output){
        if($with_ts) {
            \Illuminate\Support\Facades\Log::info(\Illuminate\Support\Carbon::now(). ' - ' . $output);
        }
        else{
            \Illuminate\Support\Facades\Log::info($output);
        }
    }


    /*public function __createItems( $count ) {
        $num = $this->itemsNumber - $count;
        $autoItems = collect( $this->autoItems )->random( $num );

        foreach( $autoItems->all() as $item ) {
            $newItem = new \App\Models\AuctionItems;
            $newItem->type_id = $item->type_id;
            $newItem->category_id = $item->category_id;
            $newItem->title = $item->title;
            $newItem->description = $item->description;
            $newItem->per_bid_price = 0.01;
            $newItem->start_price = 0.00;
            $newItem->current_price = 0.00;
            $newItem->buy_price = $item->buy_price;
            $newItem->status = 0;
            $newItem->ends_at = Carbon::now()->addMinutes( $item->ends_at );

            if( $newItem->save() ) {
                $images = \App\Models\AuctionImages::whereIn('id', $item->images)->get();

                foreach( $images as $image ) {
                    $newImage = new \App\Models\AuctionImages;
                    $newImage->item_id = $newItem->id;
                    $newImage->image = $image->image;
                    $newImage->order = $image->order;
                    $newImage->save();
                }
            }
        }
    }*/
}