<?php 
namespace App\Http\Controllers\Admin\Ajax;

use App\Facades\NodeGWManager;
use Input, NodeGW, Image, DB;
use Illuminate\Http\Request;

class AuctionController extends AjaxController {

  	public function __construct()
  	{
  		parent::__construct();
	}

	public function postImageUpload( Request $request, $section ) {


		if( 
			$request->hasFile('files') 
		) {

			$files = $request->file('files');
			$_result = [];

			foreach( $files as $f ) {
				$filename = str_random(40) . '.' . $f->getClientOriginalExtension();
				$thumbnailPath = storage_path('app/public/auction/'. $section . '/thumbnails/');

				Image::make($f->getRealPath())->resize(260, null, function($constraint) {
					$constraint->aspectRatio();
    				$constraint->upsize();
				})->save($thumbnailPath . $filename);

				$path = $f->storeAs('auction/' . $section, $filename, 'public');

				$_result[] = [
					'name' => $filename,
					'size' => '',
					'url' => url('storage/auction/' . $section . '/' . $filename),
					'thumbnailUrl' => url('storage/auction/' . $section . '/thumbnails/' . $filename)
				];
			}

			return response()->json(["files" => $_result]);
		}
	}

  	public function postItemsTable()
	{   $data = Input::all();
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);
	
		$fields = [
			'status' => 'any',
			'startDate' => false,
			'endDate' => false,
			'userID' => false,

		  ];

		$items = \App\Models\AuctionItems::select(DB::raw('auction_items.*'))->join('auction_items_translations','auction_items.id', '=', 'auction_items_translations.auction_items_id')->where('auction_items_translations.locale','en');;

		if ($winner = Input::get('winner')){
			if(strlen($winner)>3)
				$item=$items->join('auction_bids','auction_items.id', '=', 'auction_bids.item_id')
				->where('auction_items.status','=','2')
				->where('auction_bids.user_id','=',$winner);
		}
		if($category = Input::get('category')){
			if($category!='any')
				$items=$items->where('auction_items.category_id','=',"$category");
		}

		if ($title = Input::get('title')){

			$items=$items->where('auction_items_translations.title','LIKE','%'.$title.'%');
		}

		$startDate = Input::get( 'startDate', $fields['startDate'] );
        $endDate = Input::get( 'endDate', $fields['endDate'] );


        if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
            $startDate = \App\Models\Date::format( $startDate );
            $endDate = \App\Models\Date::format( $endDate );
            $items=$items->whereRaw("(DATE(auction_items.ends_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }
	
		$start = ( $start < 1 ) ? 0 : $start;
    	$length = ( $length < 1 ) ? 25 : $length;

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 0:
		        	$order_by = 'id';
		          break;
                case 1:
		        	$order_by = 'title';
		        	break;
		        case 2:
		        	$order_by = 'category_id';
		          break;
		        case 3:
		        	$order_by = 'current_price';
		          break;
		        case 4:
		        	$order_by = 'buy_price';
		          break;
                case 5:
                    $order_by = 'status';
                    break;
		        case 6:
		        	$order_by = 'ends_at';
		          break;
		        default: 
		         	$dir = 'asc';
		         	$order_by = 'id';
		        break;
		    }
			$item=$items->orderBy( $order_by, $dir );


		}

		$items_count = $items->count();
  		$items_total = $items->skip( $start )->take( $length )->count();
  		$items = $items->skip( $start )->take( $length )->get();

  		foreach( $items as $i ) {
  		    if($i->category){
                $i->category_name = $i->category->title;
            }else{
                $i->category_name = "DELETED";
            }

  			if($i->status == 2) {
                $last_bid = \App\Models\AuctionBids::where('item_id', $i->id)->orderby('id', 'desc')->first();
                if($last_bid) {
                    $last_bidder_user = \App\Models\User::where('id', $last_bid->user_id)->first();
                    if($last_bidder_user) {
                        $i->winner = $last_bidder_user->username;
                    }
                    else{
                        $i->winner = '-';
                    }
                }
                else {
                    $i->winner = '-';
                }
            }
            else{
                $i->winner = '-';
            }

            //$act_enum = \App\Models\Enums::where('model_name', 'auction_items')->where('model_field', 'status')->where('model_id', $i->status)->first();
            //clock()->info($i->getEnumStatus);
            try{
                $i->status = $i->getEnumStatus->value;
            }
            catch (\Exception $e) {
                $i->status = 'Unknown';
            }
  		}

        //$winners = \App\Models\AuctionItems::whereHas('lastBidder', function( $query ){})->get();
        //$last_bid = \App\Models\AuctionBids::where('item_id', $item->id)->orderby('id', 'desc')->first();
        //$last_bidder_user = \App\Models\User::where('id', $last_bid->user_id)->first();

        /*foreach( $items as $w ) {
            //$i->category_name = $i->category->title;
            if(!is_null($w->user())) {
                clock()->info($w->user());
            }
            //break;
        }*/

  		$response = ['draw' => Input::get('draw', 1), 'data' => $items, 'recordsFiltered' => $items_count, 'recordsTotal' => $items_total];

    	return $this->responseTable( $response );
	}

	public function postItemsDelete()
	{
		$id = Input::get('id', null);
		
		if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->first() ) )
			return response()->json(false);

		//$item->delete();
        $item->status = 4;
        $item->save();

		NodeGWManager::send('api/socketio/auction/ended', [
            'item' => $item->toArray()
        ]);

		return response()->json(true);
	}

	public function postPackagesTable()
	{
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);

		$start = ( $start < 1 ) ? 0 : $start;
    	$length = ( $length < 1 ) ? 25 : $length;

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 0:
		        	$order_by = 'id';
		          break;
		        case 1:
		        	$order_by = 'auction_packages_translations.name';
		          break;
		        case 2:
		        	$order_by = 'price';
		          break;
		        case 3:
		        	$order_by = 'bids';
		          break;
		        case 4:
		        	$order_by = 'status';
		          break;
		        default: 
		         	$dir = 'asc';
		         	$order_by = 'id';
		        break;
		    }

			$packages=\App\Models\AuctionPackages::select(DB::raw('auction_packages.*'))->join('auction_packages_translations','auction_packages.id','=','auction_packages_translations.auction_packages_id')->where('auction_packages_translations.locale','en')->orderBy( $order_by, $dir );
		}

		$packages_total = $packages->count();
  		$packages_count = $packages->skip( $start )->take( $length )->count();
  		$packages = $packages->skip( $start )->take( $length )->get();

  		clock()->info($packages);

  		$response = ['draw' => Input::get('draw', 1), 'data' => $packages, 'recordsFiltered' => $packages_count, 'recordsTotal' => $packages_total];

    	return $this->responseTable( $response );
	}

	public function postPackagesStatusChange()
	{
		$id = Input::get('id');

		$package = \App\Models\AuctionPackages::where('id', $id)->first();

		if( $package->status == 0 ) {
			$package->status = 1;
		} else $package->status = 0;

		$package->save();

		return response()->json(true);
	}

	public function postPackagesDelete()
	{
		$id = Input::get('id', null);
		
		if( is_null( $item = \App\Models\AuctionPackages::where('id', $id)->first() ) )
			return response()->json(false);

		$item->name = $item->name . '_deleted';
		$item->save();
		$item->delete();

		return response()->json(true);
	}

	public function postCategoriesTable()
	{	$data = Input::all();
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);
		$fields = [
			'startDate' => false,
			'endDate' => false,
			'userID' => false,

		  ];
		$start = ( $start < 1 ) ? 0 : $start;
    	$length = ( $length < 1 ) ? 25 : $length;

		$categories=\App\Models\AuctionCategories::select(DB::raw('auction_categories.*'))->join('auction_cat_translations','auction_categories.id', '=', 'auction_cat_translations.auction_categories_id')->where("auction_cat_translations.locale","en");

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 0:
		        	$order_by = 'id';
		          break;
		        case 1:
		        	$order_by = 'title';
		          break;
		        case 2:
		        	$order_by = 'status';
		          break;
		        case 3:
		        	$order_by = 'created_at';
		          break;
		        default: 
		         	$dir = 'asc';
		         	$order_by = 'id';
		        break;
		    }


		}
		$categories=$categories->orderBy($order_by,$dir);
		$categories_total = $categories->count();
  		$categories_count = $categories->skip( $start )->take( $length )->count();
  		$categories = $categories->skip( $start )->take( $length )->get();

  		$response = [
  			'draw' => Input::get('draw', 1), 
  			'data' => $categories, 
  			'recordsFiltered' => $categories_count, 
  			'recordsTotal' => $categories_total
  		];

    	return $this->responseTable( $response );
	}

	public function postCategoriesStatusChange()
	{
		$id = Input::get('id');

		$category = \App\Models\AuctionCategories::where('id', $id)->first();

		if( $category->status == 0 ) {
			$category->status = 1;
		} else $category->status = 0;

		$category->save();

		return response()->json(true);
	}

	public function postCategoriesDelete()
	{
		$id = Input::get('id', null);
		
		if( is_null( $category = \App\Models\AuctionCategories::where('id', $id)->first() ) )
			return response()->json(false);

		$category->title = $category->title . '_deleted';
		$category->save();
		$category->delete();

		return response()->json(true);
	}




    public function postAuctionPurchaseTable()
    {
        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);
		$fields = [
			'status' => 'any',
			'startDate' => false,
			'endDate' => false,
			'userID' => false,

		  ];

		$auctionpurchases = \App\Models\AuctionPurchases::select(DB::raw('auction_purchases.*, users.username as user, auction_items_translations.title as item_id'))
		->join('auction_items','auction_items.id', '=', 'auction_purchases.item_id')
		->join('auction_items_translations','auction_items.id', '=', 'auction_items_translations.auction_items_id')
		->join('users','users.id', '=', 'auction_purchases.user_id')
		->where('auction_items_translations.locale','en');
		if ($user = Input::get('user')){
			if(strlen($user)>3)
			$auctionpurchases=$auctionpurchases->where('auction_purchases.user_id','=',$user);
		}
		if ($item = Input::get('item')){
			if(strlen($item)>1)
				$auctionpurchases=$auctionpurchases->where('auction_items_translations.title','LIKE', '%'.$item.'%');
		}

		$startDate = Input::get( 'startDate', $fields['startDate'] );
        $endDate = Input::get( 'endDate', $fields['endDate'] );


        if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
            $startDate = \App\Models\Date::format( $startDate );
            $endDate = \App\Models\Date::format( $endDate );
            $auctionpurchases=$auctionpurchases->whereRaw("(DATE(auction_purchases.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

		$start = ( $start < 1 ) ? 0 : $start;
		$length = ( $length < 1 ) ? 25 : $length;
		

	
        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];
            switch ($_order['column']) {
                case 0:
                    $order_by = 'id';
					break;
				case 1:
					$order_by ='user_id';
					break;
                case 2:
                    $order_by = 'auction_purchases.item_id';
                    break;
                case 3:
                    $order_by = 'price';
                    break;
                case 4:
                    $order_by = 'status';
                    break;
                case 5:
                    $order_by = 'type';
                    break;
                case 6:
                    $order_by = 'created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'created_at';
                    break;
            }
        }


        $auctionpurchases = $auctionpurchases->orderBy($order_by, $dir);

        $auctionpurchases_count = $auctionpurchases->count();
        $auctionpurchases_total = $auctionpurchases->skip( $start )->take( $length )->count();
        $auctionpurchases = $auctionpurchases->skip( $start )->take( $length )->get();

        foreach( $auctionpurchases as $t ) {
            if($t->status == 1){
                $t->status = trans('status.under_process');
            }
            else if($t->status == 2){
                $t->status = trans('status.shipping');
            }
            else if($t->status == 3){
                $t->status = trans('status.delivered');
            }
            else{
                $t->status = trans('status.undetermined');
            }

            $t->price = '$ ' . number_format($t->price, 2);

          

            if($t->type == 'Online Wallet'){
                $t->type = trans('status.online_wallet');
            }
            else if ($t->type == 'Gift Card'){
                $t->type = trans('status.gift_card');
            }
            else{
                $t->type = trans('status.unknown');
            }

        }



        $response = [
            'draw' => Input::get('draw', 1),
            'data' => $auctionpurchases,
            'recordsFiltered' => $auctionpurchases_count,
            'recordsTotal' => $auctionpurchases_total
        ];

        return response()->json($response);
    }

    public function postAuctionPurchaseStatusChange()
    {
        $id = Input::get('id');
        $code = Input::get('code');

        $purchase = \App\Models\AuctionPurchases::where('id', $id)->first();

        $purchase->status = $code;

        $purchase->save();

        $user = \App\Models\User::where('id', $purchase->user_id)->first();
        $item = \App\Models\AuctionItems::where('id', $purchase->item_id)->first();


        $this->generatePurchaseShippedMessage($user, $item);

        return response()->json(true);
    }

    public function generatePurchaseShippedMessage($user, $item){
        $from = trans('messages.from.stock');
        $subject = trans('messages.subjects.purchase.status_shipped');
        $content = trans('messages.contents.purchase.status_shipped'). $item->title;

        $mail = new \App\Models\Messages;
        $mail->from = $from;
        $mail->subject = $subject;
        $mail->content = $content;
        $mail->user_id = $user->id;
        $mail->status = 0;
        $mail->starred = 0;
        $mail->category = null;
        $mail->deleted_at = null;
        $mail->save();
    }

    /*public function postAuctionPurchaseDelete()
    {
        $id = Input::get('id', null);

        if( is_null( $kyc = \App\Models\UsersKycs::where('id', $id)->first() ) )
            return response()->json(false);

        $kyc->delete();

        return response()->json(true);
    }*/
}