<?php
namespace App\Http\Controllers\Admin\Ajax;

use \App\Models\Loans;
use Input, DB, Mail, Config, Auth, Date;

class HistoryController extends AjaxController {

    public function __construct()
    {
        parent::__construct();
    }

    public function postTransactionsTable()
    {
        ini_set('memory_limit','256M');

        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);

        $fields = [
            'source' => "transactions",
            'id' => false,
            'user_id' => false,
            'type' => [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            'amount' => false,
            'note' => false,
            'startDate' => false,
            'endDate' => false,
            'processor' => false,
        ];

        //DB::enableQueryLog();

        $transactions = \App\Models\Transactions::select(
                DB::raw('transactions.id, user_id, transactions.type, transactions.status, amount, balance, transactions.created_at, note, transactions.processor_id, processors.name as processor, users.username'))
            ->leftJoin('processors', 'processors.id', '=', 'transactions.processor_id')->join('users', 'users.id', '=', 'transactions.user_id');

        if ($source = Input::get('source', $fields['source'])) {
            if ($source !== 'transactions') {
                switch ($source) {
                    case 'cashouts':
                        $fields['type'] = [7, 8, 9, 10, 11];
                        break;
                    default:
                        break;
                }
            }
        }

        if ($id = Input::get('id', $fields['id']))
            if ($id !== '')
                $transactions = $transactions->where('transactions.id', $id);

        if ($user_id = Input::get('user_id', $fields['user_id']))
            if ($user_id !== '')
                $transactions = $transactions->where('user_id', $user_id);

        if ($processor = Input::get('processor', $fields['processor']))
            if ($processor !== 'any')
                $transactions = $transactions->where('transactions.processor_id', $processor);

        $startDate = Input::get( 'startDate', $fields['startDate'] );
        $endDate = Input::get( 'endDate', $fields['endDate'] );

        if( $startDate !== '' && $endDate !== '' ) {
            $startDate = \App\Models\Date::format( $startDate );
            $endDate = \App\Models\Date::format( $endDate );
            $transactions = $transactions->whereRaw("(DATE(transactions.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

        $start = ($start < 1) ? 0 : $start;
        $length = ($length < 1) ? 25 : $length;

        if ($type = Input::get('type', $fields['type']))
            if ($type !== 'any')
                $transactions = $transactions->whereIn('transactions.type', (is_array($type)) ? $type : [$type]);
            //else
            //    $transactions = $transactions->whereIn('transactions.type', $fields['type']);

        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ($_order['column']) {
                case 3:
                    $order_by = 'transactions.amount';
                    break;
                case 8:
                    $order_by = 'transactions.created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'transactions.id';
                    break;
            }

        }

        $transactions = $transactions->orderBy($order_by, $dir);

        $transactions_count = $transactions->count();
        $transactions_total = $transactions->skip( $start )->take( $length )->count();
        $transactions = $transactions->skip( $start )->take( $length )->get();

        $typeNames = \App\Models\Transactions::$types;

        foreach( $transactions as $t ) {
            $t->typeName = $typeNames[ $t->type ];
        }

        $response = ['draw' => Input::get('draw', 1), 'data' => $transactions, 'recordsFiltered' => $transactions_count, 'recordsTotal' => $transactions_total];
        return response()->json($response);
    }

    public function postTokensTable()
    {
        ini_set('memory_limit','256M');

        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);

        $fields = [
            'id' => false,
            'user_id' => false,
            'type' => 'any',
            'amount' => false,
            'startDate' => false,
            'endDate' => false,
        ];

        $plans = \App\Models\UsersPackages::select(
                DB::raw('users_packages.id, user_id, package_id, users.username, invested_amount, users_packages.status, users_packages.created_at, packages.name as tokenName')
            )->join('users', 'users.id', '=', 'users_packages.user_id')
            ->join('packages', 'packages.id', '=', 'users_packages.package_id');

        if ($id = Input::get('id', $fields['id']))
            if ($id !== '')
                $plans = $plans->where('id', $id);

        if ($user_id = Input::get('user_id', $fields['user_id']))
            if ($user_id !== '')
                $plans = $plans->where('user_id', $user_id);

        if ($type = Input::get('type', $fields['type']))
            if ($type !== 'any')
                $plans = $plans->where('package_id', $type);

        $startDate = Input::get( 'startDate', $fields['startDate'] );
        $endDate = Input::get( 'endDate', $fields['endDate'] );

        if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
            $startDate = \App\Models\Date::format( $startDate );
            $endDate = \App\Models\Date::format( $endDate );
            $plans = $plans->whereRaw("(DATE(users_packages.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

        $start = ($start < 1) ? 0 : $start;
        $length = ($length < 1) ? 25 : $length;

        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ($_order['column']) {
                case 2:
                    $order_by = 'invested_amount';
                    break;
                case 5:
                    $order_by = 'users_packages.created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'users_packages.id';
                    break;
            }

        }

        $plans = $plans->orderBy($order_by, $dir);

        $plans_count = $plans->count();
        $plans_total = $plans->skip( $start )->take( $length )->count();
        $plans = $plans->skip( $start )->take( $length )->get();

        $response = ['draw' => Input::get('draw', 1), 'data' => $plans, 'recordsFiltered' => $plans_count, 'recordsTotal' => $plans_total];
        return response()->json($response);
    }

    public function postDepositsTable()
    {
        ini_set('memory_limit','256M');

        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);

        $fields = [
            'id' => false,
            'user_id' => false,
            'currency_type' => 'any',
            'source_type' => 'any',
            'startDate' => false,
            'endDate' => false,
        ];

        $deposits = \App\Models\UsersDeposits::select(
            DB::raw('
                users_deposits.id, 
                users_deposits.created_at,
                users_deposits.user_id,
                users_deposits.uuid,
                users.username,
                users_deposits.usdt_value,
                users_deposits.coin_rate,
                users_deposits.currency_name,
                users_deposits.currency_amount,
                users_deposits.comment,
                users_deposits.transaction_source_system,
                users_deposits.status')
                )->join('users', 'users.id', '=', 'users_deposits.user_id');
                 //   ->join('packages', 'packages.id', '=', 'users_packages.package_id');

        if ($id = Input::get('id', $fields['id']))
            if ($id !== '')
                $deposits = $deposits->where('id', $id);

        if ($user_id = Input::get('user_id', $fields['user_id']))
            if ($user_id !== '')
                $deposits = $deposits->where('user_id', $user_id);

        if ($currency_type = Input::get('currency_type', $fields['currency_type']))
            if ($currency_type !== 'any')
                $deposits = $deposits->where('currency_name', $currency_type);

        if ($source_type = Input::get('source_type', $fields['source_type']))
            if ($currency_type !== 'any')
                $deposits = $deposits->where('transaction_source_system', $source_type);

        $startDate = Input::get( 'startDate', $fields['startDate'] );
        $endDate = Input::get( 'endDate', $fields['endDate'] );

        if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
            $startDate = \App\Models\Date::format( $startDate );
            $endDate = \App\Models\Date::format( $endDate );
            $deposits = $deposits->whereRaw("(DATE(users_deposits.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

        $start = ($start < 1) ? 0 : $start;
        $length = ($length < 1) ? 25 : $length;

        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ($_order['column']) {
                case 0:
                    $order_by = 'users_deposits.id';
                    break;
                case 1:
                    $order_by = 'users_deposits.created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'users_deposits.id';
                    break;
            }

        }

        $deposits = $deposits->orderBy($order_by, $dir);

        $deposits_count = $deposits->count();
        $deposits_total = $deposits->skip( $start )->take( $length )->count();
        $deposits = $deposits->skip( $start )->take( $length )->get();

        foreach( $deposits as $t ) {
            $t->usdt_value += 0;
            $t->coin_rate += 0;
            $t->currency_amount += 0;
        }

        $response = ['draw' => Input::get('draw', 1), 'data' => $deposits, 'recordsFiltered' => $deposits_count, 'recordsTotal' => $deposits_total];
        return response()->json($response);
    }
}
