<?php
namespace App\Http\Controllers\Admin\Ajax;

use \App\Models\Loans;
use DB, Config, Input, Carbon\Carbon, Date;

class RequestsController extends AjaxController {

	public function __construct()
	{
		parent::__construct();
	}

	public function getPendingDepositsTable()
	{
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);

		$deposits = \App\Models\Transactions::select(DB::raw('transactions.*, transactions_bitcoin.data AS bitcoin_txid, users.username'))
		->join('users', 'users.id', '=', 'transactions.user_id')
		->leftJoin('transactions_bitcoin', 'transactions_bitcoin.transaction_id', '=', 'transactions.id')
		->where('status', 0)
		->whereIn('type', [1,2,3,4,5,6]);

		$start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 1:
		        	$order_by = 'transactions.id';
		          break;
		        case 2:
		        	$order_by = 'transactions.processor_id';
		          break;
		        case 3:
		        	$order_by = 'transactions.user_id';
		        	break;
		        case 4:
		        	$order_by = 'transactions.type';
		        	break;
		       	case 5:
		        	$order_by = 'transactions.fee';
		          break;
		        case 6:
		        	$order_by = 'transactions.amount';
		          break;
		        case 8:
		          $order_by = 'transactions.created_at';
		          break;
		        default:
		         	$dir = 'desc';
		         	$order_by = 'transactions.id';
		        break;
		    }

		    $deposits = $deposits->orderBy( $order_by, $dir );

		}

		$deposits_total = $deposits->count();
  	$deposits_count = $deposits->skip( $start )->take( $length )->count();
  	$deposits = $deposits->skip( $start )->take( $length )->get();

  	$typeNames = \App\Models\Transactions::$types;

  	foreach( $deposits as $d ) {
  		$d->typeName = $typeNames[ $d->type ];
  	}

  	$response = ['draw' => Input::get('draw', 1), 'data' => $deposits, 'recordsFiltered' => $deposits_count, 'recordsTotal' => $deposits_total];

    	return response()->json( $response );
	}

	public function postPendingDepositsAction()
	{
		$action = Input::get('action', false);
		$id = Input::get('id', false);

		if( !$action ) return $this->response(['error' => 'Wrong action!']);
		if( !$id ) return $this->response(['error' => 'Wrong ID!']);

		$deposit = \App\Models\Transactions::where('status', 0);

		if( is_array( $id ) ) {
			$deposit = $deposit->whereIn('id', $id);
		} else {
			$deposit = $deposit->where('id', $id);
		}

		$transactions = $deposit->get();

		if( $action == 1 ) {

			if( $deposit->update([ 'status' => 1 ]) ) {

				if( $transactions->count() > 1 ) {
					foreach( $transactions as $t ) {
						$uwallet = \App\Models\UsersBalances::where('processor_id', $t->processor_id)->where('users_id', $t->user_id)->update([
								'wallet' => DB::raw('wallet + ' . $t->netAmount())
						]);
					}
				} else {
					$uwallet = \App\Models\UsersBalances::where('processor_id', $transactions->first()->processor_id)->where('users_id', $transactions->first()->user_id)->update([
								'wallet' => DB::raw('wallet + ' . $transactions->first()->netAmount())
						]);
				}

				return $this->response(['data' => true]);

			}
		} else {
			if( $deposit->delete() ) {

				return $this->response(['data' => true]);

			}
		}

		return $this->response(['error' => 'Database error!']);
	}

	public function getPendingCashoutsTable()
	{
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);

		$cashouts = \App\Models\Transactions::select(DB::raw('transactions.*, transactions_bitcoin.data AS bitcoin_txid, users.username'))
		->join('users', 'users.id', '=', 'transactions.user_id')
		->leftJoin('transactions_bitcoin', 'transactions_bitcoin.transaction_id', '=', 'transactions.id')
		->where('status', 0)
		->whereIn('type', [7,8,9,10,11]);

		$start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 1:
		        	$order_by = 'transactions.id';
		          break;
		        case 2:
		        	$order_by = 'transactions.processor_id';
		          break;
		        case 3:
		        	$order_by = 'transactions.balance';
		        	break;
		        case 4:
		        	$order_by = 'transactions.user_id';
		        	break;
		        case 5:
		        	$order_by = 'transactions.type';
		        	break;
		       	case 6:
		        	$order_by = 'transactions.fee';
		          break;
		        case 7:
		        	$order_by = 'transactions.amount';
		          break;
		        case 9:
		          $order_by = 'transactions.created_at';
		          break;
		        default:
		         	$dir = 'desc';
		         	$order_by = 'transactions.id';
		        break;
		    }

		    $cashouts = $cashouts->orderBy( $order_by, $dir );

		}

		$cashouts_total = $cashouts->count();
  	$cashouts_count = $cashouts->skip( $start )->take( $length )->count();
  	$cashouts = $cashouts->skip( $start )->take( $length )->get();

  	$typeNames = \App\Models\Transactions::$types;
  	$balanceNames = [];

  	foreach( \App\Models\UsersBalances::$pools as $pool ) {
  		$balanceNames[ $pool['name'] ] = $pool['label'];
  	}

  	foreach( $cashouts as $d ) {
  		$d->typeName = $typeNames[ $d->type ];
  		$d->balance = $balanceNames[ $d->balance ];
  	}

  	$response = ['draw' => Input::get('draw', 1), 'data' => $cashouts, 'recordsFiltered' => $cashouts_count, 'recordsTotal' => $cashouts_total];

    	return response()->json( $response );
	}

	public function postPendingCashoutsAction() {
		$action = Input::get('action', false);
		$id = Input::get('id', false);

		if( !$action ) return $this->response(['error' => 'Wrong action!']);
		if( !$id ) return $this->response(['error' => 'Wrong ID!']);

		$cashout = \App\Models\Transactions::where('status', 0);

		if( is_array( $id ) ) {
			$cashout = $cashout->whereIn('id', $id);
		} else {
			$cashout = $cashout->where('id', $id);
		}

		$transactions = $cashout->get();

		if( $action == 1 ) {

			if( $transactions->count() > 1 ) {
				foreach( $transaction as $t ) {
					if( $t->processor_id !== 1 ) {
						$pool = $t->balance;
						$gateWay = new \App\Libraries\GatewayCashouts( $processor->id );

						if( $t->processor_id == 2 ) {
								$gateWay->setVariables([ 'account_number' => $user_processor->value, 'amount' => $processor->netAmount('cashout', $data['amount']), 'user_id' => $cashout->user_id ]);
						} elseif( $t->processor_id == 3 ) {
								$gateWay->setVariables([ 'receiver_email' => $this->user->email, 'amount' => $processor->netAmount('cashout', $data['amount']), 'user_id' => $cashout->user_id ]);
						}

						$gw_response = $gateWay->send();

						if( !$gw_response ) {
							$new_transaction->status = 0;
							return Redirect::back()->with('error', 'Gateway error!')->withInput();
						} else {
							if( is_array( $gw_response ) && array_key_exists('ERROR', $gw_response) )
								return Redirect::back()->with('error', 'Gateway error!')->withInput();
							else
								$new_transaction->status = 1;
						}
					}
				}
			} else {
				if( $transactions->first()->processor_id !== 1 ) {
					$pool = $transactions->first()->balance;

					if( $transactions->first()->processor_id == 5 ) {

						$transactions->first()->bitcoin->data = Input::get('bitcoin_txid');
						$transactions->first()->bitcoin->save();

					}
				}
			}

			if( $cashout->update([ 'status' => 1 ]) ) {

				return $this->response(['data' => true]);

			}
		} else {
			if( $cashout->delete() ) {

				if( $transactions->count() > 1 ) {
					foreach( $transactions as $t ) {

						if( $t->processor_id == 5 )
							$t->bitcoin->delete();


						$pool = $t->balance;

						$uwallet = \App\Models\UsersBalances::where('processor_id', $t->processor_id)->where('users_id', $t->user_id)->update([
								$pool => DB::raw($pool . ' + ' . $t->amountNumber)
						]);
					}
				} else {
					$transactions->first()->bitcoin->delete();
					$pool = $transactions->first()->balance;
					$uwallet = \App\Models\UsersBalances::where('processor_id', $transactions->first()->processor_id)->where('users_id', $transactions->first()->user_id)->update([
								$pool => DB::raw($pool . ' + ' . $transactions->first()->amountNumber)
						]);
				}

				return $this->response(['data' => true]);

			}
		}

		return $this->response(['error' => 'Database error!']);
	}

	public function getPendingPrincipalTable()
	{
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);

		$principal = \App\Models\UsersPlans::select(DB::raw('users_plans.*, processors.name as processorName, tokens.name as tokenName, users.username'))->join('users', 'users.id', '=', 'users_plans.user_id')->join('processors', 'processors.id', '=', 'users_plans.processor_id')->join('tokens', 'tokens.id', '=', 'users_plans.token_id')->where('users_plans.status', 2);

		$start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 1:
		        	$order_by = 'users_plans.id';
		          break;
		        case 2:
		        	$order_by = 'users_plans.processor_id';
		          break;
		        case 3:
		        	$order_by = 'users_plans.token_id';
		          break;
		        case 4:
		        	$order_by = 'users_plans.user_id';
		        	break;
		       	case 5:
		        	$order_by = 'users_plans.invested_amount';
		          break;
		        case 6:
		        	$order_by = 'users_plans.balance';
		          break;
		        case 7:
		          $order_by = 'users_plans.created_at';
		          break;
		        default:
		         	$dir = 'desc';
		         	$order_by = 'users_plans.id';
		        break;
		    }

		    $principal = $principal->orderBy( $order_by, $dir );

		}

		$principal_total = $principal->count();
  	$principal_count = $principal->skip( $start )->take( $length )->count();
  	$principal = $principal->skip( $start )->take( $length )->get();

  	$response = ['draw' => Input::get('draw', 1), 'data' => $principal, 'recordsFiltered' => $principal_count, 'recordsTotal' => $principal_total];

    	return response()->json( $response );
	}

	public function postPendingPrincipalAction()
	{
		$action = Input::get('action', false);
		$id = Input::get('id', false);

		if( !$action ) return $this->response(['error' => 'Wrong action!']);
		if( !$id ) return $this->response(['error' => 'Wrong ID!']);

		$principal = \App\Models\UsersPlans::where('status', 2);

		if( is_array( $id ) ) {
			$principal = $principal->whereIn('id', $id);
		} else {
			$principal = $principal->where('id', $id);
		}

		if( $action == 1 ) {

			if( $principal->update([ 'status' => 1 ]) ) {

				return $this->response(['data' => true]);

			}
		} else {
			if( $principal->delete() ) {

				return $this->response(['data' => true]);

			}
		}

		return $this->response(['error' => 'Database error!']);
	}

	public function getPendingBanksTable()
	{
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);

		$banks = \App\Models\UsersProcessors::select(DB::raw('users_processors.*, users.username'))->join('processors', 'processors.id', '=', 'users_processors.processor_id')->join('users', 'users.id', '=', 'users_processors.user_id')->where('users_processors.processor_id', 1)->where('users_processors.status', 0)->where('users_processors.bank_details_id', '!=', NULL);

		$start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 1:
		        	$order_by = 'users_processors.id';
		          break;
		        case 2:
		        	$order_by = 'users_processors.processor_id';
		          break;
		        case 3:
		        	$order_by = 'users_processors.user_id';
		        	break;
		        case 4:
		          $order_by = 'users_processors.created_at';
		          break;
		        default:
		         	$dir = 'desc';
		         	$order_by = 'users_processors.id';
		        break;
		    }

		    $banks = $banks->orderBy( $order_by, $dir );

		}

		$banks_total = $banks->count();
  	$banks_count = $banks->skip( $start )->take( $length )->count();
  	$banks = $banks->skip( $start )->take( $length )->get();

  	foreach($banks as $b)
  	{
  		$b->profileName = $b->bankDetails->profile_name;
  	}

  	$response = ['draw' => Input::get('draw', 1), 'data' => $banks, 'recordsFiltered' => $banks_count, 'recordsTotal' => $banks_total];

    return response()->json( $response );
	}

	public function postPendingBanksAction()
	{
		$action = Input::get('action', false);
		$id = Input::get('id', false);

		if( !$action ) return $this->response(['error' => 'Wrong action!']);
		if( !$id ) return $this->response(['error' => 'Wrong ID!']);

		$banks = \App\Models\UsersProcessors::where('processor_id', 1)->where('status', 0);

		if( is_array( $id ) ) {
			$banks = $banks->whereIn('id', $id);
		} else {
			$banks = $banks->where('id', $id);
		}

		if( $action == 1 ) {

			if( $banks->update([ 'status' => 1 ]) ) {

				return $this->response(['data' => true]);

			}
		} else {
			foreach( $banks as $b ) {
				$b->bankDetails->delete();
			}

			if( $banks->delete() ) {

				return $this->response(['data' => true]);

			}
		}

		return $this->response(['error' => 'Database error!']);
	}

	public function getPendingGatewayAccountsTable()
	{
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 25);

		$gwAccouts = \App\Models\UsersProcessors::select(DB::raw('users_processors.*, users.username'))->join('processors', 'processors.id', '=', 'users_processors.processor_id')->join('users', 'users.id', '=', 'users_processors.user_id')->where('users_processors.status', 0)->where('users_processors.processor_id', '!=', 1);

		$start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

		if ( $order && !empty( $order ) ) {
      		$_order = array_pop($order);
      		$dir = $_order['dir'];

		    switch ( $_order['column'] ) {
		      	case 1:
		        	$order_by = 'users_processors.id';
		          break;
		        case 2:
		        	$order_by = 'users_processors.processor_id';
		          break;
		        case 3:
		        	$order_by = 'users_processors.user_id';
		        	break;
		        case 4:
		          $order_by = 'users_processors.created_at';
		          break;
		        default:
		         	$dir = 'desc';
		         	$order_by = 'users_processors.id';
		        break;
		    }

		    $gwAccouts = $gwAccouts->orderBy( $order_by, $dir );

		}

		$gwAccouts_total = $gwAccouts->count();
  	$gwAccouts_count = $gwAccouts->skip( $start )->take( $length )->count();
  	$gwAccouts = $gwAccouts->skip( $start )->take( $length )->get();

  	foreach($gwAccouts as $gw)
  	{
  		$gw->value = $gw->data;
  	}

  	$response = ['draw' => Input::get('draw', 1), 'data' => $gwAccouts, 'recordsFiltered' => $gwAccouts_count, 'recordsTotal' => $gwAccouts_total];

    return response()->json( $response );
	}

	public function postPendingGatewayAccountsAction()
	{
		$action = Input::get('action', false);
		$id = Input::get('id', false);

		if( !$action ) return $this->response(['error' => 'Wrong action!']);
		if( !$id ) return $this->response(['error' => 'Wrong ID!']);

		$gwAccouts = \App\Models\UsersProcessors::where('processor_id', '!=', 1)->where('status', 0);

		if( is_array( $id ) ) {
			$gwAccouts = $gwAccouts->whereIn('id', $id);
		} else {
			$gwAccouts = $gwAccouts->where('id', $id);
		}

		if( $action == 1 ) {

			if( $gwAccouts->update([ 'status' => 1 ]) ) {

				return $this->response(['data' => true]);

			}
		} else {
			if( $gwAccouts->delete() ) {

				return $this->response(['data' => true]);

			}
		}

		return $this->response(['error' => 'Database error!']);
	}

}
