<?php 
namespace App\Http\Controllers\Admin\Ajax;

use DB, Config, Input, Carbon\Carbon, Date;

class SummaryController extends AjaxController {

    public function __construct()
    {
        parent::__construct();
    }

    public function postAccounting()
    {   
        $fields = [
            'startDate' => false,
            'endDate' => false
        ];

        $transactions = \App\Models\Transactions::select(DB::raw('SUM(COALESCE(`amount`, 0.00)) as amount, type, status'));

        $startDate = Input::get( 'startDate', $fields['startDate'] );
        $endDate = Input::get( 'endDate', $fields['endDate'] );

        if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
            $startDate = \App\Models\Date::format( $startDate );
            $endDate = \App\Models\Date::format( $endDate );
            $transactions = $transactions->whereRaw("(DATE(created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

        $transactions = $transactions->groupBy('type', 'status')->get();

        $transactions_sum = [
            'completed' => [
                'credits' => [1 => 0.00, 2 => 0.00, 3 => 0.00, 4 => 0.00, 5 => 0.00, 6 => 0.00, 'sum' => 0.00 ],
                'debits' => [7 => 0.00, 8 => 0.00, 9 => 0.00, 10 => 0.00, 11 => 0.00, 'sum' => 0.00]
            ],
            'pending' => [
                'credits' => [1 => 0.00, 2 => 0.00, 3 => 0.00, 4 => 0.00, 5 => 0.00, 6 => 0.00, 'sum' => 0.00 ],
                'debits' => [7 => 0.00, 8 => 0.00, 9 => 0.00, 10 => 0.00, 11 => 0.00, 'sum' => 0.00]
            ],
            'startDate' => $startDate,
            'endDate' => $endDate
        ];

        foreach( $transactions as $t ) {
            $status = ( $t->status == 1 ) ? 'completed' : 'pending';
            if( $t->type < 7 ) {
                $transactions_sum[$status]['credits'][ $t->type ] = $t->amountNumber;
                $transactions_sum[$status]['credits']['sum'] = $transactions_sum[$status]['credits']['sum'] + $t->amountNumber;
            }
            else {
                $transactions_sum[$status]['debits'][ $t->type ] = $t->amountNumber;
                $transactions_sum[$status]['debits']['sum'] = $transactions_sum[$status]['debits']['sum'] + $t->amountNumber;
            }
        }

        $results = [
            'credit' => [
                'processor-deposit' => number_format($transactions_sum['completed']['credits'][1], 2),
                'ewallet-cashout' => number_format($transactions_sum['completed']['credits'][2], 2),
                'interest' => number_format($transactions_sum['completed']['credits'][3], 2),
                'commission' => number_format($transactions_sum['completed']['credits'][4], 2),
                'transfer-from' => number_format($transactions_sum['completed']['credits'][5], 2),
                'misc-credit' => number_format($transactions_sum['completed']['credits'][6], 2),
                'total-credits' => number_format($transactions_sum['completed']['credits']['sum'], 2)
            ],
            'debit' => [
                'processor-cashouts' => number_format($transactions_sum['completed']['debits'][11], 2),
                'ewallet-cashouts' => number_format($transactions_sum['completed']['debits'][7], 2),
                'plans' => number_format($transactions_sum['completed']['debits'][8], 2),
                'transfer-to' => number_format($transactions_sum['completed']['debits'][9], 2),
                'misc-debit' => number_format($transactions_sum['completed']['debits'][10], 2),
                'total-debits' => number_format($transactions_sum['completed']['debits']['sum'], 2)
            ],
            'cashout' => [
                'cashout-fee' => 0.00,
                'transfer-fee' => 0.00,
                'ewallet-cashouts' => number_format($transactions_sum['completed']['debits'][7], 2) . ' / $ ' . number_format($transactions_sum['pending']['debits'][7], 2) . ' Pending',
                'processor-cashouts' => number_format($transactions_sum['completed']['debits'][11], 2) . ' / $ ' . number_format($transactions_sum['pending']['debits'][11], 2) . ' Pending'
            ]
        ];

        return response()->json( $results );
    }
}