<?php namespace App\Http\Controllers\Admin\Ajax;

use Input, DB, Sentry, Mail, Config, MailManager, NotificationManager;
use Illuminate\Http\Request;

class UsersController extends AjaxController
{

  public function postUsersTable()
  {
    $data = Input::all();
    $order = Input::get('order', false);
    $start = Input::get('start', 0);
    $length = Input::get('length', 25);

    $fields = [
      'status' => 'any',
      'startDate' => false,
      'endDate' => false,
      'userID' => false
    ];

    $users = \App\Models\User::select(DB::raw('users.id, first_name, last_name, username, email, wallet as ewalletBalance, created_at, activated_at'))->leftJoin('users_balances as ub', 'ub.users_id', '=', 'users.id');

    if ($status = Input::get('status', $fields['status'])) {
      if( $status !== 'any' )
        $users = $users->where('activated_at', '=', NULL);
    }

    $startDate  = Input::get( 'startDate', $fields['startDate'] );
    $endDate    = Input::get( 'endDate', $fields['endDate'] );

    if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
      $startDate    = \App\Models\Date::format( $startDate );
      $endDate      = \App\Models\Date::format( $endDate );
      $users        = $users->whereRaw("(DATE(users.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
    }

    if( $userID = Input::get('userID', $fields['userID'] )) {
      if( $userID !== false )
        $users = $users->where('users.id', $userID);
    }

    $start = ($start < 1) ? 0 : $start;
    $length = ($length < 1) ? 25 : $length;

    if ($order && !empty($order)) {
      $_order = array_pop($order);
      $dir = $_order['dir'];

      switch ($_order['column']) {
        case 0:
          $order_by = 'users.id';
          break;
        case 1:
          $order_by = 'first_name';
          break;
        case 2:
          $order_by = 'username';
          break;
        case 3:
          $order_by = 'email';
          break;
        case 4:
          $order_by = 'is_activated';
          break;
        case 5:
          $order_by = 'ewalletBalance';
          break;
        case 6:
          $order_by = 'users.created_at';
          break;
      }

      $users = $users->orderBy($order_by, $dir);
    }

    $users_count = \App\Models\User::count();
    $users_total = $users->skip( $start )->take( $length )->get()->count();
    $users = $users->skip( $start )->take( $length )->get();


    foreach( $users as $key => $u ) {
      $u->status = 0;
      $u->name = $u->fullName();

      if ( $u->isActivated() ) $u->status = 1;
    }

    $response = [
      'draw' => Input::get('draw', 1),
      'data' => $users,
      'recordsFiltered' => $users_count,
      'recordsTotal' => $users_total
    ];

    return response()->json($response);
  }

	public function postSearchUser()
  {
    $query = Input::get('q');
    $page_limit = Input::get('page_limit');
    $page = Input::get('page');

    $users = \App\Models\User::/*join('users_details as ud', 'ud.id', '=', 'users.id')->*/
    where('id', $query)->orWhere('first_name', 'LIKE', '%' . $query . '%')->orWhere('last_name', 'LIKE', '%' . $query . '%')->orWhere('username', 'LIKE', '%' . $query . '%')->orWhere('sponsor_code', 'LIKE', '%' . $query . '%')->orWhere('email', 'LIKE', '%' . $query . '%')->paginate($page_limit);

    //$total = $users->count();

    return response()->json($users);
  }


  # USER INFO #

  public function postAccountingData()
  {
    $fields = [
        'startDate' => false,
        'endDate' => false,
        'userID' => false
    ];

    $transactions = \App\Models\Transactions::select(DB::raw('SUM(COALESCE(`amount`, 0.00)) as amount, type, status'));

    $startDate  = Input::get( 'startDate', $fields['startDate'] );
    $endDate    = Input::get( 'endDate', $fields['endDate'] );

    if( $startDate !== '' && $endDate !== '' ) {
      $startDate    = \App\Models\Date::format( $startDate );
      $endDate      = \App\Models\Date::format( $endDate );
      $transactions = $transactions->whereRaw("(DATE(created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
    }

    if( $userID = Input::get('userID', $fields['userID'] ) !== false ) {
      $transactions = $transactions->where('user_id', $userID);
    }

    $transactions = $transactions->groupBy('type', 'status')->get();

    $transactions_sum = [
        'completed' => [
            'credits' => [1 => 0.00, 2 => 0.00, 3 => 0.00, 4 => 0.00, 5 => 0.00, 6 => 0.00, 'sum' => 0.00 ],
            'debits' => [7 => 0.00, 8 => 0.00, 9 => 0.00, 10 => 0.00, 11 => 0.00, 'sum' => 0.00]
        ],
        'pending' => [
            'credits' => [1 => 0.00, 2 => 0.00, 3 => 0.00, 4 => 0.00, 5 => 0.00, 6 => 0.00, 'sum' => 0.00 ],
            'debits' => [7 => 0.00, 8 => 0.00, 9 => 0.00, 10 => 0.00, 11 => 0.00, 'sum' => 0.00]
        ],
        'startDate' => $startDate,
        'endDate' => $endDate
    ];

    foreach( $transactions as $t ) {
        $status = ( $t->status == 1 ) ? 'completed' : 'pending';
        if( $t->type < 7 ) {
            $transactions_sum[$status]['credits'][ $t->type ] = $t->amountNumber;
            $transactions_sum[$status]['credits']['sum'] = $transactions_sum[$status]['credits']['sum'] + $t->amountNumber;
        }
        else {
            $transactions_sum[$status]['debits'][ $t->type ] = $t->amountNumber;
            $transactions_sum[$status]['debits']['sum'] = $transactions_sum[$status]['debits']['sum'] + $t->amountNumber;
        }
    }

    $results = [
        'credit' => [
            'processor-deposit' => number_format($transactions_sum['completed']['credits'][1], 2),
            'ewallet-cashout' => number_format($transactions_sum['completed']['credits'][2], 2),
            'interest' => number_format($transactions_sum['completed']['credits'][3], 2),
            'commission' => number_format($transactions_sum['completed']['credits'][4], 2),
            'transfer-from' => number_format($transactions_sum['completed']['credits'][5], 2),
            'misc-credit' => number_format($transactions_sum['completed']['credits'][6], 2),
            'total-credits' => number_format($transactions_sum['completed']['credits']['sum'], 2)
        ],
        'debit' => [
            'processor-cashouts' => number_format($transactions_sum['completed']['debits'][11], 2),
            'ewallet-cashouts' => number_format($transactions_sum['completed']['debits'][7], 2),
            'plans' => number_format($transactions_sum['completed']['debits'][8], 2),
            'transfer-to' => number_format($transactions_sum['completed']['debits'][9], 2),
            'misc-debit' => number_format($transactions_sum['completed']['debits'][10], 2),
            'total-debits' => number_format($transactions_sum['completed']['debits']['sum'], 2)
        ],
        'cashout' => [
            'cashout-fee' => 0.00,
            'transfer-fee' => 0.00,
            'ewallet-cashouts' => number_format($transactions_sum['completed']['debits'][7], 2) . ' / $ ' . number_format($transactions_sum['pending']['debits'][7], 2) . ' Pending',
            'processor-cashouts' => number_format($transactions_sum['completed']['debits'][11], 2) . ' / $ ' . number_format($transactions_sum['pending']['debits'][11], 2) . ' Pending'
        ]
    ];

    return response()->json( $results );
  }

  public function postTransactionsTable()
  {
      $order  = Input::get('order', false);
      $start  = Input::get('start', 0);
      $length = Input::get('length', 10);

      $fields = [
        'source' => "transactions",
        'id' => false,
        'user_id' => false,
        'type' => [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
        'amount' => false,
        'note' => false,
        'startDate' => false,
        'endDate' => false,
        'processor' => false,
      ];

      $transactions = \App\Models\Transactions::select(
              DB::raw('transactions.id, user_id, transactions.type, amount, balance, transactions.created_at, note, transactions.processor_id, processors.name as processor, users.username'))
          ->leftJoin('processors', 'processors.id', '=', 'transactions.processor_id')->join('users', 'users.id', '=', 'transactions.user_id');

      if ($source = Input::get('source', $fields['source'])) {
          if ($source !== 'transactions') {
              switch ($source) {
                  case 'cashouts':
                      $fields['type'] = [7, 8, 9, 10, 11];
                      break;
                  default:
                      break;
              }
          }
      }

      if ($id = Input::get('id', $fields['id']))
        if ($id !== '')
          $transactions = $transactions->where('transactions.id', $id);

      if ($user_id = Input::get('user_id', $fields['user_id']))
        if ($user_id !== '')
          $transactions = $transactions->where('user_id', $user_id);

      if ($processor = Input::get('processor', $fields['processor']))
        if ($processor !== 'any')
          $transactions = $transactions->where('transactions.processor_id', $processor);

      $startDate  = Input::get( 'startDate', $fields['startDate'] );
      $endDate    = Input::get( 'endDate', $fields['endDate'] );

      if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
        $startDate    = \App\Models\Date::format( $startDate );
        $endDate      = \App\Models\Date::format( $endDate );
        $transactions = $transactions->whereRaw("(DATE(transactions.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
      }

      $start = ($start < 1) ? 0 : $start;
      $length = ($length < 1) ? 25 : $length;

      if ($type = Input::get('type', $fields['type']))
        if ($type !== 'any')
          $transactions = $transactions->whereIn('transactions.type', (is_array($type)) ? $type : [$type]);

      if ($order && !empty($order)) {
        $_order = array_pop($order);
        $dir = $_order['dir'];

        switch ($_order['column']) {
          case 3:
            $order_by = 'transactions.amount';
            break;
          case 7:
            $order_by = 'transactions.created_at';
            break;
          default:
            $dir = 'desc';
            $order_by = 'transactions.id';
            break;
        }

      }

      $transactions = $transactions->orderBy($order_by, $dir);

      $transactions_count = $transactions->count();
      $transactions_total = $transactions->skip( $start )->take( $length )->count();
      $transactions = $transactions->skip( $start )->take( $length )->get();

      $typeNames = \App\Models\Transactions::$types;

      foreach( $transactions as $t ) {
          $t->typeName = $typeNames[ $t->type ];
      }

      $response = ['draw' => Input::get('draw', 1), 'data' => $transactions, 'recordsFiltered' => $transactions_count, 'recordsTotal' => $transactions_total];
      return response()->json($response);
  }

  public function postPackagesTable()
  {
      ini_set('memory_limit','256M');

      $order = Input::get('order', false);
      $start = Input::get('start', 0);
      $length = Input::get('length', 10);

      $fields = [
          'id' => false,
          'user_id' => false,
          'package_id' => false,
          'amount' => false,
          'startDate' => false,
          'endDate' => false,
          'type' => false
      ];

      $packages = \App\Models\UsersPackages::select(
              DB::raw('users_packages.id, user_id, package_id, users.username, invested_amount, users_packages.status, users_packages.created_at, packages.name as tokenName')
          )->join('users', 'users.id', '=', 'users_packages.user_id')
          ->join('packages', 'packages.id', '=', 'users_packages.package_id');

      if ($id = Input::get('id', $fields['id']))
          if ($id !== '')
              $packages = $packages->where('users_packages.id', $id);

      if ($user_id = Input::get('user_id', $fields['user_id']))
          if ($user_id !== '')
              $packages = $packages->where('user_id', $user_id);

      $startDate = Input::get( 'startDate', $fields['startDate'] );
      $endDate = Input::get( 'endDate', $fields['endDate'] );

      if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
          $startDate = \App\Models\Date::format( $startDate );
          $endDate = \App\Models\Date::format( $endDate );
          $packages = $packages->whereRaw("(DATE(users_packages.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
      }

      $start = ($start < 1) ? 0 : $start;
      $length = ($length < 1) ? 25 : $length;

      if ($package_id = Input::get('package_id', $fields['package_id']))
          if ($package_id !== 'any')
              $packages = $packages->whereIn('package_id', $package_id);

      if ($type = Input::get('type', $fields['type']))
          if ($type !== 'any')
              $packages = $packages->where('packages.id', $type);

      if ($order && !empty($order)) {
          $_order = array_pop($order);
          $dir = $_order['dir'];

          switch ($_order['column']) {
              case 2:
                  $order_by = 'invested_amount';
                  break;
              case 5:
                  $order_by = 'users_packages.created_at';
                  break;
              default:
                  $dir = 'desc';
                  $order_by = 'users_packages.id';
                  break;
          }

      }

      $packages = $packages->orderBy($order_by, $dir);

      $packages_count = $packages->count();
      $packages_total = $packages->skip( $start )->take( $length )->count();
      $packages = $packages->skip( $start )->take( $length )->get();

      $response = ['draw' => Input::get('draw', 1), 'data' => $packages, 'recordsFiltered' => $packages_count, 'recordsTotal' => $packages_total];

      return response()->json($response);
  }

  public function postBankProfilesTable()
  {
    $order = Input::get('order', false);
    $start = Input::get('start', 0);
    $length = Input::get('length', 25);
    $userID = Input::get('userID');

    $banks = \App\Models\UsersProcessors::select(DB::raw('users_processors.*, users.username'))->join('processors', 'processors.id', '=', 'users_processors.processor_id')->join('users', 'users.id', '=', 'users_processors.user_id')->where('users_processors.processor_id', 1)->where('user_id', $userID)->where('users_processors.bank_details_id', '!=', NULL);

    $start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

    if ( $order && !empty( $order ) ) {
          $_order = array_pop($order);
          $dir = $_order['dir'];

        switch ( $_order['column'] ) {
            case 0:
              $order_by = 'users_processors.id';
              break;
            case 1:
              $order_by = 'users_processors.processor_id';
              break;
            case 3:
              $order_by = 'users_processors.created_at';
              break;
            default:
              $dir = 'desc';
              $order_by = 'users_processors.id';
            break;
        }

        $banks = $banks->orderBy( $order_by, $dir );

    }

    $banks_total = $banks->count();
    $banks_count = $banks->skip( $start )->take( $length )->count();
    $banks = $banks->skip( $start )->take( $length )->get();

    foreach($banks as $b)
    {
      $b->profileName = $b->bankDetails->profile_name;
    }

    $response = ['draw' => Input::get('draw', 1), 'data' => $banks, 'recordsFiltered' => $banks_count, 'recordsTotal' => $banks_total];

    return response()->json( $response );
  }

  public function postBankProfilesAction()
  {
    $action = Input::get('action', false);
    $id     = Input::get('id', false);

    if( !$action )  return $this->response( ['error' => 'Wrong action!'] );
    if( !$id )      return $this->response( ['error' => 'Wrong ID!'] );

    $banks = \App\Models\UsersProcessors::where('processor_id', 1)->where('status', 0);
    $user = \App\Models\User::where('id', $banks->first()->user_id)->first();

    $banks = $banks->where('id', $id);

    if( $action == 1 ) {

      if( $banks->update([ 'status' => 1 ]) ) {

        return $this->response(['data' => true]);

      }
    } else {

      $bankDetails = \App\Models\UsersBankDetails::where('id', $banks->first()->bank_details_id)->delete();

      if( $banks->delete() ) {

        if( !is_null( $user )  ) {

          MailManager::send('wallet.deleted-profile', 'Bank Account Profile Deleted', ['id' => $userProcessor->id, 'Profile_Name' => $bankDetails->profile_name, 'Bank_Name' => $bankDetails->bank_name ], $user);

          return $this->response(['data' => true]);
        }

      }
    }

    return $this->response(['error' => 'Database error!']);
  }

    public function postKycsTable()
    {
        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 25);

        $start = ( $start < 1 ) ? 0 : $start;
        $length = ( $length < 1 ) ? 25 : $length;

        if ( $order && !empty( $order ) ) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ( $_order['column'] ) {
                case 0:
                    $order_by = 'id';
                    break;
                case 1:
                    $order_by = 'user';
                    break;
                case 2:
                    $order_by = 'type';
                    break;
                case 3:
                    $order_by = 'document_location';
                    break;
                case 4:
                    $order_by = 'status';
                    break;
                case 5:
                    $order_by = 'created_at';
                    break;
                default:
                    $dir = 'asc';
                    $order_by = 'id';
                    break;
            }



            //if($order_by != 'user'){
                $kycs = \App\Models\UsersKycs::join('users', 'users_kycs.user_id', '=', 'users.id')
                    ->where('users.kyc_pending', 1)
                    ->orderBy( $order_by, $dir )
                    ->select('users_kycs.id', 'users.username as user', 'users.id as userid', 'users_kycs.type', 'users_kycs.document_location', 'users_kycs.status', 'users_kycs.created_at');
            /*}
            else{
                $kycs = \App\Models\UsersKycs::join('users', 'users_kycs.user_id', '=', 'users.id')
                    ->where('users.kyc_pending', 1)
                    ->orderBy('users.username', $dir )
                    ->select('users_kycs.id', 'users.username', 'users_kycs.type', 'users_kycs.status', 'users_kycs.created_at');
            }*/
        }

        $kycs_total = $kycs->count();
        $kycs_count = $kycs->skip( $start )->take( $length )->count();
        $kycs = $kycs->skip( $start )->take( $length )->get();

        foreach($kycs as $k){
            $k->document_location = \App\Models\UsersKycs::where('id', $k->id)->first()->getImagePathAttribute();
        }
 
        $response = [
            'draw' => Input::get('draw', 1),
            'data' => $kycs,
            'recordsFiltered' => $kycs_count,
            'recordsTotal' => $kycs_total
        ];

        return $this->responseTable( $response );
    }

    public function postKycsStatusChange()
    {
        $id = Input::get('id');
        $code = Input::get('code');

        $kyc = \App\Models\UsersKycs::where('id', $id)->first();

        $kyc->status = $code;

        $kyc->save();

        return response()->json(true);
    }

    public function postKycsDelete()
    {
        $id = Input::get('id', null);

        if( is_null( $kyc = \App\Models\UsersKycs::where('id', $id)->first() ) )
            return response()->json(false);

        $kyc->delete();

        return response()->json(true);
    }

    public function setVerificationComplete( Request $request ){

        $id = $request->input('id');

        $user = \App\Models\User::where('id', $id)->first();

        $user->kyc_pending = 0;
        $user->kyc_approved = 1;

        $user->save();

        \App\Facades\NotificationManager::create([
            'user_id' => $user->id,
            'type' => 'user-verified',
            'content_id' => $user->id
        ], $user);

        return response()->json(true);
    }
}
