<?php 
namespace App\Http\Controllers\Admin\Ajax;

use \App\Http\Controllers\Admin\AdminController;
use DB, Config, Input, File;

class VideosController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

  public function getEmbed()
  {
    $id = Input::get('id', false);

    if( !$id ) 
      return response(['error' => 'Wrong ID.']);

    if( is_null( $video = \App\Models\Videos::where('id', $id)->where('status', 1)->first()))
      return response()->json(['error' => 'Video not found.']);

    $embedCode = '<iframe width="300" height="400" scrolling="no" frameborder="no" src="' . route("site.video-embed", $video->id_long) . '"></iframe>';

    $embed = $video->embed;

    if( !is_null( $embed ) )
      $embed = json_decode($embed->data, true);

    return response()->json(['data' => ['video' => $video, 'embed' => $embed, 'embedCode' => $embedCode]]);
  }

  public function postEmbedSave()
  {
    $inputs = Input::all();

    if( !$inputs['id'] ) 
      return response(['error' => 'Wrong ID.']);
    
    if( is_null( $video = \App\Models\Videos::where('id', $inputs['id'])->where('status', 1)->first()))
      return response()->json(['error' => 'Video not found.']);


    $embedData = $video->embed;

    if( !$embedData || is_null( $embedData ) ) {
      $embedData = new \App\Models\VideosEmbed;
      $embedData->video_id = $video->id;
    }

    $embedData->data = json_encode([
      'bg-color' => $inputs['bg-color'],
      'font-color' => $inputs['font-color'],
      'link-color' => $inputs['link-color'],
    ]);

    $embedData->save();

    return response()->json(['data' => true]);
  }

  public function getVideos()
  {
    $order = Input::get('order', false);
    $start = Input::get('start', 0);
    $length = Input::get('length', 25);

    $fields = [
      'id' => false,
      'type' => false
    ];

    $videos = \App\Models\Videos::select('id', 'id_long', 'title_hu', 'title_en', 'title_de', 'free', 'price', 'type', 'status', 'views', 'created_at');

    if( $id = Input::get( 'id', $fields['id'] ) )
      if( $id !== '' )
        $videos = $videos->where('id', $id);

    $start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

    if( $type = Input::get( 'type', $fields['type'] ) )
      if( $type !== 'any' )
        $videos = $videos->whereIn('type', ( is_array( $type ) ) ? $type : [ $type ] );
      else
        $videos = $videos->whereIn('type', $fields['type'] );

    if ( $order && !empty( $order ) ) {
      $_order = array_pop($order);
      $dir = $_order['dir'];

      switch ( $_order['column'] ) {
        case 0:
          $order_by = 'id';
          break;
        case 1:
          $order_by = 'title_' . app()->getLocale();
          break;
        case 2:
          $order_by = 'free';
          break;
        case 3:
          $order_by = 'price';
          break;
        case 4:
          $order_by = 'type';
          break;
        case 5:
          $order_by = 'status';
          break;
        case 6:
          $order_by = 'views';
          break;
        case 7:
          $order_by = 'created_at';
          break;
        default: 
          $dir = 'desc';
          $order_by = 'id';
         break;
      }

      $videos = $videos->orderBy( $order_by, $dir );

    }


    $videos_total = $videos->count();
    $videos_count = $videos->skip( $start )->take( $length )->count();
    $videos = $videos->skip( $start )->take( $length )->get();

    $title = 'title_' . app()->getLocale();

    foreach( $videos as $i => $v ) {

      $videos[ $i ]->title = $v->$title;
    }

    $response = ['draw' => Input::get('draw', 1), 'data' => $videos, 'recordsFiltered' => $videos_count, 'recordsTotal' => $videos_total];
    return response()->json( $response );
  }

  public function getVideoAudioObjects()
  {
    $id = Input::get('id');
    $lang = Input::get('lang');

    $audioFiles = \App\Models\VideosAudio::where('video_id', $id)->where('language', $lang)->first();

    return response()->json($audioFiles);
  }

  public function postRemoveFile()
  {
    $response = true;
    $type = Input::get('type');
    $id = Input::get('id');
    $fn = Input::get('filename');
    $query = null;
    $delete = false;

    switch ($type) {
      case 'audio':
        $query = \App\Models\VideosAudio::where('id', $id)->first();
        $getter = 'filename';
        $delete = true;

        $path = storage_path() . '/audios/';
      break;
      case 'video':
        $query = \App\Models\Videos::where('id', $id)->first();
        $getter = 'videoFile';

        $path = storage_path() . '/videos/';
      break;
      case 'thumbnail':
        $query = \App\Models\Videos::where('id', $id)->first();
        $getter = 'thumbnailFile';

        $path = public_path() . '/assets/videos/thumbnails/';
      break;
    }

    if( !is_null( $query ) ) {
      $filename = $path . $query->{$getter};
      if( $delete ) {
        $query->delete();
      } else {
        $query->{$getter} = NULL;
        $query->save();
      }
    } else {
      $filename = $path . $fn;
    }

    if( File::exists( $filename ) ) {
      File::delete( $filename );
    } else {
      $response = false;
    }

    return response()->json($response, 200);
  }

  public function getVideoObjects()
  {
    $id = Input::get('id');

    $order = Input::get('order', false);
  	$start = Input::get('start', 0);
  	$length = Input::get('length', 10);

  	$start = ($start < 1) ? 0 : $start;
  	$length = ($length < 1) ? 25 : $length;

    $icons = \App\Models\VideosIcon::where('video_id', $id);
    $icons_total = $icons->count();

    if ($order && !empty($order)) {
      $_order = array_pop($order);
      $dir = $_order['dir'];

      switch ($_order['column']) {
        case 0:
          $order_by = 'id';
        break;
        default: 
        	$order_by = 'id';
        break;
      }

      $icons = $icons->orderBy($order_by, $dir);
    }

    $icons_count = count( $icons->get() );
	  $icons = $icons->skip( $start )->take( $length )->get();

    $title = 'title_' . app()->getLocale();

    foreach( $icons as $i => $icon ) {

      $icon->title = $icon->$title;
    }

    $response = ['draw' => Input::get('draw', 1), 'data' => $icons, 'recordsFiltered' => $icons_count, 'recordsTotal' => $icons_total];

    return response()->json($response);
  }

  public function getVideoObject()
  {
    $id = Input::get('id');
    $icon = \App\Models\VideosIcon::where('id', $id)->first();

    return response()->json($icon);
  }

  public function postVideoObjectRemove()
  {
    $id = Input::get('id');
    $icon = \App\Models\VideosIcon::where('id', $id)->first();
    $response = true;

    if( !is_null($icon) ) $icon->delete();
    else $reponse = false;

    return response()->json($reponse);
  }

  public function getVideoObjectSave()
  {
    $inputs = Input::all();
    $id = Input::get('id');
    $icon = \App\Models\VideosIcon::where('id', $id)->first();

    $icon->title = $inputs['title'];
    $icon->coordinates = $this->explodeCoordinates( $inputs['coordinates'], $icon->video->fps );
    $icon->content = $inputs['content'];
    $icon->status = $inputs['status'];
    $icon->save();

    return response()->json(true);
  }

  private function explodeCoordinates( $coordinates, $fps ) {
    $fpsSec = 1 / $fps;

    $lines = explode(PHP_EOL, $coordinates);

    if( count($lines) > 1 ) {

      foreach( $lines as $k => $l ) {
        $data = preg_split('/\s+/', trim($l));
        $lines[ $k ] = $data;
      }

      $lines = array_map(function( $i ) use ($fpsSec) {
        return [
          'ts' => floatval(number_format($i[0] * $fpsSec, 5, '.', '')),
          'x' => floatval($i[1]),
          'y' => floatval($i[2])
        ];
      }, $lines);

    } else {
      $lines = [];
    }

    return json_encode($lines);
  }
}