<?php namespace App\Http\Controllers\Admin;

use Carbon\Carbon;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\File;
use Redirect, DB, Validator, Input, Session, Image;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;

class AuctionController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

	/* ITEMS */

	public function getItems()
	{		$categories = \App\Models\AuctionCategories::get();
		return view('admin.auction.items.index',['categories'=>$categories])->withTitle('Items - Auction Manager');
	}

	public function getItemsView( $id = null )
	{  
		if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->first() ) )
			return Redirect::route('admin.auction.items')->with('error', 'Wrong item ID');

		$categories = \App\Models\AuctionCategories::get();
        $statuses = \App\Models\Enums::getAuctionItemsEnumStatuses();

		return view('admin.auction.items.view', ['statuses' => $statuses, 'item' => $item, 'categories' => $categories])->withTitle($item->title . ' - Auction Manager');
	}

	public function postItemsView( $id = null )
	{
		if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->first() ) )
			return Redirect::route('admin.auction.items')->with('error', 'Wrong auction ID');

		$data = Input::all();

		$rules = [
			'translate.*.title' => 'required',
			'type' => 'required|in:1,2',
			'category' => 'required|exists:auction_categories,id',
			'translate.*.description' => 'required',
			'per_bid_price' => 'required',
			'start_price' => 'required',
			'buy_price' => 'required',
			'status' => 'required',
			'ends_at' => 'required|date_format:Y-m-d H:i:s',

			'images' => 'required|array',

            'user_win_price_limit' => 'required',
            'user_bidders_price' => 'required',
            'inheritance_cycle' => 'required'
		];

		$validator = Validator::make( $data, $rules );

		if ( $validator->fails() )
      		return Redirect::back()->withInput()->withErrors( $validator );
		$item->fill(request()->get('translate'));
		$item->type_id = $data['type'];
		$item->category_id = $data['category'];
		$item->per_bid_price = $data['per_bid_price'];
		$item->start_price = $data['start_price'];
		$item->buy_price = $data['buy_price'];
        $item->status = $data['status'];
		$item->ends_at = $data['ends_at'];

        $item->user_win_price_limit = $data['user_win_price_limit'];
        $item->user_bidders_price = $data['user_bidders_price'];
        $item->inheritance_cycle = $data['inheritance_cycle'];

        if(Arr::exists($data, 'bot_need_win')){
        //if(($data['bot_need_win']) {
            $item->bot_need_win = $data['bot_need_win'];
        }
        else{
            $item->bot_need_win = 0;
        }
		if(Arr::exists($data, 'auto_renew')){
			$item->auto_renew = $data['auto_renew'];
		}
		else{
			$item->auto_renew = 0;
		}
		if($data['bot_bids_target_price'] == 0){
            $item->bot_bids_target_price = $data['buy_price'] * (config('auction.bot_bids_target_price_percent') / 100);
        }
        else{
            $item->bot_bids_target_price = $data['bot_bids_target_price'];
        }


        if(!$data['bot_bids_start_from'] || $data['bot_bids_start_from'] >= $data['ends_at']){
            $item->bot_bids_start_from = Carbon::parse($data['ends_at'])->subHours(config('auction.bot_bids_start_before_end_hour'));
        }
        else{
            $item->bot_bids_start_from = $data['bot_bids_start_from'];
        }


		if( $item->save() ) {
			foreach( $item->images as $i ) {
				$i->delete();
			}

			foreach( $data['images'] as $key => $i ) {
				$images = new \App\Models\AuctionImages;
				$images->item_id = $item->id;
				$images->image = $i;
				$images->order = $key;
				$images->save();
			}

			return Redirect::route('admin.auction.items')->with('success', 'Item successfully saved.');
		}

		return Redirect::back()->with('error', 'Could not save the item.');
	}

	public function getItemsNew()
	{
		$categories = \App\Models\AuctionCategories::/*orderBy('title', 'ASC')->*/get();
        $statuses = \App\Models\Enums::getAuctionItemsEnumStatuses();

		return view('admin.auction.items.new', ['statuses' => $statuses, 'categories' => $categories])->withTitle('Create New Item - Auction Manager');
	}

	public function postItemsNew()
	{
		$data = Input::all();

		clock()->info($data);

		$rules = [
			'translate.*.title' => 'required|unique:auction_items_translations,title',
			'type' => 'required|in:1,2',
			'category' => 'required|exists:auction_categories,id',
			'translate.*.description' => 'required',
			'per_bid_price' => 'required',
			'start_price' => 'required',
			'buy_price' => 'required',
			'ends_at' => 'required|date_format:Y-m-d H:i:s',
			'images' => 'required|array',

            'user_win_price_limit' => 'required',
            'user_bidders_price' => 'required'
		];

		$validator = Validator::make($data, $rules);

		if ( $validator->fails() )
      		return Redirect::back()->withInput()->withErrors($validator);

		$item = new \App\Models\AuctionItems;
		$item->fill(request()->get('translate'));

		$item->type_id = $data['type'];
		$item->category_id = $data['category'];
		$item->per_bid_price = $data['per_bid_price'];
		$item->buy_price = $data['buy_price'];
		$item->ends_at = $data['ends_at'];

        $item->user_win_price_limit = $data['user_win_price_limit'];
        $item->user_bidders_price = $data['user_bidders_price'];

		if(Arr::exists($data, 'start_price') && Arr::exists($data, 'buy_price')){
		    if($data['start_price'] > ($data['buy_price'] * (config('auction.start_price_by_buy_price_percent') / 100))){
                $item->start_price = $data['start_price'];
            }
            else{
                $item->start_price = $data['buy_price'] * (config('auction.start_price_by_buy_price_percent') / 100);
            }
        }
        else{
            $item->start_price = 0;
        }

        $item->current_price = $data['start_price'];


        if(Arr::exists($data, 'bot_need_win')){
		//if($data['bot_need_win']) {
            $item->bot_need_win = $data['bot_need_win'];
        }
        else{
            $item->bot_need_win = 0;
        }

		if(Arr::exists($data, 'auto_renew')){
			$item->auto_renew = $data['auto_renew'];
		}
		else{
			$item->auto_renew = 0;
		}
        if($data['bot_bids_target_price'] == 0){
            $item->bot_bids_target_price = $data['buy_price'] * (config('auction.bot_bids_target_price_percent') / 100);
        }
        else{
            $item->bot_bids_target_price = $data['bot_bids_target_price'];
        }


        if(!$data['bot_bids_start_from'] || $data['bot_bids_start_from'] >= $data['ends_at']){
            $item->bot_bids_start_from = Carbon::parse($data['ends_at'])->subHours(config('auction.bot_bids_start_before_end_hour'));
        }
        else{
            $item->bot_bids_start_from = $data['bot_bids_start_from'];
        }


		if( $item->save() ) {
			foreach( $data['images'] as $key => $i ) {
				$images = new \App\Models\AuctionImages;
				$images->item_id = $item->id;
				$images->image = $i;
				$images->order = $key;
				$images->save();
			}

			return Redirect::route('admin.auction.items')->with('success', 'Item has successfully created.');
		}

		return Redirect::back()->with('error', 'Could not save the item.');
	}

	/* PACKAGES */

	public function getPackages()
	{	
		return view('admin.auction.packages.index')->withTitle('Packages - Auction Manager');
	}

	public function getPackagesView( $id = null )
	{
		if( is_null( $package = \App\Models\AuctionPackages::where('id', $id)->first() ) )
			return Redirect::route('admin.auction.packages')->with('error', 'Wrong auction package ID');

		return view('admin.auction.packages.view', ['package' => $package])->withTitle($package->name . ' - Auction Manager');
	}

	public function postPackagesView( $id = null )
	{
		if( is_null( $package = \App\Models\AuctionPackages::where('id', $id)->first() ) )
			return Redirect::route('admin.auction.packages')->with('error', 'Wrong auction package ID');

		$data = Input::all();
		$data['status'] = Input::get('status', 0);

		$rules = [
			'translate.*.name' =>'required',
			'translate.*.description' => 'required',
			//'image' => 'required',
			'status' => 'required|boolean',
			'price' => 'required',
			'bids' => 'required'
		];

		$validator = Validator::make( $data, $rules );

		if ( $validator->fails() )
      		return Redirect::back()->withInput()->withErrors( $validator );

		$package->fill(request()->get('translate'));
		$package->status = $data['status'];
		$package->price = $data['price'];
		$package->bids = $data['bids'];

		if( $package->save() )
			return Redirect::route('admin.auction.packages')->with('success', 'Package successfully saved.');

		return Redirect::back()->with('error', 'Could not save the package.');
	}

	public function getPackagesNew()
	{
		return view('admin.auction.packages.new')->withTitle('Create New Package - Auction Manager');
	}

	public function postPackagesNew()
	{
		$data = Input::all();
		$data['status'] = Input::get('status', 0);

		$rules = [
			'translate.*.name' => 'required|unique:auction_packages_translations,name',
			'translate.*.description' => 'required',
			//'image' => 'required',
			'status' => 'required|boolean',
			'price' => 'required',
			'bids' => 'required'
		];

		$validator = Validator::make($data, $rules);

		if ( $validator->fails() )
      		return Redirect::back()->withInput()->withErrors($validator);

		$package = new \App\Models\AuctionPackages;
		$package->fill(request()->get('translate'));
		$package->status = $data['status'];
		$package->price = $data['price'];
		$package->bids = $data['bids'];

		if( $package->save() )
			return Redirect::route('admin.auction.packages')->with('success', 'Package has successfully created.');

		return Redirect::back()->with('error', 'Could not save the package.');
	}

	/* Categories */

	public function getCategories()
	{
		return view('admin.auction.categories.index')->withTitle('Categories - Auction Manager');
	}

	public function getCategoriesView( $id = null )
	{
		if( is_null( $category = \App\Models\AuctionCategories::where('id', $id)->first() ) )
			return Redirect::route('admin.auction.categories')->with('error', 'Wrong category ID');

		return view('admin.auction.categories.view', ['category' => $category])->withTitle($category->title . ' - Auction Manager');
	}

	public function postCategoriesView( Request $request, $id = null )
	{
		if( is_null( $category = \App\Models\AuctionCategories::where('id', $id)->first() ) )
			return Redirect::route('admin.auction.categories')->with('error', 'Wrong category ID');

		$data = Input::all();
		$data['status'] = Input::get('status', 0);

		$rules = [
			'translate.*.title' => 'required',// Rule::unique('auction_cat_translations')->ignore($category->id)],
			'thumbnail' => 'file',
			'status' => 'required|boolean',
		];

		$validator = Validator::make( $data, $rules );

		if ( $validator->fails() )
      		return Redirect::back()->withInput()->withErrors( $validator );

        if(!File::exists(storage_path('app/public/auction/categories/thumbnails/'))) {
            File::makeDirectory(storage_path('app/public/auction/categories/thumbnails/'), 0775, true);
        }

    	$thumbnailFilename = $category->thumbnail;

      	if( 
			$request->hasFile('thumbnail')
		) {
      		$file = $request->file('thumbnail');

      		$filename = str_random(40) . '.' . $file->getClientOriginalExtension();
			$thumbnailPath = storage_path('app/public/auction/categories/');

			Image::make($file->getRealPath())->resize(400, null, function($constraint) {
				$constraint->aspectRatio();
				$constraint->upsize();
			})->save($thumbnailPath . $filename);

			//$path = $f->storeAs('auction/categories', $filename, 'public');

			$thumbnailFilename = $filename;
      	}

		//$category->title = $data['title'];
        $category->fill(request()->get('translate'));
		$category->title_url = str_slug(request()->get('translate')['en']['title']);
		$category->thumbnail = $thumbnailFilename;
		$category->status = $data['status'];

		if( $category->save() )
			return Redirect::route('admin.auction.categories')->with('success', 'Category successfully saved.');

		return Redirect::back()->with('error', 'Could not save the category.');
	}

	public function getCategoriesNew()
	{
		return view('admin.auction.categories.new')->withTitle('New Category - Auction Manager');
	}

	public function postCategoriesNew(Request $request)
	{
		$data = Input::all();
		$data['status'] = Input::get('status', 0);

		$rules = [
            'translate.*.title' => 'required|unique:auction_cat_translations,title',
			'thumbnail' => 'required|file',
			'status' => 'required|boolean',
		];


        //clock()->info($data);
        /*
        clock()->info($category);
        clock()->info($thumbnailFilename);
		*/

		$validator = Validator::make($data, $rules);

		if ( $validator->fails() )
      		return Redirect::back()->withInput()->withErrors($validator);


		$category = new \App\Models\AuctionCategories;

        $category->fill(request()->get('translate'));

		$category->title_url = str_slug(request()->get('translate')['en']['title'], '-');

        if(!File::exists(storage_path('app/public/auction/categories/thumbnails/'))) {
            File::makeDirectory(storage_path('app/public/auction/categories/thumbnails/'), 0775, true);
        }

        $thumbnailFilename = $category->thumbnail;

        if(
        $request->hasFile('thumbnail')
        ) {
            $file = $request->file('thumbnail');

            $filename = str_random(40) . '.' . $file->getClientOriginalExtension();
            $thumbnailPath = storage_path('app/public/auction/categories/');

            Image::make($file->getRealPath())->resize(400, null, function($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })->save($thumbnailPath . $filename);

            //$path = $f->storeAs('auction/categories', $filename, 'public');

            $thumbnailFilename = $filename;
        }




		$category->thumbnail = $thumbnailFilename;
		$category->status = $data['status'];

		if( $category->save() )
			return Redirect::route('admin.auction.categories')->with('success', 'Category has successfully created.');

		return Redirect::back()->with('error', 'Could not save the category.');
	}

    public function getAuctionPurchases()
    {
        return view('admin.auction.purchases.index')->withTitle('Auction Purchases - Order Manager');
    }

    public function getShippingInfo($id)
    {
        //return $id;

        $order_user = \App\Models\User::where('id', $this->user->id)->first();
        $order_purchase = \App\Models\AuctionPurchases::where('id', $id)->first();
        $order_shipping = \App\Models\Shippings::where('id', $order_purchase->shipping_id)->first();

        clock()->info($order_purchase);
        if($order_purchase->status == 1){
            $order_purchase->status = trans('status.under_process');
        }
        else if($order_purchase->status == 2){
            $order_purchase->status = trans('status.shipping');
        }
        else if($order_purchase->status == 3){
            $order_purchase->status = trans('status.delivered');
        }
        else{
            $order_purchase->status = trans('status.undetermined');
        }

        $order_purchase->price = '$ ' . number_format($order_purchase->price, 2);

        $auction_item = \App\Models\AuctionItems::where('id', $order_purchase->item_id)->first();
        $order_purchase->item_id = $auction_item->title;


        $datas = [
            'user' => $order_user,
            'purchase' => $order_purchase,
            'shipping' => $order_shipping
        ];

        clock()->info($datas);

        return view('auctionpurchase.view-shipping', $datas)->withTitle( trans('auctionpurchase.index.shipping') );
    }

}