<?php namespace App\Http\Controllers\Admin;

use Input, Lang, Redirect, DB, Str, Validator, View, Config, ApcManager;

class ConfigurationController extends AdminController {

	private $types = [
		'radio' => [

		]
	];

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		$configs = \App\Models\Configs::orderBy('id', 'asc')->orderBy('group', 'asc')->get();
		$groups = \App\Models\Configs::orderBy('group', 'asc')->groupBy('group')->pluck('group');

		$_configs = [];

		foreach( $configs as $c ) {
			if( !isset( $configs[ $c->group ] ) )
				$configs[ $c->group ] = [];

			$_configs[ $c->group ][ $c->key ] = $c;
		}

		return view('admin.configuration.index', ['configs' => $_configs, 'groups' => $groups])->withTitle('System Configuration');
	}

	public function editPost()
	{
		$group = Input::get('save', false);

		if( !$group )
			return Redirect::route('admin.configuration')->with('error', 'Group not found.');

		if( is_null( $configs = \App\Models\Configs::where('group', $group)->orderBy('id', 'asc')->orderBy('group', 'asc')->get() ) )
			return Redirect::route('admin.configuration')->with('error', 'Config not found.')->withTab($group);

		$rules = [];
		$values = [];
		$multiple = [];

		foreach( $configs as $c ) {

			switch( $c->type ) {
				case 'radio' :
					$value = Input::get( str_replace(".", "_", $c->key ), 'off' );
					$value = ( $value == 'on' ) ? true : false;
				break;
				case 'input' :

					if( !is_null( $decoded = json_decode( $c->value, true ) ) ) {
						if( is_array($decoded) )
							$multiple[] = $c->key;
					}

					$value = Input::get( str_replace( ".", "_", $c->key ) );

				break;
				case 'input-multiple':

					$multiple[] = $c->key;

					$options = json_decode( $c->options, true );
					$_value = Input::get( $c->key, '' );
					$value = [];

					foreach( $options as $key => $o ) {
						$value[ $o ] = $_value[ $key ];
					}

				break;
				case 'list-append':
					$options = json_decode( $c->options, true );
					$groups = [];

					foreach( $options as $key => $o ){
						$item = str_replace(".", "_", $c->key ) . '_' . $key;
						$input_values = Input::get( $item );
						$groups[ $key ] = array_combine(range(1, count($input_values)), array_values($input_values));
					}

					$value = $groups;

				break;
				default:
					$value = Input::get( $c->key );
				break;
			}

			$rules[ $c->key ] = $c->validation;
			$values[ $c->key ] = $value;
		}

		$validator = Validator::make( $values, $rules );

		if ( $validator->fails() )
			return Redirect::back()->withInput()->withErrors( $validator )->withTab($group);

		foreach( $values as $key => $value ) {
			if( is_bool( $value ) )
				$value = ( $value ) ? 'true' : 'false';

			if( in_array( $key, $multiple ) ) {
				if( is_array( $value ) ) {
					$value = json_encode( $value );
				} else {
					$exploded = explode(', ', $value );
					$value = json_encode( $exploded );
				}
			} 
			elseif( is_array( $value ) ) {
				$value = json_encode( $value );
			}
			
			DB::table('configs')->where('key', $key)->update(['value' => $value]);
			
		}
			
		ApcManager::storeConfig();

		return Redirect::route('admin.configuration')->with('success', 'Configuration successfully updated.')->withTab($group);

	}
}