<?php namespace App\Http\Controllers\Admin;

use App\Models\Packages;
use App\Models\Transactions;
use App\Models\User;
use App\Models\UserAddresses;
use App\Models\UsersPackages;
use App\Models\WCILastPrices;
use Illuminate\Support\Carbon;
use Spatie\Permission\Models\Role;

class DashboardController extends AdminController {


	public function __construct()
	{
		parent::__construct();
	}

	public function index($days = 15)
	{
		$totals = [
			'users' => \App\Models\User::whereIn('id', User::getUsersIdsForStatistic())->count(),
            'items' => \App\Models\AuctionItems::where('status', 0)->count(),
			//'packages' => \App\Models\UsersPackages::where('status', 1)->sum('invested_amount'),
			//'deposits' => \App\Models\Transactions::whereIn('type', [1,2,6])->where('status', 1)->sum('amount'),
			'deposits' => \App\Models\UsersDeposits::whereIn('user_id', User::getUsersIdsForStatistic())->sum('currency_amount'),
			'deposits_usdt' => \App\Models\UsersDeposits::whereIn('user_id', User::getUsersIdsForStatistic())->sum('usdt_value'),
			//'cashouts' => \App\Models\Transactions::whereIn('type', [7,10,11])->where('status', 1)->sum('amount')
            'userbids' => \App\Models\AuctionUsersPackages::whereIn('user_id', User::getUsersIdsForStatistic())->sum('bids')
		];

        $startDate = Carbon::today()->subDay();
        $endDate = Carbon::today()->subDays($days);

        $lastPrices = WCILastPrices::all();

        $commissionTypes = [
            "dc" => [
                "currencies" => [
                    "cpc",
                ],
                "name" => "Direct Commission",
                "type" => 6

            ],
            "profit" => [
                "currencies" => [
                    "cpc",
                ],
                "name" => "Profit",
                "type" => 31
            ]
        ];

        $packages = Packages::where("status", 1)->get()->pluck('name', 'id')->toArray();

        $currentDay = $startDate;

        $commissions = [];
        $packagesCount = [];
        $income = [];


        for ($i = (int)$startDate->diffInDays($endDate); $i >= 0; $i--) {

            $currentDayEnd = $currentDay->format('Y-m-d 23:59:59');
            $currentDayStart = $currentDay->format('Y-m-d 00:00:00');

            foreach ($packages as $pk => $package) {
                $packagesCount[$i][$pk] = UsersPackages::where([
                    'status' => 1,
                    'package_id' => $pk
                ])->whereBetween('created_at', [$currentDayStart, $currentDayEnd])->count();

            }
            // dd($packages);
            $income[$i] = UserAddresses::withTrashed()->whereBetween('created_at', [$currentDayStart, $currentDayEnd])->sum('received');

            foreach (array_keys($commissionTypes) as $ck) {
                $commissions[$i]['start'] = $currentDayStart;
                $commissions[$i]['end'] = $currentDayEnd;

                foreach ($commissionTypes[$ck]['currencies'] as $currency) {
                    $commissions[$i]['commissions'][$ck] = Transactions::join('remote_transactions', 'transactions.id', '=', 'remote_transactions.transaction_id')
                        ->selectRaw('SUM(remote_transactions.result_data) as "' . $currency . '", SUM(remote_transactions.amount) as usd')
                        ->where([
                            'transactions.type' => $commissionTypes[$ck]['type'],
                            'remote_transactions.status' => 2,
                            'remote_transactions.coin_type' => $currency
                        ])
                        ->whereBetween('transactions.created_at', [$currentDayStart, $currentDayEnd])
                        ->first()
                        ->toArray();
                }
            }

            $currentDay = $currentDay->subDay();
        }


        $pastIncome['last_week'] = UserAddresses::withTrashed()->whereBetween('created_at', [Carbon::today()->subDays(7)->format('Y-m-d 00:00:00'), Carbon::today()->format('Y-m-d 23:59:59')])->sum('received');
        $pastIncome['last_month'] = UserAddresses::withTrashed()->whereBetween('created_at', [Carbon::today()->subDays(30)->format('Y-m-d 00:00:00'), Carbon::today()->format('Y-m-d 23:59:59')])->sum('received');
        $pastIncome['all_time'] = UserAddresses::withTrashed()->sum('received');




        return view('admin.dashboard.index', [
            'totals' => $totals,
            'pastIncome' => $pastIncome,
            'income' => $income,
            'lastPrices' => $lastPrices,
            'commissions' => $commissions,
            'commissionTypes' => $commissionTypes,
            'packages' => $packages,
            'packagesCount' => $packagesCount,
            'days' => $days
            ])->withTitle('Dashboard');
	}

}
