<?php namespace App\Http\Controllers\Admin;

use Validator, Input, Session, Redirect, File;

class EmailTemplatesController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		return view('admin.emailtemplates.index')->withTitle('Email Templates');
	}

	public function getEdit($id = null)
	{
		if( is_null( $template = \App\Models\Admin\EmailTemplates::where('id', $id)->first() ) )
			return Redirect::route('admin.emailtemplates')->with('error', 'Wrong email template ID');

		$template->content = str_replace(['@extends("emails.template")', '@section("content")', "@stop"], ['', '', ''], $template->content);

		return view('admin.emailtemplates.edit', ['template' => $template])->withTitle($template->title);
	}

	public function postEdit($id = null)
	{
		if( is_null( $template = \App\Models\Admin\EmailTemplates::where('id', $id)->first() ) )
			return Redirect::route('admin.emailtemplates')->with('error', 'Wrong email template ID');

		$data = Input::all();

		$rules = [
			'title' => 'required|unique:admin_email_templates,title,' . $template->title . ',title',
			'content' => 'required'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      return Redirect::back()->withInput()->withErrors($validator);

		$template->title = $data['title'];
		$template->content = '@extends("emails.template") @section("content")' . $data['content'] . '@stop';

		$template->content = str_replace(['&gt;'], ['>'], $template->content);

		if( $template->path !== NULL ){
			$view_path = base_path('resources' . DIRECTORY_SEPARATOR . 'views' . DIRECTORY_SEPARATOR . 'emails' . DIRECTORY_SEPARATOR);
			$view_name = str_replace('.', DIRECTORY_SEPARATOR, $template->path);
			$_d = array_slice(explode(DIRECTORY_SEPARATOR, $view_name), 0, -1);
			$dir = implode(DIRECTORY_SEPARATOR, $_d);
			if( !File::exists( $view_path . $dir ) ) File::makeDirectory( $view_path . $dir );
			File::put($view_path . $view_name . '.blade.php', $template->content);
		}

		if( $template->save() )
			return Redirect::route('admin.emailtemplates')->with('success', 'Email template successfully saved.');

		return Redirect::back()->with('error', 'Could not save the email template!');
	}

	public function getNew()
	{
		return view('admin.emailtemplates.new')->withTitle('Create New');
	}

	public function postNew()
	{
		$data = Input::all();

		$rules = [
			'title' => 'required|unique:admin_email_templates,title',
			'content' => 'required'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      return Redirect::back()->withInput()->withErrors($validator);

		$template = new \App\Models\Admin\EmailTemplates;
		$template->title = $data['title'];
		$template->content = $data['content'];

		$template->content = str_replace(['&gt;'], ['>'], $template->content);

		if( $template->save() )
			return Redirect::route('admin.emailtemplates')->with('success', 'Email template successfully saved.');

		return Redirect::back()->with('error', 'Could not save the email template!');
	}

}
