<?php namespace App\Http\Controllers\Admin;

use Validator, Input, Session, Redirect;

class FaqManagerController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		$faqs = \App\Models\Faqs::orderBy('top', 'desc')->orderBy('id', 'asc')->get();
		return view('admin.faq-manager.index', ['faqs' => $faqs])->withTitle('F.A.Q. Manager');
	}

	public function getView($id = null)
	{
		if( is_null( $faq = \App\Models\Faqs::where('id', $id)->first() ) )
			return Redirect::route('admin.faq-manager')->with('error', 'Wrong FAQ ID');

		return view('admin.faq-manager.view', ['faq' => $faq])->withTitle($faq->title);
	}

	public function postView($id = null)
	{
		if( is_null( $faq = \App\Models\Faqs::where('id', $id)->first() ) )
			return Redirect::route('admin.faq-manager')->with('error', 'Wrong FAQ ID');

		$data = Input::all();
		$data['top'] = Input::get('top', 0);

		$rules = [
			'translate.*.question' => 'required',
			'translate.*.answer' => 'required',
			'top' => 'required|boolean'
		];


		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      return Redirect::back()->withInput()->withErrors($validator);

	  	$faq->fill(request()->get('translate'));
		$faq->top = $data['top'];

		if( $faq->save() )
			return Redirect::route('admin.faq-manager')->with('success', 'FAQ successfully updated.');

		return Redirect::back()->with('error', 'Error occured. Try to save again!');
	}

	public function getNew()
	{
		return view('admin.faq-manager.new')->withTitle('Create New');
	}

	public function postNew()
	{
		$data = Input::all();
		$data['top'] = Input::get('top', 0);

		$rules = [
			'translate.*.question' => 'required|unique:faqs_translations,question',
			'translate.*.answer' => 'required',
			'top' => 'required|boolean'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      return Redirect::back()->withInput()->withErrors($validator);


		$faq = new \App\Models\Faqs;
		$faq->top = $data['top'];
		$faq->fill(request()->get('translate'));
		
		if( $faq->save() )
			return Redirect::route('admin.faq-manager')->with('success', 'FAQ successfully created.');

		return Redirect::back()->with('error', 'Could not save FAQ. Please try again!');

	}

	public function getDelete($id = null)
	{
		if( is_null( $faq = \App\Models\Faqs::where('id', $id)->first() ) )
			return Redirect::route('admin.faq-manager')->with('error', 'Wrong FAQ ID');

		$faq->delete();

		return Redirect::back()->with('success', 'FAQ successfully deleted.');
	}
}