<?php namespace App\Http\Controllers\Admin;

use Redirect, DB, Validator, Input, Session;

class NewsController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		return view('admin.news.index')->withTitle('News Manager');
	}

	public function getEdit( $id = null )
	{
		if( is_null( $article = \App\Models\News::where('id', $id)->first() ) )
			return Redirect::route('admin.news')->with('error', 'Wrong news ID');

		return view('admin.news.edit', ['article' => $article])->withTitle($article->title . ' - News Manager');
	}

	public function postEdit($id = null)
	{
		if( is_null( $article = \App\Models\News::where('id', $id)->first() ) )
			return Redirect::route('admin.news')->with('error', 'Wrong news ID');

		$data = Input::all();

        $rules = [
            'translate.*.title' => 'required',
            'translate.*.content' => 'required',
        ];

		$validator = Validator::make($data, $rules);

        if ( $validator->fails() ){
            return Redirect::back()->withInput()->withErrors($validator);
        }

        $article->title_url = strtolower( changeForeignLetters( request('translate.en.title') ) ) .'-'. date('Y-m-d');
        $article->fill(request()->get('translate'));

		if( $article->save() )
			return Redirect::route('admin.news')->with('success', 'Article successfully saved.');

		return Redirect::back()->with('error', 'Could not save the article.');
	}

	public function getNew()
	{
		return view('admin.news.new')->withTitle('Create New');
	}

	public function postNew()
	{
		$data = Input::all();
		$rules = [
			'translate.*.title' => 'required|unique:news_translations,title',
			'translate.*.content' => 'required',
		];

		$validator = Validator::make($data, $rules);

		if ( $validator->fails() ){
            return Redirect::back()->withInput()->withErrors($validator);
        }

		$article = new \App\Models\News;
		$article->title_url = strtolower( changeForeignLetters( request('translate.en.title') ) ) .'-'. date('Y-m-d');
        $article->fill(request()->get('translate'));

		if( $article->save() )
			return Redirect::route('admin.news')->with('success', 'Article successfully created.');

		return Redirect::back()->with('error', 'Could not save the article.');
	}

}