<?php namespace App\Http\Controllers\Admin;

use Input, DB, Redirect, Validator;

class PackagesController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		return view('admin.packages.index')->withTitle('Packages');
	}

	public function getNew()
	{
		return view('admin.packages.new')->withTitle( 'Create' );
	}

	public function postNew()
	{
		$data = Input::all();

		$data['principle_return'] = Input::get('principle_return', 0);
		$data['matching_bonus'] = Input::get('matching_bonus', 0);
		$data['infinity_bonus'] = Input::get('infinity_bonus', 0);
		$rules = [
			'name' => 'required|unique:packages,name',
			'description' => 'required',
			'status' => 'required|boolean',
			'price' => 'required|numeric',
			'offset_units' => 'required|numeric',
			'resale_units' => 'required|numeric',
			'wholesale_credit' => 'required|numeric',
			'cv' => 'required|numeric',
			'matching_bonus' => 'required|boolean',
			'infinity_bonus' => 'required|boolean',
			//'status_upgrade' => 'required|boolean'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails()) {
      		return Redirect::back()->withInput()->withErrors($validator);
	    }

	    $package = new \App\Models\Packages;

	    foreach( $rules as $name => $r ) {
	    	$package->$name = $data[$name];
	    }

		if( $package->save() )
			return Redirect::route('admin.packages')->with('success', 'Package successfully saved.');

		return Redirect::back()->with('error', 'Failed to save package.');
	}

	public function view( $id = null )
	{
		if( is_null( $t = \App\Models\Packages::where('id', $id)->first() ) ) {
			return Redirect::back()->with('error', 'Wrong package ID');
		}

		return view('admin.tokens.view', ['t' => $t])->withTitle( $t->name );
	}

	public function viewPost($id = null)
	{
		if( is_null( $package = \App\Models\Packages::where('id', $id)->first() ) )
			return Redirect::route('admin.packages')->with('error', 'Wrong package ID');

		$data = Input::all();

		$data['status'] = Input::get('status', 0);
		$data['matching_bonus'] = Input::get('matching_bonus', 0);
		$data['infinity_bonus'] = Input::get('infinity_bonus', 0);
		$rules = [
			'name' => 'required|unique:packages,name',
			'description' => 'required',
			'status' => 'boolean',
			'price' => 'required|numeric',
			'offset_units' => 'required|numeric',
			'resale_units' => 'required|numeric',
			'wholesale_credit' => 'required|numeric',
			'cv' => 'required|numeric',
			'matching_bonus' => 'boolean',
			'infinity_bonus' => 'boolean',
			//'status_upgrade' => 'required|boolean'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails()) {
      		return Redirect::back()->withInput()->withErrors($validator);
   		}

	    foreach( $rules as $name => $r ) {
	    	$package->$name = $data[$name];
	    }


		if( $package->save() )
			return Redirect::route('admin.packages')->with('success', 'Package successfully updated.');

		return Redirect::back()->with('error', 'Failed to update package.');
	}
}
