<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Support\Facades\Request;
use Illuminate\Validation\Rule;
use Redirect, DB, Validator, Input, Session, Sentry, Mail, Config, MailManager, NotificationManager;

class SupportController extends AdminController {

	public $priorities = [1 => 'LOW', 'MEDIUM', 'HIGH'];

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		$this->prepareSharedData('active');

		if( $this->user->hasRole('admin') ) {

			$tickets = \App\Models\Support::active()->orderBy('id', 'desc')->get();

		} elseif( $this->user->hasRole('support') ) {

			$tickets = \App\Models\Support::active()->assigned($this->user->id)->orderBy('id', 'desc')->get();

		}

		return view('admin.support.index', ['tickets' => $tickets])->with($this->shared_data)->withTitle('Support Tickets - Active');
	}

	public function open()
	{
		$this->prepareSharedData('open');
		if( $this->user->hasRole('admin') ) {

			$tickets = \App\Models\Support::open()->orderBy('id', 'desc')->get();

		} elseif( $this->user->hasRole('support') ) {

			$tickets = \App\Models\Support::open()->assigned($this->user->id)->orderBy('id', 'desc')->get();

		}

		return view('admin.support.index', ['tickets' => $tickets])->with($this->shared_data)->withTitle('Support Tickets - Open');
	}

	public function closed()
	{
		$this->prepareSharedData('closed');
		if( $this->user->hasRole('admin') ) {

			$tickets = \App\Models\Support::closed()->orderBy('id', 'desc')->get();

		} elseif( $this->user->hasRole('support') ) {

			$tickets = \App\Models\Support::closed()->assigned($this->user->id)->orderBy('id', 'desc')->get();

		}
		return view('admin.support.index', ['tickets' => $tickets])->with($this->shared_data)->withTitle('Support Tickets - Closed');
	}

	public function view( $id = null )
	{
		if( $this->user->hasRole('admin') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		} elseif( $this->user->hasRole('support') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->assigned($this->user->id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');
		}

		if( is_null( $ticket->user ) || !$ticket->user )
			return Redirect::back()->with('error', 'User for this ticket does not exist');

		$responses = \App\Models\Admin\SupportResponses::orderBy('title', 'asc')->get();

		switch ( $ticket->status ) {
			case 0:
				$status = 'open';
				break;

			case 1:
				$status = 'active';
				break;

			case 2:
				$status = 'closed';
				break;
		}

		$this->prepareSharedData($status);

		return view('admin.support.view', ['ticket' => $ticket, 'responses' => $responses])->with($this->shared_data)->withTitle($ticket->subject);
	}

	public function delete( $id = null )
	{
		if( $this->user->hasRole('admin') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		} elseif( $this->user->hasRole('support') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->assigned($this->user->id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		}

		$replies = \App\Models\SupportReplies::where('ticket_id', $id)->get();

		$ticket->delete();

		if( $replies->count() ) \App\Models\SupportReplies::where('ticket_id', $id)->delete();

		return Redirect::back()->with('success', 'Ticket successfully deleted');

	}

	public function delete_reply( $id = null )
	{
		if( is_null( $response = \App\Models\SupportReplies::where('id', $id)->first() ) )
			return Redirect::back()->with('error', 'Reply does not exist');

		if( $this->user->hasRole('admin') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $response->ticket_id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		} elseif( $this->user->hasRole('support') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $response->ticket_id)->assigned($this->user->id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		}

		$response->delete();

		return Redirect::back()->with('success', 'Reply successfully deleted');
	}

	public function reply( $id = null )
	{
		if( $this->user->hasRole('admin') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		} elseif( $this->user->hasRole('support') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->assigned($this->user->id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');
		}

		$data = Input::all();

		$ticket->status = $data['status'];
		$ticket->save();

		$reply = new \App\Models\SupportReplies;
		$reply->ticket_id = $id;
		$reply->user_id = $this->user->id;
		if(is_null($data['content']))
			return Redirect::back()->with('error','Please write some response!');
		$reply->content = $data['content'];

		if( $reply->save() ) {
			$user = \App\Models\User::where('id', $ticket->user_id)->first();

			//$support_category = DB::table('support_tickets_categories')->where('id', $ticket->category)->first();

			\App\Facades\NotificationManager::create( [
				'type' => 'support-new-reply',
				'content_id' => $reply->ticket_id
			], $user);

      \App\Facades\MailManager::send('support.reply-ticket', 'Support Ticket Reply', [ 'id' => $ticket->id, 'subject' => $ticket->subject, 'category' => $ticket->category->name, 'priority' => $this->priorities[$ticket->priority], 'reply' => $reply->content ], $user);

			return Redirect::back()->with('success', 'Reply successfully sent.');
		}

		return Redirect::back()->with('error', 'Could not send reply.');
	}

	public function openTicket($id = null)
	{
		if( $this->user->hasRole('admin') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		} elseif( $this->user->hasRole('support') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->assigned($this->user->id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');
		}

		$ticket->status = 0;

		if( $ticket->save() ) {
			$user = \App\Models\User::where('id', $ticket->user_id)->first();

			$support_category = DB::table('support_tickets_categories')->where('id', $ticket->category_id)->first();

			NotificationManager::create( [
				'type' => 'support-reopen',
				'content_id' => $ticket->id
			], $user);

      MailManager::send('support.reopen-ticket', 'Support Ticket Re-Opened', [
				'id' => $ticket->id, 'subject' => $ticket->subject,
				'category' => $support_category->name,
				'priority' => $this->priorities[$ticket->priority] ], $user);

			return Redirect::back()->with('success', 'Support ticket successfully opened.');
		}

		return Redirect::back()->with('error', 'Could not open support ticket.');
	}

	public function closeTicket($id = null)
	{
		if( $this->user->hasRole('admin') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');

		} elseif( $this->user->hasRole('support') ) {

			if( is_null( $ticket = \App\Models\Support::where('id', $id)->assigned($this->user->id)->first() ) )
				return Redirect::route('admin.support')->with('error', 'Wrong ticket ID!');
		}

		$ticket->status = 2;

		if( $ticket->save() ) {
			$user = \App\Models\User::where('id', $ticket->user_id)->first();

			$support_category = DB::table('support_tickets_categories')->where('id', $ticket->category_id)->first();

			NotificationManager::create( [
				'type' => 'support-closed',
				'content_id' => $ticket->id
			], $user);

      MailManager::send('support.closed-ticket', 'Support Ticket Closed', [ 'id' => $ticket->id, 'subject' => $ticket->subject, 'category' => $support_category->name, 'priority' => $this->priorities[$ticket->priority] ], $user);

			return Redirect::back()->with('success', 'Support ticket successfully saved.');
		}

		return Redirect::back()->with('error', 'Could not close support ticket.');
	}

	public function canned_responses()
	{
		$responses = \App\Models\Admin\SupportResponses::orderBy('title', 'asc')->get();
		return view('admin.support.responses', ['responses' => $responses])->withTitle('Canned Support Responses');
	}

	public function view_canned_responses($id = null)
	{
		if( is_null( $response = \App\Models\Admin\SupportResponses::where('id', $id)->first() ) )
			return Redirect::route('admin.support.responses')->with('error', 'Wrong response ID!');

		return view('admin.support.responses-view', ['response' => $response])->withTitle($response->title);
	}


	public function view_canned_responsesPost($id = null)
	{
		if( is_null( $response = \App\Models\Admin\SupportResponses::where('id', $id)->first() ) )
			return Redirect::route('admin.support.responses')->with('error', 'Wrong response ID!');

		$data = Input::all();

		$rules = [
			'title' => 'required|unique:admin_support_canned,title,' . $response->title . ',title',
			'content' => 'required'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
			return Redirect::back()->withInput()->withErrors($validator);

		$response->title = $data['title'];
		$response->content = $data['content'];

		if( $response->save() )
			return Redirect::route('admin.support.responses')->with('success', 'Response successfully updated.');

		return Redirect::back()->with('error', 'Could not save response.');
	}

	public function new_canned_responses()
	{
		return view('admin.support.responses-new')->withTitle('Create New');
	}

	public function new_canned_responsesPost()
	{
		$data = Input::all();

		$rules = [
			'title' => 'required|unique:admin_support_canned,title',
			'content' => 'required'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      return Redirect::back()->withInput()->withErrors($validator);

		$response = new \App\Models\Admin\SupportResponses;
		$response->title = $data['title'];
		$response->content = $data['content'];

		if( $response->save() )
			return Redirect::route('admin.support.responses')->with('success', 'Response successfully saved.');

		return Redirect::back()->with('error', 'Could not save response!');
	}

	public function delete_canned_responses($id = null)
	{
		if( is_null( $response = \App\Models\Admin\SupportResponses::where('id', $id)->first() ) )
			return Redirect::route('admin.support.responses')->with('error', 'Wrong response ID!');

		$response->delete();

		return Redirect::back()->with('success', 'Response successfully deleted.');
	}

	public function assigned()
	{
		if( !$this->user->hasRole('admin') ) {
			return Redirect::route('admin.support');
		}

		$support_users = \App\Models\Groups::usersByPermission('support');

		$types = DB::table('support_tickets_categories')->orderBy('id', 'asc')->get();

		return view('admin.support.assigned', ['users' => $support_users, 'types' => $types])->withTitle('Assigned Users');
	}

	public function assignedPost()
	{
		if( !$this->user->hasRole('admin') ) {
			return Redirect::route('admin.support');
		}

		$data = Input::get('assigned_types');

		foreach( $data as $id => $d ) {
			DB::table('support_tickets_categories')->where('id', $id)->update(['assigned_to' => $d]);
		}

		return Redirect::back()->with('success', 'Successfully saved.');
	}

	public function getAttachment( $id = null )
	{
		if( is_null( $ticket = \App\Models\Support::where('id', $id)->whereNotNull('attachment')->first() ) )
			return Redirect::route('support')->with('error', 'Wrong ticket ID.');

		$path = storage_path() . '/uploads/support/' . $ticket->attachment;

		if( !File::exists( $path ) ) return response('Not found.', 404);

		return response()->file( $path );
	}

	private function prepareSharedData( $currentPage )
	{
		if( $this->user->hasRole('admin') ) {

			$this->active_count = \App\Models\Support::active()->count();
			$this->open_count = \App\Models\Support::open()->count();
			$this->closed_count = \App\Models\Support::closed()->count();

		} elseif( $this->user->hasRole('support') ) {

			$this->active_count = \App\Models\Support::active()->assigned($this->user->id)->count();
			$this->open_count = \App\Models\Support::open()->assigned($this->user->id)->count();
			$this->closed_count = \App\Models\Support::closed()->assigned($this->user->id)->count();

		}

		$this->ticket_categories = [];
		$ticket_categories = DB::table('support_tickets_categories')->orderBy('id', 'asc')->get();

		foreach( $ticket_categories as $tc ) {

			$category = \App\Models\SupportCategories::where('id', $tc->id)->first();
			if($category)
			$this->ticket_categories[ $tc->id ] = $category->translate('en')->name;
		}

		$this->shared_data = [
			'active_count' => $this->active_count,
			'open_count' => $this->open_count,
			'closed_count' => $this->closed_count,

			'ticket_categories' => $this->ticket_categories,
			'current' => $currentPage
		];
	}


	public function getSupportCategories()
    {
        return view('admin.support.categories.index')->withTitle('Categories - Support Manager');
    }

    public function getSupportCategoriesView( $id = null )
    {
        if( is_null( $category = \App\Models\SupportCategories::where('id', $id)->first() ) )
            return Redirect::route('admin.support.categories')->with('error', 'Wrong category ID');

        return view('admin.support.categories.view', ['category' => $category])->withTitle($category->name . ' - Support Manager');
    }

    public function postSupportCategoriesView( Request $request, $id = null )
    {
        $category = \App\Models\SupportCategories::where('id', $id)->first();

        if( is_null($category ) ){
            return redirect()->route('admin.support.categories')->with('error', 'Wrong category ID');
        }

        $data = Input::all();
        $data['status'] = Input::get('status', 0);
        $data['assigned_to'] = Input::get('assigned_to', 0);

        $rules = [
			'translate.*.name' => 'required',
            'status' => 'required|boolean',
        ];

        $validator = Validator::make( $data, $rules );

        if ( $validator->fails() )
            return Redirect::back()->withInput()->withErrors( $validator );

        /*$thumbnailFilename = $category->thumbnail;

        if(
        $request->hasFile('thumbnail')
        ) {
            $file = $request->file('thumbnail');

            $filename = str_random(40) . '.' . $file->getClientOriginalExtension();
            $thumbnailPath = storage_path('app/public/auction/categories/');

            Image::make($file->getRealPath())->resize(400, null, function($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })->save($thumbnailPath . $filename);

            //$path = $f->storeAs('auction/categories', $filename, 'public');

            $thumbnailFilename = $filename;
        }
        */
        if(!$category){
            $category = new \App\Models\SupportCategories;
        }

		$category->fill(request()->get('translate'));
        $category->status = $data['status'];
        $category->assigned_to = $data['assigned_to'];

        if( $category->save() )
            return Redirect::route('admin.support.categories')->with('success', 'Category successfully saved.');

        return Redirect::back()->with('error', 'Could not save the category.');
    }

    public function getSupportCategoriesNew()
    {
        return view('admin.support.categories.new')->withTitle('New Category - Support Manager');
    }

    public function postSupportCategoriesNew(Request $request)
    {
        $data = Input::all();
        $data['status'] = Input::get('status', 0);
        $data['name'] = Input::get('name', '');
        $data['assigned_to'] = 0;

        $rules = [
            'translate.*.name' => 'required|unique:stc_translations,name',
            'status' => 'required|boolean',
        ];


        //clock()->info($data);
        /*
        clock()->info($category);
        clock()->info($thumbnailFilename);
		*/

        $validator = Validator::make($data, $rules);

        if ( $validator->fails() )
            return Redirect::back()->withInput()->withErrors($validator);


        $category = new \App\Models\SupportCategories;
		$category->fill(request()->get('translate'));

        /*$thumbnailFilename = $category->thumbnail;

        if(
        $request->hasFile('thumbnail')
        ) {
            $file = $request->file('thumbnail');

            $filename = str_random(40) . '.' . $file->getClientOriginalExtension();
            $thumbnailPath = storage_path('app/public/auction/categories/');

            Image::make($file->getRealPath())->resize(400, null, function($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })->save($thumbnailPath . $filename);

            //$path = $f->storeAs('auction/categories', $filename, 'public');

            $thumbnailFilename = $filename;
        }



        $category->thumbnail = $thumbnailFilename;
        */
        $category->status = $data['status'];
        $category->assigned_to = $data['assigned_to'];

        if( $category->save() )
            return Redirect::route('admin.support.categories')->with('success', 'Category has successfully created.');

        return Redirect::back()->with('error', 'Could not save the category.');
    }
}
