<?php namespace App\Http\Controllers\Admin;

use Input, Redirect, Validator, Session, Queue, NotificationManager;

class ToolsController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

	public function getReportUploader()
	{
		$reports = \App\Models\Reports::where('status', 1)->orderBy('date', 'DESC')->get();

		return view('admin.tools.report-uploader', ['reports' => $reports])->withTitle('Report Uploader');
	}

	public function postReportUploader()
	{
		$rules = [
			'attachment_1' 	=> 'required|max:10000000',
			'attachment_2' 	=> 'required|max:10000000',
			'date' 					=> 'required|date',
			'week' 					=> 'required|in:1,2'
		];

		$inputs = Input::all();
		$validator = Validator::make( $inputs, $rules );

		if ( $validator->fails() )
  		return Redirect::back()->withInput()->withErrors( $validator );

   	if ( Input::hasFile('attachment_1') && Input::file('attachment_1')->isValid() ) {
   		$destinationPath = storage_path() . '/uploads/reports/';
      $extension = Input::file('attachment_1')->getClientOriginalExtension();
      $fileName = str_random(25) . '.' . $extension;
      Input::file('attachment_1')->move( $destinationPath, $fileName );

      $inputs['attachment_1'] = $fileName;
		}

		if ( Input::hasFile('attachment_2') && Input::file('attachment_2')->isValid() ) {
   		$destinationPath = storage_path() . '/uploads/reports/';
      $extension = Input::file('attachment_2')->getClientOriginalExtension();
      $fileName = str_random(25) . '.' . $extension;
      Input::file('attachment_2')->move( $destinationPath, $fileName );

      $inputs['attachment_2'] = $fileName;
		}

    $report = new \App\Models\Reports;
		$report->file_1 = $inputs['attachment_1'];
		$report->file_2 = $inputs['attachment_2'];
		$report->status = 1;
		$report->week = $inputs['week'];
		$report->date = date( 'Y-m-01', strtotime( $inputs['date'] ) );

    if( $report->save() ) {
    	return Redirect::back()->with('success', 'Report has been successfully uploaded!');
    } else return Redirect::back()->with('error', 'There was an error while trying to upload the report.');
	}

	public function getTokenTransactions()
	{
		$plans = \App\Models\Packages::orderBy('id', 'asc')->get();
		$processors = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();
		return view('admin.tools.token-transactions', ['plans' => $plans, 'processors' => $processors])->withTitle('Token Transaction');
	}

	public function postTokenTransactions()
	{
		$method = Input::get('method');
		$datas = Input::all();
		$error = false;

		switch ($method) {
			case 'interest':

				$datas['amount_format'] = Input::get('amount_format', 0);

				$rules = [
					'token_id' => 'required|exists:users_plans,id|numeric',
					'amount' => 'required|numeric',
					'amount_format' => 'required|boolean'
				];

				$validator = Validator::make(Input::except(['_token','method']), $rules);

				if ($validator->fails())
					return Redirect::back()->withInput()->withErrors($validator);

				if( !is_null($plan = \App\Models\UsersPlans::where('id', $datas['token_id'])->where('status', 1)->first() ) ) {

					if( $datas['amount_format'] == 1 ) {
						$interest = ( $plan->balance * $datas['amount'] ) / 100;
					} else {
						$interest = $datas['amount'];
					}

					$plan->balance = $plan->balance + $interest;
					$plan->save();

					$transaction = new \App\Models\Transactions;
					$transaction->processor_id = $plan->processor_id;
					$transaction->user_id = $plan->user_id;
					$transaction->type = 3;
					$transaction->amount = $interest;
					$transaction->note = 'Earned $' . $interest . ' on Plan #' . $plan->id;
					$transaction->status = 1;
					$transaction->save();

					$success = 'Token has been successfully saved.';


				} else $error = 'Error occured. Please try again!';


				break;

			case 'create':
				$datas['charge_member'] = Input::get('charge_member', 0);

				$rules = [
					'member_id' => 'required|exists:users,id|numeric',
					'amount' => 'required|numeric',
					'token_id' => 'required|exists:tokens,id',
					'processor_id' => 'required|exists:processors,id'
				];

				$validator = Validator::make(Input::except(['_token','method']), $rules);

				if ($validator->fails())
					return Redirect::back()->withInput()->withErrors($validator);

				$new_plan = new \App\Models\UsersPlans;
				$new_plan->user_id = $datas['member_id'];
				$new_plan->token_id = $datas['token_id'];
				$new_plan->processor_id = $datas['processor_id'];
				$new_plan->invested_amount = $datas['amount'];
				$new_plan->balance = 0;
				$new_plan->status = 1;
				$new_plan->last_roll = \App\Models\Date::format(date('Y-m-d', time()), 'Y-m-d');
				$new_plan->save();

				if( $datas['charge_member'] == 1 ) {
					$user_balance = \App\Models\UsersBalances::where('users_id', $datas['member_id'])->where('processor_id', $datas['processor_id'])->first();
					if( $user_balance->wallet > 0 ) {
						$user_balance->wallet = $user_balance->wallet - $datas['amount'];
						$user_balance->save();
					}
				}

				$this->dispatch( new \App\Jobs\SetPlan( $new_plan->id ) );

				$success = 'Token has been successfully created.';

				break;

			case 'expire':

				$rules = [
					'token_id' => 'required|exists:users_plans,id|numeric'
				];

				$validator = Validator::make(Input::except(['_token','method']), $rules);

				if ($validator->fails())
					return Redirect::back()->withInput()->withErrors($validator);

				$token = \App\Models\UsersPlans::where('id', $datas['token_id'])->first();
				$token->status = 0;
				$token->save();

				$success = 'Token has been successfully expired.';

				break;

			default:

				return Redirect::back()->with('error', 'Wrong method.');

				break;
		}

		if( $error ) {
			return Redirect::back()->with('error', $error)->withInput();
		}

		return Redirect::back()->with('success', $success);

	}

	public function getBankTransactions()
	{
		$processors = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();
    $pools = \App\Models\UsersBalances::$pools;

		return view('admin.tools.bank-transactions', ['processors' => $processors, 'pools' => $pools])->withTitle('Bank Transactions');
	}

	public function postBankTransactions()
	{
		$rules = [
      1 => [
          'method' => 'required',

          'member_id_to' => 'required|exists:users,id',
          'member_id_from' => 'required|exists:users,id',
          'pool_from' => '',
          'pool_to' => '',

          'processor_to' => 'exists:processors,id',
          'processor_from' => 'exists:processors,id',

          'amount' => 'required|numeric',
          'account_id' => 'numeric',
          'note' => ''
      ],
      2 => [
          'method' => 'required',

          'member_id_to' => 'required|exists:users,id',
          'pool_to' => '',

          'processor_to' => 'exists:processors,id',

          'amount' => 'required|numeric',
          'account_id' => 'numeric',
          'note' => ''
      ],
      3 => [
          'method' => 'required',

          'member_id_from' => 'required|exists:users,id',
          'pool_from' => '',

          'processor_from' => 'exists:processors,id',

          'amount' => 'required|numeric',
          'account_id' => 'numeric',
          'note' => ''
      ]
		];

		$method = Input::get('method');
		$data = Input::all();
		$error = false;

		$validator = Validator::make($data, $rules[$method]);

		if ($validator->fails())
			return Redirect::back()->withInput()->withErrors($validator);

    $pools = \App\Models\UsersBalances::$pools;

		switch ($method) {
			case 1:

			if( is_null( $member_to = \App\Models\User::where('id', $data['member_id_to'])->orWhere('username', $data['member_id_to'])->first() ) ) {
				return Redirect::back()->with('error', 'Wrong username/user ID.')->withInput();
	    }

						if( $data['member_id_from'] !== '' ) {

					    if( is_null( $member_from = \App\Models\User::where('id', $data['member_id_from'])->orWhere('username', $data['member_id_from'])->first() ) ) {
								return Redirect::back()->with('error', 'Wrong username/user ID.')->withInput();
					    }

							$user_wallet = \App\Models\UsersBalances::where('processor_id', $data['processor_from'])->where('users_id', $data['member_id_from'])->first();

              if( $user_wallet->$pools[$data['pool_from']]['name'] < $data['amount'] ) {
								return Redirect::back()->with('error', 'User does not have enough money.')->withInput();
              }

              $user_wallet->$pools[$data['pool_from']]['name'] -= $data['amount'];
              $user_wallet->save();
            }

                $member_wallet = \App\Models\UsersBalances::where('users_id', $data['member_id_to'])->where('processor_id', $data['processor_to'])->first();
                $member_wallet->$pools[$data['pool_to']]['name'] += $data['amount'];
                $member_wallet->save();

              if( $data['member_id_from'] !== '' ) {

                // Transaction to
                $new_transaction = new \App\Models\Transactions;
                $new_transaction->processor_id = $data['processor_from'];
                $new_transaction->user_id = $data['member_id_from'];
                $new_transaction->type = 9;
                $new_transaction->amount = $data['amount'];
                $new_transaction->balance = $pools[$data['pool_from']]['name'];
                $new_transaction->note = 'Sent $' . $data['amount'] . ' from balance ' . $pools[$data['pool_from']]['label'] . ' to ' . $member_wallet->user->username . ' (#' . $member_wallet->users_id . ')' . ( ( $data['note'] !== '' ) ? ' - ' . $data['note'] : '' );;
                $new_transaction->status = 1;
                $new_transaction->save();

								NotificationManager::create( [
									'type' => 'wallet.transfer-to',
									'content_id' => $new_transaction->id
								], $new_transaction->user);

              }


                // Transaction from
                $new_transaction2 = new \App\Models\Transactions;
                $new_transaction2->processor_id = $data['processor_to'];
                $new_transaction2->user_id = $member_wallet->users_id;
                $new_transaction2->type = 5;
                $new_transaction2->amount = $data['amount'];
                $new_transaction2->balance = $pools[$data['pool_to']]['name'];
                $new_transaction2->note = ( $data['note'] !== '' ) ? ' - ' . $data['note'] : '';
                $new_transaction2->status = 1;
                $new_transaction2->save();

								NotificationManager::create( [
									'type' => 'wallet.transfer-form',
									'content_id' => $new_transaction2->id
								], $new_transaction2->user);

                $success = 'Transaction successfully saved.';
                break;

						case 2:
								if( is_null( $member_to = \App\Models\User::where('id', $data['member_id_to'])->orWhere('username', $data['member_id_to'])->first() ) ) {
									return Redirect::back()->with('error', 'Wrong username/user ID.')->withInput();
						    }

								$processor = \App\Models\Processors::where('id', '!=', 4)->where('id', $data['processor_to'])->first();

                $user_wallet = \App\Models\UsersBalances::where('processor_id', $data['processor_to'])->where('users_id', $data['member_id_to'])->first();

                //if( $user_wallet->wallet < $data['amount'] ) {
                    //return Redirect::back()->with('error', trans('wallet.notenoughmoney'))->withInput();
                //}

                $user_wallet->$pools[$data['pool_to']]['name'] = ($user_wallet->wallet + $data['amount']);
                $user_wallet->save();

								$new_transaction = new \App\Models\Transactions;
                $new_transaction->processor_id = $data['processor_to'];
                $new_transaction->user_id = $data['member_id_to'];
                $new_transaction->type = 1;
                $new_transaction->fee = $processor->feeAmount('fund', $data['amount']);
                $new_transaction->amount = $processor->netAmount('fund', $data['amount']);
                $new_transaction->balance = $pools[$data['pool_to']]['name'];
                $new_transaction->note = 'Deposit of ' . $processor->netAmount('fund', $data['amount']) . ' ($' .$processor->feeAmount('fund', $data['amount']). ' fee) to balance ' . $pools[$data['pool_to']]['label'] . ' (' . $processor->name . ')';
                $new_transaction->status = 1;
                $new_transaction->save();

                $success = 'Transaction successfully saved.';
                break;

			case 3:

				if( is_null( $member_to = \App\Models\User::where('id', $data['member_id_from'])->orWhere('username', $data['member_id_from'])->first() ) ) {
					return Redirect::back()->with('error', 'Wrong username/user ID.')->withInput();
		    }

                $processor = \App\Models\Processors::where('id', '!=', 4)->where('id', $data['processor_to'])->first();

                $user_wallet = \App\Models\UsersBalances::where('processor_id', $data['processor_to'])->where('users_id', $data['member_id_from'])->first();

                if( $user_wallet->wallet < $data['amount'] ) {
                  return Redirect::back()->with('error', 'User has not enough money.')->withInput();
                }

                $user_wallet->$pools[$data['pool_to']]['name'] = ($user_wallet->wallet - $data['amount']);
                $user_wallet->save();

                $new_transaction = new \App\Models\Transactions;
                $new_transaction->processor_id = $data['processor_from'];
                $new_transaction->user_id = $data['member_id_from'];
                $new_transaction->type = 7;
                $new_transaction->fee = $processor->feeAmount('cashout', $data['amount']);
                $new_transaction->amount = $processor->netAmount('cashout', $data['amount']);
                $new_transaction->balance = $pools[$data['pool_from']]['name'];
                $new_transaction->note = 'Withdrawal of ' . $processor->netAmount('fund', $data['amount']) . ' ($' .$processor->feeAmount('cashout', $data['amount']). ' fee) from balance ' . $pools[$data['pool_from']]['label'] . ' (' . $processor->name . ')';
                $new_transaction->status = 0;
                $new_transaction->save();

								NotificationManager::create( [
									'type' => 'wallet.cashout',
									'content_id' => $new_transaction->id
								], $user);

                $success = 'Transaction successfully saved.';
    			break;

			default:
				return Redirect::back()->with('error', 'Wrong method!');
    			break;
		}

		if( $error ) {
			return Redirect::back()->with('error', $error)->withInput();
		}

		return Redirect::back()->with('success', $success);
	}
}
