<?php namespace App\Http\Controllers\Admin;

use DB, Redirect, Session, Input, Config, Validator, Excel, Image, File, MessageBag, MailManager, Carbon\Carbon;
use Illuminate\Http\Request;

class UsersController extends AdminController
{

    public function getIndex()
    {
        return view('admin.users.index')->withTitle('Users');
    }

    public function getCreate()
    {
        $groups = \App\Models\Groups::orderBy('name', 'asc')->get();
        $countries = \App\Models\Country::orderBy('id', 'asc')->get();

        return view('admin.users.create', [
            'groups' => $groups,
            'countries' => $countries
        ])->withTitle('Create User');
    }

    public function postCreate()
    {
        return DB::transaction(function () {

            $rules = [
                'first_name' => 'min:3|max:50',
                'last_name' => 'min:3|max:50',
                // 'birth_date'    => 'required|date_format:Y-m-d',
                //'country'       => 'exists:countries,id',
                // 'zip'           => 'required|max:25',
                //'city'          => 'max:100',
                //'state'         => 'required|max:100',
                //'address'       => 'required|max:200',
                //'phone'         => 'required|between:6,20',

                'username' => 'required|alpha_num|min:3|max:100|unique:users,username',
                'email' => 'required|email|min:3|max:200|unique:users,email',

                //'sponsor_code'      => '',
                //'upline_user'       => '',
                //'leg'               => 'required_with:upline_user|in:1,2',

                'groups' => 'required|array',

                'password' => 'required|between:6,32',
                'password_confirm' => 'required|between:6,32|same:password',
                //'secondary_password'    => 'required|between:6,32',
                //'secondary_password_confirm'    => 'required|between:6,32|same:secondary_password',
            ];

            //$data = Input::all();

            $validator = Validator::make(Input::all(), $rules, ['captcha.captcha' => 'Invalid captcha']);

            if ($validator->fails())
                return Redirect::back()->withErrors($validator)->withInput();

            foreach ($rules as $key => $r)
                $data[$key] = Input::get($key, NULL);

            $sponsor = null;
            $upline_user = null;
            $error = false;
            $leg = 0;

            // Sponsor code sent
            if (isset($data['sponsor_code']) && !is_null($data['sponsor_code']) && $data['sponsor_code'] !== '') {

                // Check sponsor code
                if (is_null($sponsor = \App\Models\Sponsor::select(DB::raw('sponsors.*, u.username'))
                    ->leftJoin('users AS u', 'u.id', '=', 'sponsors.user_id')
                    ->orWhere('u.id', $data['sponsor_code'])
                    ->orWhere('user_code', $data['sponsor_code'])
                    ->orWhere('username', 'LIKE', '%' . $data['sponsor_code'] . '%')
                    ->orWhere('email', 'LIKE', '%' . $data['sponsor_code'] . '%')
                    ->first()
                )) $error = 'Sponsor not found.';

            }

            // Upline user check
            if (isset($data['upline_user']) && !is_null($data['upline_user']) && $data['upline_user'] !== '') {

                if (is_null($upline_user = \App\Models\Sponsor::select(DB::raw('sponsors.*, u.username'))
                    ->leftJoin('users AS u', 'u.id', '=', 'sponsors.user_id')
                    ->orWhere('u.id', $data['upline_user'])
                    ->orWhere('user_code', $data['upline_user'])
                    ->orWhere('username', 'LIKE', '%' . $data['upline_user'] . '%')
                    ->orWhere('email', 'LIKE', '%' . $data['upline_user'] . '%')
                    ->where('children', '<', 3)
                    ->first()
                )) $error = 'Upline user not found.';
                else {
                    if ($upline_user->children !== 3) {

                        $leg = $data['leg'];

                        if ($upline_user->children == 1 && $leg == 1) $error = 'Wrong leg has been choosen.1';
                        elseif ($upline_user->children == 2 && $leg == 2) $error = 'Wrong leg has been choosen.2';

                    } else $error = 'Upline user hasn\'t got any free legs left.';
                }
            }

            if ($error) {
                //$messageBag = new MessageBag([$error]);
                return Redirect::back()->with('error', $error)->withInput();
            }

            $user = new \App\Models\User;
            $user->id = random_num(8);
            $user->first_name = $data['first_name'];
            $user->last_name = $data['last_name'];
            $user->username = $data['username'];
            $user->email = $data['email'];
            $user->sponsor_code = random_num(8);
            $user->password = $data['password'];
            $user->secondary_password = $data['password'];

            if ($user->save()) {

                $allGroups = \App\Models\Groups::all();

                foreach ($allGroups as $group) {
                    if (isset($data['groups'][$group->id]))
                        $user->addGroup($group);
                }

                $user_profile = new \App\Models\UserProfile;
                $user_profile->id = $user->id;
                //$user_profile->birth_date   = \App\Models\Date::format($data['birth_date'], 'Y-m-d');
                $user_profile->avatar = NULL;
                $user_profile->country_id = NULL;
                //$user_profile->zip          = $data['zip'];
                //$user_profile->city         = $data['city'];
                //$user_profile->state        = $data['state'];
                //$user_profile->address      = $data['address'];
                //$user_profile->phone        = $data['phone'];
                $user_profile->save();

                $processors = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();

                foreach ($processors as $p) {
                    $new_processor = new \App\Models\UsersProcessors;
                    $new_processor->user_id = $user->id;
                    $new_processor->processor_id = $p->id;
                    $new_processor->status = -1;
                    $new_processor->save();

                    $new_balance = new \App\Models\UsersBalances;
                    $new_balance->users_id = $user->id;
                    //$new_balance->processor_id = $p->id;
                    $new_balance->save();
                }

                //\App\Models\Sponsor::addToTree($sponsor, $upline_user, $user, $leg);

                MailManager::send('auth.new-account', 'New Account', [], $user);

                return redirect()->route('admin.users')->with('success', 'User has been successfully created.');
            }

        });
    }

    public function getSendNewPassword($user_id = null)
    {
        if (is_null($user = \App\Models\User::where('id', $user_id)->first()))
            return redirect()->back()->with('error', 'Wrong user ID');

        $newPassword = str_random(8);

        $user->password = $newPassword;
        $user->save();

        MailManager::send('users.new-password-system', 'New password - System Generated', [
            'newPassword' => $newPassword
        ], $user);

        return redirect()->back()->with('success', 'New password has been successfully generated.');
    }

    public function becomeMember($user_id = null)
    {
        if (!is_null($user = \App\Models\User::where('id', $user_id)->first())) {

            Session::put('becomeMember', $user->id);
            Session::put('originalAccount', $this->user->id);

            return Redirect::route('home');

        } else {
            return Redirect::back();
        }
    }

    public function getUserInfo($id, $section = 'details', $edit = false)
    {
        if ($edit && $section == 'details') $section = $section . '-' . $edit;

        $member = \App\Models\User::where('id', $id)->first();

        if (is_null($member))
            return redirect()->route('admin.users')->with('Wrong user ID');

        $datas = [
            'member' => $member,
            'section' => $section
        ];

        switch ($section) {
            case 'details':
            case 'details-edit':
                $datas['user_groups'] = \Spatie\Permission\Models\Role::orderBy('name', 'asc')->get();

                //$datas['user_group_roles'] = Spatie\Permission\Models\Group::orderBy('name', 'asc')->get();

                $datas['countries'] = \App\Models\Country::orderBy('id', 'asc')->get();
                break;

            case 'balances':
                $datas['e_wallet'] = $e_wallet = \App\Models\UsersBalances::where('users_id', $id)->first();

                $_transactions = \App\Models\Transactions::select(DB::raw('amount, type, created_at'))->whereIn('type', [1, 4, 7, 11])->whereRaw('created_at BETWEEN NOW() - INTERVAL 30 DAY AND NOW()')->where('user_id', $id)->get();
                $transactions = [1 => 0.00, 4 => 0.00, 7 => 0.00, 11 => 0.00];

                foreach ($_transactions as $t) {
                    $transactions[$t->type] += $t->amountNumber;
                }

                $tokens = \App\Models\Packages::pluck('name', 'id');
                $_users_plans = \App\Models\UsersPackages::select(DB::raw('SUM(COALESCE(`invested_amount`, 0.00)) as invested_amount, package_id'))->where('status', 1)->where('user_id', $id)->groupBy('package_id')->get();
                $users_plans = [];

                foreach ($_users_plans as $up) {
                    $users_plans[$up->token_id] = $up;
                }

                $users_processors = \App\Models\UsersProcessors::where('user_id', $member->id)->orderBy('created_at', 'desc')->get();

                $processors = \App\Models\Processors::where('id', '!=', 4)->get();

                $datas['transactions'] = $transactions;
                $datas['tokens'] = $tokens;
                $datas['users_plans'] = $users_plans;
                $datas['processors'] = $processors;
                break;

            case 'accounting':

                break;

            case 'transactions':

                $datas['processors'] = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();
                $datas['types'] = \App\Models\Transactions::$types;
                $datas['icons'] = \App\Models\Transactions::$icons;

                break;

            case 'packages':
                $datas['packages'] = \App\Models\Packages::orderBy('id', 'asc')->get();
                $datas['processors'] = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();
                break;

            case 'processors':
                $datas['processors'] = \App\Models\Processors::where('id', '!=', 1)->where('id', '!=', 4)->orderBy('id', 'asc')->get();
                $ups = \App\Models\UsersProcessors::where('processor_id', '!=', 1)->where('user_id', $member->id)->orderBy('id', 'asc')->get();

                $uprocessors = [];

                foreach ($ups as $up) {
                    $uprocessors[$up->processor_id] = $up;
                }

                $datas['uprocessors'] = $uprocessors;

                break;
        }

        //clock()->info($datas['user_groups']);

        //clock()->info('admin: ' . $member->hasRole('admin'));
        //clock()->info('support: ' . $member->hasRole('support'));
        //clock()->info('user: ' . $member->hasRole('user'));

        return view('admin.users.popup', $datas)->withTitle($member->fullName() . ' (#' . $member->id . ') - ' . $section);
    }

    public function postUserInfo($id, $section)
    {
        $user = \App\Models\User::where('id', $id)->first();

        if (is_null($user))
            return redirect()->route('admin.users')->with('Wrong user ID');

        switch ($section) {
            case 'details-edit':
                $rules = [
                    'personal' => [
                        'first_name' => 'alpha|min:3|max:50',
                        'last_name' => 'alpha|min:3|max:50',
                        'birth_date' => 'required|date_format:Y-m-d',
                        'country' => 'exists:countries,id',
                        'zip' => 'required|max:25',
                        'city' => 'max:100',
                        'state' => 'required|max:100',
                        'address' => 'required|max:200',
                        'phone' => 'required|between:6,20'
                    ],
                    'account' => [
                        //'username' => 'required|min:3|max:100|unique:users,username'
                    ],
                    'groups' => [
                        'groups' => 'required|array'
                    ],
                    'password' => [
                        'password' => 'between:6,32',
                        'password_confirmation' => 'same:password',
                        'secondary_password' => 'between:6,32',
                        'secondary_password_confirmation' => 'required_with:secondary_password|same:secondary_password'
                    ],
                    'status' => [
                        'frozen' => '',
                        'suspended' => '',
                        'suspension-end-date' => ''
                    ]
                ];

                $submit = \Illuminate\Support\Facades\Input::get('save');
                $section = (isset($rules[$submit])) ? $rules[$submit] : null;
                $data = \Illuminate\Support\Facades\Input::all();


                //clock()->info($data);

                if (is_null($section))
                    return Redirecr::back()->with('error', 'Wrong section')->with('tab', $submit);

                if ($submit == 'account') {
                    $section['username'] = 'required|alpha_num|min:3|max:100|unique:users,username,' . $user->username . ',username';
                    $section['email'] = 'required|email|min:3|max:200|unique:users,email,' . $user->email . ',email';
                }

                $validator = Validator::make($data, $section);

                if ($validator->fails())
                    return Redirect::back()->withErrors($validator)->with('tab', $submit)->withInput();

                foreach ($section as $r) {
                    $data[$r] = Input::get($r, NULL);
                }

                //clock()->info($data);

                if ($data['save'] == 'account') {
                    if (!isset($data['activated'])) {
                        $data['activated_at'] = 0;
                        $data['activated'] = 0;
                    }
                }

                //clock()->info($data);

                $method = 'save' . ucfirst($submit);
                $result = $this->$method($user, $data);

                clock()->info($method);
                clock()->info($result);

                if ($result)
                    return Redirect::back()->with('success', 'Your account data has been successfully saved!')->with('tab', $submit);
                else
                    return Redirect::back()->with('error', 'There was an error while trying to update the user.')->with('tab', $submit);
                break;

            case 'processors':
                $data = Input::all();
                $rules = [];
                $processors = \App\Models\Processors::where('id', '!=', 1)->where('id', '!=', 4)->orderBy('id', 'asc')->get();

                foreach ($processors as $p) {
                    $rules['processor_' . $p->id] = 'min:1|max:100';
                }

                $validator = Validator::make($data, $rules);

                if ($validator->fails())
                    return Redirect::back()->withInput()->withErrors($validator);

                foreach ($processors as $p) {
                    if (isset($data['processor_' . $p->id]) && trim($data['processor_' . $p->id]) !== '') {

                        $input = $data['processor_' . $p->id];
                        $userProcessor = \App\Models\UsersProcessors::where('processor_id', $p->id)->where('user_id', $user->id)->first();

                        if ($userProcessor->data !== $input) {
                            $userProcessor->data = $input;
                            $userProcessor->status = 0;
                            $userProcessor->save();
                        }

                    }
                }

                return Redirect::back()->with('success', 'You have successfully updated the gateway accounts.');
                break;
        }
    }

    public function getUserDelete($id)
    {
        if (is_null($user = \App\Models\User::where('id', $id)->first()))
            return redirect()->back()->with('error', 'Wrong user ID');

        DB::beginTransaction();
        //$user->groups()->delete();
        $groups = DB::table('users_groups')->where('user_id', $id)->delete();
        $user->profile->delete();

        $processors = $user->processors();
        if ($processors->count() > 0)
            $processors->delete();

        $balances = $user->balances();
        if ($balances->count() > 0)
            $balances->delete();

        $messages = $user->messages();
        if ($messages->count() > 0)
            $messages->delete();

        $transactions = $user->transactions();
        if ($transactions->count() > 0)
            $transactions->delete();

        $support = $user->support();
        if ($support->count() > 0)
            $support->delete();

        $SupportReplies = $user->SupportReplies();
        if ($SupportReplies->count() > 0)
            $SupportReplies->delete();

        $notifications = \App\Models\Notifications::where('user_id', $id)->delete();
        $bank_details = \App\Models\UsersBankDetails::where('user_id', $id)->delete();

        $sessions = DB::table('sessions')->where('user_id', $id)->delete();
        $throttle = DB::table('throttle')->where('user_id', $id)->delete();

        //$tree = \App\Models\Sponsor::deleteFromTree( $user );

        $user->delete();

        DB::commit();

        return response('User successfully deleted. Please close this tab!');
    }

    private function savePersonal($user, $data)
    {
        $user->first_name = $data['first_name'];
        $user->last_name = $data['last_name'];
        $user->profile->birth_date = $data['birth_date'];
        //$user->profile->gender      = $data['gender'];
        $user->profile->country_id = $data['country'];
        //$user->profile->zip         = $data['zip'];
        $user->profile->city = $data['city'];
        //$user->profile->state       = $data['state'];
        //$user->profile->address     = $data['address'];
        //$user->profile->phone       = $data['phone'];

        if ($user->save() && $user->profile->save())
            return true;

        return false;
    }

    private function saveAccount($user, $data)
    {
        //clock()->info($data);
        //clock()->info(isset($data['activated_at']) ? true : false);
        $user->username = $data['username'];
        $user->email = $data['email'];
        if (isset($data['activated'])) {
            $user->activated = $data['activated'];
        }

        if (isset($data['activated_at'])) {
            if ($data['activated_at'] == 0) {
                $user->activated_at = null;
            }
        }

        if ($user->activated_at == null && $data['activated'] == 1) {
            $user->activated = $data['activated'];
            $user->activated_at = Carbon::now();
        }

        if ($user->save()) return true;

        return false;
    }

    private function saveGroups($user, $data)
    {
        $allGroups = \App\Models\Groups::all();
        foreach ($allGroups as $group) {
            if (isset($data['groups'][$group->id]))
                $user->addGroup($group);
            else
                $user->removeGroup($group);
        }

        return true;
    }

    private function savePassword($user, $data)
    {
        $user->password = $data['password'];

        if (isset($data['secondary_password']))
            $user->secondary_password = $data['secondary_password'];

        if ($user->save()) return true;

        return false;
    }

    private function saveStatus($user, $data)
    {
        $user->frozen = (isset($data['frozen']) ? $data['frozen'] : 0);
        $user->suspended = (isset($data['suspended']) ? $data['suspended'] : 0);

        if (isset($data['suspended'])) {
            if ($data['suspended']) {
                if ($data['suspension-end-date'] != null) {
                    $user->suspended_at = Carbon::now();
                    $user->suspension_end = Carbon::parse($data['suspension-end-date']);
                }
            } else {
                $user->suspended = false;
                $user->suspended_at = null;
                $user->suspension_end = null;
            }
        } else {
            $user->suspended = false;
            $user->suspended_at = null;
            $user->suspension_end = null;
        }

        if ($user->save()) return true;

        return false;
    }

    # POPUPS #

    public function getBankProfilesView($userid, $id = null)
    {
        if (is_null($userProcessor = \App\Models\UsersProcessors::where('id', $id)->where('processor_id', 1)->where('user_id', $userid)->first()))
            return Redirect::route('admin.users.popups.bankProfiles', $userid)->with('error', 'Wrong bank profile ID');

        $bankDetails = $userProcessor->bankDetails;

        return view('admin.users.popups.bankProfiles.view', ['bankDetails' => $bankDetails])->withTitle($bankDetails->profile_name);
    }

    public function postBackProfileView($userid, $id = null)
    {
        if (is_null($userProcessor = \App\Models\UsersProcessors::where('id', $id)->where('processor_id', 1)->where('user_id', $userid)->first()))
            return Redirect::wallet('admin.users.popup', [$userid, 'processors'])->with('error', 'Wrong bank profile ID.');

        $bankDetails = $userProcessor->bankDetails;

        $rules = [
            'Profile_Name' => 'required|min:1|max:150',
            'Name_on_Account' => 'required|min:1|max:150',
            'Bank_Name' => 'required|min:1|max:150',
            'Bank_Address' => 'max:150',
            'Bank_Phone_Number' => 'max:150',
            'Institution_Number' => 'max:150',
            'Branch_Number' => 'max:150',
            'Account_Number' => 'required|max:150',
            'Routing_Number' => 'max:150',
            'IBAN' => 'max:150',
            'Swift_code' => 'required|max:150',
            'Intermediary_Bank_Info' => 'max:300'
        ];

        $data = [];

        foreach ($rules as $key => $value)
            $data[$key] = Input::get($key, '');

        $validator = Validator::make($data, $rules);

        if ($validator->fails())
            return Redirect::back()->withInput()->withErrors($validator);

        foreach ($data as $key => $d) {
            if (isset($rules[$key])) {
                $val = strtolower($key);
                $bankDetails->$val = trim($d);
            }
        }

        $checkOthers = \App\Models\UsersBankDetails::where('Account_Number', $bankDetails->account_number)->count();

        if ($checkOthers > 0)
            return Redirect::back()->with('error', 'This account number already exists in the system.')->withInput();

        if ($userProcessor->save() && $bankDetails->save())
            return Redirect::route('admin.users.popup', [$userid, 'processors'])->with('success', 'Bank Profile successfully updated.');

        return Redirect::back()->with('error', 'An error has occured. Please try again!');
    }

    public function changeUserStatus($id, $status)
    {

        if (is_null($user = \App\Models\User::where('id', $id)->first()))
            return Redirect::back()->with('error', 'User not found');

        switch ($status) {
            case 'activate':
                $user->activation_code = null;
                $user->activated = true;
                $user->activated_at = \App\Models\Date::now();
                $user->save();
                break;

            case 'unactivate':
                $user->activated = false;
                $user->activated_at = null;
                $user->getActivationCode();
                break;

            default:

                return Redirect::back()->with('error', 'Wrong action!');

                break;
        }

        return Redirect::back()->with('success', 'User status successfully changed!');
    }


    /* Categories */

    public function getKycs()
    {
        return view('admin.users.kycs.index')->withTitle('KYCs Manager');
    }
    
    public function getUserKycView($id = null){
        if( is_null( $user= \App\Models\User::where('id', $id)->first() ) ){
            return Redirect::route('admin.kycs')->with('error', 'Wrong User ID');
        }
        $kycs = \App\Models\UsersKycs::join('users', 'users_kycs.user_id', '=', 'users.id')
        ->where('users.id',$id)
        ->select('users_kycs.id', 'users.username as user', 'users.id as userid', 'users_kycs.type', 'users_kycs.document_location', 'users_kycs.status', 'users_kycs.created_at')->get();
        foreach($kycs as $k){
            $k->document_location = \App\Models\UsersKycs::where('id', $k->id)->first()->getImagePathAttribute();
        }
        $kycText='No Kyc';
        if($user->kyc_pending){
            $kycText='Kyc pending';
        }
        if($user->kyc_approved){
            $kycText='Kyc approved';
        }
        return view('admin.users.userKyc', ['user' => $user,'images'=>$kycs,'kycText'=>$kycText])->withTitle('KYCs Manager');
    }
    public function getKycsView($id = null)
    {
        if( is_null( $kyc = \App\Models\UsersKycs::where('id', $id)->first() ) ){
            return Redirect::route('admin.kycs')->with('error', 'Wrong KYC ID');
        }

        return view('admin.users.kycs.view', ['kyc' => $kyc])->withTitle('KYCs Manager');

    }
    
    public function postKycsView(Request $request, $id = null)
    {
        if( is_null( $kyc = \App\Models\UsersKycs::where('id', $id)->first() ) )
            return Redirect::route('admin.kycs')->with('error', 'Wrong KYC ID');

        $data = Input::all();
        $data['status'] = Input::get('status', 0);

        $rules = [
            'status' => 'required'
        ];

        $validator = Validator::make( $data, $rules );

        if ( $validator->fails() )
            return Redirect::back()->withInput()->withErrors( $validator );

        /*if(!File::exists(storage_path('app/public/auction/categories/thumbnails/'))) {
            File::makeDirectory(storage_path('app/public/auction/categories/thumbnails/'), 0775, true);
        }

        $thumbnailFilename = $category->thumbnail;

        if(
        $request->hasFile('thumbnail')
        ) {
            $file = $request->file('thumbnail');

            $filename = str_random(40) . '.' . $file->getClientOriginalExtension();
            $thumbnailPath = storage_path('app/public/auction/categories/');

            Image::make($file->getRealPath())->resize(400, null, function($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })->save($thumbnailPath . $filename);

            //$path = $f->storeAs('auction/categories', $filename, 'public');

            $thumbnailFilename = $filename;
        }*/

        //$category->title = $data['title'];
        /*$category->fill(request()->get('translate'));
        $category->title_url = str_slug(request()->get('translate')['en']['title']);
        $category->thumbnail = $thumbnailFilename;*/
        $kyc->status = $data['status'];
        $kyc->type = $data['type'];

        if( $kyc->save() )
            return Redirect::route('admin.kycs')->with('success', 'KYC successfully saved.');

        return Redirect::back()->with('error', 'Could not save the kyc.');
    }
}


