<?php 
namespace App\Http\Controllers\Admin;

use Input, Validator, Redirect, Config, Storage, File, Response, Illuminate\Http\Request, App, Image, DB;

class VideosController extends AdminController {

	public function __construct()
	{
		parent::__construct();
	}

	public function getIndex()
	{
		$videos = \App\Models\Videos::orderBy('created_at', 'desc')->paginate(30);
		return view('admin.videos.index', ['videos' => $videos])->withTitle('Videos');
	}

  public function getNew()
	{
		$sections = ['details', 'icons', 'files', 'subs'];
		$step = 0;
		$datas = [];

		/* Check Request */

		if( !is_null( $request = \App\Models\VideosTemp::where('user_id', $this->user->id)->first() ) ) {
			$step = $request->step + 1;
			$datas = json_decode($request->datas, true);
		}

		$subVideos = \App\Models\Videos::orderBy('id', 'asc')->get();
		$section = $sections[ $step ];
		return view('admin.videos.new-' . $section, ['request' => $request, 'subVideos' => $subVideos, 'datas' => $datas])->withTitle('Videos - New');
	}

	public function postNew()
	{
		$sections = ['details', 'icons', 'files', 'subs'];
		$rules = [
			'details' => [
				'videoType' 	 => 'required|in:0,1,2,3',
				'title_hu' 		 => 'required|min:3',
				'title_en' 		 => 'required|min:3',
				'title_de' 		 => 'required|min:3',
				'description_hu' => 'required|min:10',
				'description_en' => 'required|min:10',
				'description_de' => 'required|min:10',
				'fps' 			 => 'required',
				'length' 		 => 'required|numeric',
				'width' 		 => 'required|numeric',
				'height' 		 => 'required|numeric',
				'period' 		 => 'required_without:free|numeric',
				'price' 		 => 'required_without:free|numeric'
			],
			'icons' => [
				'icon_title_en' => 'required|array',
				'icon_title_hu' => 'required|array',
				'icon_title_de' => 'required|array',

				'icon_content_en' => 'required|array',
				'icon_content_hu' => 'required|array',
				'icon_content_de' => 'required|array',

				'icon_status' => 'array',
				'icon_coordinates' => 'required|array',
			],
			'files' => [],
			'subs' => []
		];

		$inputs = Input::all();
		$section = $inputs['section'];
		$sectionMethod = 'postNewSection_' . $section;
		$lastSection = false;
		$request = \App\Models\VideosTemp::where('user_id', $this->user->id)->first();

		//dd( $inputs );

		if( array_search( $section, $sections ) == (count( $sections ) - 1 ))
			$lastSection = true;

		$validator = Validator::make( $inputs, $rules[ $section ]);

		if( $validator->fails() )
			return Redirect::back()->withErrors( $validator )->withInput();

		if( !$lastSection && is_null( $request ) ) {
			$request = new \App\Models\VideosTemp;
			$request->user_id = $this->user->id;
			$request->datas = json_encode( [] );
			$request->step = 0;
			$request->save();
		}

		if( !method_exists( $this, $sectionMethod ) )
			return Redirect::back()->with('error', 'No section found.')->withInput();

		$_nd =  $this->$sectionMethod( $request, $inputs );

		if( $_nd === false ) return redirect()->back()->with('error', 'Something went wrong!');

		$newDatas[ $section ] = $_nd;

		$request->datas = array_merge( json_decode( $request->datas, true ), $newDatas );

		$result = true;

		if( (array_search( $section, $sections ) == 2) && ($request->datas['details']['videoType'] == 0 || $request->datas['details']['videoType'] == 3) )
			$lastSection = true;
		else
			$request->step = array_search( $section, $sections );

		if( $lastSection ) {

			$result = DB::transaction(function() use ($request ) {

				try {
					//$request->datas = json_decode( $request->datas, true );

					$video = new \App\Models\Videos;
					$video->id_long = $request->datas['details']['id_long'];
					$video->title_hu = $request->datas['details']['title_hu'];
					$video->title_en = $request->datas['details']['title_en'];
					$video->title_de = $request->datas['details']['title_de'];
					$video->description_hu = $request->datas['details']['description_hu'];
					$video->description_en = $request->datas['details']['description_en'];
					$video->description_de = $request->datas['details']['description_de'];
					$video->fps = $request->datas['details']['fps'];
					$video->width = $request->datas['details']['width'];
					$video->height = $request->datas['details']['height'];
					$video->videoFile = $request->datas['files']['video_filename'];
					$video->videoFile_original = $request->datas['files']['video_original_filename'];
					$video->thumbnailFile = $request->datas['files']['thumbnail_filename'];
					$video->thumbnailFile_original = $request->datas['files']['thumbnail_original_filename'];
					$video->length = $request->datas['details']['length'];
					$video->period = $request->datas['details']['period'];
					$video->price = $request->datas['details']['price'];
					$video->free = $request->datas['details']['free'];
					$video->type = $request->datas['details']['videoType'];
					$video->status = 0;
					$video->save();

					switch( $request->datas['details']['videoType'] ) {
						case 1:

							foreach( $request->datas['subs']['areas'] as $k => $area ) {

								if( !isset( $request->datas['subs']['floormap_name_hu'][ $k ] ) ) continue;

								$floormap = new \App\Models\VideosFloormaps;
								$floormap->video_id = $video->id;
								$floormap->name_hu = $request->datas['subs']['floormap_name_hu'][ $k ];
								$floormap->name_en = $request->datas['subs']['floormap_name_en'][ $k ];
								$floormap->name_de = $request->datas['subs']['floormap_name_de'][ $k ];
								$floormap->filename = $request->datas['subs']['floormap_filename'][ $k ];
								$floormap->floor = $k;
								$floormap->save();

								$rooms = [];

								foreach( $area as $key => $datas ) {
									foreach( $datas as $id => $val ) {

										if( !isset( $area[ $id ]['new'] ) ) continue;

										if( !isset( $rooms[ $id ] ) ) $rooms[ $id ] = [];

										if( !isset( $rooms[ $id ][ $key ] ) ) $rooms[ $id ][ $key ] = '';

										switch ( $key ) {
											case 'y':
												$rooms[ $id ]['top'] = $val;
											break;
											case 'x':
												$rooms[ $id ]['left'] = $val;
											break;
											case 'height':
												$rooms[ $id ]['height'] = $val;
											break;
											case 'width':
												$rooms[ $id ]['width'] = $val;
											break;
											case 'videoID':
												$rooms[ $id ]['id'] = $val;
											break;
											case 'order':
												$rooms[ $id ]['order'] = $val;
											break;
										}
									}
								}

								foreach( $rooms as $id => $room ) {
									$floormapData = new \App\Models\VideosFloormapsDatas;
									$floormapData->floormap_id = $floormap->id;
									$floormapData->video_id = $room['id'];
									$floormapData->top = $room['top'];
									$floormapData->left = $room['left'];
									$floormapData->right = $room['left'] + $room['width'];
									$floormapData->bottom = $room['top'] + $room['height'];
									$floormapData->order = $room['order'];
									$floormapData->save();
								}

							}

						break;

						case 2:

							foreach( $request->datas['subs']['previews'] as $k => $p ) {
								$preview = new \App\Models\VideosPreviews;
								$preview->parent_video = $video->id;
								$preview->video_id = $p;
								$preview->order = $k;
								$preview->save();
							}

						break;
					}


					foreach( $request->datas['icons'] as $icon ) {
						$videoIcons = new \App\Models\VideosIcon;
						$videoIcons->video_id = $video->id;
						$videoIcons->title_en = $icon['title_en'];
						$videoIcons->title_hu = $icon['title_hu'];
						$videoIcons->title_de = $icon['title_de'];
						//$videoIcons->coordinates = $icon['coordinates'];
						$videoIcons->content_hu = $icon['content_hu'];
						$videoIcons->content_en = $icon['content_en'];
						$videoIcons->content_de = $icon['content_de'];
						$videoIcons->status = ( isset( $icon['status'] ) ) ? $icon['status'] : 0;
						$videoIcons->save();
					}

					if( isset( $request->data['details']['status'] ) && $request->data['details']['status'] !== 1 )
						$video->status = $request->data['details']['status'];

			        if( !$video->save() ) {
			        	DB::rollback();
		            	return redirect()->back()->with('error', 'Hiba történt a videó mentése közben!');
			        }

			    $request->delete();

			    return true;

			  } catch( \Exception $e) {

			  	dd( $e );
		    	return false;

		    }

		  });

			if( $result )
				return Redirect::back()->with('success', 'Video has been successfully saved.');

		} else {

			if( $result ) {
				$request->datas = json_encode( $request->datas );

				if( $request->save() )
					return Redirect::back();
			}
		}

		return Redirect::back()->with('error', 'There was an error. Please try again to save.')->withInput();
	}

	private function postNewSection_details( $request, $inputs )
	{
		$newDatas = [];

		$newDatas['videoType'] = $inputs['videoType'];
		$newDatas['id_long'] = str_random(10);
		$newDatas['title_hu'] = $inputs['title_hu'];
		$newDatas['title_en'] = $inputs['title_en'];
		$newDatas['title_de'] = $inputs['title_de'];
		$newDatas['description_hu'] = $inputs['description_hu'];
		$newDatas['description_en'] = $inputs['description_en'];
		$newDatas['description_de'] = $inputs['description_de'];
		$newDatas['fps'] = Input::get('fps', Config::get('videos.default_fps'));
		$newDatas['width'] = $inputs['width'];
		$newDatas['height'] = $inputs['height'];
		//$newDatas['videoFile'] = $inputs['video_filename'];
		//$newDatas['videoFile_original'] = $inputs['video_original_filename'];
		//$newDatas['thumbnailFile'] = $inputs['thumbnail_filename'];
		//$newDatas['thumbnailFile_original'] = $inputs['thumbnail_original_filename'];
		$newDatas['length'] = $inputs['length'];
		$newDatas['period'] = $inputs['period'];
		$newDatas['price'] = $inputs['price'];
		$newDatas['free'] = Input::get('free', 0);
		$newDatas['type'] = $inputs['videoType'];
		//$newDatas['status'] = 0;

		return $newDatas;
	}

	private function postNewSection_icons( $request, $inputs ) {
		$icons = [];
		//$requestData = json_decode( $request->datas, true );

		foreach( $inputs as $k => $i ) {
			$split = explode('_', $k);
			if( isset($split[1]) ) {
				if( $split[0] == 'icon' ) {
					foreach( $i as $key => $val ) {

						if( !isset( $icons[$key] ) ) $icons[$key] = [];

						switch ( $split[ 1 ] ) {
							case 'coordinates':
								try {

									//$icons[ $key ]['coordinates'] = $this->explodeCoordinates( $val, $requestData['details']['ftp'] );

								} catch ( \ErrorException $e ) {

									return false;

								}
							break;
							default:
								if( isset( $split[2] ) )
									$icons[$key][$split[1] . '_' . $split[2]] = $val;
								else
									$icons[$key][$split[1]] = $val;
							break;
						}
					}
				}
			}
		}

		return $icons;
	}

	private function postNewSection_subs( $request, $inputs ) {
		$requestData = json_decode( $request->datas, true );
		$newDatas = [];

		switch( $requestData['details']['videoType'] ) {
			case 1:

				//$newDatas['areas'] = $inputs['areas'];

				$newDatas['floormaps'] = Input::except('_token');

			break;

			case 2:

				$newDatas['previews'] = $inputs['previews'];

			break;

		}

		return $newDatas;
	}

	private function postNewSection_files( $request, $inputs ) {
		/*ini_set('post_max_size', '999999M');
		ini_set('upload_max_filesize', '999999M');

		$files = [];

		$file = Input::file('video_file');

		$extension = $file->getClientOriginalExtension();
    $filename = sha1(time() . time()) . ".{$extension}";

    $upload_success = $file->move(storage_path() . '/videos/', $filename);


    $files['video']['original_filename'] = $file->getClientOriginalName();
    $files['video']['new_filename'] = $filename;


		foreach( Config::get('app.languages') as $lang ) {
			if( File::has( 'audio_' . $lang ) ) {
				$file = Input::file( 'audio_' . $lang );

				$extension = $file->getClientOriginalExtension();
			    $filename = sha1( time() . time() ) . ".{$extension}";

			    $upload_success = $file->move( storage_path() . '/audios/', $filename );

			    $files['audio'][$lang]['original_filename'] = $file->getClientOriginalName();
			    $files['audio'][$lang]['new_filename'] = $filename;
			}
		}*/

		$files = [];

		$files["video_filename"] = "a6c514443f8846af9063ae0a8857f607e0a7d307.mp4";
	  $files["video_original_filename"] = "hal.mp4";
	  $files["thumbnail_filename"] = "cd29caed7c4e3a44d920763ffb12c7481ca22356.jpg";
	  $files["thumbnail_original_filename"] = "1.jpg";

	  foreach( Config::get('app.languages') as $lang ) {
		  $files["audio_filename_" . $lang] = "7bb547dacf1a8e21c95b84d1f78fb03ce89c4127.mp3";
		  $files["audio_original_filename_" . $lang] = "Maid with the Flaxen Hair.mp3";
		}

		return $files;
	}


	public function getEmbed()
	{
		$videos = \App\Models\Videos::where('status', 1)->orderBy('id', 'asc')->get();
		return view('admin.videos.embed', ['videos' => $videos])->withTitle('Videos - Embed');
	}

	public function getView( $id = null ){
		if( is_null( $video = \App\Models\Videos::where('id', $id)->first()))
			return Redirect::route('admin.videos.index')->with('error', 'Wrong video ID');

		$files = [];

		//if( Storage::exists( $video->videoFile ) ) 
			//$files['video'] => null;
			//return Redirect::route('admin.videos.index')->with('error', 'Video file not found.');
		//else

		return view('admin.videos.view', ['video' => $video])->withTitle('Videos - ' . $video->title);
	}

  public function getEdit( $id = null, $section = '' )
  {
    if( is_null( $video = \App\Models\Videos::where('id', $id)->first()))
      return Redirect::route('admin.videos.index')->with('error', 'Wrong video ID');

    $sections = ['details', 'icons', 'files', 'subs'];

    if( !in_array($section, $sections) )
      return Redirect::route('admin.videos.index')->with('error', 'Wrong section!');

    $subVideos = \App\Models\Videos::orderBy('id', 'asc')->get();

    return view('admin.videos.edit-' . $section, ['video' => $video, 'section' => $section, 'subVideos' => $subVideos])->withTitle('Edit - ' . $video->title);
  }

  public function postEdit( $videoId = null, $section = '' ) {
    if( is_null( $video = \App\Models\Videos::where('id', $videoId)->first()))
      return Redirect::route('admin.videos.index')->with('error', 'Wrong video ID');

    $sections = ['details', 'icons', 'files', 'subs'];

    if( !in_array($section, $sections) )
      return Redirect::route('admin.videos.index')->with('error', 'Wrong section!');

    	$rules = [
			'details' => [
				//'videoType' 	 => 'required|in:0,1,2,3',
				'title_hu' 		 => 'required|min:3',
				'title_en' 		 => 'required|min:3',
				'title_de' 		 => 'required|min:3',
				'description_hu' => 'required|min:10',
				'description_en' => 'required|min:10',
				'description_de' => 'required|min:10',
				'fps' 			 => 'required',
				'length' 		 => 'required|numeric',
				'width' 		 => 'required|numeric',
				'height' 		 => 'required|numeric',
				'period' 		 => 'required_without:free|numeric',
				'price' 		 => 'required_without:free|numeric'
			],
			'icons' => [
				'icon_title_en' => 'required|array',
				'icon_title_hu' => 'required|array',
				'icon_title_de' => 'required|array',

				'icon_content_en' => 'required|array',
				'icon_content_hu' => 'required|array',
				'icon_content_de' => 'required|array',

				'icon_status' => 'array',
				'icon_coordinates' => 'required|array',
			],
			'files' => [],
			'subs' => []
		];

		$inputs = Input::all();
      	$validator = Validator::make( $inputs, $rules[ $section ]);

     	if($validator->fails())
        	return Redirect::back()->withErrors($validator)->withInput();

      	return DB::transaction(function () use ($video, $inputs, $section) {

	      	$sectionMethod = 'postEditSection_' . $section;


					$result = $this->$sectionMethod( $inputs, $video );

	        if( $result )
	          return redirect()->route('admin.videos')->with('success', 'A videó sikeresen mentve lett.');
	        else
	          return redirect()->back()->with('error', 'Hiba történt a mentés közben. Próbáld meg ismét!');
	    });
    }


  	public function postEditSection_details( $inputs, $video ) {
  		//$newDatas = [];

		//$newDatas['videoType'] = $inputs['videoType'];
		//$newDatas['id_long'] = str_random(10);
		$video->title_hu = $inputs['title_hu'];
		$video->title_en = $inputs['title_en'];
		$video->title_de = $inputs['title_de'];
		$video->description_hu = $inputs['description_hu'];
		$video->description_en = $inputs['description_en'];
		$video->description_de = $inputs['description_de'];
		$video->fps = Input::get('fps', Config::get('videos.default_fps'));
		$video->width = $inputs['width'];
		$video->height = $inputs['height'];
		//$newDatas['videoFile'] = $inputs['video_filename'];
		//$newDatas['videoFile_original'] = $inputs['video_original_filename'];
		//$newDatas['thumbnailFile'] = $inputs['thumbnail_filename'];
		//$newDatas['thumbnailFile_original'] = $inputs['thumbnail_original_filename'];
		$video->length = $inputs['length'];
		$video->period = $inputs['period'];
		$video->price = $inputs['price'];
		$video->free = Input::get('free', 0);
		//$newDatas['type'] = $inputs['videoType'];

		if( isset($inputs['status'] ) )
			$video->status = $inputs['status'];

		return $video->save();
  }

  public function postEditSection_icons( $inputs, $video ) {
  		$icons = [];
		//$requestData = json_decode( $request->datas, true );

		foreach( $inputs as $k => $i ) {
			$split = explode('_', $k);
			if( isset($split[1]) ) {
				if( $split[0] == 'icon' ) {
					foreach( $i as $key => $val ) {

						if( !isset( $icons[$key] ) ) $icons[$key] = [];

						switch ( $split[ 1 ] ) {
							case 'coordinates':
								try {

									//$icons[ $key ]['coordinates'] = $this->explodeCoordinates( $val, $video->ftp );

								} catch ( \ErrorException $e ) {

									return false;

								}
							break;
							default:
								if( isset( $split[2] ) )
									$icons[$key][$split[1] . '_' . $split[2]] = $val;
								else
									$icons[$key][$split[1]] = $val;
							break;
						}
					}
				}
			}
		}

		

			if( count( $icons ) > 0 ) {
				$videoIcons = \App\Models\VideosIcon::where('video_id', $video->id)->delete();

				foreach( $icons as $icon ) {
					$videoIcon = new \App\Models\VideosIcon;
					$videoIcon->video_id = $video->id;
					$videoIcon->title_en = $icon['title_en'];
					$videoIcon->title_hu = $icon['title_hu'];
					$videoIcon->title_de = $icon['title_de'];
					//$videoIcons->coordinates = $icon['coordinates'];
					$videoIcon->content_hu = $icon['content_hu'];
					$videoIcon->content_en = $icon['content_en'];
					$videoIcon->content_de = $icon['content_de'];
					$videoIcon->status = ( isset( $icon['status'] ) ) ? $icon['status'] : 0;
					$videoIcon->save();
				}

				return true;
			}

			return false;

  	}

  	public function postEditSection_subs( $inputs, $video ) {
  		switch ( $video->type ) {
  			case 1:
  				$flmps = \App\Models\VideosFloormaps::where('video_id', $video->id)->delete();
  				$flmpDatas = \App\Models\VideosFloormapsDatas::where('video_id', $video->id)->delete();

		  		foreach( $inputs['areas'] as $k => $area ) {

					if( !isset( $inputs['floormap_name_hu'][ $k ] ) ) continue;

					$floormap = new \App\Models\VideosFloormaps;
					$floormap->video_id = $video->id;
					$floormap->name_hu = $inputs['floormap_name_hu'][ $k ];
					$floormap->name_en = $inputs['floormap_name_en'][ $k ];
					$floormap->name_de = $inputs['floormap_name_de'][ $k ];
					$floormap->filename = $inputs['floormap_filename'][ $k ];
					$floormap->floor = $k;
					$floormap->save();

					$rooms = [];

					foreach( $area as $key => $datas ) {
						foreach( $datas as $id => $val ) {

							if( !isset( $area[ $id ]['new'] ) ) continue;

							if( !isset( $rooms[ $id ] ) ) $rooms[ $id ] = [];

							if( !isset( $rooms[ $id ][ $key ] ) ) $rooms[ $id ][ $key ] = '';

							switch ( $key ) {
								case 'y':
									$rooms[ $id ]['top'] = $val;
								break;
								case 'x':
									$rooms[ $id ]['left'] = $val;
								break;
								case 'height':
									$rooms[ $id ]['height'] = $val;
								break;
								case 'width':
									$rooms[ $id ]['width'] = $val;
								break;
								case 'videoID':
									$rooms[ $id ]['id'] = $val;
								break;
								case 'order':
									$rooms[ $id ]['order'] = $val;
								break;
							}
						}
					}

					foreach( $rooms as $id => $room ) {
						$floormapData = new \App\Models\VideosFloormapsDatas;
						$floormapData->floormap_id = $floormap->id;
						$floormapData->video_id = $room['id'];
						$floormapData->top = $room['top'];
						$floormapData->left = $room['left'];
						$floormapData->right = $room['left'] + $room['width'];
						$floormapData->bottom = $room['top'] + $room['height'];
						$floormapData->order = $room['order'];
						$floormapData->save();
					}

				}
			break;

			case 2: 

				$prvs = \App\Models\VideosPreviews::where('parent_video', $video->id)->delete();

				foreach( $inputs['previews'] as $k => $p ) {
					$preview = new \App\Models\VideosPreviews;
					$preview->parent_video = $video->id;
					$preview->video_id = $p;
					$preview->order = $k;
					$preview->save();
				}

			break;
		}

		return true;
  	}

  	private function postEditSection_files( $inputs, $video ) {

		$video->videoFile = $inputs['video_filename'];
		$video->videoFile_original = $inputs['video_original_filename'];
		$video->thumbnailFile = $inputs['thumbnail_filename'];
		$video->thumbnailFile_original = $inputs['thumbnail_original_filename'];

	  	$vidauds = \App\Models\VideosAudio::where('video_id', $video->id)->delete();

	  	foreach( Config::get('app.languages') as $lang ) {
			if( isset( $inputs['audio_filename_' . $lang] ) && $inputs['audio_filename_' . $lang] !== '' ) {
				$videoAudio = new \App\Models\VideosAudio;
				$videoAudio->video_id = $video->id;
				$videoAudio->filename = $inputs['audio_filename_' . $lang];
				$videoAudio->original_filename = $inputs['audio_original_filename_' . $lang];
				$videoAudio->language = $lang;
				$videoAudio->save();
			}
		}

		$video->save();
		
		return true;
	}


  	public function getVideo( $video = null )
	{
	    $fileContents = Storage::disk('local')->get('uploads/videos/{$name}');
	    $response = Response::make($fileContents, 200);
	    $response->header('Content-Type', "video/mp4");
	    return $response;
	}

	public function postThumbnailUpload(Request $request)
	{
		return $this->fileUpload($request, 'thumbnail', public_path() . '/assets/videos/thumbnails/', asset('assets/videos/thumbnails/'));
	}

	public function postVideoUpload(Request $request)
	{
		return $this->fileUpload($request, 'video', storage_path() . '/videos/');
	}

	public function postAudioUpload(Request $request)
	{
		return $this->fileUpload($request, 'audio', storage_path() . '/audios/');
	}

	public function postFloormapUpload(Request $request)
	{
		$data = $this->fileUpload($request, 'floormap', public_path() . '/assets/videos/floormaps/', asset('assets/videos/floormaps/'), false);

		if( $data['path'] ) {
			$img = Image::make( public_path() . '/assets/videos/floormaps/' . $data['filename'] )->resize(800, 300, function( $const ) {
				$const->aspectRatio();
				$const->upsize();
			})->save();

			return response()->json($data);
		}
	}

	/*public function postNew()
	{
		return DB::transaction(function ($t) {

			$rules = [
				'title_hu' => 'required|min:3',
				'title_en' => 'required|min:3',
				'title_de' => 'required|min:3',
				'description_hu' => 'required|min:10',
				'description_en' => 'required|min:10',
				'description_de' => 'required|min:10',
				'fps' => 'required',
				'length' => 'required|numeric',
				'width' => 'required|numeric',
				'height' => 'required|numeric',
				'period' => 'required_without:free|numeric',
				'price' => 'required_without:free|numeric',
				'videoType' => 'required|in:0,1,2,3'
			];

			$inputs = Input::all();

			$validator = Validator::make($inputs, $rules);

			if($validator->fails())
				return Redirect::back()->withErrors($validator)->withInput();

			$fps = Input::get('fps', Config::get('videos.default_fps'));

			$icons = [];

			foreach( $inputs as $k => $i ) {
				$split = explode('_', $k);
				if( isset($split[1]) ) {
					if( $split[0] == 'icons' ) {
						foreach( $i as $key => $val ) {

							if( !isset( $icons[$key] ) ) $icons[$key] = [];

							switch ($split[1]) {
								case 'coordinates':
									$icons[$key]['coordinates'] = $this->explodeCoordinates( $val, $fps );
								break;
								default:
									if( isset( $split[2] ) )
										$icons[$key][$split[1] . '_' . $split[2]] = $val;
									else
										$icons[$key][$split[1]] = $val;
								break;
							}
						}
					}
				}
			}

			$videoid = str_random(10);

			$video = new \App\Models\Videos;
			$video->id_long = $videoid;
			$video->title_hu = $inputs['title_hu'];
			$video->title_en = $inputs['title_en'];
			$video->title_de = $inputs['title_de'];
			$video->description_hu = $inputs['description_hu'];
			$video->description_en = $inputs['description_en'];
			$video->description_de = $inputs['description_de'];
			$video->fps = $fps;
			$video->width = $inputs['width'];
			$video->height = $inputs['height'];
			$video->videoFile = $inputs['video_filename'];
			$video->videoFile_original = $inputs['video_original_filename'];
			$video->thumbnailFile = $inputs['thumbnail_filename'];
			$video->thumbnailFile_original = $inputs['thumbnail_original_filename'];
			$video->length = $inputs['length'];
			$video->period = $inputs['period'];
			$video->price = $inputs['price'];
			$video->free = Input::get('free', 0);
			$video->type = $inputs['videoType'];
			$video->status = 0;
			$video->save();

			switch( $inputs['videoType'] ) {
				case 1:

					foreach( $inputs['areas'] as $k => $area ) {

						$floormap = new \App\Models\VideosFloormaps;
						$floormap->video_id = $video->id;
						$floormap->name_hu = $inputs['floormap_name_hu'][ $k ];
						$floormap->name_en = $inputs['floormap_name_en'][ $k ];
						$floormap->name_de = $inputs['floormap_name_de'][ $k ];
						$floormap->filename = $inputs['floormap_filename'][ $k ];
						$floormap->floor = $k;
						$floormap->save();

						$rooms = [];

						foreach( $area as $key => $datas ) {
							foreach( $datas as $id => $val ) {

								if( !isset( $rooms[ $id ] ) ) $rooms[ $id ] = [];

								if( !isset( $rooms[ $id ][ $key ] ) ) $rooms[ $id ][ $key ] = '';

								switch ( $key ) {
									case 'y':
										$rooms[ $id ]['top'] = $val;
									break;
									case 'x':
										$rooms[ $id ]['left'] = $val;
									break;
									case 'height':
										$rooms[ $id ]['height'] = $val;
									break;
									case 'width':
										$rooms[ $id ]['width'] = $val;
									break;
									case 'videoID':
										$rooms[ $id ]['id'] = $val;
									break;
									case 'order':
										$rooms[ $id ]['order'] = $val;
									break;
								}
							}
						}

						foreach( $rooms as $id => $room ) {
							$floormapData = new \App\Models\VideosFloormapsDatas;
							$floormapData->floormap_id = $floormap->id;
							$floormapData->video_id = $room['id'];
							$floormapData->top = $room['top'];
							$floormapData->left = $room['left'];
							$floormapData->right = $room['left'] + $room['width'];
							$floormapData->bottom = $room['top'] + $room['height'];
							$floormapData->order = $room['order'];
							$floormapData->save();
						}

					}

				break;

				case 2: 

					foreach( $inputs['previews'] as $k => $p ) {
						$preview = new \App\Models\VideosPreviews;
						$preview->parent_video = $video->id;
						$preview->video_id = $p;
						$preview->order = $k;
						$preview->save();
					}

				break;

			}

			foreach( Config::get('app.languages') as $lang ) {
				if( isset( $inputs['audio_filename_' . $lang] ) && $inputs['audio_filename_' . $lang] !== '' ) {
					$videoAudio = new \App\Models\VideosAudio;
					$videoAudio->video_id = $video->id;
					$videoAudio->filename = $inputs['audio_filename_' . $lang];
					$videoAudio->original_filename = $inputs['audio_original_filename_' . $lang];
					$videoAudio->language = $lang;
					$videoAudio->save();
				}
			}

			foreach( $icons as $icon ) {
				$videoIcons = new \App\Models\VideosIcon;
				$videoIcons->video_id = $video->id;
				$videoIcons->title_en = $icon['title_en'];
				$videoIcons->title_hu = $icon['title_hu'];
				$videoIcons->title_de = $icon['title_de'];
				$videoIcons->coordinates = $icon['coordinates'];
				$videoIcons->content_hu = $icon['content_hu'];
				$videoIcons->content_en = $icon['content_en'];
				$videoIcons->content_de = $icon['content_de'];
				$videoIcons->status = $icon['status'];
				$videoIcons->save();
			}

			if( isset($inputs['status']) && $inputs['status'] !== 1 ) {
				$video->status = $inputs['status'];
			}

			if( $video->save() )
				return redirect()->back()->with('success', 'A videó sikeresen létre lett hozva.');
			else
				return redirect()->back()->with('error', 'Hiba történt a videó létrehozása közben!');

		});
	}*/

	private function fileUpload( $request, $type, $path, $publicPath = false, $resp = true ) {
		ini_set('post_max_size', '999999M');
		ini_set('upload_max_filesize', '999999M');
		$file = $request->file;

		//$rules = array(
		//	'file' => 'required',
		//);

		//$validation = Validator::make(Input::all(), $rules);

		//if ($validation->fails())
		//{
		//	return Response::make($validation->errors()->first(), 400);
		//}

		$extension = $file->getClientOriginalExtension();
	    $filename = sha1(time() . time()) . ".{$extension}";

	    $upload_success = $file->move($path, $filename);

	    if ($upload_success) {
	    	$response = ['filename' => $filename, 'original_filename' => $file->getClientOriginalName(), 'path' => ($publicPath) ? $publicPath : '', 'type' => $type];

	    	if( $lang = $request->input('lang', false) )
	    		$response['lang'] = $lang;

	      return ( $resp ) ? Response::json($response, 200) : $response;

	    } else {
	      return ( $resp ) ? Response::json(['error' => true], 400) : $response;
	    }
	}

	private function explodeCoordinates( $coordinates, $fps ) {
		$fpsSec = 1 / $fps;

		$lines = explode(PHP_EOL, $coordinates);

		if( count($lines) > 1 ) {

			foreach( $lines as $k => $l ) {
				$data = preg_split('/\s+/', trim($l));
				$lines[ $k ] = $data;
			}

			$lines = array_map(function( $i ) use ($fpsSec) {
				return [
					'ts' => floatval(number_format($i[0] * $fpsSec, 5, '.', '')),
					'x' => floatval($i[1]),
					'y' => floatval($i[2])
				];
			}, $lines);

		} else {
			$lines = [];
		}

		return json_encode($lines);
	}
}