<?php

namespace App\Http\Controllers\Api;

use App\Facades\MailManager;
use App\Models\UserAddresses;
use App\Models\UsersDeposits;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class DepositController extends Controller
{
    private $apiKey;

    public function __construct()
    {
        parent::__construct();

        $this->apiKey = config('deposits.api_key');
    }

    public function postDeposit(Request $request){

        if ($request->get('api_key') != $this->apiKey) {
            return response()->json(['status' => 'error', 'message' => 'API_KEY_ERROR', 'description' => 'Api key dont match']);
        }

        $validator = Validator::make($request->all(), [
            'uuid' => [
                'required',
                'regex:/^[\da-f]{8}-[\da-f]{4}-[\da-f]{4}-[\da-f]{4}-[\da-f]{12}$/iD',
            ],
        ]);

        if ($validator->fails()) {
            return response()->json(['status' => 'error', 'message' => 'ADDRESS_FORMAT_ERROR', 'description' => 'Address format error.']);
        }

        $validator = Validator::make($request->all(), [
            'uuid' => 'required',
            'amount' => 'required|numeric',
            'usdt_value' => 'required|numeric',
            'coin_rate' => 'required|numeric',
            'coin_type' => 'required|in:firefly-cpc,cpc',
            'email' => 'required|email',
        ]);

        if ($validator->fails()) {
            return response()->json(['status' => 'error', 'message' => 'PAYLOAD_ERROR', implode(' ', $validator->messages()->all())]);
        }

        $address = UserAddresses::where('address', $request->get('uuid'))->first();

        if($address){

            $payment = $address->payments()->create([
                "address_id" => $address->id,
                "email" => $request->get('email'),
                "usdt_value" => $request->get('usdt_value'),
                "coin_rate" => $request->get('coin_rate'),
                "currency_name" => $request->get('coin_type'),
                "currency_amount" => $request->get('amount'),
                "transaction_source_system" => "thepeak",
                "comment" => $request->get("comment")
            ]);

            $address->received += $request->get('amount');
            $address->save();

            if (!$payment) {
                return response()->json(['status' => 'error', 'message' => 'FAILED_TO_SAVE_DEPOSIT_ERROR', 'description' => 'Failed to save deposit.']);
            }

            if ($address->amount <= $address->received) {
                $processor = \App\Models\Processors::where('id', 5)->first();

                $package = $address->package()->first();
                $user = $address->user()->first();

                $transactionDeposit = new \App\Models\Transactions;
                $transactionDeposit->processor_id = 1;
                $transactionDeposit->user_id = $address->user_id;
                $transactionDeposit->type = 6;
                $transactionDeposit->fee = $processor->fee_fund;
                $transactionDeposit->amount = $package->price;
                $transactionDeposit->translateOrNew('en')->note = 'Deposit via CPC - ' . $address->amount;
                $transactionDeposit->translateOrNew('cn')->note = '存款美元 - ' . $address->amount;
                $transactionDeposit->status = 1;
                $transactionDeposit->save();

                $address->delete();

                $new_package = new \App\Models\UsersPackages;
                $new_package->user_id = $address->user_id;
                $new_package->package_id = $package->id;
                $new_package->status = 1;
                $new_package->save();

                $new_transaction = new \App\Models\Transactions;
                $new_transaction->processor_id = 1;
                $new_transaction->user_id = $address->user_id;
                $new_transaction->type = 13;
                $new_transaction->amount = $package->price;
                $new_transaction->translateOrNew('en')->note = 'Package ID: #' . $new_package->id . ', Type: ' . $package->name;
                $new_transaction->translateOrNew('cn')->note = '套餐 ID: #' . $new_package->id . ', 类型: ' . $package->name;
                $new_transaction->status = 1;

                if ($new_transaction->save()) {

                    //try{
                        MailManager::send('packages.new-package', 'New Package Purchase', [
                            'id' => $new_package->id,
                            'package_name' => $package->name,
                            'amount' => $package->price
                        ], $user);

                        dispatch(new \App\Jobs\SetPackage($new_package->id));
                    /*}catch(\Exception $e){

                    }*/

                }
            }

            return response()->json(['status' => 'success', 'message' => 'Deposit saved!']);

        }

        $user = User::where('uuid', $request->get('uuid'))->first();

        if(!$user){
            return response()->json(['status' => 'error', 'message' => 'User not found']);
        }

        $deposit = $user->deposits()->create([
           "uuid" => $request->get('uuid'),
           "email" => $request->get('email'),
           "usdt_value" => $request->get('usdt_value'),
           "coin_rate" => $request->get('coin_rate'),
            "currency_name" => $request->get('coin_type'),
            "currency_amount" => $request->get('amount'),
            "transaction_source_system" => "thepeak",
            "comment" => $request->get("comment")
        ]);

        if(!$deposit){
            return response()->json(['status' => 'error', 'message' => 'Failed to save deposit']);
        }

        $balance = $user->balances()->first();

        if(!$balance){
            $balance = new \App\Models\UsersBalances;
            $balance->users_id = $user->id;
            $balance->save();
        }

        $balance->wallet = $balance->wallet + $request->get('usdt_value');


        if($balance->save()){
            return response()->json(['status' => 'success', 'message' => 'User balance updated: '. $balance->wallet]);
        };

        return response()->json(['status' => 'error', 'message' => 'Failed to save balance']);
    }

}
