<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Hash, Captcha;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/desk';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
        view()->share('bodyClass', 'login');
    }

    public function login(Request $request)
    {
        $rules = [
            'email' => 'required|max:255|email',
            'password' => 'required',
        ];


        if (config('app.need_captcha')) {
            $rules['captcha'] = 'required|captcha';
        }

        $message = [
            'captcha.captcha' => trans('auth.wrong_captcha'),
            'captcha.required' => trans('auth.no_captcha')];

        $this->validate($request, $rules, $message);

        if (Auth::attempt(['email' => $request->get('email'), 'password' => $request->get('password')])) {
            return redirect()->intended('/desk');
        } else {
            return redirect()->back()->withErrors(trans('auth.failed'));
        }

    }

    function authenticated(Request $request, $user)
    {
        $user->recordLogin($request);
    }

}
