<?php

namespace App\Http\Controllers\Auth;

use App\User;
use App\Http\Controllers\Controller;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

use Auth, View, Input, Event, Redirect, Session, Config, Validator, MessageBag, DB, MailManager, Hash, Cookie, Cart;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = '/desk';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except(['getRegistration', 'postRegister']);
    }

    public function getRegistration( Request $request, $ref = null )
    {
        $uid = $request->input('uid', null);
        $leg = $request->input('l', null);
        $p = $request->input('p', null);
        $useUpline = false;
        $sponsor = null;

        if( !is_null( $ref ) ) {
            $sponsor = \App\Models\TreeGenealogyNested::select(DB::raw('u.id, u.username'))
                ->leftJoin('users AS u', 'u.id', '=', 'tree_genealogy_nested.id')
                ->orWhere('u.id', $ref)
                ->orWhere('username', $ref)
                ->orWhere('email', $ref)
                ->orWhere('sponsor_code', $ref)
                ->first();
        }


        view()->share('bodyClass', 'register');
        $countries = \App\Models\Country::orderBy('name', 'asc')->get();
        $packages = \App\Models\Packages::orderBy('id', 'asc')->get();
        $cart = Cart::instance('packages');
        $selectedPackage = null;

        foreach( $packages as $package ) {
            if( $package->id == $p ) $selectedPackage = $package->id;
        }

        return view('auth.register', [
            'countries' => $countries,
            'ref' => $ref,
            'packages' => $packages,
            'cart' => $cart,
            'useUpline' => $useUpline,
            'selectedPackage' => $selectedPackage,
            'sponsor' => $sponsor
        ])->withTitle('Registration');
    }

    public function postRegister( $ref = null )
    {
        $rules = [
                'first_name' => 'min:2|max:50',
                'last_name' => 'min:2|max:50',

                'email' => 'required|email|unique:users,email',

                //'gender'                => 'required|in:1,2',

                //'birth_date'            => 'required|date_format:m/d/Y',
                'country' => 'exists:countries,id',
                //'zip'                   => 'required|max:25',
                'city' => 'max:100',
                //'state'                 => 'required|max:100',
                //'address'               => 'required|max:200',
                //'phone'                 => 'required|between:5,20',

                'password' => 'required|between:6,32',
                'password_confirm' => 'required|same:password',
                //'secondary_password' => 'required|between:4,32|different:password',
                //'secondary_password_confirm' => 'required|same:secondary_password',

                'username' => 'required|alpha_num|min:3|max:20|unique:users,username',

                //'sponsor_code'          => 'exists:tree_genealogy_nested,id',
                //'upline_user'              => '',
                //'leg'                   => 'required|in:1,2',

                'agreement' => 'accepted',

                //'captcha' => 'required|captcha'

                //'package'               => 'required|exists:packages,id'
            ];

            if(config('app.need_captcha')){
                $rules['captcha'] = 'required|captcha' ;
            }

            $message= [
                'password.between'=> trans('auth.password_length'),
                'captcha.captcha'=> trans('auth.wrong_captcha'),
                'captcha.required'=> trans('auth.no_captcha')
            ];
           
            $validator = Validator::make(Input::all(), $rules, $message);

            if ($validator->fails())
                return Redirect::back()->withErrors($validator)->withInput();

            $result = DB::transaction(function () use ($rules, $ref) {
                try {
                    $data = Input::all();

                    foreach ($rules as $key => $r)
                        $data[$key] = Input::get($key, NULL);

                    $sponsor = null;
                    $error = false;

                    if (isset($data['package'])) {
                        $cart = Cart::instance('packages');

                        $package = \App\Models\Packages::where('id', $data['package'])->first();

                        if ($cart->content()->count() > 0)
                            $cart->destroy();

                        $cart->add([
                            'id' => $package->id,
                            'name' => $package->name,
                            'qty' => 1,
                            'price' => $package->price
                        ]);
                    }

                    // Check sponsor code

                    if ($ref && is_null( $sponsor = \App\Models\TreeGenealogyNested::select(DB::raw('u.id, u.username'))
                        ->leftJoin('users AS u', 'u.id', '=', 'tree_genealogy_nested.id')
                        ->orWhere('u.id', $ref)
                        ->orWhere('username', $ref)
                        ->orWhere('email', $ref)
                        ->orWhere('sponsor_code', $ref)
                        ->first()

                    )){
                        $error = ['key' => 'Sponsor', 'value' => 'Sponsor not found.'];
                    }

                    if ($error) {
                        $messageBag = new \Illuminate\Support\MessageBag();
                        $messageBag->add($error['key'], $error['value']);
                        return redirect()->back()->with('error', $messageBag)->withInput();
                    }

                    $user = new \App\Models\User;
                    $user->id = $this->generateUniqueId(0, 8);
                    $user->first_name = $data['first_name'];
                    $user->last_name = $data['last_name'];
                    $user->username = $data['username'];
                    $user->email = $data['email'];
                    $user->sponsor_code = random_num(8);
                    $user->password = Hash::make($data['password']);
                    $user->secondary_password = null;
                    $user->uuid = Str::uuid();

                    if ($user->save()) {

                        $user->assignRole('user');

                        $user_profile = new \App\Models\UserProfile;
                        $user_profile->id = $user->id;
                        //$user_profile->birth_date   = \App\Models\Date::format($data['birth_date'], 'Y-m-d');
                        $user_profile->avatar = NULL;
                        $user_profile->country_id = $data['country'];
                        //$user_profile->zip          = $data['zip'];
                        $user_profile->city = $data['city'];
                        //$user_profile->state        = $data['state'];
                        //$user_profile->address      = $data['address'];
                        //$user_profile->phone        = $data['phone'];
                        $user_profile->currency = 'USD';//( $ref == 'wexcoin' ) ? 'WEX' : 'EUR';
                        $user_profile->save();

                        $users_platform_settings = new \App\Models\UsersPlatformSettings;
                        $users_platform_settings->id = $user->id;
                        $users_platform_settings->save();

                        $processors = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();

                        foreach ($processors as $p) {
                            $new_processor = new \App\Models\UsersProcessors;
                            $new_processor->user_id = $user->id;
                            $new_processor->processor_id = $p->id;
                            $new_processor->status = -1;
                            $new_processor->save();
                        }

                        $new_balance = new \App\Models\UsersBalances;
                        $new_balance->users_id = $user->id;
                        $new_balance->save();

                        //\App\Models\Sponsor::addToTree($sponsor, $upline_user, $user, $leg);

                        if($sponsor){
                            $tempSponsor = new \App\Models\TempSponsor;
                            $tempSponsor->user_id = $user->id;
                            $tempSponsor->sponsor_user_id = $sponsor->id;

                            $tempSponsor->save();
                        }

                        //$user->addGroup(\App\Models\Groups::findByName('Users'));

                        //MailManager::send( 'auth.new-account', 'New Account', [], $user );

                        MailManager::send( 'auth.account-confirmation', 'Account Verification', [
                            'activationCode' => $user->getActivationCode()
                        ], $user);

                        Auth::login($user, true);


                        //Session::put('selectedPackage', $package->id);


                        return redirect($this->redirectPath())->with('success', 'Registration completed! You can log in.');
                    } else {
                        return redirect()->back()->withInput()->with('error', 'Error occured! Please try again. 0');
                    }

                } catch (\Illuminate\Database\QueryException $e) {
                    dump($e);
                    return redirect()->back()->withInput()->with('error', 'Error occured! Please try again. 1');
                } catch (\Exception $e) {
                    dump($e);
                    return redirect()->back()->withInput()->with('error', 'Error occured! Please try again. 2');
                }

                return redirect()->back()->withInput()->with('error', 'Error occured! Please try again. 3');
            });

            return $result;
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required|string|min:6|confirmed',
            'captcha' => 'required|captcha'
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return User
     */
    protected function create(array $data)
    {
        return User::create([
            'name' => $data['name'],
            'email' => $data['email'],
            'password' => bcrypt($data['password']),
        ]);
    }

    public function generateUniqueId($iteration, $length){
        $iteration++;

        if($iteration > 100){
            if($length < 10) {
                $length++;
                $iteration = 0;
            }
        }

        $temp_id = random_num($length);

        $exists = \App\Models\User::where('id', $temp_id)->get();
        if(count($exists) == 0){
            return $temp_id;
        }
        else{
            $this->generateUniqueId($iteration, $length);
        }
    }
}
