<?php
namespace App\Http\Controllers\Clientzone\Ajax;

use App\Facades\NodeGWManager;
use App\Models\AuctionItems;
use App\Models\User;
use Illuminate\Support\Facades\Log;
use Input, DB, Config, Validator, Carbon\Carbon, NodeGW, Cart;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\Event;
use App\Events\ManualBidEvent;

class AuctionController extends AjaxController {

	public function __construct()
	{
		parent::__construct();
	}

	public function postPlaceBid( Request $request )
	{
		$id = $request->input('id', null);

		if( is_null( $item = \App\Models\AuctionItems::where('id', $request->input('id', null))->where('status', 0)->first() ) )
			return $this->responseNew(trans('messages.wrong_item_id'));

		if( !$this->user->hasBids() )
			return $this->responseNew(trans('messages.no_bids'));


        $last_bid = \App\Models\AuctionBids::where('item_id', $item->id)->orderby('id', 'desc')->first();


        //$bot_guard = User::where('id', User::getBotIds()[$this->__generateRandom(0, count(User::getBotIds()))])->first();

        $last_bidder_user = null;

        if($last_bid){
            $last_bidder_user = \App\Models\User::where('id', $last_bid->user_id)->first();
        }

        if(($item->isbusy == 1 || !is_null($item->busied_at)) && Carbon::now()->diffInMinutes(Carbon::parse($item->busied_at)) > \Illuminate\Support\Facades\Config::get('auction.item_is_released_after_minutes', 5)){
            $item->isbusy = 0;
            $item->busied_at = null;
            $item_old_id = $item->id;
            $item->save();
            $item = \App\Models\AuctionItems::where('id', $item_old_id)->where('status', 0)->first();
        }


        if($item->isbusy == 0) {

            $newBid = new \App\Models\AuctionBids;
            $newBid->item_id = $item->id;
            $newBid->user_id = $this->user->id;
            $newBid->type = 1;
            $newBid->price = $item->per_bid_price;

            ////////
            if(!in_array($this->user->id, User::getBotIds())){
                $item->user_bidders_price = $item->user_bidders_price + $item->per_bid_price;
            }
            ////////

            $deduces = $this->user->deduceBid($item->per_bid_price);
            $newBid->package_bid_price = $deduces[0]['price'];

            if ($newBid->save()) {

                $item->current_price = $item->current_price + $newBid->price;
                $item->ends_at = $item->ends_at->addSeconds(10);
                $item->last_bid_id = $newBid->id;

                if ($item->save()) {

                    $newBid->username = $this->user->username;
                    $newBid->created_at_format = $newBid->created_at->format('Y.m.d H:i:s');
                    //$item->ends_at_ts = $item->ends_at->timestamp;

                    $data = $item->toArray();
                    $data["ends_at_ts"] = $item->ends_at->timestamp;


                    //$last_bid_id = \App\Models\AuctionBids::where('item_id', $item->id)->where('user_id', $this->user->id)->orderby('id', 'desc')->first()->id;


                    //Log::error('ajax/Noti user id: '. $this->user->id);
                    //Log::error('ajax/Noti last bidder '. $last_bidder_user->id);

                    if($last_bidder_user && $last_bidder_user->id != $this->user->id) {

                        //Log::error('ajax/In noti process '. $last_bidder_user->id);
                        \App\Facades\NotificationManager::create([
                            'user_id' => $last_bidder_user->id,
                            'type' => 'auction-overbid',
                            //'content_id' => $last_bid_id
                            'content_id' => $item->id
                        ], $last_bidder_user);
                    }

                    NodeGWManager::send('api/socketio/auction/newBid', [
                        'item' => $data,
                        'newBids' => [$newBid->toArray()]
                    ]);

                    event(new ManualBidEvent($this->user->id, $item->id));


                    /*if($item->bot_need_win == 1 && (!in_array($last_bidder_user->id, User::getBotIds()))){
                        $this->autoBid($item, $bot_guard, $last_bidder_user);
                    }*/


                    return $this->responseNew(null, [
                        'bid' => $newBid->id,
                        'currentPrice' => $item->current_price,
                        'endsAt' => $item->ends_at->timestamp,
                        'lastBidder' => [
                            'id' => $this->user->id,
                            'username' => $this->user->username
                        ]
                    ]);
                }
            }
        }
        else
        {
            $waiting_stack = new \App\Models\WaitingUserBids;
            $waiting_stack->item_id = $item->id;
            $waiting_stack->user_id = $this->user->id;
            $waiting_stack->type = 1;

            $item->current_price = $item->current_price + $item->per_bid_price;

            if($waiting_stack->save())
            {

                /*if($item->bot_need_win == 1 && (!in_array($last_bidder_user->id, User::getBotIds()))){
                    $this->autoBid($item, $bot_guard, $last_bidder_user);
                }*/

                return $this->responseNew(null, [
                    'bid' => $waiting_stack->id . '(Queued)',
                    'currentPrice' => $item->current_price,
                    'endsAt' => $item->ends_at->timestamp,
                    'lastBidder' => [
                        'id' => $this->user->id,
                        'username' => $this->user->username
                    ]
                ]);
            }
        }

	}
/*
    public function __generateRandom( $min, $max ) {
        return (int) random_int( (int) $min, (int) $max );
    }

    public function autoBid($item, $bot, $last_bidder_user)
    {
        //$this->info(Carbon::now() . ' - Bidding: ' . $bot->id);
        if (!isset($bot))
            return false;
        if (!$bot->hasBids()) {
            $new_package = new \App\Models\AuctionUsersPackages;
            $new_package->user_id = $bot->id;
            $new_package->package_id = 2;
            $new_package->bids = 1000000;
            $new_package->save();
        }

        $rate = $this->__generateRandom(0, 3);

        if($rate == 0) {

            /*if(Carbon::parse($item->ends_at)->subSeconds(1) >  $last_bid->created_at){
                $new_bid_time = Carbon::parse($item->ends_at)->subSeconds(1);
            }
            else{
                $new_bid_time = $item->ends_at;
            }*/
/*
            $newBid = new \App\Models\AuctionBids;
            $newBid->item_id = $item->id;
            $newBid->user_id = $bot->id;
            $newBid->type = 1;
            $newBid->price = $item->per_bid_price;
            //$newBid->created_at = $new_bid_time;
            //$newBid->updated_at = $new_bid_time;


            $deduces = $bot->deduceBid($item->per_bid_price);
            $newBid->package_bid_price = $deduces[0]['price'];


            if ($newBid->save()) {

                $item->current_price = $item->current_price + $newBid->price;
                $item->ends_at = $item->ends_at->addSeconds(10);

                if ($item->save()) {
                    $newBid->username = $bot->username;
                    $newBid->created_at_format = $newBid->created_at->format('Y.m.d H:i:s');

                    $data = $item->toArray();
                    $data["ends_at_ts"] = $item->ends_at->timestamp;

                    sleep($this->__generateRandom(0, 3));

                    \App\Facades\NotificationManager::create([
                        'user_id' => $last_bidder_user->id,
                        'type' => 'auction-overbid',
                        //'content_id' => $last_bid_id
                        'content_id' => $item->id
                    ], $last_bidder_user);


                    NodeGWManager::send('api/socketio/auction/newBid', [
                        'item' => $data,
                        'newBids' => [$newBid->toArray()]
                    ]);

                    event(new ManualBidEvent($bot->id, $item->id));
                }
            }
        }
        else{
            $bidButler = new \App\Models\AuctionBidButlers;
            $bidButler->item_id = $item->id;
            $bidButler->user_id = $bot->id;
            $bidButler->counter = $this->__generateRandom(1, 5);
            if($bidButler->save()){
                if(is_null( $item->lastBidder ) || $item->lastBidder->user->id != $bot->id){
                    event(new ManualBidEvent($bot->id, $item->id));
                }

                return $this->responseNew(null, [
                    'result' => true,
                    'counter' => $bidButler->counter,
                    'multiplier' => $item->per_bid_price / config('auction.bid_price')
                ]);
            }
        }

    }*/

	public function postBuyItNow( Request $request ) {
	    //Bypassed because of security reasons #noge
	    return false;

		$id = $request->input('id', null);

		if( is_null( $item = \App\Models\AuctionItems::where('id', $request->input('id', null))->where('status', 0)->first() ) )
			return $this->responseNew(trans('messages.wrong_item_id'));

		$prevBids = \App\Models\AuctionBids::where('item_id', $item->id)->where('user_id', $this->user->id)->sum('package_bid_price');

		$buyPrice = $item->buy_price - $prevBids;

		$newBid = new \App\Models\AuctionBids;
		$newBid->item_id = $item->id;
		$newBid->user_id = $this->user->id;
		$newBid->type = 3;
		$newBid->price = $buyPrice;

		if( $newBid->save() ) {
			$item->current_price = $item->buy_price;
			$item->status = 2;
			$item->ends_at = Carbon::now();

            $item->last_bid_id = $newBid->id;

			if( $item->save() ) {
				$newBid->username = $this->user->username;
				$newBid->created_at_format = $newBid->created_at->format('Y.m.d H:i:s');
				//$item->ends_at_ts = $item->ends_at->timestamp;

                $data = $item->toArray();
                $data["ends_at_ts"] = $item->ends_at->timestamp;

				NodeGWManager::send('api/socketio/auction/newBid', [
                    'item' => $data,
                    'ended' => true,
                    'newBids' => [$newBid->toArray()]
                ]);

				return $this->responseNew(null, [
					'bid' => $newBid->id, 
					'currentPrice' => $item->current_price,
					'endsAt' => $item->ends_at->timestamp,
					'lastBidder' => [
						'id' => $this->user->id,
						'username' => $this->user->username
					]
				]);
			}
		}	
	}

	public function postGetBuyPrice( Request $request ) {
		$id = $request->input('id', null);

		if( is_null( $item = \App\Models\AuctionItems::where('id', $request->input('id', null))->where('status', 0)->first() ) )
			return $this->responseNew(trans('messages.wrong_item_id'));

		$prevBids = \App\Models\AuctionBids::where('item_id', $item->id)->where('user_id', $this->user->id)->sum('package_bid_price');

		return $this->responseNew(null, [
			'prevBids' => $prevBids,
			'buyItNowPrice' => $item->buy_price
		]);
	}

	public function postSetBidButler( Request $request )
	{
		$id = $request->input('id', null);

		if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->where('status', 0)->first() ) )
			return $this->responseNew(trans('messages.wrong_item_id'));

        if( !$this->user->hasBids() )
            return $this->responseNew(trans('messages.no_bids'));

		if( !is_null( $bidButler = \App\Models\AuctionBidButlers::where('item_id', $id)->where('user_id', $this->user->id)->first() ) ) {
			$bidButler->counter = $bidButler->counter + $request->input('counter', 1);
			$bidButler->save();
		} else {
			$bidButler = new \App\Models\AuctionBidButlers;
			$bidButler->item_id = $item->id;
			$bidButler->user_id = $this->user->id;
			$bidButler->counter = $request->input('counter', 1);
			$bidButler->save();
		}

        if(is_null( $item->lastBidder ) || $item->lastBidder->user->id != $this->user->id){
            event(new ManualBidEvent($this->user->id, $item->id));
        }

		return $this->responseNew(null, [
			'result' => true,
			'counter' => $bidButler->counter,
            'multiplier' => $item->per_bid_price / config('auction.bid_price')
		]);
	}

	public function postSaveWatchlist( Request $request ) {
		$id = $request->input('id', null);

		if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->first() ) )
			return $this->responseNew(trans('messages.wrong_item_id'));

		$watchlist = \App\Models\AuctionWatchlist::where('item_id', $item->id)->where('user_id', $this->user->id)->first();

		if( !is_null($watchlist = \App\Models\AuctionWatchlist::where('item_id', $item->id)->where('user_id', $this->user->id)->first() )) {
			$watchlist->delete();
			$watchlist = null;
		} else {
			$watchlist = new \App\Models\AuctionWatchlist;
			$watchlist->item_id = $item->id;
			$watchlist->user_id = $this->user->id;
			$watchlist->save();
		}

		return $this->responseNew(null, [
			'result' => (is_null($watchlist)) ? false : true
		]); 
	}

	public function postPackagesAddTo( Request $request ) {
		$id = $request->input('id', null);
        $quantity = $request->input('quantity', 1);

        if(is_null( $package = \App\Models\AuctionPackages::where('id', $id)->first() ) ) {
            return $this->responseNew(trans('messages.wrong_package_id'));
        }

        if( $quantity > 10 ) {
            $quantity = 10;
        } elseif( $quantity < 1 ) {
            $quantity = 1;
        }

        $cart = Cart::instance('auction.packages');


        /*$checkCarts = $cart->search(function ( $cartItem, $rowId ) use ($id) {
            return $cartItem->id === $id;
        });*/
        $cart->destroy();
        

        //if( $this->user->isNewUser() && isset( $checkCarts) && $checkCarts->count() ) {
        //    $item = $checkCarts->first();
        //    $cart->update($item->rowId, $quantity);
        //} else {
            $cart->add(
                ['id' => $package->id, 'name' => $package->name, 'qty' => $quantity, 'price' => $package->price, 'bids' => $package->bids]
            );
        //}

        return $this->response([ 'data' => [
            'result' => true,
            'packageName' => $package->name,
            'packageQuantity' => $quantity,
            'totalCost' => $cart->total()
        ]]);
	}


    public function postBidButlerTable()
    {
        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);

        $fields = [
            'id' => false,
            'startDate' => false,
            'endDate' => false,
        ];

        //DB::enableQueryLog();

        $auction_bid_butlers = \App\Models\AuctionBidButlers::select(
            DB::raw('auction_bid_butlers.id, user_id, auction_bid_butlers.counter, auction_bid_butlers.created_at, auction_items.title'))
            ->leftJoin('auction_items', 'auction_items.id', '=', 'auction_bid_butlers.item_id')->where('user_id', $this->user->id);


        if ($id = Input::get('id', $fields['id']))
            if ($id !== '')
                $auction_bid_butlers = $auction_bid_butlers->where('auction_bid_butlers.id', $id);


        $startDate = Input::get('startDate', $fields['startDate']);
        $endDate = Input::get('endDate', $fields['endDate']);

        if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {
            $startDate = \App\Models\Date::format($startDate, 'Y-m-d');
            $endDate = \App\Models\Date::format($endDate, 'Y-m-d');
            $auction_bid_butlers = $auction_bid_butlers->whereRaw("(DATE(auction_bid_butlers.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

        $start = ($start < 1) ? 0 : $start;
        $length = ($length < 1) ? 25 : $length;


        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ($_order['column']) {
                case 1:
                    $order_by = 'auction_bid_butlers.counter';
                    break;
                case 6:
                    $order_by = 'auction_bid_butlers.created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'auction_bid_butlers.id';
                    break;
            }

        }

        $auction_bid_butlers = $auction_bid_butlers->orderBy($order_by, $dir);

        $auction_bid_butlers_count = $auction_bid_butlers->count();
        $auction_bid_butlers_total = $auction_bid_butlers->skip( $start )->take( $length )->count();
        $auction_bid_butlers = $auction_bid_butlers->skip( $start )->take( $length )->get();


        $response = ['draw' => Input::get('draw', 1), 'data' => $auction_bid_butlers, 'recordsFiltered' => $auction_bid_butlers_count, 'recordsTotal' => $auction_bid_butlers_total];
        return response()->json($response);
    }

    public function postClearBidButler( Request $request ){
        $id = $request->input('id', null);

        if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->where('status', 0)->first() ) )
            return $this->responseNew('Wrong item ID');

        if( !is_null( $bidButler = \App\Models\AuctionBidButlers::where('item_id', $id)->where('user_id', $this->user->id)->first() ) ) {
            //$bidButler->counter = $bidButler->counter + $request->input('counter', 1);
            //$bidButler->save();
            $bidButler->forceDelete();
        }

        /*} else {
            $bidButler = new \App\Models\AuctionBidButlers;
            $bidButler->item_id = $item->id;
            $bidButler->user_id = $this->user->id;
            $bidButler->counter = $request->input('counter', 1);
            $bidButler->save();
        }*/

        /*if(is_null( $item->lastBidder ) || $item->lastBidder->user->id != $this->user->id){
            event(new ManualBidEvent($this->user->id, $item->id));
        }*/

        return $this->responseNew(null, [
            'result' => true,
            'counter' => 0,
            'multiplier' => $item->per_bid_price / config('auction.bid_price')
        ]);

    }
}