<?php 
namespace App\Http\Controllers\Clientzone\Ajax;

use \App\Models\Messages;
use DB, Config, Input;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;

class MailboxController extends AjaxController {

	public function __construct()
	{
		parent::__construct();
		$this->mailbox_list_length = Config::get('app.mailbox_list_length');
	}

	public function postMessages()
	{
		$paginator = [
			'all' => 0,
			'totalPages' => 1,
			'currentPage' => Input::get('page', 1),
			'perPage' => Config::get('paginator.ajaxMailbox', 25)
		];

		$c = Input::get('c', 'inbox');

		$length = ( $paginator['currentPage'] == 1 ) ? $paginator['perPage'] : ( $paginator['currentPage'] * $paginator['perPage'] );
		$start = ( $paginator['currentPage'] == 1 ) ? 0 : $length - $paginator['perPage'];

		$messages = Messages::select(DB::raw(
			'messages.id, messages.subject, messages.from, messages.content, messages.status, messages.starred, messages.created_at'
		))
		->where('user_id', $this->user->id)
		->orderBy('messages.id', 'desc');

		switch ($c) {
			case 'inbox':
				$messages = $messages->where('category', NULL);
				break;

			case 'important':
				$messages = $messages->where('starred', 1); // TODO
				break;
			
			case 'promotions':
				$messages = $messages->where('category', 1);
				break;

			case 'trash':
				$messages = $messages->where('category', 2);
				break;
		}

		$messages_total = $messages->get()->count();

		$paginator['all']	= $messages_total;
		if( ( $totalPages = ceil( $paginator['all'] / $paginator['perPage'] ) ) > 0 ) {
			$paginator['totalPages'] = ceil( $paginator['all'] / $paginator['perPage'] );
		}
		$messages = $messages->skip( $start )->take( $paginator['perPage'] )->get();

		foreach( $messages as $key => $m ) {
			$m->status = ( $m->status == 0 ) ? 'unread' : '';
			$m->date = $m->created_at->format('m/d/y h:i A');
			unset($m->created_at);
		}

		$this->countMessages();

		$this->results['data']['messages'] = $messages;
		$this->results['paginator'] = $paginator;

		return response()->json( $this->results );
	}

	public function postViewMessage()
	{
		if( is_null( $message = \App\Models\Messages::select(DB::raw('messages.id, messages.subject, messages.from, messages.content, messages.starred, messages.created_at'))->where('id', Input::get('id', 0))->where('user_id', $this->user->id)->first() ) ) {

			$this->results['error'] = trans('messages.wrong_message_id');

		} else {
			if( !$message->status ) {
				$message->status = 1;
				$message->save();
			}

			$this->countMessages();
			
			$message->date = $message->created_at->format('m/d/y h:i A');
			unset($message->created_at);

			$this->results['data']['message'] = $message;
		}

		//$this->results['paginator'] = $paginator;

		return response()->json( $this->results );
	}

	public function postSetImportant()
	{
	    $req_ids = Input::get('ids', 0);

        foreach( $req_ids as $key => $value ) {
            //clock()->info($key);
            //clock()->info($value);

            if( is_null( $message = \App\Models\Messages::where('id', $value)->where('user_id', $this->user->id)->first() ) ) {

                $this->results['error'] = trans('messages.wrong_message_id');

            } else {
                if( $message->starred ) {
                    $message->starred = 0;
                } else {
                    $message->starred = 1;
                }

                if($message->category == 2){
                    $message->category = null;
                }

                $message->save();

                $this->results['data']['status'] = $message->starred;

                $this->countMessages();

            }

            //$this->results['paginator'] = $paginator;

        }

        $this->countMessages();

        //$this->results['data']['status'] = 0;
		return response()->json( $this->results );
	}

    public function postTrashMessage()
    {
        $req_ids = Input::get('ids', 0);

        foreach( $req_ids as $key => $value ) {
            if (is_null($message = \App\Models\Messages::where('id', $value)->where('user_id', $this->user->id)->first())) {
                $this->results['error'] = trans('messages.wrong_message_id');
            } else {
                $message->category = 2;
                $message->starred = 0;
                $message->status = 1;
                $message->save();
                $this->results['data']['status'] = $message->starred;
                $this->countMessages();
            }

        }
        return response()->json( $this->results );
    }

    public function postDeleteMessage()
    {
        $req_ids = Input::get('ids', 0);

        foreach( $req_ids as $key => $value ) {
            if (is_null($message = \App\Models\Messages::where('id', $value)->where('user_id', $this->user->id)->first())) {
                $this->results['error'] = trans('messages.wrong_message_id');
            } else {
                $message->category = 3;
                $message->starred = 0;
                $message->status = 1;
                $message->deleted_at = Carbon::now();
                $message->save();
                $this->results['data']['status'] = $message->starred;
                $this->countMessages();
            }
        }
        return response()->json( $this->results );
    }

	protected function countMessages()
	{
		$counters = [
			'inbox' => 0,
			'trash' => 0,
			'important' => 0,
			'promotions' => 0
		];

		$messages = \App\Models\Messages::select(DB::raw('SUM(IF(status = 0, 1, 0)) as counter, SUM(IF(starred = 1, 1, 0)) as counter_starred, category'))->where('user_id', $this->user->id)->groupBy('category')->get();

		foreach( $messages as $m ) {
			$counters['important'] = $counters['important'] + $m->counter_starred;

			switch ( $m->category ) {
				case 1:
					$counters['promotions'] = $counters['promotions'] + $m->counter;
					break;
				case 2:
					$counters['trash'] = $counters['trash'] + $m->counter;
					break;
				default:
					$counters['inbox'] = $counters['inbox'] + $m->counter;
					break;
			}
		}

		$this->results['data']['counters'] = $counters;
	}
}