<?php 
namespace App\Http\Controllers\Clientzone\Ajax;

use App\Models\Packages;
use Illuminate\Support\Facades\Artisan;
use Input, DB, Config, Cart, Carbon, MailManager;

class PackagesController extends AjaxController {

	public function __construct()
	{
		parent::__construct();
	}

	public function postPackagesTable()
	{
        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);

        $packages = \App\Models\UsersPackages::select(DB::raw('users_packages.*, packages.price As amount'))->where('user_id', $this->user->id)->join('packages', 'packages.id', '=', 'users_packages.package_id');

        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ($_order['column']) {
                case 0:
                    $order_by = 'id';
                    break;
                case 1:
                    $order_by = 'package_id';
                    break;
                case 3:
                    $order_by = 'amount';
                    break;
                case 6:
                    $order_by = 'status';
                    break;
                case 7:
                    $order_by = 'created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'created_at';
                    break;
            }

        }

        $packages = $packages->orderBy($order_by, $dir);

        $packages_count = $packages->count();
        $packages_total = $packages->skip( $start )->take( $length )->count();
        $packages = $packages->skip( $start )->take( $length )->get();

        $statuses = [1 => trans('packages.status.active'), 0 => trans('packages.status.expired')];

        foreach( $packages as $t ) {
            $t->packageName = trans('packages.packageNames.' . $t->packages->id);
            ///$t->interest = $t->packages->interest;
            $t->statusName = $statuses[ $t->status ];
            $t->packages = null;
        }

        $response = [
            'draw' => Input::get('draw', 1), 
            'data' => $packages, 
            'recordsFiltered' => $packages_count, 
            'recordsTotal' => $packages_total
        ];

        return response()->json($response);
	}

    public function postPackagesAddTo() {
        $id = Input::get('id', null);
        $quantity = Input::get('quantity', 1);

        if(is_null( $package = \App\Models\Packages::where('id', $id)->first() ) ) {
            return $this->responseNew('Wrong package id');
        }

        /*if( $quantity > 10 ) {
            $quantity = 10;
        } elseif( $quantity < 1 ) {*/
            $quantity = 1;
        //}

        $cart = Cart::instance('packages');

        if( $this->user->isNewUser() ) {
            $quantity = 1;
            $cart->destroy();
        } else {
            /*$checkCarts = $cart->search(function ( $cartItem, $rowId ) use ($id) {
                return $cartItem->id === $id;
            });*/
            $cart->destroy();
        }

        //if( $this->user->isNewUser() && isset( $checkCarts) && $checkCarts->count() ) {
        //    $item = $checkCarts->first();
        //    $cart->update($item->rowId, $quantity);
        //} else {
            $cart->add(
                ['id' => $package->id, 'name' => $package->name, 'qty' => $quantity, 'price' => $package->price]
            );
        //}

        return $this->response([ 'data' => [
            'result' => true,
            'totalCost' => $cart->total()
        ]]);

    }

    public function postPackagesPeakStatus() {
        $address = Input::get('address', null);

        if( !is_null( $p = \App\Models\UserAddresses::where('user_id', $this->user->id)->withTrashed()->where('address', $address)->first() ) ) {
            if( $p->hasExpired() && is_null( $p->deleted_at ) ) {
                $p->delete();

                return response()->json([
                    'status' => 2
                ]);
            } else {
                $percent = $p->receivedPercent();

                if( $p->received < $p->amount ) {

                    if( $percent >= 100 ) {
                        $percent = 99;
                    }

                    return response()->json([
                        'status' => 1,
                        'percent' => $percent,
                        'received' => $p->received
                    ]);
                } else {
                    if( $p->user->newUser == 1 ) {
                        return response()->json([
                            'status' => 0
                        ]);
                    } else {
                        $p->delete();

                        return response()->json([
                            'status' => 1,
                            'percent' => $percent,
                            'received' => $p->received
                        ]);

                    }
                }
            }
        }

        return response()->json([
            'status' => 0
        ]);
    }
}