<?php
namespace App\Http\Controllers\Clientzone\Ajax;

use Input, DB, Config, Validator, Carbon\Carbon;
use Illuminate\Validation\Rule;

class TradingPlatformController extends AjaxController {

	public function __construct()
	{
		parent::__construct();
	}

	public function postPlatformData()
	{

		$bookBuy = \App\Models\TradingBook::select(DB::raw('ANY_VALUE(price) as price, SUM(amount) as amount'))->where('type', 1)->where('status', 0)->groupBy('price')->take(8)->get();
		$bookSell = \App\Models\TradingBook::select(DB::raw('ANY_VALUE(price) as price, SUM(amount) as amount'))->where('type', 2)->where('status', 0)->groupBy('price')->take(8)->get();

		$escrowedUnits = \App\Models\TradingBook::select(DB::raw('SUM(amount) as amount'))->where('type', 1)->where('user_id', $this->user->id)->where('status', 1)->whereRaw('escrowed_until > NOW()')->first();

		$sumOptions = \App\Models\TradingBook::select(DB::raw('MIN(price) as min, MAX(price) as max, SUM(amount) as volume'))->where('user_id', $this->user->id)->first();
		$countOptions = \App\Models\TradingBook::select(DB::raw('type, COUNT(*) as counter'))->where('user_id', $this->user->id)->groupBy('type')->get();

		$myBookBuy = \App\Models\TradingBook::select('id', 'amount', 'price')->where('user_id', $this->user->id)->where('type', 1)->where('status', 0)->get();
		$myBookSell = \App\Models\TradingBook::select('id', 'amount', 'price')->where('user_id', $this->user->id)->where('type', 2)->where('status', 0)->get();

		$escrowedUnits = ( is_null( $escrowedUnits ) ) ? 0.00 : $escrowedUnits->amount;

		$balance = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
	
	    return $this->response([
			'data' => [
				'balances' => [
					'resaleUnits' => $balance->resale_units,
					'onlineAccount' => $balance->online_account,
          'mandatoryAccount' => $balance->mandatory_account,
					'escrowedUnits' => $escrowedUnits,
					'sumOptions' => $sumOptions,
					'countOptions' => $countOptions,
					'totalUnits' => $balance->resale_units + $escrowedUnits 
				],
				'options' => [
					'buy' => $bookBuy, 
					'sell' => $bookSell
				],
				'myOptions' => [
					'buy' => $myBookBuy,
					'sell' => $myBookSell
				]
			]	
	    ]);
  	}

    public function postListOrders()
    {

      $orders = \App\Models\TradingBook::select('id', 'amount', 'price')->where('type', Input::get('type', 1))->where('status', 0)->get();

      return $this->response([
        'data' => $orders
      ]);

    }

  	public function postBuy() {
  		$rules = [
  			'amount' => 'required|numeric|min:1',
  			'price' => 'required'
  		];

  		$validator = Validator::make( Input::all(), $rules );

  		if( $validator->fails() )
  			return $this->response(['error' => 'Check your field inputs.']);

  		$price = ( Input::get('price') == 0 ) ? 0.072 : Input::get('price');

      $balance = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

      if( ($balance->mandatory_account + $balance->online_account) < (Input::get('amount') * $price) )
        return $this->response(['error' => 'You don\'t have enough balance.']);

  		$buy = new \App\Models\TradingBook;
  		$buy->user_id = $this->user->id;
  		$buy->type = 1;
  		$buy->amount = Input::get('amount');
  		$buy->price = $price;
  		$buy->status = 0;
  		$buy->save();

  		$amount = Input::get('amount');

  		$sellOptions = \App\Models\TradingBook::where('user_id', '!=', $this->user->id)->where('price', $price)->where('type', 2)->where('status', 0)->get();

  		if( !is_null($sellOptions) && $sellOptions->count() > 0 ) {

  			$sumAmount = $sellOptions->sum('amount');

  			foreach( $sellOptions as $o ) {
  				if( $o->amount <= $amount ) {
  					$amount = $amount - $o->amount;
  					$o->linked_id = $buy->id;
  					$o->status = 1;
  					$o->save();
  				} else {
  					$restAmount = $o->amount - $amount;
  					$amount = 0;
            $o->amount = $amount;
  					$o->linked_id = $buy->id;
  					$o->status = 1;
  					$o->save();

  					$newBuy = new \App\Models\TradingBook;
			  		$newBuy->user_id = $o->user_id;
			  		$newBuy->type = 2;
			  		$newBuy->amount = $restAmount;
			  		$newBuy->price = $price;
			  		$newBuy->status = 0;
			  		$newBuy->save();
  				}

  				$obalance = \App\Models\UsersBalances::where('users_id', $o->user_id)->first();
          $obalance->online_account = $obalance->online_account + ($o->amount * $price);
          $obalance->save();

  				if( $amount == 0 )
  					break;
  			}

  			if( $amount == 0 ) {
          $buy->escrowed_until = Carbon::now()->addDays(28);
  				$buy->status = 1;

  				//$balance->resale_units = $balance->resale_units + Input::get('amount');
  				$balance = $this->deduceEqually( $balance, Input::get('amount') * $price );
  			} else {
  				//$balance->resale_units = $balance->resale_units + $amount;
  				$balance = $this->deduceEqually( $balance, $amount * $price );

  				$restBuy = new \App\Models\TradingBook;
  				$restBuy->user_id = $this->user->id;
  				$restBuy->type = 1;
  				$restBuy->amount = $amount;
  				$restBuy->price = $price;
  				$restBuy->status = 0;
  				$restBuy->save();
  			}

  			$buy->save();
  			$balance->save();
  		}

  		return $this->response([
        'data' => true
		  ]);
  	}

    private function deduceEqually( $balances, $amount ) {
      $rest = $amount;

      if( $balances->mandatory_account > 0 ) {
        if( $balances->mandatory_account >= $amount ){
          $rest = 0;
          $balances->mandatory_account = $balances->mandatory_account - $amount;
        } else {
          $rest = $amount - $balances->mandatory_account;
          $balances->mandatory_account = 0;
        }
      }

      if( $rest > 0 ) {
        $balances->online_account = $balances->online_account - $rest;
      }

      return $balances;
    }

  	public function postSell() {
  		$rules = [
  			'amount' => 'required|numeric|min:1',
  			'price' => 'required'
  		];

  		$validator = Validator::make( Input::all(), $rules );

  		if( $validator->fails() )
  			return $this->response(['error' => 'Check your field inputs.']);

  		$price = ( Input::get('price') == 0 ) ? 0.072 : Input::get('price');

      $balance = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
      $amount = Input::get('amount');

      if( $balance->resale_units < $amount )
        return $this->response(['error' => 'You don\'t have enough units.']);

  		$sell = new \App\Models\TradingBook;
  		$sell->user_id = $this->user->id;
  		$sell->type = 2;
  		$sell->amount = Input::get('amount');
  		$sell->price = $price;
  		$sell->status = 0;
  		$sell->save();

  		$buyOptions = \App\Models\TradingBook::where('user_id', '!=', $this->user->id)->where('price', $price)->where('type', 1)->where('status', 0)->get();

  		if( !is_null($buyOptions) && $buyOptions->count() > 0 ) {
  			
  			$sumAmount = $buyOptions->sum('amount');

  			foreach( $buyOptions as $o ) {
  				$obalance = \App\Models\UsersBalances::where('users_id', $o->user_id)->first();

  				if( $o->amount <= $amount ) {
  					$amount = $amount - $o->amount;
  					$o->linked_id = $sell->id;
  					$o->status = 1;
  					$o->save();
  					$obalance->resale_units = $obalance->resale_units + $o->amount;

  				} else {
  					$restAmount = $o->amount - $amount;
  					$amount = 0;
  					$o->linked_id = $sell->id;
  					$o->status = 1;
  					$o->save();

  					$newSell = new \App\Models\TradingBook;
			  		$newSell->user_id = $o->user_id;
			  		$newSell->type = 1;
			  		$newSell->amount = $restAmount;
			  		$newSell->price = $price;
			  		$newSell->status = 0;
			  		$newSell->save();

			  		$obalance->resale_units = $obalance->resale_units + Input::get('amount');
  				}

				$obalance->save();

  				if( $amount == 0 )
  					break;
  			}

  			if( $amount == 0 ) {
  				$sell->status = 1;

  				$balance->resale_units = $balance->resale_units - Input::get('amount');
  				$balance->online_account = $balance->online_account + ( Input::get('amount') * $price );
  			} else {
  				$balance->resale_units = $balance->resale_units - $amount;
  				$balance->online_account = $balance->online_account + ( (Input::get('amount') - $amount) * $price );

  				$restSell = new \App\Models\TradingBook;
  				$restSell->user_id = $this->user->id;
  				$restSell->type = 2;
  				$restSell->amount = $amount;
  				$restSell->price = $price;
  				$restSell->status = 0;
  				$restSell->save();
  			}

  			$sell->save();
  			$balance->save();
  		} else {
  			$balance->resale_units = $balance->resale_units - $amount;
  			$balance->save();
  		}

  		return $this->response([
			'data' => true
		]);

  	}

  	public function postCancel() {
  		$user = $this->user;
  		$id = Input::get('id', null);

  		$rules = [
  			'id' => [
		        'required',
		        Rule::exists('trading_book')->where(function ($query) use ($user, $id) {
		            $query->where('id', $id)->where('user_id', $user->id)->where('status', 0);
		        })
		    ]
  		];

  		$validator = Validator::make( Input::all(), $rules );

  		if( $validator->fails() )
  			return $this->response(['error' => 'Wrong order id.']);

  		$order = \App\Models\TradingBook::where('id', $id)->where('user_id', $this->user->id)->where('status', 0)->first();

  		$order->delete();

  		return $this->response([
			'data' => true
		]);
  	}
}
