<?php
namespace App\Http\Controllers\Clientzone\Ajax;

use App\Models\Transactions;
use Input, DB, Config, Validator;

class TransactionsController extends AjaxController {

	public function __construct()
	{
		parent::__construct();
	}

	public function postTransactionsTable()
	{
        ini_set('memory_limit','256M');

        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);

        $fields = [
            'source' => "transactions",
            'id' => false,
            'type' => [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            'startDate' => false,
            'endDate' => false,
            'processor' => false,
        ];

        //DB::enableQueryLog();

        $transactions = Transactions::select(DB::raw('transactions.id, user_id, transactions.status, transactions.type, transactions.fee, transactions.amount, transactions_translation.note as note, transactions.created_at, transactions.processor_id, processors.name as processor'))
            ->leftJoin('transactions_translation', 'transactions.id', '=', 'transactions_translation.transactions_id')
            ->leftJoin('processors', 'processors.id', '=', 'transactions.processor_id')
            ->where('transactions_translation.locale', Config::get('app.locale'))
            ->where('user_id', $this->user->id);


        $start = ($start < 1) ? 0 : $start;
        $length = ($length < 1) ? 25 : $length;


        $transactions_count = $transactions->count();
        $transactions_total = $transactions->skip( $start )->take( $length )->count();
        $transactions = $transactions->skip( $start )->take( $length )->get();

        $typeNames = \App\Models\Transactions::$types;

        foreach( $transactions as $t ) {
            $t->typeName = $t->typeName;
            $t->typeIcon = $t->typeIcon;
			$t->statusName = ( ( $t->status == 0 ) ? 'Pending' : 'Completed' );
        }

        $response = ['draw' => Input::get('draw', 1), 'data' => $transactions, 'recordsFiltered' => $transactions_count, 'recordsTotal' => $transactions_total];
        return response()->json($response);
	}

    public function postAccountSummaryData()
    {
        $fields = [
            'startDate' => false,
            'endDate' => false
        ];

        $transactions = \App\Models\Transactions::select(DB::raw('SUM(COALESCE(`amount`, 0.00)) as amount, type, status'))->where('user_id', $this->user->id);

        $startDate = Input::get('startDate', $fields['startDate']);
        $endDate = Input::get('endDate', $fields['endDate']);

        if( $startDate !== '' && $endDate !== '' ) {
            $startDate = \App\Models\Date::format($startDate, 'Y-m-d');
            $endDate = \App\Models\Date::format($endDate, 'Y-m-d');
            $transactions = $transactions->whereRaw("(DATE(created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

        $transactions = $transactions->groupBy('type', 'status')->get();

        $transactions_sum = [
            'completed' => [
                'credits' => [1 => 0.00, 2 => 0.00, 3 => 0.00, 4 => 0.00, 5 => 0.00, 6 => 0.00, 'sum' => 0.00 ],
                'debits' => [7 => 0.00, 8 => 0.00, 9 => 0.00, 10 => 0.00, 11 => 0.00, 'sum' => 0.00]
            ],
            'pending' => [
                'credits' => [1 => 0.00, 2 => 0.00, 3 => 0.00, 4 => 0.00, 5 => 0.00, 6 => 0.00, 'sum' => 0.00 ],
                'debits' => [7 => 0.00, 8 => 0.00, 9 => 0.00, 10 => 0.00, 11 => 0.00, 'sum' => 0.00]
            ],
            'startDate' => $startDate,
            'endDate' => $endDate
        ];

        foreach( $transactions as $t ) {
            $status = ( $t->status == 1 ) ? 'completed' : 'pending';
            if( $t->type < 7 ) {
                $transactions_sum[$status]['credits'][ $t->type ] = $t->amountNumber;
                $transactions_sum[$status]['credits']['sum'] = $transactions_sum[$status]['credits']['sum'] + $t->amountNumber;
            }
            else {
                $transactions_sum[$status]['debits'][ $t->type ] = $t->amountNumber;
                $transactions_sum[$status]['debits']['sum'] = $transactions_sum[$status]['debits']['sum'] + $t->amountNumber;
            }
        }

        $results = [
            'credit' => [
                'processor-deposit' => number_format($transactions_sum['completed']['credits'][1], 2),
                'ewallet-cashout' => number_format($transactions_sum['completed']['credits'][2], 2),
                'commission' => number_format($transactions_sum['completed']['credits'][4], 2),
                'transfer-from' => number_format($transactions_sum['completed']['credits'][5], 2),
                'misc-credit' => number_format($transactions_sum['completed']['credits'][6], 2),
                'total-credits' => number_format($transactions_sum['completed']['credits']['sum'], 2)
            ],
            'debit' => [
                'processor-cashouts' => number_format($transactions_sum['completed']['debits'][11], 2),
                'ewallet-cashouts' => number_format($transactions_sum['completed']['debits'][7], 2),
                'plans' => number_format($transactions_sum['completed']['debits'][8], 2),
                'transfer-to' => number_format($transactions_sum['completed']['debits'][9], 2),
                'misc-debit' => number_format($transactions_sum['completed']['debits'][10], 2),
                'total-debits' => number_format($transactions_sum['completed']['debits']['sum'], 2)
            ],
            'cashout' => [
                'cashout-fee' => 0.00,
                'transfer-fee' => 0.00,
                'ewallet-cashouts' => number_format($transactions_sum['completed']['debits'][7], 2) . ' / $ ' . number_format($transactions_sum['pending']['debits'][7], 2) . ' Pending',
                'processor-cashouts' => number_format($transactions_sum['completed']['debits'][11], 2) . ' / $ ' . number_format($transactions_sum['pending']['debits'][11], 2) . ' Pending'
            ]
        ];

        return response()->json( $results );
    }

		public function postTransactionsBitcoinTXID() {
			$id = Input::get('pk');
			$txid = Input::get('value');

			$rules = [
				'value' => 'required|max:300',
				'pk' => 'required|exists:transactions,id'
			];

			$validator = Validator::make(Input::all(), $rules);

			if ($validator->fails())
	      return response()->json($validator->errors()->all()[0]);

			$transaction = \App\Models\Transactions::where('id', $id)->where('user_id', $this->user->id)->where('processor_id', 5)->first();

			if( is_null( $transaction ) ) return response()->json('Wrong transaction ID!');

			$bitcoin_transaction = \App\Models\TransactionsBitcoin::where('transaction_id', $transaction->id)->first();

			if( is_null( $bitcoin_transaction )) {
				$bitcoin_transaction = new \App\Models\TransactionsBitcoin;
				$bitcoin_transaction->transaction_id = $id;
			}

			$bitcoin_transaction->data = $txid;
			$bitcoin_transaction->save();

			return response()->json( true );
		}



    public function postActivationHistoryTable()
    {
        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);

        $history = \App\Models\ActivationHistory::where('user_id', $this->user->id);

        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ($_order['column']) {
                case 0:
                    $order_by = 'type';
                    break;
                case 1:
                    $order_by = 'status';
                    break;
                case 2:
                    $order_by = 'created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'created_at';
                    break;
            }

        }

        $history = $history->orderBy($order_by, $dir);

        $history_count = $history->count();
        $history_total = $history->skip( $start )->take( $length )->count();
        $history = $history->skip( $start )->take( $length )->get();

        if( $history_count > 0 ) {
            foreach( $history as $h ) {
                $h->date = $h->created_at->endOfWeek()->format('d. F Y');
                $h->typeName = $h->typeName;
            }
        }

        $response = ['draw' => Input::get('draw', 1), 'data' => $history, 'recordsFiltered' => $history_count, 'recordsTotal' => $history_total];
        return response()->json($response);
    }
}
