<?php 
namespace App\Http\Controllers\Clientzone\Ajax;

use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Input, DB, Config;

class TreeController extends AjaxController {

	public function __construct()
	{
		parent::__construct();
	}

    public function postDownlineTable( Request $request )
    {
        $root = $this->user->sponsor;

        $dataTable = DataTables::init( $request, [
            'model' => '\\App\\Models\\Sponsor',
            'order' => [
                0 => 'transactions.id',
                1 => 'transactions.username',
                2 => 'transactions.level',
                3 => 'transactions.created_at'
            ],
            'defaults' => [
                'level' => range(0,50),
                'search' => ['users.username', 'users.first_name', 'users.last_name', 'users.id']
            ]
        ])->dbJoin( function( $db ) use ( $root ) {
            return $db->select(DB::raw('users.id, users.created_at, users.first_name, users.last_name, users.username, sponsor_level AS level, users.sponsor_code'))
                ->leftJoin('users', 'users.id', '=', 'sponsors.user_id')
                ->where('user_id', '!=', $this->user->id)
                ->whereBetween('left', [ $root->left, $root->right ])
                ->where('sponsor_group', $root->sponsor_group);

        })->get(function( $t ) use ($root) {
            $t->levelCalc = $t->level - $root->sponsor_level;
            $t->date = Carbon::createFromFormat('Y-m-d H:i:s', $t->created_at)->format('d/m/y H:i');

            return $t;
        });

        return response()->json( $dataTable );
    }

    public function postSearchDownline( Request $request ) {
        $query = Input::get('q');
        $root = $this->user->upline;

        $users = \App\Models\TreeUplineNested::select(DB::raw('users.id, users.username'))
            ->join('users', 'users.id', '=', 'tree_upline_nested.id')
            ->where(function( $q ) use ( $query ) {
                $q->where('users.id', $query)
                    ->orWhere('first_name', 'LIKE', '%' . $query . '%')
                    ->orWhere('last_name', 'LIKE', '%' . $query . '%')
                    ->orWhere('username', 'LIKE', '%' . $query . '%')
                    ->orWhere('users.id', 'LIKE', '%' . $query . '%')
                    ->orWhere('email', 'LIKE', '%' . $query . '%');
            })
            ->where('tree_upline_nested.id', '!=', $this->user->id)
            ->whereBetween('tree_upline_nested._lft', [ $root->_lft, $root->_rgt ])
            ->get();

        return response()->json($users);
    }

    public function postGetTreeView( Request $request ) {
        $id = Input::get('id', '#');

        $level = Input::get('level', 0);


        $results = [];

        clock()->info($this->user->id);

        if( $id == '#' ) {
            $root = \App\Models\TreeGenealogyNested::where('id', $this->user->id)/*->with('user.balances')*/->first();

            $members = [$root];
        } else {
            $me = $this->user->genealogy;

            $members = \App\Models\TreeGenealogyNested::where('id', $id)->first()->children()/*->with('user.balances')*/->get()->toFlatTree();

        }

        foreach( $members as $m ) {

            $act_user_packages_amount = \App\Models\UsersPackages::select(DB::raw('SUM(packages.price_real) AS amount'))->join('packages', 'packages.id', '=', 'users_packages.package_id')->where('user_id', $m->user->id)->where('users_packages.status', 1)->pluck('amount')->first();

            clock()->info($level . ' - ' . $m->user->username . ' - ' . $act_user_packages_amount);
            //clock()->info('Generation: ' . $lvl);


            if(($this->user->id != $m->id) && ($level != 0)) {
                $res = [
                    'id' => $m->id,
                    'parent' => $id,
                    'text' => '<span class="m-badge m-badge--danger m-badge--wide m-badge--rounded m--font-boldest">' . $level . '</span> <span class="m-badge m-badge--success m-badge--wide m-badge--rounded m--font-boldest">' . $m->user->username . '</span><span class="m-badge m-badge--metal m-badge--wide m-badge--rounded"> ' . $this->getPercent($level) . '%</span><span class="m-badge m-badge--info m-badge--wide m-badge--rounded">$' . $act_user_packages_amount * ($this->getPercent($level) / 100) . '</span>',
                    'state' => [
                        'opened' => false,
                        'disabled' => false,
                        'selected' => false,
                    ],
                    'li_attr' => [],
                    'a_attr' => []
                ];
            }
            else {
                $res = [
                    'id' => $m->id,
                    'parent' => $id,
                    'text' => '<span class="m-badge m-badge--primary m-badge--wide m-badge--rounded m--font-boldest">' . $m->user->username . '</span> <span class="m-badge m-badge--danger m-badge--wide m-badge--rounded m--font-boldest">' . trans('tree.genealogyTree.level') . '</span> <span class="m-badge m-badge--success m-badge--wide m-badge--rounded m--font-boldest">' . trans('tree.genealogyTree.user_name') . '</span><span class="m-badge m-badge--metal m-badge--wide m-badge--rounded">' . trans('tree.genealogyTree.commission_percent') . ' (%)</span><span class="m-badge m-badge--info m-badge--wide m-badge--rounded">' . trans('tree.genealogyTree.commission_value') . ' ($)</span>',
                    'state' => [
                        'opened' => false,
                        'disabled' => false,
                        'selected' => false,
                    ],
                    'li_attr' => [],
                    'a_attr' => []
                ];
            }


            if( $m->hasChildren() )
                $res['children'] = true;

            $results[] = (object)$res;
        }

        return response()->json( $results );
    }

    public function getPercent($generation){

        foreach(config('mlm.commission_percent_levels') as $key => $value){
            if($generation <= $key){
                return $value;
            }
        }

        return 0;
    }

    public function postCheckAddMember() {
        $uid = Input::get('uid', null);
        $leg = Input::get('leg', null);
        $result = false;
        $error = false;
        $data = [];

        if( is_null( $leg ) )
            $error = 'Wrong leg selection.';
        else {
            if( !is_null( $findUpline = \App\Models\Sponsor::select(DB::raw('sponsors.*, u.username'))
                ->join('users AS u', 'u.id', '=', 'sponsors.user_id')
                ->where('sponsors.user_id', $uid)
                ->first()
            ) ) {

                if( $findUpline->children < 3 && $findUpline->children !== $leg ) {
                    $packages = \App\Models\Packages::where('status', 1)->orderBy('price', 'ASC')->get();

                    foreach( $packages as $p ) {
                        $data[] = [
                            'id' => $p->id,
                            'name' => $p->name,
                            'price' => $p->price,
                            'icon' => asset('images/icons/' . $p->icon),
                            'route' => route('auth.register', $this->user->username) . '?uid=' . $findUpline->user_id . '&l=' . $leg . '&p=' . $p->id
                        ];
                    }

                    $result = true;
                }
            }
        }

        return $this->response([
            'error' => $error,
            'data' => $data,
            'result' => $result
        ]);
    }

	/*public function postCommissionsTable()
	{
        $order = Input::get('order', false);
        $start = Input::get('start', 0);
        $length = Input::get('length', 10);
        $fields = [
			'startDate' => false,
			'endDate' => false,
			'level' => false,
			'search' => false
		];

        $root = $this->user->sponsor;

		$downline_list = \App\Models\Sponsor::select(DB::raw('users.id, users.created_at, users.first_name, users.last_name, level, users.username, users.email, sponsor_level, users.sponsor_code'))->leftJoin('users', 'users.id', '=', 'sponsors.user_id')->where('id', '!=', $this->user->id);

		if ($search = Input::get('search', $fields['search'])) {
            $downline_list = $downline_list->where(function($query) use ($search) {
              	$query->where('users.id', $search);
              	$query->orWhere('username', 'LIKE', '%' . $search . '%');
              	$query->orWhere('first_name', 'LIKE', '%' . $search . '%');
              	$query->orWhere('last_name', 'LIKE', '%' . $search . '%');
              	$query->orWhere('email', 'LIKE', '%' . $search . '%');
            });

        }

        if ($level = Input::get('level', $fields['level']))
			if ($level !== 0)
				if($level <= $root->sponsor_level )
					$downline_list = $downline_list->where('sponsor_level', '=', $level + 1);

        $downline_list = $downline_list->whereBetween('left', [ $root->left, $root->right ])
            ->where('sponsor_group', $root->sponsor_group);
            //->where('sponsor_level', '<=', $root->sponsor_level );

        $startDate = Input::get('startDate', $fields['startDate']);
        $endDate = Input::get('endDate', $fields['endDate']);

        if( $startDate && $startDate !== '' && $endDate && $endDate !== '' ) {

        	$startDate = \App\Models\Date::format(Input::get('startDate', $fields['startDate']));
        	$endDate = \App\Models\Date::format(Input::get('endDate', $fields['endDate']));

        	$downline_list = $downline_list->whereRaw("(DATE(users.created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");
        }

        $downline_list = $downline_list->orderBy('left', 'ASC');

        if ($order && !empty($order)) {
            $_order = array_pop($order);
            $dir = $_order['dir'];

            switch ($_order['column']) {
                case 2:
                    $order_by = 'sponsor_level';
                    break;
                case 3:
                    $order_by = 'created_at';
                    break;
                default:
                    $dir = 'desc';
                    $order_by = 'sponsor_level';
                    break;
            }

        }

        $downline_list = $downline_list->orderBy($order_by, $dir);

        $downline_list_count = $downline_list->count();
        $downline_list_total = $downline_list->skip( $start )->take( $length )->count();
        $downline_list = $downline_list->skip( $start )->take( $length )->get();

        foreach( $downline_list as $dl ) {
            $dl->levelCalc =  $dl->sponsor_level - $this->user->sponsor->sponsor_level;
        }

        $response = ['draw' => Input::get('draw', 1), 'data' => $downline_list, 'recordsFiltered' => $downline_list_count, 'recordsTotal' => $downline_list_total];
        return response()->json($response);
	}

    public function postCheckAddMember() {
        $uid = Input::get('uid', null);
        $leg = Input::get('leg', null);
        $result = false;
        $error = false;
        $data = [];

        if( is_null( $leg ) )
            $error = 'Wrong leg selection.';
        else {
            if( !is_null( $findUpline = \App\Models\Sponsor::select(DB::raw('sponsors.*, u.username'))
                ->join('users AS u', 'u.id', '=', 'sponsors.user_id')
                ->where('sponsors.user_id', $uid)
                ->first()
            ) ) {

                if( $findUpline->children < 3 && $findUpline->children !== $leg ) {
                    $packages = \App\Models\Packages::where('status', 1)->orderBy('price', 'ASC')->get();

                    foreach( $packages as $p ) {
                        $data[] = [
                            'id' => $p->id,
                            'name' => $p->name,
                            'price' => $p->price,
                            'icon' => asset('images/icons/' . $p->icon),
                            'route' => route('auth.register', $this->user->username) . '?uid=' . $findUpline->user_id . '&l=' . $leg . '&p=' . $p->id
                        ];
                    }

                    $result = true;
                }
            }
        }

        return $this->response([
            'error' => $error,
            'data' => $data,
            'result' => $result
        ]);
    }*/
}