<?php 

namespace App\Http\Controllers\Clientzone;

use \App\Http\Controllers\Controller;

use Input, DB;

class AjaxController extends Controller {

	public function __construct()
	{
		parent::__construct();
	}

	public function getSponsor()
	{
		$result = false;
		$response = [];
		$user_code = Input::get('sponsor_code', null);

		if( !is_null( $user_code ) ) {
			$sponsor = \App\Models\Sponsor::select(DB::raw('sponsors.*, u.username'))
			->leftJoin('users AS u', 'u.id', '=', 'sponsors.user_id')
			->orWhere('u.id', $user_code)
			->orWhere('user_code', $user_code)
			->orWhere('username', 'LIKE', '%' . $user_code . '%')
			->orWhere('email', 'LIKE', '%' . $user_code . '%')
			->first();

			if( !is_null( $sponsor ) ) {
				$result = true;
				$legs = 0;

				if( $sponsor->children < 3 ) {
					if( $sponsor->children == 1 ) {
						$legs = 2;
					} elseif( $sponsor->children == 2 ){
						$legs = 1;
					}
				}

				$response['sponsor'] = $sponsor;
				$response['legs'] = $legs;

			} else $response['message'] = 'Sponsor not found.';
		} else $response['message'] = 'Wrong sponsor code.';


		$response['result'] = $result;

		return response()->json( $response );
	}

	public function getUpline()
	{
		$result = false;
		$response = [];
		$user_code = Input::get('upline_user', null);

		if( !is_null( $user_code ) ) {
			$upline = \App\Models\Sponsor::select(DB::raw('sponsors.*, u.username'))
			->leftJoin('users AS u', 'u.id', '=', 'sponsors.user_id')
			->orWhere('u.id', $user_code)
			->orWhere('user_code', $user_code)
			->orWhere('username', 'LIKE', '%' . $user_code . '%')
			->orWhere('email', 'LIKE', '%' . $user_code . '%')
			->first();

			if( !is_null( $upline ) ) {
					$result = true;
					$response['upline'] = $upline;

			} else $response['message'] = 'Upline user not found.';
		} else $response['message'] = 'Wrong upline code.';


		$response['result'] = $result;

		return response()->json( $response );
	}

	public function searchCashout()
	{

		return $this->searchTransaction( function( $transactions ) {

			$extra_fields = ['status' => false, 'type' => [2,7,11]];

			$order = Input::get('order', false);

			if( $type = Input::get( 'type', $extra_fields['type'] ) )
				if( $type !== 'any' )
					$transactions = $transactions->whereIn('transactions.type', ( is_array( $type ) ) ? $type : [ $type ] );
				else
					$transactions = $transactions->whereIn('transactions.type', $extra_fields['type'] );

			if( $status = Input::get( 'status', $extra_fields['status'] ) )
				if( $status !== 'any' )
					$transactions = $transactions->where('transactions.status', $status);

				if ( $order && !empty( $order ) ) {
	      $_order = array_pop($order);
	      $dir = $_order['dir'];

	      switch ( $_order['column'] ) {
	        case 6:
	        	$order_by = 'transactions.status';
	          break;
	        case 7:
	          $order_by = 'transactions.created_at';
	          break;
	         default:
	         	$dir = 'desc';
	         	$order_by = 'transactions.id';
	         break;
	      }

	    }

      return $transactions = $transactions->orderBy( $order_by, $dir );

		}, 'amount, type, created_at, note, transactions.id, processors.name as processor, fee, status');
	}

	public function searchTransaction( $extra = false, $extra_select = false, $extra_order = false )
	{
		$order = Input::get('order', false);
		$start = Input::get('start', 0);
		$length = Input::get('length', 10);

		$fields = [
			'id' => false,
			'startDate' => \App\Date::now('Y-m-d'),
			'endDate' => \App\Date::now('Y-m-d'),
			'type' => [1,2,3,4,5,6,7,8,9,10,11],
			'processor' => false,
		];

		$transactions = \App\Models\Transactions::select(
			( $extra_select ) ?
				DB::raw($extra_select) :
				DB::raw('amount, type, created_at, note, transactions.id, processors.name as processor')
		)->join('processors', 'processors.id', '=', 'transactions.processor_id')->where('user_id', $this->user->id);

		if( $id = Input::get( 'id', $fields['id'] ) )
			if( $id !== '' )
				$transactions = $transactions->where('transactions.id', $id);

		if( $processor = Input::get( 'processor', $fields['processor'] ) )
			if( $processor !== 'any' )
				$transactions = $transactions->where('transactions.processor_id', $processor);

		$startDate = \App\Date::format (Input::get( 'startDate', $fields['startDate'] ) );
		$endDate = \App\Date::format( Input::get( 'endDate', $fields['endDate'] ) );
		$transactions = $transactions->whereRaw("(DATE(created_at) BETWEEN '" . $startDate . "' AND '" . $endDate . "')");

		$start = ( $start < 1 ) ? 0 : $start;
    $length = ( $length < 1 ) ? 25 : $length;

    if( $extra )
			$transactions = call_user_func( $extra, $transactions );
		else {

			if( $type = Input::get( 'type', $fields['type'] ) )
				if( $type !== 'any' )
					$transactions = $transactions->whereIn('transactions.type', ( is_array( $type ) ) ? $type : [ $type ] );
				else
					$transactions = $transactions->whereIn('transactions.type', $fields['type'] );

			if ( $order && !empty( $order ) ) {
	      $_order = array_pop($order);
	      $dir = $_order['dir'];

	      switch ( $_order['column'] ) {
	        case 2:
	        	$order_by = 'transactions.amount';
	          break;
	        case 5:
	          $order_by = 'transactions.created_at';
	          break;
	         default:
	         	$dir = 'desc';
	         	$order_by = 'transactions.id';
	         break;
	      }

	    }

      $transactions = $transactions->orderBy( $order_by, $dir );

    }


		$transactions_total = $transactions->count();
    $transactions_count = $transactions->skip( $start )->take( $length )->count();
    $transactions = $transactions->skip( $start )->take( $length )->get();

    $response = ['draw' => Input::get('draw', 1), 'data' => $transactions, 'recordsFiltered' => $transactions_count, 'recordsTotal' => $transactions_total];
    return response()->json( $response );
	}

}
