<?php 

namespace App\Http\Controllers\Clientzone;

use App\Models\AuctionItems;
use Auth;
use App\Facades\NotificationManager;
use Illuminate\Http\Request, Cart, Session, Validator;
use Illuminate\Support\Arr;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Redirect;

class AuctionController extends ClientzoneController {

	public function __construct()
	{
		parent::__construct();
	}

	public function getIndex( Request $request ) {
		$endingSoon = \App\Models\AuctionItems::where('status', 0)->endingSoon()->take(12)->get();

        $openItems = \App\Models\AuctionItems::where('status', 0)->orderBy('ends_at', 'desc')->paginate(12);
        //$openItems = \App\Models\AuctionItems::orderBy('ends_at', 'desc')->take(100)->get();

		$featured = \App\Models\AuctionItems::where('status', 0)->where('type_id', 2)->take(12)->get();
		$closedItems = \App\Models\AuctionItems::where('status', 2)->orderBy('ends_at', 'desc')->take(12)->get();
		$categories = \App\Models\AuctionCategories::where('status', 1)/*->orderBy('title', 'ASC')*/->get();
        $priceCategores = config('auction.price_categories');

		return view('auction.index',
		[
			'endingSoon' => $endingSoon,
            'categories' => $categories,
            'priceCategories' => $priceCategores,
			'closedItems' => $closedItems,
            'openItems' => $openItems,
            'featured' => $featured
		]
		)->withTitle('Auction');
	}

	public function getCategory( Request $request, $title = null ) {
		if( is_null( $category = \App\Models\AuctionCategories::where('title_url', $title)->first() ) )
			return redirect()->back()->with('error', trans('auction.error.Category not found'));

		$items = \App\Models\AuctionItems::where('status', 0)->where('category_id', $category->id)->notEnded()->orderBy('ends_at', 'ASC')->get();

		return view('auction.category',
		[
			'items' => $items,
			'category_title' => $category->title
		]
		)->withTitle('Auction');
	}
    public function getPriceCategory( Request $request, $categoryN = 1 ) {
		if( $categoryN <1 || $categoryN > count(config('auction.price_categories')))
			return redirect()->route('auction.index')->with('error', 'Category not found.');
        $category = config('auction.price_categories')[$categoryN];
        $min = $category[0];
        $max = $category[1];
        $column = config('auction.use_buy_price_for_categories') ? 'buy_price' : 'current_price';
        $items = \App\Models\AuctionItems::where('status', 0)->whereRaw($column.' >= '. $min. ' and '.$column.' <'.$max)->notEnded()->orderBy('ends_at', 'ASC')->get();
        $prev = null;
        $next = null;
        if($categoryN>1) $prev =$categoryN-1;
        if($categoryN<count(config('auction.price_categories'))) $next =$categoryN+1;
        return view('auction.category',
		[
			'items' => $items,
            'category_title' => $min.'$ -'.$max .'$',
            'prev' => $prev,
            'next' => $next
		]
		)->withTitle('Auction');
	}

	public function getPackages( Request $request )
    {
        $cart = Cart::instance('auction.packages');
        $selectedPackage = Session::get('selectedAuctionPackage', null);

        if (!is_null($selectedPackage)) {
            $package = \App\Models\AuctionPackages::where('id', $selectedPackage)->first();

            if ($cart->content()->count() > 0)
                $cart->destroy();

            $cart->add([
                'id' => $package->id,
                'name' => $package->name,
                'qty' => 1,
                'price' => $package->price,
                'bids' => $package->bids
            ]);
        }


        $userid = $this->user->id;
        $balances = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

        $e_wallet_balance = $balances->wallet;

        $bids_count = \App\Models\AuctionUsersPackages::where('user_id', $this->user->id)->sum('bids');

        $won_items_count = \App\Models\AuctionItems::whereHas('lastBidder', function ($query) use ($userid) {
            $query->where('user_id', $this->user->id);
        })->where('status', 2)->count();

        $watchlist_count = \App\Models\AuctionWatchlist::select('auction_watchlist.*')->where('user_id', $this->user->id)->count();


        $total_bids = \App\Models\AuctionUsersPackages::where('user_id', $this->user->id)->sum('bids');
        $mypackages = \App\Models\AuctionUsersPackages::where('user_id', $this->user->id)->orderBy('created_at', 'ASC')->get();

        /*if ($mypackages->count() > 0) {

            foreach ($mypackages as $p) {
                if($p->is_mlm_package){
                    $p->package->name = \App\Models\Packages::where('id', $p->package->id)->first()->name;
                }
            }
        }*/

		$packages = \App\Models\AuctionPackages::where('status', 1)->orderBy('price', 'asc')->get();
		//$ewallets = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
        
        //$_processors = \App\Models\Processors::where('id', 4)->where('fee_fund_status', 1)->orderBy('id', 'asc')->get();

		return view('auction.packages', [
		    'count_bids' => $bids_count,
            'count_won_items' => $won_items_count,
            'count_watchlist' => $watchlist_count,
            'e_wallet_balance' => $e_wallet_balance,
            'total_bids' => $total_bids,
            'packages' => $packages,
            'mypackages' => $mypackages,
            'cart' => $cart])->withTitle('Packages');
	}

	public function getBasket()
	{
		$cart = Cart::instance('auction.packages');

		//if( $cart->content()->count() == 0 ) 
		//	return redirect()->route('auction.packages');
		//$_processors = \App\Models\Processors::where('id', 4)->where('fee_fund_status', 1)->orderBy('id', 'asc')->get();

        $_processors = \App\Models\Processors::where('fee_fund_status', 1)->orderBy('id', 'asc')->get();

		$wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

        $total_cards = \App\Models\GiftCards::where('owner_id', $this->user->id)->where('deleted', 0)->sum('amount_left');

		return view('auction.basket', ['cart' => $cart, 'ewallets' => $wallet, 'giftcards' => $total_cards, 'processors' => $_processors])->withTitle('Auction - Basket');
	}

	public function postBasket( Request $request )
	{
		$data = $request->all();
		$rules = [
            'processor' => 'required|exists:processors,id'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
            return redirect()->back()->withInput()->withErrors($validator);

        $cart = Cart::instance('auction.packages');

        if( $cart->content()->count() == 0 )
            //return redirect()->back()->withInput()->with('error', trans('auction.error.Please select one package at least') );
            return redirect()->back()->withInput()->with('error', trans('messages.least_one_package'));

        $price = (float)$cart->total(2, '.', '');
        $cart_item_count = 0;
        foreach($cart->content() as $row) {
            $cart_item_count += $row->qty;
        }

        $item = $cart->content()->first();

        $package = \App\Models\AuctionPackages::where('id', $item->id)->first();
        //$user_processor = \App\Models\UsersProcessors::where('processor_id', $data['processor'])->where('data', '!=', NULL)->where('user_id', $this->user->id)->where('status', 1)->first();
        //$processor = \App\Models\Processors::where('id', $data['processor'])->first();



        $user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

        if($data['processor'] == 4) {

            if ($user_wallet->wallet < ($package->price * $cart_item_count))
                return redirect()->back()->with('error', trans('messages.no_money'))->withInput();
            //return redirect()->back()->with('error', trans('auction.error.You dont have enough money') )->withInput();

            for ($i = 0; $i < $cart_item_count; $i++) {
                $user_wallet->wallet = $user_wallet->wallet - $package->price;
                $user_wallet->save();

                if (is_null($new_package = \App\Models\AuctionUsersPackages::where('is_mlm_package', 0)->where('package_id', $item->id)->where('user_id', $this->user->id)->first())) {

                    $new_package = new \App\Models\AuctionUsersPackages;
                    $new_package->user_id = $this->user->id;
                    $new_package->package_id = $package->id;
                    $new_package->bids = $package->bids;
                } else {
                    $new_package->bids = $new_package->bids + $package->bids;
                }

                $new_package->save();

                $new_transaction = new \App\Models\Transactions;
                $new_transaction->balance = 'wallet';
                $new_transaction->processor_id = $data['processor'];
                $new_transaction->user_id = $this->user->id;
                $new_transaction->type = 13;
                $new_transaction->amount = $package->price;
                $new_transaction->note = 'Auction Package ID: #' . $new_package->id . ', Type: ' . $new_package->package->name;
                $new_transaction->status = 1;
                $new_transaction->save();
                $price = 0;

            }


        }

        else if($data['processor'] == 6){
            $total_cards = \App\Models\GiftCards::where('owner_id', $this->user->id)->where('deleted', 0)->sum('amount_left');
            if ($total_cards < (float)$cart->total(2, '.', ''))
                return Redirect::back()->with('error', trans('auctionpurchase.shared.noGift'));

            $gift_cards = \App\Models\GiftCards::where('owner_id', $this->user->id)->where('deleted', 0)->orderBy('amount_left', 'asc')->get();
            $price = (float)$cart->total(2, '.', '');



            for ($i = 0; $i < $cart_item_count; $i++) {

                foreach ($gift_cards as $gift) {

                    if ((float)$gift->amount_left <= (float)$price && (float)$price > 0) {
                        $gf = \App\Models\GiftCards::where('id', $gift->id)->first();
                        //$gf->amount = 0;
                        $gf->deleted_at = Carbon::now();
                        $gf->deleted = 1;
                        $gf->used_by = $this->user->id;
                        if($gf->amount == $gf->amount_left) {
                            $price = (float)$price - (float)$gf->amount;
                        }
                        else{
                            $price = (float)$price - (float)$gf->amount_left;
                        }
                        $gf->save();
                    } else if ((float)$gift->amount > (float)$price && (float)$price > 0) {
                        $gf = \App\Models\GiftCards::where('id', $gift->id)->first();
                        $gf->amount_left = (float)$gf->amount - (float)$price;
                        $price = (float)$price - (float)$gf->amount;
                        $gf->save();
                    } else {
                        break;
                    }

                }


                if (is_null($new_package = \App\Models\AuctionUsersPackages::where('package_id', $item->id)->where('user_id', $this->user->id)->first())) {

                    $new_package = new \App\Models\AuctionUsersPackages;
                    $new_package->user_id = $this->user->id;
                    $new_package->package_id = $package->id;
                    $new_package->bids = $package->bids;
                } else {
                    $new_package->bids = $new_package->bids + $package->bids;
                }

                $new_package->save();

                $new_transaction = new \App\Models\Transactions;
                $new_transaction->balance = 'giftcard';
                $new_transaction->processor_id = $data['processor'];
                $new_transaction->user_id = $this->user->id;
                $new_transaction->type = 13;
                $new_transaction->amount = $package->price;
                $new_transaction->note = 'Auction Package ID: #' . $new_package->id . ', Type: ' . $new_package->package->name;
                $new_transaction->status = 1;
                $new_transaction->save();
            }
        }

        $cart->destroy();

        if( $price <= 0 ) {


            NotificationManager::create([
                'user_id' => $this->user->id,
                'type' => 'user-auction-package-purchase',
                'content_id' => $package->id
            ], $this->user);


            return redirect()->route('auction.packages')->with('success', trans('messages.package_purchased'));
            //return redirect()->route('auction.packages')->with('success', trans('auction.success.You have successfully purchased a package') );

        } else
            return redirect()->back()->with('error', trans('messages.purchase_error'));
            //return redirect()->back()->with('error', trans('auction.error.There was an error while trying to save your purchase'));
	}

	public function getBasketRemove( $id = null ) {
        $cart = Cart::instance('auction.packages');

        if( is_null( $item = $cart->get( $id ) ) )
            return redirect()->back()->withError(trans('messages.wrong_item_id'));
            //return redirect()->back()->withError(trans('auction.error.Wrong item ID'));

        $cart->remove( $id );
        return redirect()->back();
    }

	public function getView( Request $request, $id = null ) {
		if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->first() ) )
			return redirect()->back()->with('error', trans('messages.item_not_found'));
			//return redirect()->back()->with('error', trans('auction.error.Item not found'));

		$relatedItems = \App\Models\AuctionItems::where('status', 0)->where('auction_items.id', '!=', $item->id)->where(function($query) use ($item) {
			$query
			->orWhere('auction_items.category_id', '=', $item->category_id)
			->orWhere('auction_items.start_price', '<=', $item->start_price + 200)
			->orWhere('auction_items.start_price', '>=', $item->start_price - 200)
			->orWhere('auction_items.buy_price', '<=', $item->buy_price + 200)
			->orWhere('auction_items.buy_price', '>=', $item->buy_price - 200);
		})
		->notEnded()
		->take(5)
		->get();

		if( $relatedItems->count() < 5) {
			$restRelatedItems = \App\Models\AuctionItems::where('status', 0)->
			where('id', '!=', $item->id)
			->orderBy('ends_at', 'DESC')
			->take( 5 - $relatedItems->count() );

			if( $relatedItems->count() > 0 ) {
				$ids = $relatedItems->pluck('id');
				$restRelatedItems = $restRelatedItems->whereNotIn('id', $ids->all());
			}

			$restRelatedItems = $restRelatedItems->get();
			$relatedItems = $relatedItems->merge( $restRelatedItems );
		}

		$myBidButlerCounter = ( is_null( $item->myBidButlers ) ) ? 0 : $item->myBidButlers->counter;

		return view('auction.view', ['item' => $item, 'relatedItems' => $relatedItems, 'myBidButlerCounter' => $myBidButlerCounter])->withTitle('Auction - ' . $item->title);
	}

	public function getWonItems( Request $request ) {
		$userid = $this->user->id;

		clock()->info($userid);

		$items = \App\Models\AuctionItems::whereHas('lastBidder', function( $query ) use ( $userid ) {
			$query->where('user_id',  $this->user->id);
		})->where('status', 2)->orderBy('ends_at', 'DESC')->get();

		//$items = AuctionItems::getItemsByWinner($userid);


		//$items = \App\Models\AuctionItems::where('status', 2)->where()

        clock()->info($items);

		return view('auction.won-items',
		[
			'items' => $items/*,
            'enableRefund' => config('auction.enable_refund')*/
		]
		)->withTitle('Auction - Won Items');
	}

	public function getWatchlist( Request $request ) {
		$userid = $this->user->id;

		$items = \App\Models\AuctionWatchlist::select('auction_watchlist.*')->with(['item' => function($q) {
			$q->orderBy('ends_at', 'DESC');
		}])->where('user_id', $this->user->id)->get();

		return view('auction.watchlist',
		[
			'items' => $items
		]
		)->withTitle('Auction - My Watchlist');
	}

    public function getBidButler(){
        return view('auction.bidbutler')->withTitle( trans('bidbutler.index.title') );

    }

    public function getButlers(){

        $bids_count = \App\Models\AuctionUsersPackages::where('user_id', $this->user->id)->sum('bids');

        //$butlers = \App\Models\AuctionBidButlers::where('user_id', $this->user->id)->get();

        $watchlist_items = \App\Models\AuctionWatchlist::where('user_id', $this->user->id)->pluck('item_id');

        $bidded_items = \App\Models\AuctionBids::where('user_id', $this->user->id)->groupBy('item_id')->pluck('item_id');


        $merged_items = $watchlist_items->merge($bidded_items)->unique();



        //clock()->info($bidded_items);

        //$item_ids = [];

        /*foreach($butlers as $index=>$butler){
            $item_ids[$index] = $butler->item_id;
        }*/

        //clock()->info($item_ids);

        //array_unique(array_merge($watchlist_items, $bidded_items));

        //$items = \App\Models\AuctionItems::whereIn('id', $item_ids)->get();
        //$items = \App\Models\AuctionItems::whereIn('id', $bidded_items)->where('status', 0)->get();

        $items = \App\Models\AuctionItems::whereIn('id', $merged_items)->where('status', 0)->get();

        foreach($items as $item){
           $item['my_butlers'] = ( is_null( $item->myBidButlers ) ) ? 0 : $item->myBidButlers->counter;
        }

        //clock()->info($items);

        //$myBidButlerCounter = ( is_null( $item->myBidButlers ) ) ? 0 : $item->myBidButlers->counter;


        return view('auction.butlers', [
            'bids_count' => $bids_count,
            'items' => $items
        ])->withTitle( trans('butlers.title') );

    }
    public function postButlers(){
        //return view('auction.bidbutler')->withTitle( trans('bidbutler.index.title') );

    }
    public function reFund($id)
	{ 
        if(!config('auction.enable_refund'))
            return redirect()->route('auction-purchase.purchase')->with('error',  trans('auction.error.no_refund'));
        if( is_null( $item = \App\Models\AuctionItems::where('id', $id)->first() ) )
            return redirect()->route('auction-purchase.purchase')->with('error',  trans('auction.error.invalid_auction'));

        if(!(Auth::user() && $item->isEnded() && $item->lastBidder && $item->lastBidder->user_id == Auth::user()->id ))
            return redirect()->route('auction-purchase.purchase')->with('error',  trans('auction.error.not_winner'));

        $refund_amount=((($item->buy_price) - ($item->current_price))*( config('auction.refund_percentage')/100));
        if($refund_amount<0) $refund_amount=0;
 
        $item->status = 5;
        if($item->save()){
            if($this->refundAmount($item->lastBidder->user_id,$refund_amount,$item->id, $item->title)){
                //return redirect()->route('auction.won')->with('success',  trans('auction.success.Refund Succesfull').$refund_amount.'$');
                return redirect()->route('auction-purchase.purchase')->with('success',  trans('auction.success.Refund Succesfull'). '$'.$refund_amount);

            }
        }
        return redirect()->route('auction-purchase.purchase')->with('error',  trans('auction.error.unknown'));
           
  
    }
    private function refundAmount($user, $amount, $item, $title)
    {   //get user balance info
        $userbalance = \App\Models\UsersBalances::where('users_id', $user)->first();


        //save bonus to user balance
        if ($userbalance) {

                $userbalance->wallet += $amount;
          
                if($userbalance->save()){
                    $transaction = new \App\Models\Transactions;
                    //$transaction->processor_id = $processorid;
                    $transaction->user_id = $user;
                    $transaction->type = 14;
                    $transaction->amount = $amount;
                    $transaction->balance = "wallet";
                    $transaction->note = 'Refund for item #'.$item.': '.$title;
                    $transaction->status = 1;
                    return $transaction->save();

             }
        }
    }
}