<?php 

namespace App\Http\Controllers\Clientzone;

use Validator, Input, Config, PDF,File;

class CertificatesController extends ClientzoneController {

	public function __construct()
	{
		parent::__construct();
	}

	public function donateTree() {

        $wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

        return view('certificates.donate-tree', ['wallet' => $wallet])->withTitle(trans('certificates.donateTree.title'));
    }

    public function postDonateTree() {
    	$treecount = Input::get('treecount', 0);
    	$secondaryPassword = Input::get('password');
    	$data = Input::all();

    	$rules = [
			'treecount' 	=> 'required|min:1',
			'password' 	=> 'required|max:250'
		];

		$validator = Validator::make( $data, $rules );

		if ( $validator->fails() )
			return Redirect::back()->withInput()->withErrors( $validator );

		if( !$this->user->checkPassword($data['password'], 'secondary_password') )
			return Redirect::back()->with('error', trans('certificates.shared.wrongPassword'))->withInput();

    	$wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
    	$floor = floor( $wallet->resale_units / 100 );

    	if( $floor < $treecount )
    		return redirect()->back()->with('error', trans('certificates.shared.noUnits'));

    	$wallet->resale_units = $wallet->resale_units - ( $treecount * 100 );
    	$wallet->save();

    	$new_transaction = new \App\Models\Transactions;
	    $new_transaction->user_id = $this->user->id;
	    $new_transaction->type = 21;
	    $new_transaction->fee = 0;
	    $new_transaction->amount = $treecount * 100;
	    $new_transaction->note = 'Donated ' . $treecount . ' tree(s)';
	    $new_transaction->status = 1;
	    $new_transaction->save();

    	return redirect()->back()->with('success', trans('certificates.donateTree.messages.success', ['count' => $treecount]) );
    }

	public function issue()
	{
		$wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
		$countries = \App\Models\Countries::get();
		return view('certificates.issue', ['wallet' => $wallet, 'countries' => $countries])->withTitle(trans('certificates.issue.title'));
	}

	public function postIssue()
	{
		$rules = [
			'description' => 'max:200',
			'amount' => 'required|numeric',
			'name' => 'required|max:30',
			'address' => 'required|max:100',
			'city' => 'required|max:30',
			'postcode' => 'required|max:20',
			'state' => 'max:30',
			'country' => 'required|exists:countries,id',
			'language' => 'required|in:'. implode(',', array_keys(Config::get('app.languages')))
		];

		$validator = Validator::make(Input::all(), $rules);

		if ( $validator->fails() )
			return redirect()->back()->withInput()->withErrors( $validator );

		$wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

		if( $wallet->resale_units < Input::get('amount') )
	    	return redirect()->back()->with('error', trans('certificates.shared.noUnits'))->withInput();

	    $wallet->resale_units = ($wallet->resale_units - Input::get('amount'));
	    $wallet->save();

	    $certificate = new \App\Models\Certificates;
	    $certificate->user_id = $this->user->id;
	    $certificate->description = Input::get('description', NULL);
	    $certificate->amount = Input::get('amount');
	    $certificate->name = Input::get('name');
	    $certificate->address = Input::get('address');
	    $certificate->city = Input::get('city');
	    $certificate->postcode = Input::get('postcode');
	    $certificate->state = Input::get('state', NULL);
	    $certificate->country_id = Input::get('country');
	    $certificate->language_id = Input::get('language');
	    $certificate->save();

		return redirect()->route('certificates.list')->with('success', trans('certificates.issue.messages.success'));
	}

	public function view( $id = null ) {
		if( is_null( $certificate = \App\Models\Certificates::where('id', $id)->where('user_id', $this->user->id)->first()) )
			return redirect()->route('certificates.list')->with('error', trans('certificates.shared.wrongCertID'));

		/*return view('certificates.pdf-template', [
			'id' => $certificate->id,
			'name' => $certificate->name,
			'created_at' => $certificate->created_at,
			'description' => $certificate->description,
			'units' => $certificate->amount,
			'city' => $certificate->city,
			'address' => $certificate->address,
			'postcode' => $certificate->postcode,
			'state' => $certificate->state,
			'country' => $certificate->country->name,
		]);*/

		$pdf = PDF::loadView('certificates.pdf-template', [
			'id' => $certificate->id,
			'name' => $certificate->name,
			'created_at' => $certificate->created_at,
			'description' => $certificate->description,
			'units' => $certificate->amount,
			'city' => $certificate->city,
			'address' => $certificate->address,
			'postcode' => $certificate->postcode,
			'state' => $certificate->state,
			'country' => $certificate->country->name,
		]);
		return $pdf->stream('AusanteCertificate-' . $certificate->id);

		//return view('certificates.pdf-template');
	}

	public function download( $id = null ) {
		if( is_null( $certificate = \App\Models\Certificates::where('id', $id)->where('user_id', $this->user->id)->first()) )
			return redirect()->route('certificates.list')->with('error', trans('certificates.shared.wrongCertID'));

		//return view('certificates.pdf-template');

		$pdf = PDF::loadView('certificates.pdf-template', [
			'id' => $certificate->id,
			'name' => $certificate->name,
			'created_at' => $certificate->created_at,
			'description' => $certificate->description,
			'units' => $certificate->amount,
			'city' => $certificate->city,
			'address' => $certificate->address,
			'postcode' => $certificate->postcode,
			'state' => $certificate->state,
			'country' => $certificate->country->name,
		]);

		return $pdf->download('AusanteCertificate-' . $certificate->id);
	}

	public function list()
	{
		return view('certificates.list')->withTitle(trans('certificates.list.title'));
	}

}