<?php 

namespace App\Http\Controllers\Clientzone;

use App\Libraries\NodeGWManager;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use DB, File, Session, Cookie, Auth;

class DashboardController extends ClientzoneController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{

	    if(is_null($this->user->uuid))
        {
            $act_user = \App\Models\User::where('id', $this->user->id)->first();
            $temp_uuid = Str::uuid();
            $act_user->uuid = $temp_uuid;
            $act_user->save();

            $this->user->uuid = $temp_uuid;
        }

        if($this->user->isInTree()){
            if(!File::exists($this->user->profile->getQRCodeAttribute())){
                $this->user->profile->generateQRCode();
            }
        }



		$reports = \App\Models\Reports::where('status', 1)->orderBy('date', 'DESC')->get();

		$pending_cashouts = \App\Models\Transactions::where('user_id', $this->user->id)->where('status', 0)->whereIn('type', [7,11])->sum('amount');
		$balances = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

		$e_wallet_balance = $balances->wallet;

		$cash_balance = ($balances->online_account + $balances->resale_units + $balances->offset_units);

		$active_plans = \App\Models\UsersPackages::select(DB::raw('SUM(COALESCE(`invested_amount`, 0.00)) as amount'))->where('user_id', $this->user->id)->where('status', 1)->pluck('amount');

		$my_referrer = 'None';
		
		if( is_null( $this->user->sponsor ) )
			$my_referrer_id = 0;
		else
			$my_referrer_id = $this->user->sponsor->sponsor_id;

		if( $my_referrer_id !== 0 )
			$my_referrer = \App\Models\User::where('id', $this->user->sponsor->sponsor_id)->pluck('username');

		$news = \App\Models\News::orderBy('id', 'desc')->take(15)->get();
		$tickets = \App\Models\Support::where('user_id', $this->user->id)->orderBy('id', 'desc')->take(20)->get();

        //New tiles
        $userid = $this->user->id;


        $bids_count = \App\Models\AuctionUsersPackages::where('user_id', $this->user->id)->sum('bids');

        $won_items_count = \App\Models\AuctionItems::whereHas('lastBidder', function( $query ) use ( $userid ) {
            $query->where('user_id',  $this->user->id);
            })->where('status', 2)->count();

        $watchlist_count =  \App\Models\AuctionWatchlist::select('auction_watchlist.*')->where('user_id', $this->user->id)->count();
        //

		return view('dashboard.index', [
		    'count_bids' => $bids_count,
		    'count_won_items' => $won_items_count,
		    'count_watchlist' => $watchlist_count,
		    'my_referrer' => $my_referrer,
            'my_referrer_id' => $my_referrer_id,
            'pending_cashouts' => $pending_cashouts,
            'e_wallet_balance' => $e_wallet_balance,
            'cash_balance' => $cash_balance,
            'balances' => $balances,
            'active_plans' => $active_plans,
            'news' => $news,
            'tickets' => $tickets,
            'reports' => $reports
        ])->withTitle( trans('dashboard.index.title') );
	}

    public function getActivate(Request $request, $activationCode = null)
    {

        $activated = false;

        if( !is_null( $user = \App\Models\User::where('activation_code', $activationCode)->first() ) ) {

            $activated = true;

            if( $user->attemptActivation( $activationCode ) ) {
                Auth::login( $user, true );
                return redirect( '/desk' );
            }
        }


        if( $this->user->isActivated() )
            return redirect()->route('home');

        return view('dashboard.activate')->withTitle( trans('dashboard.index.title') );
    }

    public function postActivate(\Illuminate\Http\Request $request)
    {
        $inputs = $request->all();

        if( !$this->user->isActivated() ) {
            $rules = [
                'activationCode'	=> 'required'
            ];

            $validator = Validator::make( $request->all(), $rules );

            if ( $validator->fails() )
                return redirect()->back()->withInput()->withErrors( $validator );

            if( !$this->user->attemptActivation( $inputs['activationCode'] ) )
                return redirect()->back()->with('error', trans('dashboard.activate.wrongCode'));

            //return redirect()->route('home');
        }

        return redirect()->route('home');
    }

    public function getResendCode(\Illuminate\Http\Request $request){
        //$user = Auth::user();
       // $locale = $user->locale;

        if(!$user = \App\Models\User::where('id', Auth::id())->first()) {
            return redirect()->back()->with('error', trans('mail.account_confirmation.user_not_found'));
        }

        \App\Facades\MailManager::send( 'auth.account-confirmation', trans('mail.account_confirmation.title'), [
            'activationCode' => $user->getActivationCode()
        ], $user);

        return redirect()->back()->with('success', trans('messages.activation_email_sent'));
    }
}
