<?php 

namespace App\Http\Controllers\Clientzone;

use Config, DB, Input, Validator, Session, Redirect, Mail, Queue, MailManager, Cart;

class GiftCardsController extends ClientzoneController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		$total_cards = \App\Models\GiftCards::where('owner_id', $this->user->id)->where('deleted', 0)->sum('amount_left');
		return view('giftcards.index', ['total_cards' => $total_cards])->withTitle( trans('giftcards.index.title') );
	}

    public function activated()
    {
        return redirect()->route('gift-cards')->with('success', trans('giftcards.shared.gift_card_success_activation'));
    }

    public function getBasketRemove( $id ) {
        $cart = Cart::instance('giftcards');

        if( is_null( $item = $cart->get( $id ) ) )
            return Redirect::back()->with('error', trans('giftcards.shared.wrongID'));

        $cart->remove( $id );
        return Redirect::back();
    }

    public function getProcess()
    {
        $cart = Cart::instance('giftcards');
        if( $cart->content()->count() == 0 ) 
            return redirect()->route('gift-cards');

        $wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

        return view('giftcards.process', ['cart' => $cart, 'wallet' => $wallet])->withTitle( trans('giftcards.process.title') );
    }

    public function postProgress()
    {
        $data = Input::all();
        $rules = [
            'wallet' => 'required|in:1'
        ];

        $wallets = [1 => 'wallet'];
        $walletName = $wallets[ $data['wallet'] ];

        $cart = Cart::instance('giftcards');
        if( $cart->content()->count() == 0 ) 
            return redirect()->route('gift-cards');

        $user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

        if( $user_wallet->$walletName < (float)$cart->total(2, '.', '') )
            return Redirect::back()->with('error', trans('giftcards.shared.noMoney'));

        $user_wallet->$walletName = $user_wallet->$walletName - (float)$cart->total(2, '.', '');
        $user_wallet->save();

        $items = $cart->content();
        $item = $items->first();
        $id = $item->id;
        $qty = $item->qty;
        $price = $item->price;
        $name = $item->name;

        for( $i = 0; $i < $qty; $i++ ) {
            $giftcard = new \App\Models\GiftCards;
            $giftcard->owner_id = $this->user->id;
            $giftcard->package_id = $id;
            $giftcard->hash = md5(uniqid(rand(), TRUE));
            $giftcard->amount = $price;
            $giftcard->amount_left = $price;
            $giftcard->deleted = 0;
            $giftcard->title = $name;
            $giftcard->save();
        }

        $cart->destroy();

        return redirect()->route('gift-cards')->with('success', trans('giftcards.process.messages.success'));
    }

	public function getPurchase()
	{
		$total_cards = \App\Models\GiftCards::where('owner_id', $this->user->id)->where('deleted', 0)->sum('amount_left');
		$packages = \App\Models\Packages::where('status', 1)->orderBy('id', 'asc')->get();
		$ewallets = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
        $cart = Cart::instance('giftcards');

		return view('giftcards.purchase', ['ewallets' => $ewallets, 'total_cards' => $total_cards, 'packages' => $packages, 'cart' => $cart])->withTitle(trans('giftcards.purchase.title'));
	}

	public function postPurchase()
	{
		$data = Input::all();
        $cart = Cart::instance('giftcards');

        $id = Input::get('item', null);
        $quantity = Input::get('quantity', 1);

        $rules = [
            'item'      => 'required',
            'quantity'  => 'required|min:1'
        ];

        $validator = Validator::make( $data, $rules );

        if ( $validator->fails() )
            return Redirect::back()->withErrors( $validator );

        if(is_null( $package = \App\Models\Packages::where('id', $id)->first() ) )
            return Redirect::back()->with('error', trans('giftcards.shared.wrongPackage'));

        if( $quantity > 10 ) {
            $quantity = 10;
        } elseif( $quantity < 1 ) {
            $quantity = 1;
        }

        $cart = Cart::instance('giftcards');
        $cart->destroy();
        $cart->add(
            ['id' => $package->id, 'name' => $package->name, 'qty' => $quantity, 'price' => $package->price,
                'options' => [
                    'img'           => $package->image
                ]
            ]
        );

        return Redirect::back();
	}
}
