<?php 

namespace App\Http\Controllers\Clientzone;

use Carbon\Carbon, Hash, Validator, Input;

class GreenshillingController extends ClientzoneController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		return view('greenshilling.index')->withTitle( trans('greenshilling.index.title') );
	}

	public function buyMiningSlots()
	{
		$wallet = $this->user->balances;

		return view('greenshilling.buy', ['wallet' => $wallet])->withTitle( trans('greenshilling.buyMiningSlots.title') );
	}

	public function postBuyMiningSlots()
	{
		$rules = [
			'slotsAmount' => 'required|numeric|min:1',
			'walletAddress' => 'required|min:6',
			'secondaryPassword' => 'required'
		];

		$validator = Validator::make(Input::all(), $rules);

		if ( $validator->fails() )
			return redirect()->back()->withInput()->withErrors( $validator );

		if( !Hash::check( Input::get('secondaryPassword'), $this->user->secondary_password ) ) {
            return trans('greenshilling.buyMiningSlots.messages.errorPassword');
        }

		$wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

		$units = Input::get('slotsAmount') * 100;

		if( $wallet->resale_units < $units )
	    	return redirect()->back()->with('error', trans('greenshilling.buyMiningSlots.messages.noUnits'))->withInput();

	    $wallet->resale_units = ($wallet->resale_units - $units);
	    $wallet->save();

	    $history = new \App\Models\GreenShillingHistory;
	    $history->user_id = $this->user->id;
	    $history->slots = Input::get('slotsAmount');
	    $history->units = $units;
	    $history->period_from = Carbon::now();
	    $history->period_to = Carbon::now()->addDays(2);  
	    $history->wallet = Input::get('walletAddress');
	    $history->save();

		return redirect()->back()->with('success', trans('greenshilling.buyMiningSlots.messages.success') );
	}
}
