<?php 

namespace App\Http\Controllers\Clientzone;

use App\Facades\NotificationManager;
use App\Models\WCILastPrices;
use Config, DB, Input, Validator, Session, Redirect, Mail, Queue, MailManager, Cart, Illuminate\Http\Request;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\ServerException;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Artisan;
use Ramsey\Uuid\Uuid;

class PackagesController extends ClientzoneController {

    public function __construct()
    {
        parent::__construct();
    }

    public function index()
    {
        $total_packages = \App\Models\UsersPackages::select(DB::raw('SUM(packages.price) AS amount'))->join('packages', 'packages.id', '=', 'users_packages.package_id')->where('user_id', $this->user->id)->where('users_packages.status', 1)->pluck('amount')->first();

        return view('packages.index', ['total_packages' => $total_packages])->withTitle( trans('packages.index.title') );
    }

    public function getPurchase($package_id = null)
    {
        $selectedPackage = Session::get('selectedPackage', null);

        if(!is_null($package_id)){
            $selectedPackage = $package_id;
        }

        $cart = Cart::instance('packages');

        if( !is_null( $selectedPackage ) ) {
            $package = \App\Models\Packages::where('id', $selectedPackage )->where('id', '!=', 2)->first();

            /* Check pending package */
            if( !is_null( $p = \App\Models\UserAddresses::where('user_id', $this->user->id)->orderBy('id', 'DESC')->first() ) ) {
                if( $p->hasExpired() ) {
                    $p->delete();
                }else{
                    return redirect()->route('packages.purchase.btc', $p->address);
                }
            }

            if(!$package->userCanPurchase()){
                return redirect()->route('packages');
            }

            if( $cart->content()->count() > 0 ){
                $cart->destroy();
            }

            $cart->add([
                'id' => $package->id,
                'name' => $package->name,
                'qty' => 1,
                'price' => $package->price
            ]);
        }

        $packages = \App\Models\Packages::where('status', 1)->orderBy('id', 'asc')->get();

        $total_packages = \App\Models\UsersPackages::select(DB::raw('SUM(packages.price) AS amount'))->join('packages', 'packages.id', '=', 'users_packages.package_id')->where('user_id', $this->user->id)->where('users_packages.status', 1)->pluck('amount')->first();

        $ewallets = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
        $_processors = \App\Models\Processors::where('id', 5)->orderBy('id', 'asc')->get();

        return view('packages.purchase', ['ewallets' => $ewallets, 'total_packages' => $total_packages, 'packages' => $packages, 'cart' => $cart, 'processors' => $_processors])->withTitle( trans('packages.purchase.title') );
    }

    public function postPurchase(){


        if( !is_null( $p = \App\Models\UserAddresses::where('user_id', $this->user->id)->orderBy('id', 'DESC')->first() ) ) {
            if( $p->hasExpired() ) {
                $p->delete();
            } else return redirect()->route('packages.purchase.peak', $p->address);
        }

        $data = Input::all();
        $rules = [
            //'type' => 'required|exists:packages,id',
            //'amount' => 'required|numeric'
            'processor' => 'required|exists:processors,id'
        ];

        $validator = Validator::make($data, $rules);

        if ($validator->fails())
            return Redirect::back()->withInput()->withErrors($validator);

        $cart = Cart::instance('packages');

        if( $cart->content()->count() == 0 )
            return redirect()->back()->withInput()->with('error', trans('packages.purchase.messages.errorPackage') );

        $item = $cart->content()->first();

        $package = \App\Models\Packages::where('id', $item->id)->first();



        if(!$package->userCanPurchase()){
            return redirect()->route('packages');
        }

        $processor = \App\Models\Processors::where('id', '!=', 4)->where('id', $data['processor'])->first();

        switch( $processor->id ) {

            case 5:
                $lastAddress = \App\Models\UserAddresses::where('user_id', $this->user->id)->orderBy('id', 'DESC')->take(1)->first();

                if($lastAddress){
                    $lastAddress->delete();
                }

                $cpc_rate = WCILastPrices::where('market', 'CPC/USD')->first();

                $address = new \App\Models\UserAddresses;
                $address->package_id = $item->id;
                $address->user_id = $this->user->id;
                $address->address = (string) Uuid::uuid4();
                $address->amount = $package->price / $cpc_rate->price;
                $address->fee = 0;
                $address->currency = "cpc";

                $address->save();

                if(  $address ) {
                    $cart->destroy();

                    return redirect()->route('packages.purchase.peak', $address->address);
                } else
                    return redirect()->back()->with('error', trans('packages.purchase.messages.error4'));

                break;

            default: 

            break;
        }
    }

    public function getPurchasePeak( $address = null ) {
        if( is_null( $address = \App\Models\UserAddresses::where('user_id', $this->user->id)->where('address', $address)->orderBy('id', 'DESC')->first() ) )
            return redirect()->route('packages')->with('error', trans('packages.purchaseUsd.messages.wrongAddress'));

        if( $address->hasExpired() ) {
            $address->delete();
            return redirect()->route('packages.purchase');
        }

        $processor = \App\Models\Processors::where('id', 5)->first();

        return view('packages.purchase-peak', ['address' => $address, 'processor' => $processor])->withTitle( trans('packages.purchaseUsd.title') );
    }

    public function getCancelPurchasePeak( Request $request ) {
        if( !is_null( $p = \App\Models\UserAddresses::where('user_id', $this->user->id)->orderBy('id', 'DESC')->first() ) ) {
            $p->delete();

            return redirect()->route('packages.purchase')->with('success', trans('packages.purchaseUsd.messages.canceled'));
        }

        return redirect()->back()->with('error', 'You don\'t have any pending purchases with Usd.');
    }

    protected function processPackagePurchase( $processor, $package, $userBtcAddress ) {
        $transactionDeposit = new \App\Models\Transactions;
        $transactionDeposit->processor_id = 1;
        $transactionDeposit->user_id = $this->user->id;
        $transactionDeposit->type = 1;
        $transactionDeposit->fee = $processor->fee_fund;
        $transactionDeposit->amount = $package->price;
        $transactionDeposit->translateOrNew('en')->note = 'Deposit via Bitcoin (BTC) - ' . $userBtcAddress->amount_btc;
        $transactionDeposit->translateOrNew('cn')->note = '通过比特币存款 - ' . $userBtcAddress->amount_btc;

        $transactionDeposit->status = 1;
        $transactionDeposit->save();

        //$userBtcAddress->delete();

        $new_package = new \App\Models\UsersPackages;
        $new_package->user_id = $this->user->id;
        $new_package->package_id = $package->id;
        $new_package->status = 1;
        $new_package->save();

        $new_transaction = new \App\Models\Transactions;
        $new_transaction->processor_id = 1;
        $new_transaction->user_id = $this->user->id;
        $new_transaction->type = 1;
        $new_transaction->amount = $package->price;
        $new_transaction->translateOrNew('en')->note =  'Package ID: #' . $new_package->id . ', Type: ' . $package->name;
        $new_transaction->translateOrNew('cn')->note =  '套餐ID: #' . $new_package->id . ', 类型: ' . $package->name;
        $new_transaction->status = 1;

        if( $new_transaction->save() ) {

            MailManager::send('packages.new-package', trans('packages.purchase.emailTitle'), [
                'id' => $new_package->id, 
                //'package_name' => $package->name,
                'package_name' => trans('packages.packageNames.' . $package->id, [], $this->user->getUserLanguage()),
                'amount' => $package->price
            ], $this->user);



            $this->dispatch( new \App\Jobs\SetPackage( $new_package->id ) );


            /*NotificationManager::create([
                'user_id' => $this->user->id,
                'type' => 'user-auction-package-purchase',
                'content_id' => $package->id
            ], $this->user);*/

        }
    }
}
