<?php 

namespace App\Http\Controllers\Clientzone;


use Redirect, DB, Validator, Input, Session, Mail, Config, MailManager, File;

class SupportController extends ClientzoneController
{
	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		$this->prepareSharedData( $currentPage = 'active' );
		$tickets = \App\Models\Support::active()->where('user_id', $this->user->id)->orderBy('id', 'desc')->get();
		return view('support.index', ['tickets' => $tickets])->with($this->shared_data)->withTitle(trans('support.title.Support Tickets').' - '.trans('support.title.Active'));
	}

	public function open()
	{
		$this->prepareSharedData( $currentPage = 'open' );
		$tickets = \App\Models\Support::open()->where('user_id', $this->user->id)->orderBy('id', 'desc')->get();
		return view('support.index', ['tickets' => $tickets])->with($this->shared_data)->withTitle(trans('support.title.Support Tickets').' - '.trans('support.title.Open'));
	}

	public function closed()
	{
		$this->prepareSharedData( $currentPage = 'closed' );
		$tickets = \App\Models\Support::closed()->where('user_id', $this->user->id)->orderBy('id', 'desc')->get();
		return view('support.index', ['tickets' => $tickets])->with($this->shared_data)->withTitle(trans('support.title.Support Tickets').' - '.trans('support.title.Closed'));
	}

	public function view( $id = null )
	{
		if( is_null( $ticket = \App\Models\Support::where('id', $id)->where('user_id', $this->user->id)->first() ) )
			return Redirect::route('support')->with('error', trans('messages.wrong_ticket_id'));
			//return Redirect::route('support')->with('error', trans('support.error.Wrong ticket ID'));

		switch ( $ticket->status ) {
			case 0:
				$status = 'open';
				break;

			case 1:
				$status = 'active';
				break;

			case 2:
				$status = 'closed';
				break;
		}

		$this->prepareSharedData( $status );

		return view('support.view', ['ticket' => $ticket])->with($this->shared_data)->withTitle(trans('support.title.Support Tickets').' - ' . $ticket->subject);
	}

	public function postReply( $id = null )
	{
		if( is_null( $ticket = \App\Models\Support::where('id', $id)->where('user_id', $this->user->id)->first() ) )
			return Redirect::back()->with('error', trans('messages.wrong_ticket_id'));
			//return Redirect::back()->with('error', trans('support.error.Wrong ticket ID'));

		//if( $ticket->replies->last()->user_id == $this->user->id )
		//	return Redirect::back()->with('error', 'You can not send reply to this ticket.');

		$rules = [
			'content' => 'required|min:5|max:1000'
		];


		$inputs = Input::all();
		$validator = Validator::make($inputs, $rules);

		if ($validator->fails())
			return Redirect::back()->withInput()->withErrors($validator);

		$reply = new \App\Models\SupportReplies;
		$reply->ticket_id = $id;
		$reply->user_id = $this->user->id;
		$reply->content = $inputs['content'];

		if( $reply->save() )
			return Redirect::back()->with('success', trans('messages.reply_success'));
			//return Redirect::back()->with('success', trans('support.success.Your reply to this support ticket has been saved'));

		return Redirect::back()->with('error', trans('messages.reply_error'));
		//return Redirect::back()->with('error', trans('support.error.There was an error while trying to save your reply'));
	}

	public function getNew()
	{
		$this->prepareSharedData( );
		$tickets_categories = \App\Models\SupportCategories::orderBy('id', 'asc')->get();
		return view('support.new', ['tickets_categories' => $tickets_categories])->with($this->shared_data)->withTitle('New ticket');
		//$tickets_categories = DB::table('support_tickets_categories')->orderBy('id', 'asc')->get();
		//return view('support.new', ['tickets_categories' => $tickets_categories])->with($this->shared_data)->withTitle(trans('support.title.New ticket'));
	}

	public function postNew()
	{
		$rules = [
			'category' => 'required|exists:support_tickets_categories,id',
			'subject'  		=> 'required|min:3|max:100',
			'priority' 		=> 'required|in:1,2,3',
			'content' 		=> 'required|min:10|max:1000',
			'attachment' 	=> 'mimes:jpeg,jpg,png,txt,pdf|max:10000'
		];
		$inputExcepts = ['category'];
		$inputs = Input::all();

		$validator = Validator::make($inputs, $rules);

		if ($validator->fails())
  		return Redirect::back()->withInput()->withErrors($validator);

   	if ( Input::hasFile('attachment') && Input::file('attachment')->isValid() ) {

        if(!\Illuminate\Support\Facades\File::exists(storage_path('uploads/support/'))) {
            \Illuminate\Support\Facades\File::makeDirectory(storage_path('uploads/support/'), 0775, true);
        }


   		$destinationPath = storage_path() . '/uploads/support/';
      $extension = Input::file('attachment')->getClientOriginalExtension();
      $fileName = str_random(25) . '.' . $extension;
      Input::file('attachment')->move( $destinationPath, $fileName );

      $inputs['attachment'] = $fileName;
		}

		$category = \App\Models\SupportCategories::where('id', $inputs['category'])->first();

    $ticket = new \App\Models\Support;
    $ticket->user_id = $this->user->id;
    $ticket->assigned_to = $category->assigned_to;
    $ticket->category_id = $inputs['category'];

    foreach( $rules as $key => $value ) {
    	if( !in_array($key, $inputExcepts) ) $ticket->$key = ( ( isset( $inputs[ $key ] ) ) ? $inputs[ $key ] : NULL );
    }

    if( $ticket->save() ) {

    	MailManager::client(['username' => config('app.support_mail_username'), 'password' => config('app.support_mail_password')])->from(['address' => config('app.support_mail_address'), 'name' => 'Support - Colibri Auction'])->send('support.new-ticket', 'Support Ticket Created', [
    		'first_name' => $this->user->first_name,
    		'last_name' => $this->user->last_name,
    		'id' => $ticket->id,
    		'subject' => $ticket->subject,
    		'category' => $ticket->category->name,
    		'priority' => $ticket->priorityName()
    	], $this->user);

    	return Redirect::route('support.open')->with('success', trans('messages.ticket_success'));
    	//return Redirect::route('support.open')->with('success', trans('support.succes.Ticket has been successfully created'));

    } else return Redirect::back()->with('error', trans('messages.ticket_error'));
    //} else return Redirect::back()->with('error', trans('support.error.There was an error while trying to save your ticket'));
	}

	public function getAttachment( $id = null )
	{
		if( is_null( $ticket = \App\Models\Support::where('id', $id)->whereNotNull('attachment')->where('user_id', $this->user->id)->first() ) )
			return Redirect::route('support')->with('error', trans('messages.wrong_ticket_id'));

		$path = storage_path() . '/uploads/support/' . $ticket->attachment;

		if( !File::exists( $path ) ) return response('Not found.', 404);

		return response()->file( $path );
	}

	private function prepareSharedData( $currentPage = '' )
	{
		$this->active_count = \App\Models\Support::active()->where('user_id', $this->user->id)->count();
		$this->open_count = \App\Models\Support::open()->where('user_id', $this->user->id)->count();
		$this->closed_count = \App\Models\Support::closed()->where('user_id', $this->user->id)->count();

		$this->ticket_categories = [];
		$ticket_categories = \App\Models\SupportCategories::orderBy('id', 'asc')->get();

		foreach( $ticket_categories as $tc ) {
			$this->ticket_categories[ $tc->id ] = $tc->name;
		}

		$this->shared_data = [
			'active_count' => $this->active_count,
			'open_count' => $this->open_count,
			'closed_count' => $this->closed_count,

			'ticket_categories' => $this->ticket_categories,
			'current' => $currentPage
		];
	}

}
