<?php 

namespace App\Http\Controllers\Clientzone;

use Config, DB, Validator, Input, Redirect, Session;

class TreeController extends ClientzoneController {

	public function __construct()
	{
		parent::__construct();
	}



    public function indexNew( $id = null )
    {
        $root = \App\Models\TreeUplineNested::where('id', $this->user->id)->first();

        if( !is_null( $id ) ) {
            if( is_null( $root = \App\Models\TreeUplineNested::where('id', $id)->first() ) )
                $root = \App\Models\TreeUplineNested::where('id', $this->user->id)->first();
        }

        $children = $root->children()->with('user.balances')->get();

        $leftChild = $children->where('position', 1)->first();
        $rightChild = $children->where('position', 2)->first();
        $leftChildCount = 0;
        $rightChildCount = 0;

        if( !is_null( $leftChild ) )
            $leftChildCount = 1 + $leftChild->descendants->count();

        if( !is_null( $rightChild ) )
            $rightChildCount = 1 + $rightChild->descendants->count();

        $buildStructure = function() use ( $root ) {
            return $this->makeNew( [$root], true );
        };

        $bottomRightNode = $this->GetBottomRightNode($root);
        $bottomLeftNode = $this->GetBottomLeftNode($root);
        // if( in_array($id, ['bottom-left', 'bottom-right']) ) {
        // 	return view('tree.index-new', [
        // 			'me' => $this->user,
        // 			'buildStructure' => $buildStructure,
        // 			'leftChildCount' => $leftChildCount,
        // 			'rightChildCount' => $rightChildCount,
        // 			// 'bottomRightNodeId' => $bottomRightNode == false ? false : $bottomRightNode->id
        // 	])->withTitle( trans('tree.index.title') );
        // }

        return view('tree.index-new', [
            'me' => $this->user,
            'buildStructure' => $buildStructure,
            'leftChildCount' => $leftChildCount,
            'rightChildCount' => $rightChildCount,
            'bottomRightNodeId' => $bottomRightNode == false ? false : $bottomRightNode->id,
            'bottomLeftNodeId' => $bottomLeftNode == false ? false : $bottomLeftNode->id,
            'oneUp' => $root->parent
        ])->withTitle( trans('tree.index.title') );
    }

    public function placement() {
        return view('tree.placement')->withTitle( trans('tree.placement.title') );
    }

    public function postPlacement() {
        $rules = [
            'BinaryLeg' => 'required|in:1,2'
        ];

        $validator = Validator::make(Input::all(), $rules);

        if ($validator->fails())
            return Redirect::back()->withInput()->withErrors($validator);

        $this->user->sponsor->networkPlacement = Input::get('BinaryLeg', 1);
        $this->user->sponsor->save();

        $placementDisplay = (Input::get('BinaryLeg', 1) == 1) ? 'Left' : 'Right';

        return redirect()->back()->with('success', trans('tree.placement.messages.success') . $placementDisplay);
    }

    public function genealogyTree()
    {

        $root = \App\Models\TreeGenealogyNested::where('id', $this->user->id)->first();

        if($root) {

            $level = 1;
            $parent = $root;

            if ($parent->hasChildren()) {
                $children = $parent->children()->with('user.balances')->get()->toFlatTree();
                $root->children = $children;
            } else $root->children = [];

            $buildGenealogyStructure = function () use ($root) {

                return $this->makeGenealogyTree([$root], true);
            };

            return view('tree.genealogyTree', ['buildGenealogyStructure' => $buildGenealogyStructure])->withTitle( trans('tree.genealogyTree.title') );
        }
        else {

            return redirect()->back()->with('error', trans('tree.genealogyTree.cannot_load_tree'));
        }
    }

    public function makeGenealogyTree( $array, $first = true, $lvl = 1, $parentId = null ) {
        $content = '<ul>';
        $isEmpty = false;
        $maxLevel = 1;

        if( !$first )
            $isEmpty = (is_array($array) && !empty($array)) ? false : true;

        if( $isEmpty )
            $content = '';
        else {
            foreach ( $array as $key => $value ) {

                $content .= '<li>
                            <span class="m-badge m-badge--success m-badge--wide m-badge--rounded m--font-boldest">' . $value->user->username . '</span> <span class="m-badge m-badge--metal m-badge--wide m-badge--rounded">' . trans('tree.genealogyTree.leftLeg') . ' ' . $value->user->balances->leg_left . '</span> <span class="m-badge m-badge--info m-badge--wide m-badge--rounded">' . trans('tree.genealogyTree.rightLeg') . ' ' . $value->user->balances->leg_right . '</span>';

                if( $maxLevel > $lvl && !empty( $value->children ) )
                    $content .= $this->makeGenealogyTree( $value->children, false, $lvl++, $value->id);

                $content .= '</li>';
            }

            $content .= "</ul>";
        }

        return $content;
    }

    public function makeNew($root, $first = true, $lvl = 1, $parentId = null ) {
        $content = '<ul class="networkTreeLvl' . $lvl .' list-inline">';
        $hasChildren = false;

        if( !$first ) {
            if( is_array( $root ) )
                $hasChildren = count( $root );
            else {
                $hasChildren = $root->count() > 0;
                if( $hasChildren )
                    $root = $root->sortBy('position');
            }
        }

        if( !$first && !$hasChildren )
            $content .= $this->addPlusSpace( $nodes, 0 );
        else {
            if( !is_array( $root ) )
                $childrenCount = $root->count();
            elseif( $first )
                $childrenCount = $root[0]->children->count();

            foreach ( $root as $key => $value ) {
                if( is_string($value) && $value == 'more' ) {
                    $countMore = array_count_values( $root )['more'];

                    if( $countMore == 1 && $key == 2 )
                        $content .= $this->addPlusSpace( $parentId, 1 );

                    $content .= '<li class="">
                                    <a href="#" class="pin pinMore" data-leg="' . $key . '" data-toggle="modal" title="More">
                                    </a>
                                </li>';

                    if( $countMore == 1 && $key == 1 )
                        $content .= $this->addPlusSpace( $parentId, 2 );

                } else {
                    $orientation = ( $value->position == 1 ) ? 'left' : 'right';

                    if( !$first && $childrenCount == 1 && $orientation == 'right' )
                        $content .= $this->addPlusSpace( $value->parent_id, 1 );

                    $content .= '<li class="' . $orientation . '">
                        <a class="f16 pinTooltip pin ' .($value->user->isSuspended() ? 'pinSuspended' :'') .' '.  (( !$value->user->newUser ) ? (( !$value->user->hasRole('user-migrated') ) ? 'pin00' : 'pinMigrated') : 'pinInactive' ) . '" href="' . route('tree-new', $value->id) . '" data-toggle="m-popover" data-trigger="hover" data-html="true" data-placement="bottom" data-container="body" data-content=\'<table class="table table-condensed table-striped table-bordered small list-group-item-text"><tr>
                            <td>'. trans('tree.index.leftLegPoints') . '</td>
                            <td>'. $value->user->balances->leg_left . '</td>
                        </tr><tr>
                            <td>'. trans('tree.index.rightLegPoints') . '</td>
                            <td>'. $value->user->balances->leg_right . '</td>
                        </tr><tr>
                            <td>'. trans('tree.index.sponsorUsername') . '</td>
                            <td>'. ((is_null($value->user->genealogy->parent )) ? 'None' : $value->user->genealogy->parent->user->username) . '</td>
                        </tr>
                        <tr>
                            <td>'. trans('tree.index.uplineUsername') . '</td>
                            <td>'. ((is_null( $value->user->upline->parent )) ? 'None' : $value->user->upline->parent->user->username) . '</td>
                        </tr><tr>
                            <td>'. trans('tree.index.memberStatus') . '</td>
                            <td>'. (($value->user->newUser == 0 ) ? trans('tree.index.active') : trans('tree.index.inactive')) . '</td>
                        </tr></table>\' data-original-title=\'<span class="btn btn-xs btn-block"><span class="glyphicon glyphicon-user"></span> ' . $value->user->username . '<br>(#'. $value->user->id .')</span>\' style="opacity: 1;">
                        <span class="flag us pull-right"></span>
                        </a>';

                    $countChildren = $value->children->count();

                    if( $lvl == 4 ) {
                        if( $countChildren == 0 ) {
                            $next = $value->user_id;
                        } else {
                            if( $countChildren == 2 ) {
                                $value->children = [1 => 'more', 2 => 'more'];
                            } else {
                                $cc = $value->children->first();
                                $pos = $cc->position;

                                if( $pos == 1 ) {
                                    $value->children = [1 => 'more'];
                                }
                                else {
                                    $value->children = [2 => 'more'];
                                }
                            }

                            $next = $value->children;
                        }
                    }

                    if( $countChildren > 0 && $value->has('children') && $lvl <= 4 )
                        $content .= $this->makeNew( $value->children, false, $lvl + 1, $value->id);

                    $content .= '</li>';

                    if( !$first && $childrenCount == 1 && $orientation == 'left' )
                        $content .= $this->addPlusSpace( $value->parent_id, 2 );
                }
            }
        }

        $content .= "</ul>";

        return $content;
    }


    private function addPlusSpace( $uplineUserId, $leg ) {
        $content = '<li class="">
                    <a href="#" class="pin pinEmpty">
                    </a>
                </li>';
        $result = '';

        if( $leg == 0 ) {
            for($i = 1; $i < 3; $i++) {
                $result .= str_replace([':uid:', ':leg:'], [$uplineUserId, $i], $content);
            }
        } else
            $result = str_replace([':uid:', ':leg:'], [$uplineUserId, $leg], $content);

        return $result;
    }

    public function getDownline()
    {
        $total_commission = \App\Models\Transactions::where('type', 4)->where('user_id', $this->user->id)->where('status', 1)->sum('amount');

        return view('tree.downline', ['me' => $this->user, 'total_commission' => $total_commission])->withTitle( trans('tree.downline.title') );
    }

    // PARAMS: $root: TreeUplineNested
    private function GetBottomRightNode($root) {
        if( ! is_null( $rightChild = $this->GetRightChild($root) ) ) {
            $rightChildren = $rightChild->descendants()->where('position', 2)->reversed()->first();

            return $rightChildren;
        }

        return false;
    }

    // PARAMS: $root: TreeUplineNested
    private function GetRightChild($root) {
        return $root->children->where('position', 2)->first();
    }

    // PARAMS: $root: TreeUplineNested
    private function GetBottomLeftNode($root) {
        if( ! is_null( $leftChild = $this->GetLeftChild($root) ) ) {
            $leftChildren = $leftChild->descendants()->where('position', 1)->reversed()->first();

            return $leftChildren;
        }

        return false;
    }

    // PARAMS: $root: TreeUplineNested
    private function GetLeftChild($root) {
        return $root->children->where('position', 1)->first();
    }


    public function commission()
    {
        $total_commission = \App\Models\Transactions::where('type', 4)->where('user_id', $this->user->id)->where('status', 1)->sum('amount');

        return view('tree.commission', ['me' => $this->user, 'total_commission' => $total_commission])->withTitle( trans('tree.commission.title') );
    }

	/*public function index( $id = null )
	{
		$root = $this->user->sponsor;
		$downline_list = \App\Models\Sponsor::select(DB::raw('users.id, users.first_name, users.last_name, users.username, users.email, users.sponsor_code'))
        ->leftJoin('users', 'users.id', '=', 'sponsors.user_id')
        ->whereBetween('upline_left', [ $root->upline_left, $root->upline_right ])
        ->where('group_id', $root->group_id)
        //->where('level', '<=', $root->level + Config::get('mlm.maxDisplayLevel') )
        ->orderBy('upline_left', 'ASC')
        ->orderBy('level', 'ASC')
        ->orderBy('position', 'ASC')
        ->get();

        if( !is_null( $id ) ) {

            $my_level = $this->user->sponsor->level;
            $my_group = $this->user->sponsor->group_id;
            $search = null;

            if( in_array($id, ['bottom-left', 'bottom-right']) ) {
                $first_leg = \App\Models\Sponsor::select(DB::raw('`position`, `upline_left`, `upline_right`, `group_id`, `user_id`'))
                ->where('upline_user_id', $root->user_id)
                ->where('level', ($root->level + 1))
                ->where('group_id', $root->group_id);

                switch( $id ) {
                    case 'bottom-left':
                        $first_leg = $first_leg->where('position', 1)->first();
                        if( is_null( $first_leg ) || (($first_leg->upline_right - $first_leg->upline_left + 1) / 2) < 7 ) {
                            break;
                        }

                        $search = \App\Models\Sponsor::whereRaw('upline_right = upline_left + 1 AND (children = 0 OR children = 2) AND upline_left > ' . $root->upline_left . ' AND upline_right < ' . $root->upline_right)
                            ->where('position', 1)
                            ->where('level', '>', $root->level + Config::get('mlm.maxDisplayLevel'))
                            ->where('group_id', $root->group_id)
                            ->orderBy('upline_left', 'ASC')
                            ->first();
                        dd($search);
                    break;
                    case 'bottom-right':
                        $first_leg = $first_leg->where('position', 1)->first();
                        if( is_null( $first_leg ) || (($first_leg->upline_right - $first_leg->upline_left + 1) / 2) < 7 ) {
                            break;
                        }

                        $search = \App\Models\Sponsor::whereRaw('upline_right = upline_left + 1 AND (children = 0 OR children = 1) AND upline_left > ' . $root->upline_left . ' AND upline_right < ' . $root->upline_right)
                            ->where('position', 2)
                            ->where('level', '>', $root->level + Config::get('mlm.maxDisplayLevel'))
                            ->where('group_id', $root->group_id)
                            ->orderBy('upline_right', 'DESC')
                            ->first();
                            dd($search);
                    break;
                }
            } else {
                $search = \App\Models\Sponsor::where('user_id', '=', $id)->first();
            }

            if( 
                !is_null( $search ) && 
                $search->group_id == $my_group && $search->level >= $my_level )
                $root = $search;
            else
                $root = $this->user->sponsor;

        } else {
            $root = \App\Models\Sponsor::where('user_id', '=', $this->user->id)->first();
        }

        $result = \App\Models\Sponsor::select(DB::raw('sponsors.*, CONCAT_WS(" ", users.first_name, users.last_name) AS full_name, users.username, users.created_at, users.email, users_profiles.avatar, users_balances.leg_left, users_balances.leg_right, su.username as sponsor_username, uu.username as upline_username, positions.name as memberPosition'))
        ->leftJoin('users', 'users.id', '=', 'sponsors.user_id')
        ->leftJoin('users as su', 'su.id', '=', 'sponsors.sponsor_id')
        ->leftJoin('users as uu', 'uu.id', '=', 'sponsors.upline_user_id')
        ->leftJoin('users_profiles', 'users_profiles.id', '=', 'sponsors.user_id')
        ->leftJoin('users_balances', 'users_balances.users_id', '=', 'sponsors.user_id')
        ->leftJoin('users_positions', 'users_positions.user_id', '=', 'users.id')
        ->leftJoin('positions', 'users_positions.position_id', '=', 'positions.id')
        //->leftJoin('users_plans', 'users_plans.user_id', '=', 'users.id')
        //->leftJoin(DB::raw('( SELECT SUM(invested_amount) AS invested_amount, MAX(token_id) AS token_id, processor_id ) FROM users_plans '))
        ->whereBetween('upline_left', [ $root->upline_left, $root->upline_right ])
        ->where('group_id', $root->group_id)
        ->where('level', '<=', $root->level + Config::get('mlm.maxDisplayLevel'))
        ->orderBy('upline_left', 'ASC')
        ->orderBy('level', 'ASC')
        ->orderBy('position', 'ASC')
        ->get();

        $lookup = [];

        foreach( $result as $r ) {
            $lookup[ $r->user_id ] = $r;
        }

        /*$first_legs = \App\Models\Sponsor::select(DB::raw('`position`, `upline_left`, `upline_right`'))
        ->leftJoin('users_plans', 'users_plans.user_id', '=', 'sponsors.user_id')
        ->where('upline_user_id', $root->user_id)
        ->where('level', ($root->level + 1))
        ->where('group_id', $root->group_id)
        ->where('users_plans.status', 1)
        ->groupBy('sponsors.user_id')
        ->get();
        $children_count = $first_legs->count();*/

	/*
        $first_legs = \App\Models\Sponsor::select(DB::raw('`position`, `upline_left`, `upline_right`, `group_id`, `user_id`'))
        ->where('upline_user_id', $root->user_id)
        ->where('level', ($root->level + 1))
        ->where('level', '<=', $root->level + Config::get('mlm.maxDisplayLevel'))
        ->where('group_id', $root->group_id)
        ->orderBy('position', 'asc')
        ->get();

        $children_count = $first_legs->count();

        $root_balance = \App\Models\UsersBalances::select(DB::raw('leg_left as sales_left, `leg_right` as sales_right'))
        ->where('users_id', $root->user_id)
        ->first();

        $total_left_sales = $root_balance->sales_left;
        $total_right_sales = $root_balance->sales_right;
        $total_left_members = 0;
        $total_right_members = 0;

        if( $children_count ) {
            foreach( $first_legs as $fl ) {

                /*$leg_members = \App\Models\Sponsor::select(DB::raw('`position`, `upline_left`, `upline_right`'))
                ->whereBetween('upline_left', [ $fl->upline_left, $fl->upline_right ])
                ->where('level', '<=', Config::get('mlm.displayLevels') - 1)
                ->where('group_id', $fl->group_id)
                ->get();
                $leg_members_count = $leg_members->count();*/
                //DB::enableQueryLog();
/*
                $leg_members = \App\Models\Sponsor::select(DB::raw('COUNT(`id`) AS `counter`, SUM(leg_left) AS sales_left, SUM(leg_right) AS sales_right'))
                ->join('users_balances', 'users_balances.users_id', '=', 'sponsors.user_id')
                ->whereBetween('upline_left', [ $fl->upline_left, $fl->upline_right ])
                //->where('level', '<=', $root->level + Config::get('mlm.maxDisplayLevel'))
                ->where('sponsors.user_id', '!=', $fl->user_id)
                ->where('sponsors.group_id', $fl->group_id)
                ->first();

                if( $fl->position == 1 ) {
                    $total_left_members = 1;
                    if( !is_null($leg_members) ) {
                        $total_left_members = $total_left_members + $leg_members->counter;
                        $total_left_sales = $total_left_sales + $leg_members->sales_left;
                    }
                } else {
                    $total_right_members = 1;
                    if( !is_null($leg_members) ) {
                        $total_right_members = $total_right_members + $leg_members->counter;
                        $total_right_sales = $total_right_sales + $leg_members->sales_right;
                    }
                }
            }
        }

        $buildStructure = function() use ($lookup, $root) {
            $results = $this->buildTree($lookup, $root->upline_user_id);

            return $this->make($results, true);
        };

		return view('tree.index', ['me' => $this->user, 'downline_list' => $downline_list, 
            'buildStructure' => $buildStructure,
            'total_left_sales' => $total_left_sales,
            'total_right_sales' => $total_right_sales,
            'total_left_members' => $total_left_members,
            'total_right_members' => $total_right_members
        ])->withTitle('My Structure');
	}

    public function placement() {
        return view('tree.placement')->withTitle( trans('tree.placement.title') );
    }

    public function postPlacement() {
        $rules = [
            'BinaryLeg' => 'required|in:1,2'
        ];

        $validator = Validator::make(Input::all(), $rules);

        if ($validator->fails())
            return Redirect::back()->withInput()->withErrors($validator);

        $this->user->sponsor->networkPlacement = Input::get('BinaryLeg', 1);
        $this->user->sponsor->save(); 

        $placementDisplay = (Input::get('BinaryLeg', 1) == 1) ? 'Left' : 'Right';

        return redirect()->back()->with('success', trans('tree.placement.messages.success') . $placementDisplay);
    }

    public function buildTree(array $elements, $parentId = 0) {
        $result = [];

        foreach ($elements as $element) {
            //$element->_children = [];

            if ($element->upline_user_id == $parentId) {
                $children = $this->buildTree($elements, $element->user_id);

                if ( count( $children ) ) {
                    $element->_children = $children;
                } else $element->_children = [];

                $result[] = $element;
            }
        }

        return $result;
	}

    public function make($array, $first = true, $lvl = 1, $parentId = null ) {
        $content = '<ul class="networkTreeLvl' . $lvl .' list-inline">';
        $isEmpty = false;

        if( !$first )
            $isEmpty = (is_array($array) && !empty($array)) ? false : true;


        if( $isEmpty )
            $content .= $this->addPlusSpace( $array, 0 );
        else {
            $countArray = count( $array );

            foreach ( $array as $key => $value ) {
                if( is_string($value) && $value == 'more' ) {
                    $countMore = array_count_values( $array )['more'];

                    if( $countMore == 1 && $key == 2 )
                        $content .= $this->addPlusSpace( $parentId, 1 );

                    $content .= '<li class="">
                            <a href="#" class="pin pinMore" data-leg="' . $key . '" data-toggle="modal" title="More">
                            </a>
                        </li>';

                    if( $countMore == 1 && $key == 1 )
                        $content .= $this->addPlusSpace( $parentId, 2 );

                } else {

                    $orientation = ( $value->position == 1 ) ? 'left' : 'right';

                    if( !$first && $countArray == 1 && $orientation == 'right' )
                        $content .= $this->addPlusSpace( $value->upline_user_id, 1 );

                    $content .= '<li class="' . $orientation . '">
                        <a class="f16 pinTooltip pin pin00" href="' . route('tree', $value->user_id) . '" data-toggle="popover" data-trigger="hover" data-html="true" data-placement="bottom" data-container="body" data-content=\'<table class="table table-condensed table-striped table-bordered small list-group-item-text"><tr>
                            <td>Left Leg Points</td>
                            <td>'. $value->leg_left . '</td>
                        </tr><tr>
                            <td>Right Leg Points</td>
                            <td>'. $value->leg_right . '</td>
                        </tr><tr>
                            <td>Pin Position</td>
                            <td>' . ( (is_null($value->memberPosition)) ? 'Member' : $value->memberPosition) .'</td>
                        </tr><tr>
                            <td>Sponsor Username</td>
                            <td>'. ((is_null($value->sponsor_username)) ? 'None' : $value->sponsor_username) . '</td>
                        </tr>
                        <tr>
                            <td>Upline Username</td>
                            <td>'. ((is_null($value->upline_username)) ? 'None' : $value->upline_username) . '</td>
                        </tr><tr>
                            <td>Member Status</td>
                            <td>Active</td>
                        </tr></table>\' data-original-title=\'<span class="btn btn-xs btn-block"><span class="glyphicon glyphicon-user"></span> ' . $value->username . '</span>\' style="opacity: 1;">
                        <span class="flag us pull-right"></span>
                        </a>';

                    if( $value->children == 0 ) {
                        $next = $value->user_id;
                    } else {
                        $cc = count( $value->_children );

                        if( $value->children == 3 && $cc == 0 ) {
                            $value->_children = [1 => 'more', 2 => 'more'];
                        } elseif( 
                            ($value->children == 1 || $value->children == 2) 
                            &&
                            $cc == 0
                        ) {
                            if( $value->children == 1 ) {
                                $value->_children = [1 => 'more'];
                            }
                            else {
                                $value->_children = [2 => 'more'];
                            }
                        }

                        $next = $value->_children;
                    }

                    $content .= $this->make( $next, false, $lvl++, $value->user_id);

                    $content .= '</li>';

                    if( !$first && $countArray == 1 && $orientation == 'left' )
                        $content .= $this->addPlusSpace( $value->upline_user_id, 2 );
                }

            }
        }

        $content .= "</ul>";

        return $content;

    }

    private function addPlusSpace( $uplineUserId, $leg ) {
        $content = '<li class="">
                    <a href="#" class="pin pinAdd" data-uid=":uid:" data-leg=":leg:" data-toggle="modal" title="ADD">
                    </a>
                </li>';
        $result = '';

        if( $leg == 0 ) {
            for($i = 1; $i < 3; $i++) {
                $result .= str_replace([':uid:', ':leg:'], [$uplineUserId, $i], $content);
            }
        } else
            $result = str_replace([':uid:', ':leg:'], [$uplineUserId, $leg], $content);

        return $result; 
    }*/


}
