<?php 

namespace App\Http\Controllers\Clientzone;

use Redirect, Auth, Input, Validator, Session, Hash, Storage, Config;
use Illuminate\Http\Request;

class UserController extends ClientzoneController {

	private $rules = [
        'personal'  => [
            'first_name'   	=> 'required|alpha|min:2|max:50',
            'middle_name'   => 'max:50',
            'last_name'    	=> 'required|alpha|min:2|max:50',
            'birthdate-y'   => 'required|date_format:Y',
            'birthdate-m'   => 'required|date_format:m',
            'birthdate-d'   => 'required|date_format:d',
            'country'       => 'required|exists:countries,id',
            'zip'           => 'required|max:25',
            'city'          => 'required|max:100',
            'state'         => 'required|max:100',
            'address'       => 'required|max:200',
            'phone'         => 'required|between:6,20'
        ],
        'password'  => [
            'old_password'          		=> 'required',
            'new_password'          		=> 'required|between:6,32',
            'new_password_confirmation'  	=> 'required|same:new_password',
           // 'secondary_password'    		=> 'required'
        ],
        'secpassword'  => [
            'old_primary_password'                      => 'required',
            'old_secondary_password'                    => 'required',
            'new_secondary_password'                    => 'required|between:6,32',
            'new_secondary_password_confirmation'       => 'required|same:new_secondary_password',
        ],
        'privacy'    => []
    ];

	public function __construct()
	{
		parent::__construct();
	}

    public function downloadQRCode() {
        $file = \Illuminate\Support\Facades\Storage::disk('public')->get(
            'usersdata/qrcodes/' . $this->user->id . '.png'
        );

        $response = response( $file, 200 );
        $response->header('Content-Type', 'image/png');
        $response->header('Content-Disposition', 'attachment; filename="' . $this->user->id . '.png"');

        return $response;
    }

	public function getProfile()
	{
		return view('user.profile')->withTitle(trans('user.title.Profile'));
	}

    public function getDocuments()
    {
        return view('user.documents', ['user' => $this->user])->withTitle(trans('user.title.Documents'));
    }

    public function getLocation()
    {
        return view('user.location', ['user' => $this->user])->withTitle(trans('user.title.Address'));
    }

	public function getSettings()
	{
        $countries = \App\Models\Country::orderBy('id', 'asc')->get();
		return view('user.settings', ['countries' => $countries])->withTitle(trans('user.title.Settings'));
	}

	public function postSettings( Request $request )
	{
		$submit     = Input::get('save');
        $section    = (isset($this->rules[$submit])) ? $this->rules[$submit] : null;
        $data       = Input::all();
        $user       = Auth::user();

        if( is_null( $section ) ) {
            Session::flash('error', trans('messages.save_account_error'));
            //Session::flash('error', trans('user.error.Could not save account data'));
            return Redirect::back()->with('tab', $submit);
        }

        if( $submit == 'personal' ) {
            $mandator_fields = Config::get('users.signupRequirements');

            if( $mandator_fields && !empty( $mandator_fields ) ) {
                foreach( $mandator_fields as $mf ) {
                    if( in_array($mf, $section) )
                        $section[ $mf ] = $section[ $mf ] . '|required';
                }
            }

            //$section['username'] = 'required|alpha_num|min:3|max:20|unique:users,username,' . $user->username . ',username';
        }

        $validator = Validator::make($data, $section);

        if( $validator->fails() ) {
            Session::flash('error', $validator->messages());
            return Redirect::back()->withErrors($validator)->with('tab', $submit)->withInput();
        }

        foreach( $section as $r ) {
            $data[ $r ] = Input::get( $r, NULL );
        }

        /* Sms verification */
        /*$smsFields = Config::get('users.smsRequiredFields');
        $personalDataTypes = [
            'country'       => 'profile',
            'zip'           => 'profile',
            'city'          => 'profile',
            'state'         => 'profile',
            'address'       => 'profile',
            'phone'         => 'profile',
            'birthdate'     => 'profile'
        ];

        /*if( !is_null( $smsFields ) && !empty( $smsFields ) ) {
            $dataToUpdate = [];

            foreach( $smsFields as $sf ) {
                if( isset( $data[ $sf ] ) ){
                    if( isset( $personalDataTypes[ $sf ] ) ) {
                        $pdt = $personalDataTypes[ $sf ];
                        $oldValue = $user->$pdt->$sf;
                    } else
                        $oldValue = $user->$sf;

                    if( $data[ $sf ] != $oldValue ) {
                        $dataToUpdate[ $sf ] = $data[ $sf ];
                    }
                }
            }

            if( !empty( $dataToUpdate ) ) {

                if( $this->user->sendToken( $dataToUpdate, $request->path() ) ) {
                    return Redirect::back();
                } else return Redirect::back()->with('error', $result);
            }
        }*/


        $method = 'save' . ucfirst($submit);
        $result = $this->$method( $user, $data );

        Session::flash('tab', $submit);

        if( !is_string($result) )
        	return Redirect::back()->with('success', trans('messages.save_account_success'))->with('tab', $submit);
        	//return Redirect::back()->with('success', trans('user.success.Your account data has been successfully saved'))->with('tab', $submit);
        else
        	return Redirect::back()->with('error', $result);
	}

	private function savePersonal( $user, $data )
    {
        $user->first_name = $data['first_name'];
        $user->middle_name = $data['middle_name'];
        $user->last_name = $data['last_name'];
        $user->profile->birth_date = $data['birthdate-y'] . '-' . $data['birthdate-m'] . '-' . $data['birthdate-d'];
        $user->profile->country_id = $data['country'];
        $user->profile->zip = $data['zip'];
        $user->profile->city = $data['city'];
        $user->profile->state = $data['state'];
        $user->profile->address = $data['address'];
        $user->profile->phone = $data['phone'];

        return ($user->save() && $user->profile->save());
    }

    private function savePassword( $user, $data )
    {
    	//if( Hash::check( $user->secondary_password, $data['secondary_password'] ) ) {
		    if( Hash::check( $data['old_password'], $user->password ) ) {

		    	$user->password = Hash::make($data['new_password']);

		    	if( $user->save() ) return true;
		    } else  {
		    	return trans('messages.password_match_error');
		    	//return trans('user.error.Your current password does not match');
	    	}
		//}

    	return trans('messages.save_account_success');
    	//return  trans('user.error.Could not save your account data');
    }

    private function saveSecpassword( $user, $data )
    {
        //if( Hash::check( $user->secondary_password, $data['secondary_password'] ) ) {
            if( !Hash::check( $data['old_primary_password'], $user->password ) ) {
                return trans('messages.password_match_error');
                //return  trans('user.error.Your current primary password does not match');
            }

            if( !Hash::check( $data['old_secondary_password'], $user->secondary_password ) ) {
                return trans('messages.password_match_error');
                //return  trans('user.error.Your current secondary password does not match');
            }

            $user->secondary_password = Hash::make($data['new_secondary_password']);

            if( $user->save() ) return true;
        //}
    }
}
