<?php 

namespace App\Http\Controllers\Clientzone;

use Config, DB, Input, Validator, Session, Redirect, Mail, MailManager, NotificationManager;
use Illuminate\Support\Facades\Crypt;

use Coinbase\Wallet\Client;
use Coinbase\Wallet\Configuration;
use Coinbase\Wallet\Resource\Checkout;
use Coinbase\Wallet\Value\Money;

class WalletController extends ClientzoneController {

	public function __construct()
	{
		parent::__construct();
	}

	public function index()
	{
		$_processors = \App\Models\Processors::where('id', '!=', 4)->where('fee_fund_status', 1)->orderBy('id', 'asc')->get();
		$processors = [];
		$pending_wire = \App\Models\Transactions::where('user_id', $this->user->id)->where('status', 0)->where('type', 1)->where('processor_id', 1)->orderBy('id', 'desc')->get();
		$pending_bitcoin = \App\Models\Transactions::where('user_id', $this->user->id)->where('status', 0)->where('type', 1)->where('processor_id', 5)->orderBy('id', 'desc')->get();
		$e_wallet_balance = \App\Models\UsersBalances::select(DB::raw('SUM(COALESCE(`wallet`, 0.00)) as sum'))->where('users_id', $this->user->id)->first();

		foreach( $_processors as $w ) {
			$processors[ $w->id ] = $w;
		}

		return view('wallet.index', ['processors' => $processors, 'e_wallet_balance' => $e_wallet_balance, 'pending_wire' => $pending_wire, 'pending_bitcoin' => $pending_bitcoin])->withTitle( trans('wallet.index.title') );
	}

	public function postIndex()
	{
		$data = Input::all();

		$rules = [
			'processor' => 'required|exists:processors,id',
			'amount' => 'required_unless:processor,6',
			'gc-hash' => 'required_if:processor,6|alpha_num|min:10'
		];

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      		return Redirect::back()->withInput()->withErrors($validator);

	    $user_wallet = \App\Models\UsersProcessors::where('processor_id', $data['processor'])->where('data', '!=', NULL)->where('user_id', $this->user->id)->where('status', 1)->first();

	    if( is_null( $user_wallet ) && $data['processor'] !== 6 ) {
	    	$user_wallet = new \App\Models\UsersProcessors;
	    	$user_wallet->processor_id = $data['processor'];
	    	$user_wallet->user_id = $this->user->id;
	    	$user_wallet->data = NULL;
	    	$user_wallet->save();
	    }

	    $processor = \App\Models\Processors::where('id', '!=', 4)->where('id', $data['processor'])->first();

	  switch ($data['processor']) {
	  	case 1:

	  		$new_transaction = new \App\Models\Transactions;
		    $new_transaction->processor_id = 1;
		    $new_transaction->user_id = $this->user->id;
		    $new_transaction->type = 1;
		    $new_transaction->fee = $processor->feeAmount('fund', $data['amount']);
		    $new_transaction->amount = $processor->netAmount('fund', $data['amount']);
		    $new_transaction->note = 'Deposit of ' . $processor->netAmount('fund', $data['amount']) . ' (Bank Wire)';
		    $new_transaction->status = 0;
		    $new_transaction->save();

		    $user = $this->user;

		    MailManager::send('transactions.pending-fund', 'New Deposit', ['id' => $new_transaction->id, 'processor' => $processor->name, 'fee' => $new_transaction->fee, 'amount' => $new_transaction->amount, 'note' => $new_transaction->note ], $user);

	  		return Redirect::route('wallet.deposit.wire')->with('memo', $new_transaction->id);

	  	break;
			case 5:
				/*$processorData = json_decode( $processor->data, true );
				$configuration = Configuration::apiKey($processorData['api_key'], $processorData['api_secret']);//'8LrDryKSsiItJKTG', 'epuVrF2uoahB0SYoyx4tat1oauKg25jL');
				//$configuration->setApiUrl( Configuration::SANDBOX_API_URL );
				$client = Client::create( $configuration );

				$params = array(
					'name'               => 'Deposit',
					'amount'             => new Money($data['amount'], 'USD'),
					'metadata'           => [
																		'userID' => $this->user->id,
																		'fee' => $processor->feeAmount('fund', $data['amount']),
																		'amount' => $processor->netAmount('fund', $data['amount'])
																	],
					'success_url' 			 => route('wallet.deposit.success-coinbase'),
					'cancel_url' 			 	 => route('wallet.deposit.cancel-coinbase'),
					'notifications_url'  => route('api.ipn', 5),
					'auto_redirect' 		 => true
				);

				$checkout = new Checkout( $params );
				$client->createCheckout( $checkout );
				$code = $checkout->getEmbedCode();

				return redirect()->to("https://sandbox.coinbase.com/checkouts/" . $code);

				*/

				$new_transaction = new \App\Models\Transactions;
			    $new_transaction->processor_id = 5;
			    $new_transaction->user_id = $this->user->id;
			    $new_transaction->type = 1;
			    $new_transaction->fee = $processor->feeAmount('fund', $data['amount']);
			    $new_transaction->amount = $processor->netAmount('fund', $data['amount']);
			    $new_transaction->note = 'Deposit of ' . $processor->netAmount('fund', $data['amount']) . ' (Bitcoin)';
			    $new_transaction->status = 0;
			    $new_transaction->save();

			    $user = $this->user;

				$bitcoin_transaction = new \App\Models\TransactionsBitcoin;
				$bitcoin_transaction->transaction_id = $new_transaction->id;
				$bitcoin_transaction->data = '';
				$bitcoin_transaction->save();

		    //MailManager::send('transactions.pending-fund', 'New Deposit', ['id' => $new_transaction->id, 'processor' => $processor->name, 'fee' => $new_transaction->fee, 'amount' => $new_transaction->amount, 'note' => $new_transaction->note ], $user);

				return redirect()->route('wallet.deposit.pending-bitcoin')->with('memo', $new_transaction->id);
			break;
			case 6:
				$giftCardAmount = 0;

				if( is_null( $giftCard = \App\Models\GiftCards::where('hash', $data['gc-hash'])->first() ) ) {
					return redirect()->back()->with('error', trans('wallet.index.gcWrong'));
				}

				$user_balance = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
				$user_balance->wallet = $user_balance->wallet + $giftCard->amount;
				$user_balance->save();

				$giftCardAmount = $giftCard->amount;

				$new_transaction = new \App\Models\Transactions;
			    $new_transaction->processor_id = 6;
			    $new_transaction->user_id = $this->user->id;
			    $new_transaction->type = 1;
			    $new_transaction->fee = $processor->feeAmount('fund', $giftCardAmount);
			    $new_transaction->amount = $processor->netAmount('fund', $giftCardAmount);
			    $new_transaction->note = 'Deposit of ' . $processor->netAmount('fund', $giftCardAmount) . ' - ' . $giftCard->hash . ' (Gift Card)';
			    $new_transaction->status = 1;
			    $new_transaction->save();

				$giftCard->used_by = $this->user->id;
				$giftCard->save();

				$giftCard->delete();

				return redirect()->back()->with('success', trans('wallet.index.gcSuccess') . $giftCardAmount);
		break;
	  }
	}

	public function depositSuccess()
	{
		if( Input::get('PAYMENT_BATCH_NUM', false ) == false )
			return redirect()->route('wallet.index');

		return view('wallet.deposit.success')->withTitle(trans('wallet.index.depositSuccess'));
	}

	public function depositPendingBitcoin()
	{
		$data = json_decode(\App\Models\Processors::where('id', 5)->lists('data')[0], true);
		return view('wallet.deposit.pending-bitcoin', ['data' => $data])->withTitle(trans('wallet.index.depositPending'));
	}

	public function depositSuccessCcex()
	{
		$result = Input::get('result', false);
		$id = Input::get('ccexmpurchaseid', false);

		if( $id === false || $result === false )
			return redirect()->route('wallet.index');

		try {

			$decrypt = Crypt::decrypt( $id );

		} catch ( DecryptException $e ) {
			return redirect()->route('wallet.index');
		}


		$decrypt = json_decode( $decrypt, true );

		if( $result == 1 )
			$title = trans('wallet.index.depositSuccess');
		else
			$title = trans('wallet.index.depositCanceled');

		return view('wallet.deposit.success-ccex', [
			'result' => $result,
			'decrypt' => $decrypt
		])->withTitle( $title );
	}

	public function depositCancelCoinbase()
	{
		return view('wallet.deposit.cancel-coinbase')->withTitle( trans('wallet.index.depositCanceled'));
	}

	public function getDepositWire()
	{
		$data = json_decode(\App\Models\Processors::where('id', 1)->lists('data')[0], true);
		return view('wallet.deposit.wire', ['data' => $data])->withTitle(trans('wallet.index.bankwireFund'));
	}

	public function getDepositWireCancel($id = null)
	{
		if( is_null( $transaction = \App\Models\Transactions::where('user_id', $this->user->id)->where('id', $id)->where('processor_id', 1)->first() ) )
			return Redirect::route('wallet.index')->with('error', trans('wallet.error.Wrong transaction ID'));

		$transaction->delete();

		return Redirect::route('wallet.index')->with('success', trans('wallet.success.You have successfully canceled this bank wire request'));
	}

	public function getDepositBitcoinCancel($id = null)
	{
		if( is_null( $transaction = \App\Models\Transactions::where('user_id', $this->user->id)->where('id', $id)->where('processor_id', 5)->first() ) )
			return Redirect::route('wallet.index')->with('error', trans('wallet.error.Wrong transaction ID'));

		if( !is_null( $bitcoin_transaction = $transaction->bitcoin ) ) {
			$bitcoin_transaction->delete();
		}

		$transaction->delete();

		return Redirect::route('wallet.index')->with('success', trans('wallet.succes.You have successfully canceled this bitcoin  request'));
	}

	// Gateway accounts

	public function getGatewayAccounts()
	{
		$processors = \App\Models\Processors::where('id', '!=', 1)->where('id', '!=', 4)->orderBy('id', 'asc')->get();
		$ups = \App\Models\UsersProcessors::where('processor_id', '!=', 1)->where('user_id', $this->user->id)->orderBy('id', 'asc')->get();

		$uprocessors = [];

		foreach( $ups as $up ) {
			$uprocessors[ $up->processor_id ] = $up;
		}

		return view('wallet.gatewayAccounts.index', ['processors' => $processors, 'uprocessors' => $uprocessors ])->withTitle(trans('wallet.index.myGatewayAccounts'));
	}

	public function postGatewayAccounts()
	{
		$data = Input::all();
		$rules = [];
		$processors = \App\Models\Processors::where('id', '!=', 1)->where('id', '!=', 4)->orderBy('id', 'asc')->get();

		foreach( $processors as $p ) {
			$rules[ 'processor_' . $p->id ] = 'min:1|max:100';
		}

		$validator = Validator::make($data, $rules);

		if ( $validator->fails() )
      return Redirect::back()->withInput()->withErrors($validator);

		foreach( $processors as $p ) {
			if( isset( $data[ 'processor_' .  $p->id ] ) && trim($data[ 'processor_' .  $p->id ]) !== '' ) {

				$input = $data[ 'processor_' .  $p->id ];


				//$user_processor = new \App\Models\UsersProcessors;
				$userProcessor = \App\Models\UsersProcessors::where('processor_id', $p->id)->where('user_id', $this->user->id)->first();

				if( $userProcessor->data !== $input ) {
					$userProcessor->data = $input;
					$userProcessor->status = 0;
					$userProcessor->save();
				}

			}
		}

		return Redirect::back()->with('success', trans('wallet.success.You have successfully updated your gateway accounts'));
	}




	// Bank details

	public function getBankProfiles()
	{
		$bankProfiles = \App\Models\UsersProcessors::where('processor_id', 1)
			->where('bank_details_id', '!=', NULL)
			->where('user_id', $this->user->id)
			->whereRaw('(status = 1 OR status = 0)')
			->orderBy('id', 'asc')
			->get();

		return view('wallet.bankProfiles.index', ['bankProfiles' => $bankProfiles])->withTitle(trans('wallet.index.myBankProfiles'));
	}

	public function getBankProfilesNew()
	{
		return view('wallet.bankProfiles.new')->withTitle(trans('wallet.index.createNew'));
	}

	public function postBankProfilesNew()
	{
		$rules = [
			'Profile_Name' => 'required|min:1|max:150',
			'Name_on_Account' => 'required|min:1|max:150',
			'Bank_Name' => 'required|min:1|max:150',
			'Bank_Address' => 'max:150',
			'Bank_Phone_Number' => 'max:150',
			'Institution_Number' => 'max:150',
			'Branch_Number' => 'max:150',
			'Account_Number' => 'required|max:150',
			'Routing_Number' => 'max:150',
			'IBAN' => 'max:150',
			'Swift_code' => 'required|max:150',
			'Intermediary_Bank_Info' => 'max:300',
			'password' => 'required|min:1|max:100'
		];

		$data = [];

		foreach( $rules as $key => $value ) {
			$data[ $key ] = Input::get( $key, '');
		}

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      return Redirect::back()->withInput()->withErrors($validator);

		if( !$this->user->checkPassword($data['password'], 'secondary_password') )
			return Redirect::back()->with('error', trans('wallet.error.Wrong secondary password'))->withInput();

		unset($data['password']);

		$bankDetails = new \App\Models\UsersBankDetails;
		$bankDetails->user_id = $this->user->id;

		foreach( $data as $key => $d ) {
			if( isset( $rules[ $key ] ) ) {
				$val = strtolower( $key );
				$bankDetails->$val = trim( $d );
			}
		}

		$checkOthers = \App\Models\UsersBankDetails::where('Account_Number', $bankDetails->account_number)->count();

		if( $checkOthers > 0 )
			return Redirect::back()->with('error', trans('wallet.error.This account number already exists in the system'))->withInput();

		if( !$bankDetails->save() )
			return Redirect::back()->with('error', trans('wallet.error.An error has occured. Please try again'))->withInput();


		if( is_null( $userProcessor = \App\Models\UsersProcessors::where('user_id', $this->user->id)->where('processor_id', 1)->where('bank_details_id', '=', NULL)->first() ) )
			$userProcessor = new \App\Models\UsersProcessors;

		$userProcessor->processor_id = 1;
		$userProcessor->user_id = $this->user->id;
		$userProcessor->bank_details_id = $bankDetails->id;
		$userProcessor->status = 0;

		if( $userProcessor->save() ) {

			$user = $this->user;

			MailManager::send('wallet.new-profile', 'Bank Account Profile Created', [ 'id' => $userProcessor->id, 'Profile_Name' => $data['Profile_Name'], 'Bank_Name' => $data['Bank_Name'] ], $user);

			return Redirect::route('wallet.bank-profiles')->with('success', trans('wallet.success.You have successfully added a new bank profile'));
		}

		return Redirect::back()->with('error', trans('wallet.error.An error has occured. Please try again'))->withInput();
	}

	public function getBankProfilesView($id = null)
	{
		if( is_null( $userProcessor = \App\Models\UsersProcessors::where('id', $id)->where('processor_id', 1)->where('status', 1)->orWhere('status', 0)->where('user_id', $this->user->id)->first() ) )
			return Redirect::route('wallet.bank-profiles')->with('error', trans('wallet.error.Wrong bank profile ID'));

		$bankDetails = $userProcessor->bankDetails;

		return view('wallet.bankProfiles.view', ['bankDetails' => $bankDetails])->withTitle($bankDetails->profile_name);
	}

	public function postBankProfilesView($id = null)
	{
		if( is_null( $userProcessor = \App\Models\UsersProcessors::where('id', $id)->where('processor_id', 1)->where('user_id', $this->user->id)->orWhere('status', 0)->where('status', 1)->first() ) )
			return Redirect::route('wallet.bank-profiles')->with('error', trans('wallet.error.Wrong bank profile ID'));

		$bankDetails = $userProcessor->bankDetails;

		$rules = [
			'Profile_Name' => 'required|min:1|max:150',
			'Name_on_Account' => 'required|min:1|max:150',
			'Bank_Name' => 'required|min:1|max:150',
			'Bank_Address' => 'max:150',
			'Bank_Phone_Number' => 'max:150',
			'Institution_Number' => 'max:150',
			'Branch_Number' => 'max:150',
			'Account_Number' => 'required|max:150',
			'Routing_Number' => 'max:150',
			'IBAN' => 'max:150',
			'Swift_code' => 'required|max:150',
			'Intermediary_Bank_Info' => 'max:300',
			'password' => 'required|min:1|max:100'
		];

		$data = [];

		foreach( $rules as $key => $value )
			$data[ $key ] = Input::get( $key, '');

		$validator = Validator::make($data, $rules);

		if ($validator->fails())
      return Redirect::back()->withInput()->withErrors($validator);

    if( !$this->user->checkPassword($data['password'], 'secondary_password') )
			return Redirect::back()->with('error',trans('wallet.error.Wrong secondary password'))->withInput();

    unset( $data['password'] );

		foreach( $data as $key => $d ) {
			if( isset( $rules[ $key ] ) ) {
				$val = strtolower( $key );
				$bankDetails->$val = trim( $d );
			}
		}

		$checkOthers = \App\Models\UsersBankDetails::where('Account_Number', $bankDetails->account_number)->count();

		if( $checkOthers > 0 )
			return Redirect::back()->with('error', trans('wallet.error.This account number already exists in the system'))->withInput();

		$userProcessor->status = 0;

		if( $userProcessor->save() && $bankDetails->save() )
			return Redirect::route('wallet.bank-profiles')->with('success', trans('wallet.error.Bank Profile successfully updated'));

		return Redirect::back()->with('error', trans('wallet.error.An error has occured. Please try again'));
	}

	public function getBankProfilesDelete($id = null)
	{
		if( is_null( $userProcessor = \App\Models\UsersProcessors::where('id', $id)->where('user_id', $this->user->id)->first() ) )
			return Redirect::route('wallet.bank-profiles')->with('error', trans('wallet.error.Wrong bank profile ID.'));

		$user = $this->user;

		MailManager::send('wallet.deleted-profile', 'Bank Account Profile Deleted', [ 'id' => $userProcessor->id, 'Profile_Name' => $userProcessor->bankDetails->profile_name, 'Bank_Name' => $userProcessor->bankDetails->bank_name ], $user);

    $userProcessor->bankDetails->delete();
    $userProcessor->delete();

		return Redirect::back()->with('success', trans('wallet.succes.You have successfully deleted this bank profile'));
	}

	public function getTradeToMember()
	{
		//$processors = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();
		$user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
		return view('wallet.trade-to-member', ['user_wallet' => $user_wallet])->withTitle(trans('wallet.tradeToMember.title'));
	}

	public function postTradeToMember()
	{
		if( !Config::get('cashouts.t2mtransfer') )
			return Redirect::back();

		$data = Input::all();
		$rules = [
			'user_id' 	=> 'required',
			'amount' 		=> 'required|numeric',
			//'processor' => 'required|exists:processors,id',
			'note' 			=> 'max:250',
			'password' 	=> 'required|max:250'
		];

		$validator = Validator::make( $data, $rules );

		if ( $validator->fails() )
			return Redirect::back()->withInput()->withErrors( $validator );

	    if( is_null( $member = \App\Models\User::where('id', $data['user_id'])->orWhere('username', $data['user_id'])->first() ) )
				return Redirect::back()->with('error', trans('wallet.shared.wrongUN'))->withInput();

	    $user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

	    if( $user_wallet->resale_units < $data['amount'] )
	    	return Redirect::back()->with('error', trans('wallet.shared.noUnits'))->withInput();

	    if( !$this->user->checkPassword($data['password'], 'secondary_password') )
				return Redirect::back()->with('error', trans('wallet.shared.wrongSecPass'))->withInput();

	    $user_wallet->resale_units = ($user_wallet->resale_units - $data['amount']);
	    $user_wallet->save();

	    $member_wallet = \App\Models\UsersBalances::where('users_id', $member->id)->first();
	    $member_wallet->resale_units = ($member_wallet->resale_units + $data['amount']);
	    $member_wallet->save();

	    // Transaction to
	    $new_transaction = new \App\Models\Transactions;
	    //$new_transaction->processor_id = NULL;
	    $new_transaction->user_id = $this->user->id;
	    $new_transaction->type = 9;
	    $new_transaction->amount = $data['amount'];
	    $new_transaction->note = 'Sent ' . $data['amount'] . ' units (0.00 fee) to ' . $member->username . ' (#' . $member->id . ')';
	    $new_transaction->status = 1;
	    $new_transaction->save();

	    $user = $this->user;

		NotificationManager::create( [
			'type' => 'wallet.transfer-to',
			'content_id' => $new_transaction->id
		], $user);

	    MailManager::send('transactions.transfer-to', 'Trade Transfer To Member', [ 'id' => $new_transaction->id, 'processor' => 'Trade Account', 'member_name' => $member->username, 'member_id' => $member->id, 'fee' => 0.00, 'amount' => $new_transaction->amount, 'note' => $data['note'] ], $member);

	    // Transaction from
	    $new_transaction = new \App\Models\Transactions;
	   // $new_transaction->processor_id = NULL;
	    $new_transaction->user_id = $member->id;
	    $new_transaction->type = 5;
	    $new_transaction->amount = $data['amount'];
	    $new_transaction->note = 'Received from ' . $this->user->username . ' (#' . $this->user->id .')' . ( ( $data['note'] !== '' ) ? ' - ' . $data['note'] : '');
	    $new_transaction->status = 1;
	    $new_transaction->save();

		NotificationManager::create( [
				'type' => 'wallet.transfer-from',
				'content_id' => $new_transaction->id
			], $new_transaction->user);

	    MailManager::send('transactions.transfer-from', 'Trade Account Transfer From Member', [ 'id' => $new_transaction->id, 'processor' => 'Trade Account', 'member_name' => $user->username, 'member_id' => $user->id, 'fee' => 0.00, 'amount' => $new_transaction->amount, 'note' => $data['note'] ], $member);

	    return Redirect::route('wallet.index')->with('success', trans('wallet.tradeToMember.messages.success'));
	}

	// TRANSFER
	public function getTransfer()
	{
		$processors = \App\Models\Processors::where('id', '!=', 4)->orderBy('id', 'asc')->get();
		$user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
		return view('wallet.transfer', ['user_wallet' => $user_wallet])->withTitle(trans('wallet.transfer.title'));
	}

	public function postTransfer()
	{
		if( !Config::get('cashouts.m2mtrasfer') )
			return Redirect::back();

		$data = Input::all();
		$rules = [
			'user_id' 	=> 'required',
			'amount' 		=> 'required|numeric',
			//'processor' => 'required|exists:processors,id',
			'note' 			=> 'max:250',
			'password' 	=> 'required|max:250'
		];

		$validator = Validator::make( $data, $rules );

		if ( $validator->fails() )
			return Redirect::back()->withInput()->withErrors( $validator );

	    if( is_null( $member = \App\Models\User::where('id', $data['user_id'])->orWhere('username', $data['user_id'])->first() ) )
				return Redirect::back()->with('error', trans('wallet.shared.wrongUN'))->withInput();

	    $user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

	    if( $user_wallet->online_account < $data['amount'] )
	    	return Redirect::back()->with('error', trans('wallet.shared.noMoney'))->withInput();

	    if( !$this->user->checkPassword($data['password'], 'secondary_password') )
				return Redirect::back()->with('error', trans('wallet.shared.wrongSecPass'))->withInput();

	    $user_wallet->online_account = ($user_wallet->online_account - $data['amount']);
	    $user_wallet->save();

	    $member_wallet = \App\Models\UsersBalances::where('users_id', $member->id)->first();
	    $member_wallet->online_account = ($member_wallet->online_account + $data['amount']);
	    $member_wallet->save();

	    // Transaction to
	    $new_transaction = new \App\Models\Transactions;
	    //$new_transaction->processor_id = NULL;
	    $new_transaction->user_id = $this->user->id;
	    $new_transaction->type = 9;
	    $new_transaction->amount = $data['amount'];
	    $new_transaction->note = 'Sent $' . $data['amount'] . ' ($0.00 fee) to ' . $member->username . ' (#' . $member->id . ')';
	    $new_transaction->status = 1;
	    $new_transaction->save();

	    $user = $this->user;

		NotificationManager::create( [
			'type' => 'wallet.transfer-to',
			'content_id' => $new_transaction->id
		], $user);

	    MailManager::send('transactions.transfer-to', 'Online Account Transfer To Member', [ 'id' => $new_transaction->id, 'processor' => 'Online Account', 'member_name' => $member->username, 'member_id' => $member->id, 'fee' => 0.00, 'amount' => $new_transaction->amount, 'note' => $data['note'] ], $member);

	    // Transaction from
	    $new_transaction = new \App\Models\Transactions;
	    //$new_transaction->processor_id = NULL;
	    $new_transaction->user_id = $member->id;
	    $new_transaction->type = 5;
	    $new_transaction->amount = $data['amount'];
	    $new_transaction->note = 'Received from ' . $this->user->username . ' (#' . $this->user->id .')' . ( ( $data['note'] !== '' ) ? ' - ' . $data['note'] : '');
	    $new_transaction->status = 1;
	    $new_transaction->save();

		NotificationManager::create( [
				'type' => 'wallet.transfer-from',
				'content_id' => $new_transaction->id
			], $new_transaction->user);

	    MailManager::send('transactions.transfer-from', 'Online Account Transfer From Member', [ 'id' => $new_transaction->id, 'processor' => 'Online Account', 'member_name' => $user->username, 'member_id' => $user->id, 'fee' => 0.00, 'amount' => $new_transaction->amount, 'note' => $data['note'] ], $member);

	    return Redirect::route('wallet.index')->with('success', trans('wallet.transfer.messages.success'));
	}

	public function getTransferBalance()
	{
		$user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
		return view('wallet.transfer-balance', ['user_wallet' => $user_wallet])->withTitle(trans('wallet.transferBalance.title'));
	}

	public function postTransferBalance()
	{
		$data = Input::all();
		$rules = [
			'balance' 	=> 'required|in:1,2',
			'amount' 	=> 'required|numeric',
			'password' 	=> 'required|max:250'
		];

		$validator = Validator::make( $data, $rules );

		if ( $validator->fails() )
			return Redirect::back()->withInput()->withErrors( $validator );

		$wallets = [1 => 'wallet', 2 => 'online_account'];
        $walletName = $wallets[ $data['balance'] ];
        $toWallet = $wallets[ (($data['balance'] == 1) ? 2 : 1) ];

	    $user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

	    if( !$this->user->checkPassword($data['password'], 'secondary_password') )
			return Redirect::back()->with('error', trans('wallet.shared.wrongSecPass'))->withInput();

	    if( $user_wallet->$walletName < $data['amount'] )
	    	return Redirect::back()->with('error', trans('wallet.shared.noMoney'))->withInput();

	    $user_wallet->$walletName = ($user_wallet->$walletName - $data['amount']);
		$user_wallet->$toWallet = ($user_wallet->$toWallet + $data['amount']);
	    $user_wallet->save();

	    return Redirect::back()->with('success', trans('wallet.transferBalance.messages.success'));
	}


	// CASHOUTS

	public function getCashout()
	{
		$processors = \App\Models\Processors::whereNotIn('id', [4,6])->where('fee_cashout_status', 1)->orderBy('id', 'asc')->get();
		$processor_ewallet = \App\Models\Processors::where('id', 4)->first();

		$pending_cashouts = \App\Models\Transactions::where('user_id', $this->user->id)->where('status', 0)->whereIn('type', [7,11])->where('processor_id', 1)->orderBy('id', 'desc')->get();
		$e_wallet_balance = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();

		$_users_processors = \App\Models\UsersProcessors::whereRaw('user_id = ' . $this->user->id . ' AND ( bank_details_id IS NOT NULL OR data IS NOT NULL)')->orderBy('processor_id')->where('status', 1)->get();
		$users_processors = [];

		foreach( $_users_processors as $up ) {
			if( !isset( $users_processors[ $up->processor_id ] ) ) $users_processors[ $up->processor_id ] = [];

			$users_processors[ $up->processor_id ][] = $up;
		}

		return view('wallet.cashout', ['processors' => $processors, 'pending_cashouts' => $pending_cashouts, 'users_processors' => $users_processors, 'e_wallet_balance' => $e_wallet_balance, 'processor_ewallet' => $processor_ewallet])->withTitle(trans('wallet.cashout.title'));
	}

	public function postCashout()
	{
		$data = Input::all();

		$rules = [
			'amount' => 'required|numeric',
			//'action' => 'required|in:2,11',
			'processor' => 'required|exists:processors,id',
			//'pool_selector' => 'required|array',
			'password' => 'required|max:250',
			//'processor_group' => 'required_if:processor,1|required_if:action,11|array'
		];

		$validator = Validator::make($data, $rules);

		if ( $validator->fails() )
      		return Redirect::back()->withInput()->withErrors($validator);

		if( ( isset( $data['processor'] ) && $data['processor'] == 1 ) ) {
			if( !isset( $data['processor_group'] ) || !is_array( $data['processor_group'] ) ) {
				return Redirect::back()->with('error', trans('wallet.cashout.messages.errorGateway'))->withInput();
			}
		}

    //if( $data['action'] == '2' ) {
    //	$processor = \App\Models\Processors::where('id', 4)->where('fee_cashout_status', 1)->first();
   // } else {
    	$processor = \App\Models\Processors::where('id', $data['processor'])->where('fee_cashout_status', 1)->first();
    //}

    if( is_null( $processor ) )
			return Redirect::back()->with('error', trans('wallet.cashout.messages.errroDisabled'))->withInput();

    if( $processor->limit_cashout_min !== 0 && $data['amount'] < $processor->limit_cashout_min )
			return Redirect::back()->with('error', trans('wallet.cashout.messages.errorMin') . ' $' . $processor->limit_cashout_min)->withInput();

    if( $processor->limit_cashout_max !== 0 && $data['amount'] > $processor->limit_cashout_max )
			return Redirect::back()->with('error', trans('wallet.cashout.messages.errorMax') . ' $' . $processor->limit_cashout_max)->withInput();

		if( !$this->user->checkPassword($data['password'], 'secondary_password') )
			return Redirect::back()->with('error', trans('wallet.shared.wrongSecPass'))->withInput();

	  //$pools = ['online_account', 'offset_units', 'resale_units'];
	  //$pool = $data['pool_selector'][ $data['processor'] ];

	  //if( !in_array($pool, $pools) )
	  //  return Redirect::back()->with('error', 'Wrong pool')->withInput();

	  //if( $data['action'] != 2 ) {
		  	$cashout_limit = Config::get('cashouts.limit');
		  	$cashout_limit_type = Config::get('cashouts.limit_type');
		  	$cashout_limit_span = Config::get('cashouts.limit_span');
		  	$interval_indexes = ['d' => ' DAY','m' => ' MONTH','y' => ' YEAR'];

		  	$mysql_interval = str_replace( array_keys( $interval_indexes ), array_values( $interval_indexes ), $cashout_limit_span );

		  	$cashout_query = \App\Models\Transactions::where('user_id', $this->user->id)->whereIn('type', [2,7,11])->where('created_at', '>=', DB::raw('NOW() - INTERVAL ' . $mysql_interval));

		  	if( $cashout_limit_type == 'count' ) { // Count
		      	$cashout_count = $cashout_query->count();
		      	if( $cashout_count >= $cashout_limit ) {
		          	return Redirect::back()->with('error', trans('wallet.cashout.messages.errorLimit'))->withInput();
		      	}
		  	} elseif( $cashout_limit_type == 'value' ) { // Value
		      	$cashout_total = $cashout_query->sum('amount');
		      	if( $cashout_total >= $cashout_limit ) {
		          	return Redirect::back()->with('error', trans('wallet.error.Withdrawal limit exceeded'))->withInput();
		      	}
		  	}
	//	}

	  	$user_wallet = \App\Models\UsersBalances::where('users_id', $this->user->id)->first();
	  	//$cash_pool = $pool;

	  	if( $user_wallet->wallet < $data['amount'] )
	    	return Redirect::back()->with('error', trans('You dont have enough money'))->withInput();

	  	if( isset( $data['processor_group'][ $data['processor'] ] ) ) {
	  		if( is_null( $user_processor = \App\Models\UsersProcessors::where('id', $data['processor_group'][ $data['processor'] ])->where('user_id', $this->user->id)->first() ) )
	  				return Redirect::back()->with('error', trans('wallet.erro.Processor not found'))->withInput();
	 	}

	  $new_transaction = new \App\Models\Transactions;
	  $new_transaction->processor_id = $processor->id;
	  $new_transaction->user_id = $this->user->id;

	  /*switch ( $data['action'] ) {
      case 2: // Ewallet
          $new_transaction->type = 2;
          $new_transaction->fee = $processor->feeAmount('cashout', $data['amount']);
          $new_transaction->amount = $data['amount'];
          $new_transaction->balance = $cash_pool;
          $new_transaction->note = '$ ' . number_format($data['amount'], 2) . ' Cashout ($' . $new_transaction->fee . ' Fee)';
          $new_transaction->status = 1;
          $new_transaction->save();

          $user_wallet->$cash_pool = $user_wallet->$cash_pool - $data['amount'];

          $user_wallet->wallet = $user_wallet->wallet + $processor->netAmount('cashout', $data['amount']);
          $user_wallet->save();
          break;

      case 11: // Processor*/
        $new_transaction->fee = $processor->feeAmount('cashout', $data['amount']);
        $new_transaction->type = 11;
        $new_transaction->amount = $data['amount'];
        $new_transaction->balance = $cash_pool;
        $new_transaction->note = '$ ' . number_format($data['amount'], 2) . ' Cashout ($' . $new_transaction->fee . ' Fee)';

        if( $processor->id == 1 ) {
            $new_transaction->user_proc_id = $user_processor->id;
						if( $processor->auto_cashout == 0 ) {
		            $new_transaction->status = 0;
		        } else {
							$new_transaction->status = 1;
						}
        } else {

	        if( $processor->auto_cashout == 0 ) {
	            $new_transaction->status = 0;
	        } else {

	            $gateWay = new \App\Libraries\GatewayCashouts( $processor->id );

	            if( $processor->id == 2 ) {
	                $gateWay->setVariables([ 'account_number' => $user_processor->value, 'amount' => $processor->netAmount('cashout', $data['amount']), 'user_id' => $cashout->user_id ]);
	            } elseif( $processor->id == 3 ) {
	                $gateWay->setVariables([ 'receiver_email' => $this->user->email, 'amount' => $processor->netAmount('cashout', $data['amount']), 'user_id' => $cashout->user_id ]);
	            }

	            $gw_response = $gateWay->send();

	            if( !$gw_response ) {
	              $new_transaction->status = 0;
	              return Redirect::back()->with('error', trans('wallet.error.Gateway error'))->withInput();
	            } else {
	            	if( is_array( $gw_response ) && array_key_exists('ERROR', $gw_response) )
	            		return Redirect::back()->with('error',  trans('wallet.error.Gateway error'))->withInput();
	            	else
									$new_transaction->status = 1;
	            }
	        }

	      }

        $user_wallet->$cash_pool = $user_wallet->$cash_pool - $data['amount'];//$processor->netAmount('cashout', $data['amount']);
        $user_wallet->save();

				$new_transaction->save();

				if( $processor->id == 5 ) {
					$bitcoin_transaction = new \App\Models\TransactionsBitcoin;
					$bitcoin_transaction->transaction_id = $new_transaction->id;
					$bitcoin_transaction->data = '';
					$bitcoin_transaction->save();
				}

				NotificationManager::create( [
					'type' => 'wallet.cashout',
					'content_id' => $new_transaction->id
				], $this->user);

      /*  break;
      }*/

      return Redirect::back()->with('success', trans('wallet.success.Cashout request successfully sent'));

      //$user_wallet->wallet = ($user_wallet->wallet - $data['amount']);
      //$user_wallet->save();
	}
}
