<?php

namespace App\Http\Controllers;

use App\Facades\MailManager;
use chillerlan\QRCode\QROptions;
use Config, Session, Input, Validator, DB, Hash, Carbon\Carbon, Excel;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;

class HomeController extends Controller
{

    public function __construct()
    {
        parent::__construct();
    }

    public function index()
    {
        $featured = \App\Models\AuctionItems::where('status', 0)->where('type_id', 2)->take(4)->get();
        $endingSoon = \App\Models\AuctionItems::where('status', 0)->endingSoon()->take(8)->get();
        $latest = \App\Models\AuctionItems::where('status', 0)->orderBy('id', 'desc')->take(8)->get();

        return view('site.index', compact("endingSoon", "latest", "featured"));
    }

    public function getQRCodeUrl( Request $request ) {
        $url = $request->query('url', null);

        $code_options = new QROptions([
            'outputType' => \chillerlan\QRCode\QRCode::OUTPUT_IMAGE_PNG,
            'eccLevel'   => \chillerlan\QRCode\QRCode::ECC_H,
            'scale' => 5,
            'imageBase64' => false,
            'quietzoneSize' => false
        ]);

        $code = new \chillerlan\QRCode\QRCode($code_options);

        return $code->render($url);
    }

    public function getQRCodeStr( Request $request ) {
        $str = $request->query('str', null);

        $code_options = new QROptions([
            'outputType' => \chillerlan\QRCode\QRCode::OUTPUT_IMAGE_PNG,
            'eccLevel'   => \chillerlan\QRCode\QRCode::ECC_H,
            'scale' => 5,
            'imageBase64' => false,
            'quietzoneSize' => false
        ]);

        $code = new \chillerlan\QRCode\QRCode($code_options);

        return $code->render($str);
    }

    public function aboutUs()
    {
        return view('site.aboutUs')->withTitle(trans('home.aboutUs.title'));
    }

    public function shop()
    {
        $publicCategory = \App\Models\ProductCategories::where('name', 'Public')->where('status', 1)->first();
        $regCategory = \App\Models\ProductCategories::where('name', 'Registration')->where('status', 1)->first();

        return view('site.shop', ['publicCategory' => $publicCategory, 'regCategory' => $regCategory])->withTitle(trans('home.shop.title'));
    }

    public function carbonIndustry()
    {
        return view('site.carbon-industry')->withTitle(trans('home.carbonIndustry.title'));
    }

    public function vision()
    {
        return view('site.vision')->withTitle(trans('home.vision.title'));
    }

    public function contactReferrer( $ref = null )
    {
        if( !is_null( $ref ) ) {
            if( !is_null( $referrer = \App\Models\Sponsor::select(DB::raw('sponsors.*, users.username'))->join('users', 'users.id', '=', 'sponsors.user_id')->where('username', strtolower($ref) )->first() ) )
                $ref = $referrer;
            else return redirect()->route('site.contact-referrer');
        }

        return view('site.contact-referrer', ['referrer' => $ref])->withTitle(trans('home.contactReferrer.title'));
    }

    public function postContactReferrer( $ref = null )
    {
        $validator = Validator::make(['referrer' => Input::get('referrer', $ref)], [
            'referrer' => 'required|min:2'
        ]);

        if( $validator->fails() )
            return redirect()->back()->withErrors($validator);

        if( is_null( $referrer = \App\Models\Sponsor::select(DB::raw('sponsors.*, users.username'))->join('users', 'users.id', '=', 'sponsors.user_id')->where('username', strtolower($ref) )->first() ) )
            return redirect()->route('site.contact-referrer')->with('error', trans('home.contactReferrer.messages.error'));

        if( Input::get('send', false) ) {
            /*$validator = Validator::make(['referrer' => Input::get('referrer', $ref), [
                'referrer' => 'required|min:2'
            ]);

            if( $validator->fails() )
                return redirect()->back()->withErrors($validator);*/

            return redirect()->back()->with('success', trans('home.contactReferrer.messages.success'));
        }

        return redirect()->route('site.contact-referrer', $referrer->username);
    }

    public function terms()
    {   
        
		if(app()->getLocale())
            return view('site.terms_'.app()->getLocale())->withTitle(trans('home.terms.title'));
        else
        return view('site.terms_en')->withTitle(trans('home.terms.title'));
    }

    public function compansationPlan()
    {
        return view('site.compensation-plan')->withTitle(trans('home.compansationPlan.title'));
    }

    public function cookiePolicy()
    {
        return view('site.cookie-policy')->withTitle(trans('home.cookiePolicy.title'));
    }

    public function privacyPolicy()
    {
        return view('site.privacy-policy')->withTitle(trans('home.privacyPolicy.title'));
    }

    public function contactSupport()
    {
        return view('site.contactSupport')->withTitle(trans('home.contactSupport.title'));
    }
	public function faq()
	{
		$top_faqs = \App\Models\Faqs::top()->orderBy('id', 'asc')->get();
		$faqs = \App\Models\Faqs::notTop()->orderBy('id', 'asc')->get();
		return view('site.faq', ['top_faqs' => $top_faqs, 'faqs' => $faqs])->withTitle("F.A.Q.'s");
	}
    public function postContactSupport( \Illuminate\Http\Request $request ){

        $data = \Illuminate\Support\Facades\Input::all();
        clock()->info($data);

        /*$captcha_message  = [
            'valid_captcha' => 'Wrong code. Try again please.'
        ];*/

        $rules = [
            'email' => 'required',
            'subject' => 'required|min:5',
            'content' => 'required|min:10',
            'captcha' => 'required|captcha'
        ];

        //clock()->info($request);
        //$code = $request->input('captcha');
        //clock()->info($code);
        //$isHuman = captcha_validate($code);
        //clock()->info($isHuman);

        $validator = \Illuminate\Support\Facades\Validator::make($data, $rules);

        if( $validator->fails() )
            return redirect()->back()->withInput()->withErrors($validator);

        $from_mail = $data['email'];

        Mail::send('admin.emailtemplates.contact-support',
            array(
                'name' => 'Contact Support Visitor',
                'email' => $data['email'],
                'user_subject' => $data['subject'],
                'user_message' => $data['content']
            ), function($message) use ($data)
            {
                $message->from(config('app.support_mail_address'));
                $message->to(config('app.support_mail_address'), 'Admin')->subject('Colibri Auction - Contact Support Message');
            });

        return redirect()->back()->with('success', trans('home.contactSupport.success'));
        //return 0;
    }

    public function changeLanguage( $lang = '' )
    {
        if( !in_array($lang, array_keys(Config::get('app.languages')))) 
            $lang = Config::get('app.locale');

        Session::put('locale', $lang);

        return redirect()->back();
    }
}
