<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Contracts\Auth\Guard;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use Session;
use Cookie;
use Auth;

class CheckPermission
{
    private $available_urls_at_frozen = [];
    protected $auth;

    public function __construct(Guard $auth)
    {
        $this->auth = $auth;
    }

    public function handle($request, Closure $next, $permission = null)
    {
        $routeActions = $request->route()->getAction();
        $redirectTo = ( isset( $routeActions['redirect'] ) ) ? $routeActions['redirect'] : 'home';

        if ( !$this->auth->guest()) {

            $url_parse = parse_url($request->url());
            $current_url_path = ( isset( $url_parse['path'] ) ) ? $url_parse['path'] : '/';
            $becomeMember       = Session::pull('becomeMember', false);
            $originalAccount    = Session::pull('originalAccount', false);

            //dd( $becomeMember, $originalAccount );

            //if( $originalAccount !== false ) {
            //  $becomeMember = true;
            //}

            if( $becomeMember == false && $originalAccount == false ) {
              //Suspended
              if($request->user()->isSuspended() ) { //&& $current_url_path != '/suspended') {
                  $this->auth->logout();
                  return redirect()->route('auth.login')->with('error', 'Your account is suspended.');
              }

              //Frozen
              if($request->user()->isFrozen()) {
                  if(!in_array($current_url_path, $this->available_urls_at_frozen)) {
                      $this->auth->logout();
                      return redirect()->route('auth.login')->with('error', 'Your account is frozen.');
                  }
              }
            }

            if( $becomeMember ) {
                $originalAccount    = Session::pull('originalAccount', false);
                $newUser = \App\Models\User::find( $becomeMember );
                //$this->auth->logout();
                $this->auth->login( $newUser );

                Session::put( 'originalAccount', $originalAccount );
                return $next( $request );
            }

            if( $originalAccount !== false )
              Session::put('originalAccount', $originalAccount);



            if ( $this->auth->user()->hasAccess( $permission ) ) {
                $request->session()->put('userid', $request->user()->id);
                return $next( $request );
            } else {
                return $request->ajax ? response('Unauthorized.', 401) : redirect()->route( $redirectTo );
            }


        } else {
            return $request->ajax ? response('Unauthorized, please log in!.', 401) : redirect()->guest('desk/login');
        }
    }
}
