<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Support\Facades\DB;
use Session;
use Cookie;
use Auth;

class RoleMiddleware
{

    private $available_urls_at_frozen = [];

    public function handle($request, Closure $next, ...$role)
    {
        if (Auth::guest()) {
            return redirect()->route('auth.login');
        }

        $becomeMember = Session::get('becomeMember', false);
        $originalAccount = Session::get('originalAccount', false);

        if ($becomeMember == false && $originalAccount == false) {
            //Suspended
            if ($request->user()->isSuspended()) { //&& $current_url_path != '/suspended') {
                Auth::logout();
                return redirect()->route('auth.login')->with('error', 'Your account is suspended.');
            }

            //Frozen
            if ($request->user()->isFrozen()) {
                if (!in_array($current_url_path, $this->available_urls_at_frozen)) {
                    Auth::logout();
                    return redirect()->route('auth.login')->with('error', 'Your account is frozen.');
                }
            }
        }

        //Cookie::queue('sessData', Session::getId(), 360000, '/', '.purple24.net');

        if ($becomeMember) {
            $newUser = \App\Models\User::find($becomeMember);
            Auth::login($newUser);

            Session::put('originalAccount', $originalAccount);

            return $next($request);
        }

        if ($originalAccount !== false)
            Session::put('originalAccount', $originalAccount);

        $role = is_array($role)
            ? $role
            : explode('|', $role);

        if (!Auth::user()->hasAnyRole($role)) {

            return redirect()->back();
        } else {
            $request->session()->put('userid', $request->user()->id);
        }

        if ($request->user()->waitingForMLM() && !request()->is('*desk/packages*') && !request()->is('*desk/ajax/packages*') && !request()->is('*activate*')) {
            return redirect()->route('packages.purchase');
        }

        return $next($request);
    }
}