<?php

namespace App\Jobs;

use App\Exceptions\RequestCoinExpcetion;
use App\Facades\PeakHandler;
use App\Models\AuctionBids;
use App\Models\RemoteTransactions;
use App\Models\WCILastPrices;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\DB;

class ArchiveData extends Job implements ShouldQueue
{
    use InteractsWithQueue, SerializesModels;

    public $tries = 1;

    /**
     * Which table data should be archived by this job
     */
    protected $table;

    /**
     * How many of last rows should we keep in live
     */
    protected $keep_live;

    /**
     * Which attribute of rows should we archive by
     */
    protected $dependency_attribute;

    /**
     * Which value of data should the attribute be equals
     */
    protected $dependency_value;

    /**
     *
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($table, $keep_live, $dependency_attribute = null, $dependency_value = null)
    {
        $this->table = $table;
        $this->keep_live = $keep_live;
        $this->dependency_attribute = $dependency_attribute;
        $this->dependency_value = $dependency_value;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        if($this->table == 'auction_bids'){
            if(!is_null($this->dependency_attribute) && !is_null($this->dependency_value)){
                $collection = AuctionBids::where($this->dependency_attribute, $this->dependency_value)->orderBy('created_at', 'desc')->get();
                if($collection->count() > $this->keep_live){
                    $collection = $collection->slice($this->keep_live);
                    foreach($collection as $element){
                        DB::table($this->table . '_archive')->insert([
                                'id_live' => $element->id,
                                'item_id' => $element->item_id,
                                'user_id' => $element->user_id,
                                'type' => $element->type,
                                'price' => $element->price,
                                'package_bid_price' => $element->package_bid_price,
                                'created_at_live' => $element->created_at,
                                'updated_at_live' => $element->updated_at,
                                'deleted_at_live' => $element->deleted_at
                        ]);
                    }
                    foreach($collection as $element){
                        $element->forceDelete();
                    }
                }
            }
        }
    }
}
