<?php

namespace App\Jobs;

use App\Exceptions\RequestCoinExpcetion;
use App\Facades\PeakHandler;
use App\Models\RemoteTransactions;
use App\Models\WCILastPrices;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;

class RequestCoin extends Job implements ShouldQueue
{
    use InteractsWithQueue, SerializesModels;

    public $tries = 1;
    protected $remoteTransaction;
    protected $rateSource = "WCI";
    /**
     *
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(RemoteTransactions $remoteTransaction, $rateSource = "WCI")
    {
        $this->remoteTransaction = $remoteTransaction;
        $this->rateSource = $rateSource;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $remoteTransaction = $this->remoteTransaction;
        $remoteTransaction->status = 1;
        $remoteTransaction->save();


        if($this->job){
            $jobId = $this->job->getJobId();

            if($jobId){
                $remoteTransaction->job_id = $this->job->getJobId();
            }else{
                $remoteTransaction->job_id = null;
            }
        }

        $user = $remoteTransaction->user()->first();

        try{
            $rate = $remoteTransaction->coin_rate;

            if(is_null($rate)){
                $market = strtoupper($remoteTransaction->coin_type) ."/USD";

                $lastPrice = WCILastPrices::where('market', $market)->first();

                if($lastPrice){
                    $rate = $lastPrice->price;
                }
            }

            if(!\App\Facades\PeakHandler::userIsRegistered($user)) {
                \App\Facades\PeakHandler::registerUser($user);
            }

            $result = PeakHandler::requestCoinAsync($remoteTransaction->coin_type, $remoteTransaction->currency_type, $user, $remoteTransaction->amount, $rate, $remoteTransaction->note, $this->rateSource);
            $remoteTransaction->status = 2;

        } catch (RequestException $e) {
            $result = "User: ". $user->email. " ";

            if ($e->hasResponse()){
                $result .= $e->getResponse()->getBody()->getContents();
            }

            $remoteTransaction->status = 3;
        }catch( \Exception $e ) {
            $result = $e->getTraceAsString();
            $remoteTransaction->status = 3;
        }

        $remoteTransaction->result_data = $result;
        $remoteTransaction->save();

        if($remoteTransaction->status == 3){
            abort(500);
        }

    }
}
