<?php

namespace App\Jobs;

use App\Jobs\Job;
use App\Libraries\DirectCommission;
use App\Libraries\ProfitShare;
use App\Libraries\UserAwards;
use App\Models\PackageFSB;
use App\Models\UsersCvHistory;
use App\Models\WCILastPrices;
use Composer\Package\Package;
use Illuminate\Contracts\Mail\Mailer;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Client;
use Log, MailManager, DB, Carbon, PeakHandler;

use Config, ApcManager, Mail, NotificationManager;

class SetNonCommssionPackage extends Job implements ShouldQueue
{
    use InteractsWithQueue, SerializesModels;

    protected $name = 'setpackage';
    protected $description = 'Sets upline, referral data on a package purchase';

    public $tries = 1;

    public $packageId;

    protected $companyFee = 0.9;
    protected $commissionPool;
    protected $commissionCPCPool;
    protected $CPCPool = 0;
    protected $exchangerPool;

    protected $userpackage;
    protected $package;
    protected $user;

    protected $lastBtcPayIn;

    protected $directComissionBonus = 0;
    protected $fastStartBonus = 0;

    protected $userLoginToken;
    protected $sponsorUserLoginToken;

    protected $genealogyParentWallet;
    protected $uplineParentWallet;
    protected $coin_rate = null;

    public function __construct( $packageId )
    {
        $this->packageId = $packageId;
    }

    public function handle()
    {
        $skip = false;

        Log::debug('JOB - SETPACKAGE: 001 - ' . $this->packageId);

        $this->userpackage  = \App\Models\UsersPackages::where('id', $this->packageId )->first();
        $this->user         = $this->userpackage->user;
        $this->package      = $this->userpackage->packages;

        if( $this->user->isNewUser() ) {
            $this->activateUser(!$this->package->status);
        }

        /*$this->commissionPool = round($this->package->commission_pool);


        if( $this->commissionPool > 0) {
            $transactionCommission            = new \App\Models\Transactions;
            $transactionCommission->user_id   = $this->user->id;
            $transactionCommission->type      = 22;
            $transactionCommission->amount    = $this->commissionPool;
            $transactionCommission->translateOrNew('en')->note = 'Pool Transaction: $' . $this->commissionPool . ' in '. strtoupper(config('blockchain.transaction_currency')) .' coin';
            $transactionCommission->translateOrNew('cn')->note = '赠送价值: $' . $this->commissionPool . ' '. strtoupper(config('blockchain.transaction_currency')) .' ACC';
            $transactionCommission->status    = 1;
            $transactionCommission->save();

            $remoteTransaction = new \App\Models\RemoteTransactions;
            $remoteTransaction->status = 0;
            $remoteTransaction->transaction_id = $transactionCommission->id;
            $remoteTransaction->user_id = $this->user->id;
            $remoteTransaction->amount = $this->commissionPool;
            $remoteTransaction->coin_rate = 0.1;
            $remoteTransaction->coin_type = config('blockchain.transaction_currency');
            $remoteTransaction->currency_type = "usdt";
            //$remoteTransaction->note = strtoupper(config('blockchain.transaction_currency')) . ' pool transaction: '. $this->commissionPool;
            $remoteTransaction->note = $transactionCommission->translateOrNew($this->user->getUserLanguage())->note;
            $remoteTransaction->save();

            try{
                dispatch(new RequestCoin($remoteTransaction, $rateSource = "INTERNAL"));
            }catch (\Exception $exception){
                Log::debug('FAILED TO SET '. config('blockchain.transaction_currency') .' POOL - ' . $this->user->id);
            }
        }*/
    }

    private function activateUser( $justActivate = false ) {
        if( $justActivate ) {

            Log::debug('PACKAGE PURCHASE: INFO - Just activate user - ' . $this->user->id);

            $this->user->newUser = 0;
            $this->user->save();

        } else {
            $tempSponsor = \App\Models\TempSponsor::where('user_id', $this->user->id)->first();

            if(!$tempSponsor){
                return false;
            }

            $findSponsor = \App\Models\TreeGenealogyNested::where('id', $tempSponsor->sponsor_user_id)->first();

            if(!$findSponsor){
                return false;
            }

            if( !is_null( $findSponsor )) {
                $insertGenealogy = \App\Models\TreeGenealogyNested::addToTree( $findSponsor->id, $this->user->id );

                if( $insertGenealogy) {

                    $this->user->newUser = 0;
                    $this->user->save();

                    $tempSponsor->delete();

                    return true;

                } else return false;

            } else \Illuminate\Support\Facades\Log::debug('PACKAGE PURCHASE: ERROR 002 - NO SPONSOR FOUND - ' . $this->userpackage->id);

            return false;
        }
    }
}
