<?php

namespace App\Jobs;

use App\Jobs\Job;
use App\Libraries\DirectCommission;
use App\Libraries\ProfitShare;
use App\Libraries\UserAwards;
use App\Models\PackageFSB;
use App\Models\UsersCvHistory;
use App\Models\WCILastPrices;
use Composer\Package\Package;
use Illuminate\Contracts\Mail\Mailer;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Client;
use Log, MailManager, DB, Carbon, PeakHandler;

use Config, ApcManager, Mail, NotificationManager;

class SetPackage extends Job implements ShouldQueue
{
    use InteractsWithQueue, SerializesModels;

    protected $name = 'setpackage';
    protected $description = 'Sets upline, referral data on a package purchase';

    public $tries = 1;

    public $packageId;

    protected $companyFee = 0.9;
    protected $commissionPool;
    protected $commissionCPCPool;
    protected $CPCPool = 0;
    protected $exchangerPool;

    protected $userpackage;
    protected $package;
    protected $user;

    protected $lastBtcPayIn;

    protected $directComissionBonus = 0;
    protected $fastStartBonus = 0;

    protected $userLoginToken;
    protected $sponsorUserLoginToken;

    protected $genealogyParentWallet;
    protected $uplineParentWallet;
    protected $coin_rate = null;

    public function __construct( $packageId )
    {
        $this->packageId = $packageId;
    }

    public function handle()
    {
        $skip = false;

        Log::debug('JOB - SETPACKAGE: 001 - ' . $this->packageId);

        $this->userpackage  = \App\Models\UsersPackages::where('id', $this->packageId )->first();
        $this->user         = $this->userpackage->user;
        $this->package      = $this->userpackage->packages;



        //Log::debug('JOB - SETPACKAGE: userpackage - ' . $this->userpackage->id);
        //Log::debug('JOB - SETPACKAGE: user - ' . $this->user->id);

        //Deactivate current active package if exits
        \App\Models\UsersPackages::where('id','!=', $this->packageId)->where('user_id', $this->user->id)->where('status', 1)->update(['status' => 0]);

        /*if( $this->user->isNewUser() ) {
            $this->activateUser();
        }*/

        if(!$this->user->isInTree()){
            $this->activateUser();
        }


        $mlm_package = \App\Models\Packages::where('id', $this->userpackage->package_id )->first();

        $new_bid_package = new \App\Models\AuctionUsersPackages();
        $new_bid_package->user_id = $this->user->id;
        $new_bid_package->package_id = $this->userpackage->package_id;
        $new_bid_package->bids = $mlm_package->bids;
        $new_bid_package->is_mlm_package = 1;
        $new_bid_package->save();


       $directCommission = new DirectCommission($this->user);
       $directCommission->send();

       $profitShare = new ProfitShare($this->user);
       $profitShare->send();

    }

    private function activateUser() {

        Log::debug('PACKAGE PURCHASE: INFO - Just activate user - ' . $this->user->id);

        $this->user->newUser = 0;
        $this->user->save();

        $tempSponsor = \App\Models\TempSponsor::where('user_id', $this->user->id)->first();


        //Log::debug('JOB - SETPACKAGE: tempsonsor - ' . $tempSponsor->id);

        if(!$tempSponsor){

            $node = new \App\Models\TreeGenealogyNested;
            $node->id = $this->user->id;

            return $node->save();
        }

        $findSponsor = \App\Models\TreeGenealogyNested::where('id', $tempSponsor->sponsor_user_id)->first();

        //Log::debug('JOB - SETPACKAGE: $findSponsorIngenealogy - ' . $findSponsor->id);

        /*if(!$findSponsor){
            return false;
        }*/

        if( $findSponsor) {
            $insertGenealogy = \App\Models\TreeGenealogyNested::addToTree( $findSponsor->id, $this->user->id );

            if( $insertGenealogy) {

                $this->user->newUser = 0;
                $this->user->save();

                $tempSponsor->delete();

                return true;

            } else return false;

        } else \Illuminate\Support\Facades\Log::debug('PACKAGE PURCHASE: ERROR 002 - NO SPONSOR FOUND - ' . $this->userpackage->id);

        return false;
    }
}
