<?php

namespace App\Libraries;

use Illuminate\Auth\Passwords\DatabaseTokenRepository;
use Illuminate\Support\Str;
use Illuminate\Support\Carbon;
use Illuminate\Database\ConnectionInterface;
use Illuminate\Contracts\Hashing\Hasher as HasherContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;

use App\Models\User;

class CustomDatabaseTokenRepository extends DatabaseTokenRepository
{

    public function create(CanResetPasswordContract $user)
    {

        $user = User::where('id', $user->id)->first();

        $username = $user->getUsernameForPasswordReset();

        $this->deleteExisting($user);

        // We will create a new, random token for the user so that we can e-mail them
        // a safe link to the password reset form. Then we will insert a record in
        // the database so that we can verify the token within the actual reset.
        $token = $this->createNewToken();

        $this->getTable()->insert($this->getPayload($username, $token));

        return $token;
    }

    protected function deleteExisting(CanResetPasswordContract $user)
    {
        $user = User::where('id', $user->id)->first();
        return $this->getTable()->where('username', $user->getUsernameForPasswordReset())->delete();
    }


    protected function getPayload($username, $token)
    {
        return ['username' => $username, 'token' => $this->hasher->make($token), 'created_at' => new Carbon];
    }


    public function exists(CanResetPasswordContract $user, $token)
    {
        $user = User::where('id', $user->id)->first();
        $record = (array) $this->getTable()->where(
            'username', $user->getUsernameForPasswordReset()
        )->first();

        return $record &&
               ! $this->tokenExpired($record['created_at']) &&
                 $this->hasher->check($token, $record['token']);
    }
}
