<?php namespace App\Libraries;

use Illuminate\Http\Request;

class DataTables {

	protected $request;
	protected $order;
	protected $page;
	protected $start;
	protected $length;
	protected $modelClass;
	protected $options;

	protected $_defaults = [
		'order' 		=> [],
		'pagination' 	=> [
			'page' 		=> 0,
			'pages' 	=> 0,
			'perpage' 	=> 25
		],
		'query' => []
	];
	protected $_fieldDefaults = [
	];
	protected $_fieldFilters = [];

	public function __construct() {

	}

	public function init( Request $request, array $options = [] ) {
		$this->request = $request;
		$this->options = $options;

		$this->modelClass = new $this->options['model'];

		foreach( $this->_defaults as $key => $item ) {
			if( !isset( $this->options[ $key ] ) )
				$this->options[ $key ] = $this->_defaults[ $key ];
		}

		$this->sort 		= $this->request->input('sort', $this->options['order']);
		$this->pagination 	= $this->request->input('pagination', $this->options['pagination']);
		$this->query 		= $this->request->input('query', $this->options['query']);

		$this->pagination['page'] = (int) $this->pagination['page'];

		$this->start = ($this->pagination['page'] - 1) * $this->pagination['perpage'];
    	$this->length = ($this->pagination['page'] - 1) * $this->pagination['perpage'] + $this->pagination['perpage'];


    	if(!$this->length){
            $this->length = $this->pagination['perpage'];
        }

    	if( isset( $options['defaults'] ) && is_array( $options['defaults'] ) )
    		$this->_fieldDefaults = $options['defaults'];

    	return $this;
	}

	public function dbJoin( $fn ) {
		$this->modelClass = call_user_func_array( 
			$fn, 
			[ $this->modelClass ]
		);

		return $this;
	}

	public function filter( array $filters ) {
		$this->_fieldFilters = $filters;

		return $this;
	}

	public function get( $map = null ) {

		if( !empty( $this->_fieldDefaults ) ) {
			foreach( $this->_fieldDefaults as $k => $v ) {
				$query = false;

				if( !is_string( $k ) ) {
					$k = $v;
					$v = false;
				}

				$exploded = explode( '.', $k );
				$original = $k;
				$k = last( $exploded );

				if( isset( $this->query[ $k ] ) )
					$input = trim( $this->query[ $k ] );
				else 
					$input = $v;


				if( $k === 'search' ) {

					if( gettype( $v ) === 'string' )
						$v = [$v];

					if( isset( $this->query[ $k ] ) ) {

						$input = $this->query[ $k ];

						
						$query = $this->modelClass->where(function($query) use ( $v, $input ) {
							foreach( $v as $i ) {
								$query->orWhere($i, 'LIKE', '%' . $input . '%');
							}
						});

					}
				} else {
					if( $input !== '' ) {
						if( isset( $this->_fieldFilters[ $k ] ) ) {

							$query = call_user_func_array( 
								$this->_fieldFilters[ $k ], 
								[ $this->modelClass, $this->request ]
							);

						} else {
							switch ( gettype( $input ) ) {
								case 'array':
									$query = $this->modelClass->whereIn( $original, $input );
								break;
								case 'string':
									$query = $this->modelClass->where( $original, $input );
								break;
							}
						}
					}
				}

				if( $query !== false )
					$this->modelClass = $query;
			}
		}

		$orderBy = 'id';
		$dir = 'desc';

		if ( $this->sort && is_array( $this->sort ) 
			&& !empty( $this->sort )
			&& isset( $this->sort['field'] )
			&& isset( $this->sort['sort'] )
			&& !empty( $this->options['order'] )
		) {
      		$_order = $this->sort['field'];
      		$dir = strtoupper( $this->sort['sort'] );

      		if( isset( $this->options['order'][ $_order ] ) ) {
      			$orderBy = $this->options['order'][ $_order ];
      		}

           // $orderBy = $this->options['order'][ $_order ];

        }

		$model = $this->modelClass->orderBy( $orderBy, $dir );

		$datas_total = $model->count();
  		$datas_count = $model->skip( $this->start )->take( $this->pagination['perpage'] )->count();
  		$datas = $model->skip( $this->start )->take( $this->pagination['perpage'] )->get();

  		if( !is_null( $map ) && is_callable( $map ) )
  			$datas->map( $map );

  		return [
  			'data' 				=> $datas,
  			'meta'				=> [
  				'field' 	=> $orderBy,
				'page' 		=> $this->pagination['page'],
				'pages' 	=> floor( $datas_total / $this->length ),
				'perpage' 	=> $this->pagination['perpage'],
				'sort' 		=> strtolower( $dir ),
				'total' 	=> $datas_total
  			]
  		];
	}
}