<?php namespace App\Libraries;

class Date {

	public function __construct() {}

	public function now($format = null)
	{
		$now = new \DateTime('NOW');
		return ($format !== null) ? $now->format($format) : ($now->getTimestamp() + $now->getOffset());
	}

	public function format( $date, $format = null )
	{
		$date = new \DateTime( $date );
		return ( $format == null ) ? $date->format('Y-m-d H:i:s') : $date->format( $format );
	}

	public function format_short( $date )
	{
		$date = new \DateTime( $date );
		$now = new \DateTime('NOW');

		$diff = $date->diff($now);

		if($diff->days > 1)
		{
			$format = $date->format('h:i A');
		} else {
			$format = $date->format('M j');
		}

		return $format;
	}

	public function dateDiff($end)
	{
		$end = new \DateTime($end);
		$now = new \DateTime('NOW');
		
		$diff = $end->diff($now);
		
		if($diff->days <= 1)
		{
			//$left = $diff->h . ' hours left';
            $left = $diff->h . ' ' . trans('date.hours_left');
		} else {
			//$left = $diff->days . ' days left';
            $left = $diff->days . ' ' . trans('date.days_left');
		}
		
		return $left;
	}
	
	public function difference($format = 'd', $date)
	{
		$date = new \DateTime($date);
		$now = new \DateTime('NOW');
		
		$diff = $date->diff($now);
		$return = $diff->$format;
				
		return $return;
	}

	public function convertDuration( $sec, $nice = false )
	{
		$sec = round($sec);
		$hours = $sec / 3600;
		$minutes = $sec / 60 % 60;
		$seconds = $sec % 60;

		$hours = floor($hours);
		$minutes = floor($minutes);
		//$seconds = floor($seconds);

		if( $nice ) {
			$string = '';

			if( $hours > 0 ) $string .= $hours . ' ' . trans('date.hour') . ( ( $hours > 1 ) ? trans('date.s') : '' );
			if( $minutes > 0 ) $string .= ( ( $string !== '' ) ? ', ' : '' ) .$minutes . ' ' . trans('date.minute') . ( ( $minutes > 1 ) ? trans('date.s') : '' );
			//if( $seconds > 0 ) $string .= ( ( $string !== '' ) ? ' and ' : '' ) . $seconds . ' second' . ( ( $seconds > 1 ) ? 's' : '' );

			return $string;

		} else {
			$string = '';
			if( $hours > 0 ) $string .= '%02d:';

			$string .= '%02d:%02d';

			return ( $hours > 0 ) ? sprintf( $string, $hours, $minutes, $seconds ) : sprintf( $string, $minutes, $seconds );
		}
	}
	
	public function ago($timestamp, $append_full_date = false)
	{
		if( is_null( $timestamp ) ) return trans('date.never');
		
		$now = new \DateTime('NOW');
		$date = new \DateTime($timestamp);
		
		$diff = $now->diff($date);
		
		if($diff->i < 1)
		{
			$ago = ($diff->s > 3) ? $diff->s . ' ' . trans('date.seconds_ago') : trans('date.a_moment_ago');
		}
		elseif($diff->h < 1)
		{
			$ago = $diff->i . ( ($diff->i > 1) ? ' ' . trans('date.minutes_ago') : ' ' . trans('date.minute_ago') );
		} 
		elseif($diff->d < 1)
		{
			$ago = $diff->h . ( ($diff->h > 1) ? ' ' . trans('date.hours_ago') : ' ' . trans('date.hour_ago') );
		} 
		else {
			$ago = $diff->days . ( ($diff->days > 1) ? ' ' . trans('date.days_ago') : ' ' . trans('date.day_ago') );
		}
		
		return ($append_full_date) ? $ago . '(' . $timestamp . ')' : $ago;
	}
	
	public function time_elapsed_string($datetime, $full = false) {
		$now = new \DateTime;
		$ago = new \DateTime($datetime);
		$diff = $now->diff($ago);
	
		$diff->w = floor($diff->d / 7);
		$diff->d -= $diff->w * 7;
	
		$string = array(
			'y' => trans('date.year'),
			'm' => trans('date.month'),
			'w' => trans('date.week'),
			'd' => trans('date.day'),
			'h' => trans('date.hour'),
			'i' => trans('date.minute'),
			's' => trans('date.second'),
		);
		foreach ($string as $k => &$v) {
			if ($diff->$k) {
				$v = $diff->$k . ' ' . $v . ($diff->$k > 1 ? trans('date.s') : '');
			} else {
				unset($string[$k]);
			}
		}
	
		if (!$full) $string = array_slice($string, 0, 1);
		return $string ? implode(', ', $string) . ' ' . trans('date.ago') : trans('date.just_now');
	}
}