<?php namespace App\Libraries;

use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;

class MailManager {

	protected $defaults = [
		'port' => 465,
		'encryption' => 'ssl',
		'host' => 'localhost'
	];

	protected $from = [

	];

	public function __construct()
	{
		$this->defaults['host'] = Config::get('mail.host');
		$this->defaults['password'] = Config::get('mail.password');
		$this->defaults['username'] = Config::get('mail.username');
        $this->defaults['from'] = Config::get('mail.from');
	}

	public function client( array $data = [] ) {
		Config::set('port', ( isset( $data['port'] ) ? $data['port'] : $this->defaults['port'] ) );
		Config::set('host', ( isset( $data['host'] ) ? $data['host'] : $this->defaults['host'] ) );
		Config::set('username', $data['username'] );
		Config::set('password', $data['password'] );
		Config::set('encryption', ( isset( $data['encryption'] ) ? $data['encryption'] : $this->defaults['encryption'] ) );

		return $this;
	}

	public function from( array $data = [] ) {
        $this->defaults['from'] = $data;

		return $this;
	}

    public function send( $view, $subject, array $data = [], $user = null ) {

        Log::debug('Mail - in send method....');

        if( is_null( $user ) ) {
            if( !Auth::check() ) throw new \Exception('Must provide user to send email.');

            $user = Auth::user();
        }

        $data = array_merge( $data, [
            'user' => $user
        ]);

        /*
        $currentLocale = App::getLocale();

        if( $currentLocale !== $user->profile->language_id )
            App::setLocale( $user->profile->language_id );

        */
        $currentLocale = Config::get('app.locale');


        Log::debug('Mail - start sending....');


        Log::debug($this->defaults['from']['address']);
        Log::debug(Config::get('mail.username'));
        Log::debug($this->defaults['from']['name']);
        Log::debug(Config::get('app.name'));
        Log::debug($user->email);
        Log::debug($user->fullName());
        Log::debug($subject);
        Log::debug($view);

        Log::debug($data);
        Log::debug($user);



        Mail::send(
            'emails.' . $view,
            $data,
            function( $message ) use ( $user, $subject ) {

                $message->from( ( isset( $this->defaults['from']['address'] ) ? $this->defaults['from']['address'] : Config::get('mail.username') ), ( isset( $this->defaults['from']['name'] ) ? $this->defaults['from']['name'] : Config::get('app.name') ) );
                $message->to( $user->email, $user->fullName() );
                $message->subject( $subject );

            });


        Log::debug('Mail - sent');
        App::setLocale( $currentLocale );
    }


}
