<?php namespace App\Libraries;

use Config, Mail, Redis, Carbon\Carbon;
use Illuminate\Support\Facades\Log;
use Symfony\Component\Console\Output\ConsoleOutput;

class NotificationManager {

	protected $templates = [];

	public function __construct()
	{
         $this->templates = [
             'messages' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'icon' => 'fa fa-envelope-o',
                 'route' => 'mailbox.inbox',
                 //'route_params' => ['id' => 'linked_content_id'],
                 'route_append' => ['id' => 'linked_content_id']
             ],
             'support-new-reply' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'New reply received (#%linked_content_id%)',
                 'content' => trans('notification.support-new-reply') . ' (#%linked_content_id%)',

                 'icon' => 'fa fa-tags',
                 'route' => 'support.view',
                 'route_params' => ['id' => 'linked_content_id']
                 //'route_append' => ['id' => 'linked_content_id']
             ],
             'support-reopen' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'Support ticket reopened (#%linked_content_id%)',
                 'content' => trans('notification.support-reopen') . ' (#%linked_content_id%)',
                 'icon' => 'fa fa-tags',
                 'route' => 'support.view',
                 'route_params' => ['id' => 'linked_content_id']
                 //'route_append' => ['id' => 'linked_content_id']
             ],
             'support-closed' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'Support ticket closed (#%linked_content_id%)',
                 'content' => trans('notification.support-closed') . ' (#%linked_content_id%)',
                 'icon' => 'fa fa-tags',
                 'route' => 'support.view',
                 'route_params' => ['id' => 'linked_content_id']
                 //'route_append' => ['id' => 'linked_content_id']
             ],
             /*'wallet.transfer-to' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 'content' => 'Money Sent to member (#%linked_content_id%)',
                 'icon' => 'fa fa-money',
                 'route' => 'transactions.index',
                 'route_append' => ['id' => 'linked_content_id']
             ],
             'wallet.transfer-from' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 'content' => 'Money Received from member (#%linked_content_id%)',
                 'icon' => 'fa fa-money',
                 'route' => 'transactions.index',
                 'route_append' => ['id' => 'linked_content_id']
             ],
             'wallet.cashout' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 'content' => 'Cashout requested (#%linked_content_id%)',
                 'icon' => 'fa fa-money',
                 'route' => 'transactions.index',
                 'route_append' => ['id' => 'linked_content_id']
             ],
             'token.interest' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 'content' => 'Earned profit on plan (#%linked_content_id%)',
                 'icon' => 'fa fa-pie-chart',
                 'route' => 'transactions.index',
                 'route_append' => ['id' => 'linked_content_id']
             ],
             'mlm.referral-commission' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 'content' => 'Received referral commission (#%linked_content_id%)',
                 'icon' => 'fa fa-bar-chart',
                 'route' => 'transactions.index',
                 'route_append' => ['id' => 'linked_content_id']
             ],
             'mlm.binary-commission' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 'content' => 'Received binary commission (#%linked_content_id%)',
                 'icon' => 'fa fa-bar-chart',
                 'route' => 'transactions.index',
                 'route_append' => ['id' => 'linked_content_id']
             ],


             'test_type' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 'content' => 'Something happened (#%linked_content_id%)',
                 'icon' => 'fa fa-bar-chart',
                 'route' => 'auction/',
                 'route_append' => ['id' => 'linked_content_id']
             ]*/
             'user-verified' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'Your auction item overbidded (#%linked_content_id%)',
                 'content' => trans('notification.user-verified'). ' (#%linked_content_id%)',
                 'icon' => 'fa fa-certificate',
                 'route' => 'user.profile',
                 //'route_params' => []
             ],

             'auction-overbid' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'Your auction item overbidded (#%linked_content_id%)',
                 'content' => trans('notification.auction-overbid') . ' (#%linked_content_id%)',
                 'icon' => 'fa fa-gavel',
                 'route' => 'auction.view',
                 'route_params' => ['id' => 'linked_content_id']
             ],
             'auction-wonitem' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'Congratulation! You have won an item (#%linked_content_id%)',
                 'content' => trans('notification.auction-wonitem') . ' (#%linked_content_id%)',
                 'icon' => 'fa fa-gavel',
                 'route' => 'auction.view',
                 'route_params' => ['id' => 'linked_content_id']
             ],
             'user-deposit' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'You have a new deposit (#%linked_content_id%)',
                 'content' => trans('notification.user-deposit') . ' (#%linked_content_id%)',
                 'icon' => 'fa fa-gavel',
                 'route' => 'deposit.view',
                 'route_params' => ['id' => 'linked_content_id']
             ],
             'user-auction-package-purchase' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'You have successfully purchased a new auction bid package! (#%linked_content_id%)',
                 'content' => trans('notification.user-auction-package-purchase') . ' (#%linked_content_id%)',
                 'icon' => 'fa fa-money',
                 'route' => 'auction.packages',
                 //'route_params' => ['id' => 'linked_content_id']
             ],
             'user-auction-package-runout' => [
                 'vars' => ['id', 'avatar', 'content_id', 'fullname', 'content', 'date', 'status'],
                 'content_vars' => ['linked_content_id'],
                 //'content' => 'You have run out of an auction bid package! (#%linked_content_id%)',
                 'content' => trans('user-auction-package-runout') . ' (#%linked_content_id%)',
                 'icon' => 'fa fa-money',
                 'route' => 'auction.packages',
                 //'route_params' => ['id' => 'linked_content_id']
             ],


         ];
	}

	public function create( array $data = [], $user) {


        //clock()->info("In noti creation: " . $user->id);
       // Log::error('Found id: '. $user->id);
        //Log::error('Found user: '. print_r($user, true));
	    //if($user) {
            $notification = [
                'user_id' => $user->id,
                'type' => $data['type'],
                'linked_user_id' => (isset($data['linked_user_id']) ? $data['linked_user_id'] : NULL),
                'linked_content_id' => (isset($data['content_id']) ? $data['content_id'] : NULL),
                'status' => 0,
                'created_at' => \App\Models\Date::format(),
                'updated_at' => \App\Models\Date::format()
            ];
        //}
        /*else{
            $notification = [
                'user_id' => $data['user']->id,
                'type' => $data['type'],
                'linked_user_id' => (isset($data['linked_user_id']) ? $data['linked_user_id'] : NULL),
                'linked_content_id' => (isset($data['content_id']) ? $data['content_id'] : NULL),
                'status' => 0,
                'created_at' => \App\Models\Date::format(),
                'updated_at' => \App\Models\Date::format()
            ];
        }*/

		if( $n = \App\Models\Notifications::insert( $notification ) ) {
			//Redis::publish('notifications', json_encode( $this->build( $n ) ) );

            //if($user) {

                \App\Facades\NodeGWManager::send('api/socketio/user/newNotification', [
                    'user' => $user->id
                ]);
            //}
            /*else{
                \App\Facades\NodeGWManager::send('api/socketio/user/newNotification', [
                    'user' => $data['user']->id
                ]);
            }*/
  	}

  	return $this;
	}


	public function build( $notification ) {
        //clock()->info("In manager");
		if( isset( $notification->type ) && isset( $this->templates[$notification->type]['vars'] ) ) {


			//if( $notification->status == 0 ) $counter++;
			$result = [];

			foreach( $this->templates[$notification->type]['vars'] as $var ) {
				switch ( $var ) {
					case 'date':
						$value = \App\Facades\Date::ago($notification->created_at);
						break;
					case 'avatar':
						$value = asset('avatars/avatar.png');
						break;
					default:
						$value = ( isset($notification->$var) ) ? $notification->$var : '';
						break;
				}

				$result[ $var ] = $value;
			}

			$result[ 'icon' ] = isset($this->templates[$notification->type]['icon']) ? $this->templates[$notification->type]['icon'] : 'fa fa-user';

			if( isset( $this->templates[ $notification->type ]['content'] ) ) {
				if( isset( $this->templates[ $notification->type ]['content_vars'] ) ){
					$contentVars = [];
					$replaceVars = [];
					foreach( $this->templates[ $notification->type ]['content_vars'] as $rv ) {
						$contentVars[] = '%' . $rv . '%';
						$replaceVars[] = $notification->$rv;
					}
					$result[ 'content' ] = str_replace($contentVars, $replaceVars, $this->templates[ $notification->type ]['content']);
				}
				else
					$result[ 'content' ] = $this->templates[ $notification->type ]['content'];
			}

			if( isset($this->templates[$notification->type]['route']) ) {
				$route_params = [];
				$route_append = '';

				if( isset($this->templates[$notification->type]['route_params']) && !empty($this->templates[$notification->type]['route_params']) ) {
					$route_params = $this->templates[$notification->type]['route_params'];
					array_walk($route_params, function(&$item, $key) use ($notification) {
						$item = $notification->$item;
					});
				} elseif( isset($this->templates[$notification->type]['route_append']) && !empty($this->templates[$notification->type]['route_append']) ) {
					$route_append = $this->templates[$notification->type]['route_append'];
					array_walk($route_append, function(&$item, $key) use ($notification) {
						$item = $key . '=' . $notification->$item;
					});
					$route_append = '?' . implode('&', $route_append);
				}

				$result[ 'link' ] = route( $this->templates[$notification->type]['route'], $route_params ) . $route_append;
			}

			return $result;
		}
	}

}
