<?php 

namespace App\Libraries;

use App\Models\User;
use DB, GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Support\Facades\Log;

class PeakHandler {

	protected $url;
	protected $useDB = false;

	public function __construct() {
        $this->url = config('peak.api.url');
	}

    public function requestCoinAsync($coinType, $currency, $user, $amount = 0, $rate = null , $comment = null, $rateSource = "WCI") {

        $payload = [
            "coinType" => $coinType,
            "currencyType" => $currency,
            "currencyAmount" => $amount,
            "userEmail" => $user->email,
            "payoutType" => "COLIBRI",
            "rateSource" => $rateSource
        ];

        if( !is_null( $rate ) ){
            $payload['rate'] = $rate;
        }

        if( !is_null( $comment ) ){
            $payload['comment'] = $comment;
        }

        $cpcRequest = $this->sendRequestAsync( 'sendcoin', $payload);

        return $cpcRequest;
    }


	public function registerUser(User $user, $password = null){

	    if(is_null($password)){
	        $password = str_random(16);
        }

        $phone = "+7824784278";

        if($user->profile()->first()){
            $phone = $user->profile()->first()->phone;
        }

	    $payload = [
            "email" => $user->email,
            "username" => $user->username,
            "phoneNumber" => $phone,
            "firstname" => $user->first_name,
            "lastname" => $user->last_name,
            "credentials" => $password,
        ];

        return $this->sendRequest('user', $payload);

    }

    public function userIsRegistered($user){
        $payload = [
            "email" => $user->email
        ];

        return $this->sendRequest('checkuser', $payload, 'POST');
    }


	private function sendRequest( $name, $data , $method = "POST") {
		try {
			$request  = new Client;

	        $peakRequest  = $request->request($method, $this->url . '/' . $name, [
	            'connect_timeout' => 60,
	            'json' => $data,
                //'debug' => true,
	        ]);

            $resultJson = json_decode( $peakRequest->getBody(), true );

            Log::error($peakRequest->getBody());
            Log::error($resultJson);

	        return $resultJson;
        } catch (RequestException $e) {

            if ($e->hasResponse()){
                Log::error(print_r($data, true) .' - '. $e->getResponse()->getBody()->getContents());
            }

            return null;

	    } catch( \Exception $e ) {
            Log::error($e->getTraceAsString());
	    	return null;
	    }
	}

    private function sendRequestAsync( $name, $data , $method = "POST") {
        $request  = new Client;
        $peakRequest  = $request->request($method, $this->url . '/' . $name, [
            'connect_timeout' => 60,
            'json' => $data,
        ]);

        $resultJson = json_decode( $peakRequest->getBody(), true );

        return $resultJson;

    }
}