<?php

namespace App\Listeners;

use App\Events\ManualBidEvent;
use App\Events\ManualBidEventNow;
use App\Facades\NodeGWManager;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Support\Facades\Log;
use Session;
use Symfony\Component\Console\Output\ConsoleOutput;

class ManualBidEventNowListener implements ShouldQueue
{

    public $queue = 'default';
    public $connection = 'database';
    public $delay = 0;

    public $tries = 1;

    /**
     * Create the event listener.
     *
     * @return void
     */
    public function __construct(/*ManualBidEvent $event*/)
    {
        /*if($event->immediately == true){
            $this->delay = 0;
        }*/
    }

    /**
     * Handle the event.
     *
     * @param  ManualBidEvent  $event
     * @return void
     */
    public function handle(ManualBidEventNow $event)
    {
        if (!is_null($event->user_id) && !is_null($event->item_id)) {

            $item = \App\Models\AuctionItems::where('id', $event->item_id)->first();

            if(!$item->isbusy){

                $item->isbusy = 1;
                $item->busied_at = Carbon::now();
                $item->save();

                $last_bid = \App\Models\AuctionBids::where('item_id', $item->id)->orderby('id', 'desc')->first();
                $last_bidder = null;

                if($last_bid) {
                    $last_bidder = \App\Models\User::where('id', $last_bid->user_id)->first();
                }

                if(!$last_bidder) {
                    $last_bidder = \App\Models\User::where('id', $event->user_id)->first();
                }

                $bid_butlers = \App\Models\AuctionBidButlers::where('item_id', $item->id)->orderBy('id');

                if($last_bidder){
                    $bid_butlers->where('user_id','!=', $last_bidder->id);
                }

                $bid_butlers = $bid_butlers->get();

                if (count($bid_butlers) > 0) {

                    //The auction item is not busy
                    foreach ($bid_butlers as $butler) {

                        $act_user = \App\Models\User::where('id', $butler->user_id)->first();
                        $act_item = \App\Models\AuctionItems::where('id', $butler->item_id)->first();

                        if ($butler->counter > 0) {
                            $butler->counter = $butler->counter - 1;
                            $butler->save();
                            //$last_bidder = $act_user;

                            $this->pushBid($act_user, $act_item, true, $last_bidder);
                        }
                        else{
                            $butler->delete();
                            //$bid_butlers->forget($butler);
                        }
                    }
                }

                $waiting_bids = \App\Models\WaitingUserBids::where('item_id', $item->id)->orderBy('id')->get();

                if (count($waiting_bids) > 0) {

                    foreach ($waiting_bids as $bid) {
                        $act_user = \App\Models\User::where('id', $bid->user_id)->first();
                        $act_item = \App\Models\AuctionItems::where('id', $bid->item_id)->first();


                        $this->pushBid($act_user, $act_item, false, $last_bidder);

                        $bid->delete();

                    }
                }

                $item->isbusy = 0;
                $item->busied_at = null;
                $item->save();

                if(count($bid_butlers) > 0)
                {
                    event(new ManualBidEvent($last_bidder->id, $item->id));
                }
                else{

                    $bot_guard = User::where('id', User::getBotIds()[$this->__generateRandom(0, count(User::getBotIds()) - 1)])->first();

                    if($item->bot_need_win == 1 && (!in_array($last_bidder->id, User::getBotIds()))){
                        $this->autoBid($item, $bot_guard, $last_bidder);
                    }
                }
            }

        }
    }

    public function pushBid($user, $item, $is_butler, $last_bidder)
    {
        $newBid = new \App\Models\AuctionBids;
        $newBid->item_id = $item->id;
        $newBid->user_id = $user->id;

        if($is_butler) {
            $newBid->type = 2;
        }
        else{
            $newBid->type = 1;
        }

        $newBid->price = $item->per_bid_price;

        $deduces = $user->deduceBid($item->per_bid_price);

        if($deduces === false) {
            return false;
        }

        $newBid->package_bid_price = $deduces[0]['price'];

        if ($newBid->save()) {

            $item->current_price = $item->current_price + $newBid->price;
            $item->ends_at = $item->ends_at->addSeconds(10);

            if ($item->save()) {
                $newBid->username = $user->username;
                $newBid->created_at_format = $newBid->created_at->format('Y.m.d H:i:s');
                //$item->ends_at_ts = $item->ends_at->timestamp;

                $data = $item->toArray();
                $data["ends_at_ts"] = $item->ends_at->timestamp;

                //$last_bid_id = \App\Models\AuctionBids::where('item_id', $item->id)->where('user_id', $user->id)->orderby('id', 'desc')->first()->id;

                //Log::error('evtlistener/Noti user id: '. $user->id);
                //Log::error('evtlistener/Noti last bidder '.$last_bidder->id);

                if($last_bidder && $last_bidder != $user) {
                    //Log::error('evtlistener/In noti process '. $user->id);
                    \App\Facades\NotificationManager::create([
                        'user_id' => $last_bidder->id,
                        'type' => 'auction-overbid',
                        //'content_id' => $last_bid_id
                        'content_id' => $item->id
                    ], $last_bidder);
                }

                NodeGWManager::send('api/socketio/auction/newBid', [
                    'item' => $data,
                    'newBids' => [$newBid->toArray()],
                    'is_butler' => $is_butler
                ]);

            }
        }
    }

    public function __generateRandom( $min, $max ) {
        return (int) random_int( (int) $min, (int) $max );
    }

    public function autoBid($item, $bot, $last_bidder_user)
    {
        //$this->info(Carbon::now() . ' - Bidding: ' . $bot->id);
        if (!isset($bot))
            return false;
        if (!$bot->hasBids()) {
            $new_package = new \App\Models\AuctionUsersPackages;
            $new_package->user_id = $bot->id;
            $new_package->package_id = 2;
            $new_package->bids = 1000000;
            $new_package->save();
        }

        //$last_bid = \App\Models\AuctionBids::where('item_id', $item->id)->orderby('id', 'desc')->first();
        //$last_bidder = null;

        /*if($last_bid) {
            $last_bidder_user = \App\Models\User::where('id', $last_bid->user_id)->first();
        }*/

        $rate = $this->__generateRandom(0, 3);

        if($rate == 0) {

            sleep($this->__generateRandom(1, 3));

            if(is_null( $item->lastBidder ) || $item->lastBidder->user->id != $bot->id) {

                /*if(Carbon::parse($item->ends_at)->subSeconds(1) >  $last_bid->created_at){
                    $new_bid_time = Carbon::parse($item->ends_at)->subSeconds(1);
                }
                else{
                    $new_bid_time = $item->ends_at;
                }*/

                $newBid = new \App\Models\AuctionBids;
                $newBid->item_id = $item->id;
                $newBid->user_id = $bot->id;
                $newBid->type = 1;
                $newBid->price = $item->per_bid_price;
                //$newBid->created_at = $new_bid_time;
                //$newBid->updated_at = $new_bid_time;


                $deduces = $bot->deduceBid($item->per_bid_price);
                $newBid->package_bid_price = $deduces[0]['price'];


                if ($newBid->save()) {

                    $item->current_price = $item->current_price + $newBid->price;
                    $item->ends_at = $item->ends_at->addSeconds(10);

                    if ($item->save()) {
                        $newBid->username = $bot->username;
                        $newBid->created_at_format = $newBid->created_at->format('Y.m.d H:i:s');

                        $data = $item->toArray();
                        $data["ends_at_ts"] = $item->ends_at->timestamp;


                        \App\Facades\NotificationManager::create([
                            'user_id' => $last_bidder_user->id,
                            'type' => 'auction-overbid',
                            //'content_id' => $last_bid_id
                            'content_id' => $item->id
                        ], $last_bidder_user);


                        NodeGWManager::send('api/socketio/auction/newBid', [
                            'item' => $data,
                            'newBids' => [$newBid->toArray()]
                        ]);

                        event(new ManualBidEvent($bot->id, $item->id));
                    }
                }
            }
        }
        else{

            if( !is_null( $bidButler = \App\Models\AuctionBidButlers::where('item_id', $item->id)->where('user_id', $bot->id)->first() ) ) {
                $bidButler->counter = $bidButler->counter + $this->__generateRandom(1, 5);
                $bidButler->save();
            } else {
                $bidButler = new \App\Models\AuctionBidButlers;
                $bidButler->item_id = $item->id;
                $bidButler->user_id = $bot->id;
                $bidButler->counter = $this->__generateRandom(1, 5);
            }

            if($bidButler->save()){
                if(is_null( $item->lastBidder ) || $item->lastBidder->user->id != $bot->id){
                    event(new ManualBidEventNow($bot->id, $item->id));
                }

                return $this->responseNew(null, [
                    'result' => true,
                    'counter' => $bidButler->counter,
                    'multiplier' => $item->per_bid_price / config('auction.bid_price')
                ]);
            }
        }

    }
}
