<?php
namespace App\Models;

use DB, Carbon, Auth;

class AuctionItems extends \Illuminate\Database\Eloquent\Model {

    use \Astrotomic\Translatable\Translatable;

    protected $table = 'auction_items';
  	protected $dates = ['created_at', 'ends_at', 'updated_at'];
    public $ends_at_ts = null;
    public $translatedAttributes = ['title', 'description'];


    public function getEnumStatus(){
        return $this->hasOne('\App\Models\Enums', 'model_id', 'status')
            ->where('model_name', '=', 'auction_items')
            ->where('model_field', '=', 'status')
            ->where('status', '=', 1);
    }

  	public function applyRoles() {

  	}

    public function isEnded() {
      return $this->ends_at->timestamp <= Carbon::now()->timestamp;
    }

  	public function category() {
  		return $this->belongsTo('\App\Models\AuctionCategories', 'category_id', 'id');
  	}

  	public function bids() {
  		return $this->hasMany('\App\Models\AuctionBids', 'item_id', 'id');
  	}

    public function limitedBids() {
      return $this->hasMany('\App\Models\AuctionBids', 'item_id', 'id')->orderBy('id', 'DESC')->take(10);
    }

    public function bidButlers() {
      return $this->hasMany('\App\Models\AuctionBidButlers', 'item_id', 'id');
    }

    public function myBidButlers() {
      return $this->hasOne('\App\Models\AuctionBidButlers', 'item_id', 'id')->where('user_id', Auth::user()->id);
    }

  	public function lastBidder() {
  		//return $this->hasOne('\App\Models\AuctionBids', 'item_id', 'id')->orderBy('id', 'DESC');

        return $this->hasOne('\App\Models\AuctionBids', 'id', 'last_bid_id')->orderBy('id', 'DESC');
  	}

    /*public function getLastBidderUser() {
        return User::where('id', $this->lastBidder()->first()->user_id);
    }*/

    public function watchlist() {
      return $this->hasOne('\App\Models\AuctionWatchlist', 'item_id', 'id')->where('user_id', Auth::user()->id);
    }

    public function isWatching() {
      if( !is_null( $this->watchlist ) )
        return true;

      return false;
    }

  	public function images() {
  		return $this->hasMany('\App\Models\AuctionImages', 'item_id', 'id');
  	}

  	public function scopeEndingSoon( $query ) {
  		return $query->orderBy('ends_at', 'ASC');
  	}

  	public function scopeNotEnded( $query ) {
  		return $query->where('ends_at', '>', DB::raw('NOW()'));
  	}

  	public function thumbnail( ) {
  		return $this->hasOne('\App\Models\AuctionImages', 'item_id', 'id')->where('order', 0);
  	}

  	public function user() {
  		return $this->belongsTo('\App\Models\User', 'user_id', 'id');
  	}


}