<?php namespace App\Models;

class Date {

	public static function now($format = null)
	{
		$now = new \DateTime('NOW');
		return ($format !== null) ? $now->format($format) : ($now->getTimestamp() + $now->getOffset());
	}

	public static function format( $date = false, $format = null )
	{
		if( !$date ) $date = new \DateTime;
		else $date = new \DateTime( $date );
		
		return ( $format == null ) ? $date->format('Y-m-d H:i:s') : $date->format( $format );
	}

	public static function format_short( $date )
	{
		$date = new \DateTime( $date );
		$now = new \DateTime('NOW');

		$diff = $date->diff($now);

		if($diff->days > 1)
		{
			$format = $date->format('h:i A');
		} else {
			$format = $date->format('M j');
		}

		return $format;
	}

	public static function dateDiff($end)
	{
		$end = new \DateTime($end);
		$now = new \DateTime('NOW');
		
		$diff = $end->diff($now);
		
		if($diff->days <= 1)
		{
			$left = $diff->h . ' hours left';
		} else {
			$left = $diff->days . ' days left';
		}
		
		return $left;
	}
	
	public static function difference($format = 'd', $date)
	{
		$date = new \DateTime($date);
		$now = new \DateTime('NOW');
		
		$diff = $date->diff($now);
		$return = $diff->$format;
				
		return $return;
	}

	public static function convertDuration( $sec, $nice = false )
	{
		$sec = round($sec);
		$hours = $sec / 3600;
		$minutes = $sec / 60 % 60;
		$seconds = $sec % 60;

		$hours = floor($hours);
		$minutes = floor($minutes);
		//$seconds = floor($seconds);

		if( $nice ) {
			$string = '';

			if( $hours > 0 ) $string .= $hours . ' hour' . ( ( $hours > 1 ) ? 's' : '' );
			if( $minutes > 0 ) $string .= ( ( $string !== '' ) ? ', ' : '' ) .$minutes . ' minute' . ( ( $minutes > 1 ) ? 's' : '' );
			//if( $seconds > 0 ) $string .= ( ( $string !== '' ) ? ' and ' : '' ) . $seconds . ' second' . ( ( $seconds > 1 ) ? 's' : '' );

			return $string;

		} else {
			$string = '';
			if( $hours > 0 ) $string .= '%02d:';

			$string .= '%02d:%02d';

			return ( $hours > 0 ) ? sprintf( $string, $hours, $minutes, $seconds ) : sprintf( $string, $minutes, $seconds );
		}
	}
	
	public static function ago($timestamp, $append_full_date = false)
	{
		if( is_null( $timestamp ) ) return 'Never';
		
		$now = new \DateTime('NOW');
		$date = new \DateTime($timestamp);
		
		$diff = $now->diff($date);
		
		if($diff->i < 1)
		{
			$ago = ($diff->s > 3) ? $diff->s . ' seconds ago' : 'A moment ago';
		}
		elseif($diff->h < 1)
		{
			$ago = $diff->i . ( ($diff->i > 1) ? ' minutes ago' : ' minute ago' );
		} 
		elseif($diff->d < 1)
		{
			$ago = $diff->h . ( ($diff->h > 1) ? ' hours ago' : ' hour ago' );
		} 
		else {
			$ago = $diff->days . ( ($diff->days > 1) ? ' days ago' : ' day ago' );
		}
		
		return ($append_full_date) ? $ago . '(' . $timestamp . ')' : $ago;
	}
	
	public static function time_elapsed_string($datetime, $full = false) {
		$now = new \DateTime;
		$ago = new \DateTime($datetime);
		$diff = $now->diff($ago);
	
		$diff->w = floor($diff->d / 7);
		$diff->d -= $diff->w * 7;
	
		$string = array(
			'y' => 'year',
			'm' => 'month',
			'w' => 'week',
			'd' => 'day',
			'h' => 'hour',
			'i' => 'minute',
			's' => 'second',
		);
		foreach ($string as $k => &$v) {
			if ($diff->$k) {
				$v = $diff->$k . ' ' . $v . ($diff->$k > 1 ? 's' : '');
			} else {
				unset($string[$k]);
			}
		}
	
		if (!$full) $string = array_slice($string, 0, 1);
		return $string ? implode(', ', $string) . ' ago' : 'just now';
	}
}