<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

use DB;

class Groups extends Model
{
    protected $table = 'groups';

    public static function findByName($name)
    {
        $model = self::createModel();

        if ( ! $group = $model->newQuery()->where('name', '=', $name)->first())
        {
            throw new \Exception("A group could not be found with the name [$name].");
        }

        return $group;
    }

    public static function findByPermission($permission)
    {
        $model = self::createModel();

        if ( ! $group = $model->newQuery()->where('permissions', 'LIKE', '%"'. $permission . '":1%')->get() )
        {
            throw new \Exception("A group could not be found with permission: [$permission].");
        }

        return $group;
    }

    public static function usersByPermission($permission)
    {
        $groups = self::findByPermission($permission);
        $users = false;

        foreach( $groups as $g ) {
            if( !$users ) $users = $g->users()->get();
            else $users = $users->merge( $g->users()->get() );
        }

        return $users->all();
    }

    public static function usersByName($name)
    {
        return self::findByName($name)->users()->get();
    }

    public static function createModel()
    {
        $class = '\\'.ltrim(Groups::class, '\\');

        return new $class;
    }

    public function getId()
    {
        return $this->getKey();
    }

    public function getName()
    {
        return $this->name;
    }

    public function getPermissions()
    {
        return $this->permissions;
    }

    public function Users()
    {
        return $this->belongsToMany('\App\Models\User', 'users_groups');
    }

    public function getPermissionsAttribute($permissions)
    {
        if ( ! $permissions)
        {
            return array();
        }

        if (is_array($permissions))
        {
            return $permissions;
        }

        if ( ! $_permissions = json_decode($permissions, true))
        {
            throw new \InvalidArgumentException("Cannot JSON decode permissions [$permissions].");
        }

        return $_permissions;
    }
    
    public function setPermissionsAttribute(array $permissions)
    {
        // Merge permissions
        $permissions = array_merge($this->getPermissions(), $permissions);

        // Loop through and adjust permissions as needed
        foreach ($permissions as $permission => &$value)
        {
            // Lets make sure their is a valid permission value
            if ( ! in_array($value = (int) $value, $this->allowedPermissionsValues))
            {
                throw new \InvalidArgumentException("Invalid value [$value] for permission [$permission] given.");
            }

            // If the value is 0, delete it
            if ($value === 0)
            {
                unset($permissions[$permission]);
            }
        }

        $this->attributes['permissions'] = ( ! empty($permissions)) ? json_encode($permissions) : '';
    }
}