<?php namespace App\Models;

use DB;

class Sponsor extends \Illuminate\Database\Eloquent\Model
{

	public $table = 'sponsors';
	public $timestamps = false;
	public $incrementing = false;
	protected $primaryKey = 'user_id';
	protected $dates = ['lastActivationDate'];

    public function user()
    {
        return $this->belongsTo('App\Models\User', 'user_id', 'id');
    }

    public function isStatusActive() {
        return (bool) $this->activeStatus;
    }

	public function child() {
		return ( $this->right - $this->left - 1 );
	}

    public function directs() {
        return $this->hasMany('\App\Models\Sponsor', 'sponsor_id', 'user_id');
    }

	public function availablePositions()
	{
		$result = DB::table('sponsors AS s')
		->select(DB::raw('s.*, u.username'))
		->where('children', '<', '3')
		//->whereBetween('left', [ $this->left, $this->right ])
		->orderBy('left', 'ASC')
		->orderBy('level', 'asc')
		->orderBy('position', 'asc');

		$result->leftJoin('users AS u', 'u.id', '=', 's.user_id');

		return $result->get();
	}

	public static function addToTree( $sponsor, $upline_user, $me, $leg ) {
    // Table structure
    //        user_id, user_code, 
    //        sponsor_id, sponsor_code, left, right,
    //        upline_user_id, upline_user_code, upline_left, upline_right, children, level, position, group_id
        //DB::beginTransaction();

        $treeData = [
            'user_id' => $me->id,
            'user_code' => $me->sponsor_code,

            'sponsor_id' => 0,
            'sponsor_code' => NULL,
            'left' => 1,
            'right' => 2,
            'sponsor_level' => 1,
            'sponsor_group' => $me->id,

            'upline_user_id' => 0,
            'upline_user_code' => NULL,
            'upline_left' => 1,
            'upline_right' => 2,
            'children' => 0,
            'level' => 1,
            'position' => 1,
            'group_id' => $me->id
        ];

        $treeData['sponsor_id'] = $sponsor->user_id;
        $treeData['sponsor_code'] = $sponsor->user_code;
        $treeData['left'] = $sponsor->right;
        $treeData['right'] = ($sponsor->right + 1);
        $treeData['sponsor_level'] = ($sponsor->sponsor_level + 1);
        $treeData['sponsor_group'] = $sponsor->sponsor_group;

        DB::update("UPDATE sponsors SET `right` = `right` + 2 WHERE `sponsor_group` = '" . $sponsor->sponsor_group . "' AND `right` >= " . $sponsor->right );
        DB::update("UPDATE sponsors SET `left` = `left` + 2 WHERE `sponsor_group` = '" . $sponsor->sponsor_group . "' AND `left` >= " . $sponsor->right );

        $treeData['upline_user_id'] = $upline_user->user_id;
        $treeData['upline_user_code'] = $upline_user->user_code;

        switch ( $upline_user->children ) {
            case 0:
                $children_num = $leg;
            break;
            case 1:
            case 2:
                $children_num = 3;
            break;
        }

        $upline_user->children = $children_num;
        $upline_user->save();

        if( $leg == 1 ) {

            $treeData['upline_left'] = ($upline_user->upline_left + 1);
            $treeData['upline_right'] = ($upline_user->upline_left + 2);
            $treeData['level'] = ($upline_user->level + 1);
            $treeData['position'] = $leg;
            $treeData['group_id'] = $upline_user->group_id;

            DB::update("UPDATE sponsors SET `upline_left` = `upline_left` + 2 WHERE `group_id` = '" . $upline_user->group_id . "' AND `upline_left` > " . $upline_user->upline_left );
            DB::update("UPDATE sponsors SET `upline_right` = `upline_right` + 2 WHERE `group_id` = '" . $upline_user->group_id . "' AND `upline_right` >= " . $upline_user->upline_left );

        } else {

            $treeData['upline_left'] = $upline_user->upline_right;
            $treeData['upline_right'] = ($upline_user->upline_right + 1);
            $treeData['level'] = ($upline_user->level + 1);
            $treeData['position'] = $leg;
            $treeData['group_id'] = $upline_user->group_id;

            DB::update("UPDATE sponsors SET `upline_left` = `upline_left` + 2 WHERE `group_id` = '" . $upline_user->group_id . "' AND `upline_left` >= " . $upline_user->upline_right );
            DB::update("UPDATE sponsors SET `upline_right` = `upline_right` + 2 WHERE `group_id` = '" . $upline_user->group_id . "' AND `upline_right` >= " . $upline_user->upline_right );

        }

        $new_sponsor = new \App\Models\Sponsor;
        foreach( $treeData as $key => $value ) $new_sponsor->$key = $value;
        $new_sponsor->save();

        //DB::commit();
    }

    public static function deleteFromTree( $user ) {
        DB::beginTransaction();

        // User old datas
        $user_old = clone $user->sponsor;

        // Differences for the update
        $sponsor_difference     = $user_old->right - $user_old->left + 1;
        $upline_difference      = $user_old->upline_right - $user_old->upline_left + 1;
        $sponsor_rebuild_diff   = $user_old->left;
        $upline_rebuild_diff    = $user_old->upline_left;

        // Old upline user
        $old_upline = \App\Models\Sponsor::where('user_id', $user_old->upline_user_id)->where('group_id', $user_old->group_id)->first();
        // First downline legs
        $downline_legs = \App\Models\Sponsor::where('upline_user_id', $user_old->user_id)->where('level', ($user_old->level + 1))->where('group_id', $user_old->group_id)->orderBy('position', 'asc')->get();

        // Old sponsor user
        $old_sponsor = \App\Models\Sponsor::where('user_id', $user_old->sponsor_id)->where('sponsor_group', $user_old->sponsor_group)->first();
        // First sponsor legs
        $sponsor_legs = \App\Models\Sponsor::where('sponsor_id', $user_old->user_id)->where('sponsor_group', $user_old->sponsor_group)->orderBy('left', 'asc')->get();

        // build new tree to each children if any
        if( $downline_legs->count() ) {

            foreach( $downline_legs as $dl ) {
                if( $dl->children > 0 ) {
                    $num_children = DB::update("UPDATE sponsors SET `upline_left` = `upline_left` - ". ($dl->upline_left - 1) .", `upline_right` = `upline_right` - ". ($dl->upline_left - 1) . ", `level` = `level` - " . ($dl->level - 1) . ", `group_id` = '" . $dl->user_id . "' WHERE group_id = '" . $user_old->group_id . "' AND `upline_left` >= " . $dl->upline_left . " AND `upline_right` <= " . $dl->upline_right);
                } else {
                    $num_children = 0;
                }

                $dl->upline_left = 1;
                $dl->upline_right = ( $num_children > 0 ) ? ( ($num_children * 2) + 2 ) : 2;
                $dl->level = 1;
                $dl->position = 1;
                $dl->group_id = $dl->user_id;
                $dl->upline_user_id = 0;
                $dl->upline_user_code = NULL;
                $dl->save();
            }
        }

        // Close the missing tree
        DB::update("UPDATE sponsors SET `upline_left` = `upline_left` - " . $upline_difference . " WHERE `group_id` = '" . $user_old->group_id . "' AND `upline_left` > " . $user_old->upline_right);
        DB::update("UPDATE sponsors SET `upline_right` = `upline_right` - " . $upline_difference . " WHERE `group_id` = '" . $user_old->group_id . "' AND `upline_right` > " . $user_old->upline_right);

        // build new sponsor tree to each children if any
        if( $sponsor_legs->count() ) {

            foreach( $sponsor_legs as $sl ) {
                $num_children = DB::update("UPDATE sponsors SET `left` = `left` - ". ($sl->left - 1) .", `right` = `right` - ". ($sl->left - 1) . ", `sponsor_level` = `sponsor_level` - " . ($sl->sponsor_level - 1) . ", `sponsor_group` = '" . $sl->user_id . "' WHERE `sponsor_group` = '" . $user_old->sponsor_group . "' AND `left` > " . $sl->left . " AND `right` < " . $sl->right);

                $sl->left = 1;
                $sl->right = ( $num_children > 0 ) ? ( ($num_children * 2) + 2 ) : 2;
                $sl->sponsor_level = 1;
                $sl->group_id = $sl->user_id;
                $sl->sponsor_id = 0;
                $sl->sponsor_code = NULL;
                $sl->save();
            }

        }

        // Close the missing tree
        DB::update("UPDATE sponsors SET `left` = `left` - " . $sponsor_difference . " WHERE `sponsor_group` = '" . $user_old->sponsor_group . "' AND `left` > " . $user_old->right);
        DB::update("UPDATE sponsors SET `right` = `right` - " . $sponsor_difference . " WHERE `sponsor_group` = '" . $user_old->sponsor_group . "' AND `right` > " . $user_old->right);

        // Update user upline children counter
        if( !is_null( $old_upline ) ) {
            if( $old_upline->children == 1 || $old_upline->children == 2 ) {
                $old_upline->children = 0;
            } elseif( $old_upline->children == 3 ) {
                $old_upline->children = ($old_upline->children - $user_old->position);
            }

            $old_upline->save();
        }

        // Delete user
        $user_old->delete();

        DB::commit();
    }

    public static function replaceInTree( $new_sponsor, $me ) {
        //DB::enableQueryLog();
        DB::beginTransaction();

        $my_old         = clone $me->sponsor;
        $difference     = $my_old->right - $my_old->left + 1;
        $old_sponsor    = null;
        $old_upline     = null;
        $sponsor_action = 0;
        $upline_action  = 0;
        $setgroup = '';
        $setlevel = '';

        if( $new_sponsor !== NULL )
            $direction = ( $my_old->right > $new_sponsor->right ) ? 0 : 1; // 0 left 1 right
        else 
            $direction = 1;

        if( $my_old->sponsor_id !== 0 ) { // 2-4

            $old_sponsor = \App\Models\Sponsor::where('user_id', $my_old->sponsor_id)
                ->where('sponsor_group', $my_old->sponsor_group)
                ->first();

            if( $new_sponsor !== NULL ) { // 2-3
                if( $my_old->sponsor_group == $new_sponsor->sponsor_group ) { // 2
                    $sponsor_action = 2;
                    //$setgroup = ", `sponsor_group` = '" . $new_sponsor->sponsor_group . "'";

                    if( $old_sponsor->sponsor_level > $new_sponsor->sponsor_level )
                        $setlevel = ", `sponsor_level` = `sponsor_level` - " . ($old_sponsor->sponsor_level - $new_sponsor->sponsor_level);
                    elseif( $old_sponsor->sponsor_level < $new_sponsor->sponsor_level )
                        $setlevel = ", `sponsor_level` = `sponsor_level` + " . ($new_sponsor->sponsor_level - $old_sponsor->sponsor_level);

                } else { // 3
                    $sponsor_action = 3;

                    $setgroup = ", `sponsor_group` = '" . $new_sponsor->sponsor_group . "'";

                    if( $old_sponsor->sponsor_level > $new_sponsor->sponsor_level )
                        $setlevel = ", `sponsor_level` = `sponsor_level` - " . ($old_sponsor->sponsor_level - $new_sponsor->sponsor_level);
                    elseif( $old_sponsor->sponsor_level < $new_sponsor->sponsor_level )
                        $setlevel = ", `sponsor_level` = `sponsor_level` + " . ($new_sponsor->sponsor_level - $old_sponsor->sponsor_level);
                }
            } else { // 4
                $sponsor_action = 4;
                $setgroup = ", `sponsor_group` = '" . $me->id . "'";
                $setlevel = ", `sponsor_level` = (`sponsor_level` + 1) - " . $my_old->sponsor_level;
            }
        } else { // 1
            $sponsor_action = 1;
            $setgroup = ", `sponsor_group` = '" . $new_sponsor->sponsor_group . "'";
            if( $my_old->sponsor_level > $new_sponsor->sponsor_level )
                $setlevel = ", `sponsor_level` = `sponsor_level` - " . ($my_old->sponsor_level - $new_sponsor->sponsor_level);
            elseif( $my_old->sponsor_level < $new_sponsor->sponsor_level )
                $setlevel = ", `sponsor_level` = `sponsor_level` + " . ($my_old->sponsor_level - $old_sponsor->sponsor_level);
        }


        if( $my_old->upline_user_id !== 0 ) {
            $old_upline = \App\Models\Sponsor::where('user_id', $my_old->upline_user_id)
                ->where('group_id', $my_old->group_id)
                ->first();

            if( $new_upline !== NULL ) { // 2-3
                if( $my_old->group_id == $new_upline->group_id ) // 2
                    $upline_action = 2;
                else  // 3
                    $upline_action = 3;
            } else // 4
                $upline_action = 4;
        } else // 1
            $upline_action = 1;

        /*$my_downline = \App\Models\Sponsor::where('user_id', '!=', $me->id)
            ->whereBetween('left', [ $my_old->left, $my_old->right ])
            ->where('sponsor_group', $my_old->sponsor_group)
            ->orderBy('sponsor_level', 'ASC')
            ->orderBy('left', 'ASC')
            ->get();*/

        DB::update("UPDATE sponsors SET 
            `left` = `left` * -1, 
            `right` = `right` * -1
            WHERE `left` >= " . $my_old->left . " AND `right` <= " . $my_old->right . " AND `sponsor_group` = '" . $my_old->sponsor_group . "'");

        switch ( $sponsor_action ) {
            // No sponsor
            case 1:

                DB::update("UPDATE sponsors SET `right` = `right` + " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `right` >= " . $new_sponsor->right );
                DB::update("UPDATE sponsors SET `left` = `left` + " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `left` >= " . $new_sponsor->right );

                break;
            // Came from different sponsor group
            case 3:

                DB::update("UPDATE sponsors SET `right` = `right` + " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `right` >= " . $new_sponsor->right );
                DB::update("UPDATE sponsors SET `left` = `left` + " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `left` >= " . $new_sponsor->right );

                DB::update("UPDATE sponsors SET `left` = `left` - " . $difference . " WHERE `sponsor_group` = '" . $my_old->sponsor_group . "' AND `left` > " . $my_old->right);
                DB::update("UPDATE sponsors SET `right` = `right` - " . $difference . " WHERE `sponsor_group` = '" . $my_old->sponsor_group . "' AND `right` > " . $my_old->right);

                break;
            // Replaced in the same group
            case 2:

                if( $direction ) { // Move right

                    DB::update("UPDATE sponsors SET `left` = `left` - " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `left` > " . $my_old->right);
                    DB::update("UPDATE sponsors SET `left` = `left` + " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `left` > " . $new_sponsor->right);
                    DB::update("UPDATE sponsors SET `right` = `right` - " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `right` > " . $my_old->right . " AND `right` < " . $new_sponsor->right );

                } else { // Move left

                    DB::update("UPDATE sponsors SET `left` = `left` + " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `left` >= " . $new_sponsor->right . " AND `left` < " . $my_old->right);

                    DB::update("UPDATE sponsors SET `right` = `right` + " . $difference . " WHERE `sponsor_group` = '" . $new_sponsor->sponsor_group . "' AND `right` >= " . $new_sponsor->right . " AND `right` < " . $my_old->left );

                }
            
                break;
            // Remove
            case 4:

                DB::update("UPDATE sponsors SET `left` = `left` - " . $difference . " WHERE `sponsor_group` = '" . $my_old->sponsor_group . "' AND `left` > " . $my_old->right);
                DB::update("UPDATE sponsors SET `right` = `right` - " . $difference . " WHERE `sponsor_group` = '" . $my_old->sponsor_group . "' AND `right` > " . $my_old->right);

                break;

            default:
                throw new \Exception('Wrong action!');
                break;
        }

        if( $sponsor_action < 4 ) {
            $me->sponsor->sponsor_id = $new_sponsor->user_id;
            $me->sponsor->sponsor_code = $new_sponsor->user_code;
            $me->sponsor->sponsor_group = $new_sponsor->sponsor_group;
            $me->sponsor->sponsor_level = $new_sponsor->sponsor_level + 1;
        }


        //if( $my_downline->count() ) {

        if( $sponsor_action == 1 ) {
            if( $direction ) { // Move right
                $downline_difference = $my_old->left - ($new_sponsor->right - $difference);
                DB::update("UPDATE sponsors SET 
                    `left` = (`left` - " . $downline_difference . ") * -1, 
                    `right` = (`right` - " . $downline_difference . ") * -1" . $setgroup . $setlevel . "
                    WHERE `left` < 0 AND `sponsor_group` = '" . $my_old->sponsor_group . "'");
            }
            else { // Move left
                $downline_difference = $my_old->left - $new_sponsor->right;
                DB::update("UPDATE sponsors SET 
                    `left` = (`left` + " . $downline_difference . ") * -1, 
                    `right` = (`right` + " . $downline_difference . ") * -1 " . $setgroup . $setlevel . "
                    WHERE `left` < 0 AND `sponsor_group` = '" . $my_old->sponsor_group . "'");
            }
        } else {

            if( $direction ) { // Move right
                $downline_difference = $my_old->left - ($new_sponsor->right - $difference);
                DB::update("UPDATE sponsors SET 
                    `left` = (`left` + " . $downline_difference . ") * -1, 
                    `right` = (`right` + " . $downline_difference . ") * -1" . $setgroup . $setlevel . "
                    WHERE `left` < 0 AND `sponsor_group` = '" . $my_old->sponsor_group . "'");
            }
            else { // Move left
                $downline_difference = $my_old->left - $new_sponsor->right;
                DB::update("UPDATE sponsors SET 
                    `left` = (`left` + " . $downline_difference . ") * -1, 
                    `right` = (`right` + " . $downline_difference . ") * -1 " . $setgroup . $setlevel . "
                    WHERE `left` < 0 AND `sponsor_group` = '" . $my_old->sponsor_group . "'");
            }

        }

            $me->sponsor->save();

        //}

            //echo '<pre>';
            //print_r(DB::getQueryLog());
            //exit;

        DB::commit();
    }

	public function getDates()
	{
	 return [];
	}

}