<?php namespace App\Models;

use Dimsav\Translatable\Translatable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Str;

class Transactions extends \Illuminate\Database\Eloquent\Model {

  use SoftDeletes;
  use Translatable;

  protected $dates = ['created_at', 'updated_at', 'deleted_at'];
  protected $table = 'transactions';

  public $translatedAttributes = ['note'];

  public static $types = [
    // Credits 1-11
  	1 => 'Package Purchase',
          'Marketing Commission', //2
      3 => 'Direct Commission', //3
          'Associate Team Rewards', //4
          'Matching Bonus', //5
      6 => 'Deposit', //6
          'Logistic Pool Bonus', //7
          'Transfer From Member', //8
          'Transfer From Member', //9
          'Misc Account Credits', //10
          'Sell Product', //11

          // Debits 12-21
          'Withdrawal', //12
      13 => 'Purchase Package', //13
          'Transfer To Member', //14
          'Transfer To Member', // 15
          'Misc Account Debits', //16
          'Buy Product', //17
          'Buy Giftcard', //18
          'Create Certificate (retiring)', //19
          'Create Certificate (retailing)', //20
          'Donate Tree', //21
      31 => 'Profit Share',

  ];

  public $currencies = [
    '$ :val' => [1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 16, 17, 18],
    //':val CV' => [3],
    ':val Units' => [9, 15, 19, 20, 21]
  ];

  public static $icons = [
    1 => 'briefcase',
    'briefcase',
    'chart_curve',
    'group',
    'arrow_switch',
    'application_add',
    'briefcase',
    'briefcase',
    'briefcase',
    'briefcase',
    'briefcase',

    'briefcase',
    'package',
    'arrow_switch',
    'application_delete',
    'money_delete',
    'briefcase',
    'briefcase',
    'briefcase',
    'briefcase',
    'briefcase'
  ];

  public function netAmount()
  {
    //$fee = ( $this->fee / 100 ) * $this->amount;
    return $this->amount - $this->fee;
  }

  public function getTypeNameAttribute( )
  {
    return self::$types[ $this->type ];
  }

  public function getTypeIconAttribute( $val )
  {
    return self::$icons[ $this->type ];
  }

  public function getAmountNumber( $val ) {
    return $this->amount;
  }

  public function getAmountAttribute( $val )
  {
    foreach( $this->currencies as $k => $c ) {
      if( in_array($this->type, $c) )
        $current = $k;
    }

    return Str::replaceFirst(':val', $val, $current);
  }

  public function icon()
  {
    return $this->icons[ $this->type ];
  }

  public function user()
  {
  	return $this->belongsTo('\App\Models\User', 'user_id', 'id');
  }

  public function processors()
  {
    return $this->belongsTo('\App\Models\Processors', 'processor_id', 'id');
  }

  public function userProc()
  {
    return $this->hasOne('\App\Models\UsersProcessors', 'id', 'user_proc_id');
  }

}
