<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Config;

class UserAddresses extends Model
{
    use SoftDeletes;

    protected $table = 'user_addresses';
    public $timestamps = true;

    public function user() {
        return $this->belongsTo('\App\Models\User', 'user_id', 'id');
    }

    public function hasExpired() {
        return ( $this->created_at->addHours( Config::get('blockchain.address.expiration') )->timestamp <= Carbon::now()->timestamp );
    }
    public function package() {
        return $this->belongsTo('\App\Models\Packages', 'package_id', 'id');
    }

    public function payments() {
        return $this->hasMany('\App\Models\UserPayments', 'address_id', 'id');
    }

    public function scopePast24H( $query ) {
        return $query->whereRaw('NOW() >= DATE_ADD(deleted_at, INTERVAL 24 HOUR)');
    }

    public function uppercaseCurrency(){
        return strtoupper($this->currency);
    }

    public function receivedPercent(){
        return bcdiv($this->received / $this->amount * 100, 1, 2);

    }
}
